@@ -22,6 +22,7 @@
 */
 
 #include <config.h>
+
 #include <opm/simulators/flow/ActionHandler.hpp>
 
 #include <opm/common/OpmLog/OpmLog.hpp>
@@ -29,6 +30,7 @@
 #include <opm/common/TimingMacros.hpp>
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
+
 #include <opm/input/eclipse/Schedule/Action/ActionContext.hpp>
 #include <opm/input/eclipse/Schedule/Action/Actions.hpp>
 #include <opm/input/eclipse/Schedule/Action/ActionX.hpp>
@@ -40,6 +42,8 @@
 #include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 
 #include <opm/simulators/wells/BlackoilWellModelGeneric.hpp>
+
+#include <opm/simulators/utils/ParallelCommunication.hpp>
 #include <opm/simulators/utils/ParallelSerialization.hpp>
 
 #include <chrono>
@@ -51,6 +55,7 @@
 
 #include <fmt/chrono.h>
 #include <fmt/format.h>
+#include <fmt/ranges.h>
 
 namespace {
     std::string formatActionDate(const Opm::TimeStampUTC& timePoint,
@@ -70,9 +75,20 @@ namespace {
                            time_point, reportStep, reportStep + 1);
     }
 
-    void logActiveAction(const std::string&              actionName,
-                         const std::vector<std::string>& matchingWells,
-                         const std::string&              timeString)
+    void logActivePyAction(const std::string& actionName,
+                           const std::string& timeString)
+    {
+        const auto message =
+            fmt::format("Action {} (Python) triggered at {}",
+                        actionName, timeString);
+
+        Opm::OpmLog::info("ACTION_TRIGGERED", message);
+    }
+
+    template <typename WellNameRange>
+    void logActiveAction(const std::string&   actionName,
+                         const WellNameRange& matchingWells,
+                         const std::string&   timeString)
     {
         const auto wellString = matchingWells.empty()
             ? std::string{}
@@ -87,6 +103,16 @@ namespace {
         Opm::OpmLog::info("ACTION_TRIGGERED", message);
     }
 
+    void logInactivePyAction(const std::string& actionName,
+                             const std::string& timeString)
+    {
+        const auto message =
+            fmt::format("Action {} (Python) NOT triggered at {}.",
+                        actionName, timeString);
+
+        Opm::OpmLog::debug("NAMED_ACTION_NOT_TRIGGERED", message);
+    }
+
     void logInactiveAction(const std::string& actionName,
                            const std::string& timeString)
     {
@@ -108,6 +134,34 @@ namespace {
 
         Opm::OpmLog::debug("ACTION_NOT_TRIGGERED", message);
     }
+
+    template <typename Scalar, class WellModel>
+    std::unordered_map<std::string, Scalar>
+    fetchWellPI(const WellModel&                             wellModel,
+                const std::vector<std::string>&              wellpi_wells,
+                const Opm::Parallel::Communication           comm)
+    {
+        auto wellpi = std::unordered_map<std::string, Scalar> {};
+
+        if (wellpi_wells.empty()) {
+            return wellpi;
+        }
+
+        auto wellPI = std::vector<Scalar>(wellpi_wells.size());
+        for (auto i = 0*wellpi_wells.size(); i < wellpi_wells.size(); ++i) {
+            if (wellModel.hasLocalWell(wellpi_wells[i])) {
+                wellPI[i] = wellModel.wellPI(wellpi_wells[i]);
+            }
+        }
+
+        comm.max(wellPI.data(), wellPI.size());
+
+        for (auto i = 0*wellpi_wells.size(); i < wellpi_wells.size(); ++i) {
+            wellpi.emplace(wellpi_wells[i], wellPI[i]);
+        }
+
+        return wellpi;
+    }
 } // Anonymous namespace
 
 namespace Opm {
@@ -145,40 +199,63 @@ applyActions(const int reportStep,
     const auto now = TimeStampUTC{ schedule_.getStartTime() } + std::chrono::duration<double>(sim_time);
     const auto ts  = formatActionDate(now, reportStep);
 
-    bool commit_wellstate = false;
+
+    SimulatorUpdate sim_update;
     for (const auto& pyaction : actions.pending_python(actionState_)) {
-        auto sim_update = schedule_.runPyAction(reportStep, *pyaction, actionState_,
-                                                ecl_state_, summaryState_);
-        this->applySimulatorUpdate(reportStep, sim_update, commit_wellstate, transUp);
+        // The std::unordered_map<std::string, Scalar> wellpi contains the well production indices from the last
+        // timestep. This map is needed for the keyword WELPI. For a PyAction, we do not know which wells are
+        // affected by the PyAction, thus we get the production indices for all wells.
+        const std::vector<std::string> wellpi_wells = schedule_[reportStep].well_order().names();
+        const auto wellpi = fetchWellPI<Scalar>(this->wellModel_, wellpi_wells, this->comm_);
+        auto sim_update_current = schedule_.runPyAction(reportStep, *pyaction, actionState_,
+                                                ecl_state_, summaryState_, wellpi);
+
+        if (const auto pyRes = this->actionState_.python_result(pyaction->name());
+            !pyRes.has_value() || !*pyRes)
+        {
+            logInactivePyAction(pyaction->name(), ts);
+            continue;
+        }
+        else {
+            logActivePyAction(pyaction->name(), ts);
+        }
+
+        sim_update.append(sim_update_current);
     }
 
     auto non_triggered = 0;
     const auto simTime = asTimeT(now);
-    for (const auto& action : actions.pending(actionState_, simTime)) {
+    for (const auto& action : actions.pending(this->actionState_, simTime)) {
         const auto actionResult = action->eval(context);
-        if (! actionResult) {
+        if (! actionResult.conditionSatisfied()) {
             ++non_triggered;
             logInactiveAction(action->name(), ts);
             continue;
         }
 
-        const auto& matching_wells = actionResult.wells();
+        const auto& matches = actionResult.matches();
 
-        logActiveAction(action->name(), matching_wells, ts);
+        logActiveAction(action->name(), matches.wells(), ts);
 
-        const auto wellpi = this->fetchWellPI(reportStep, *action, matching_wells);
+        // The std::unordered_map<std::string, Scalar> wellpi contains the well production indices from the last
+        // timestep. This map is needed for the keyword WELPI. For an ActionX, we know which wells are affected by
+        // the ActionX, thus fetch the well production indices only for the affected wells.
+        const std::vector<std::string> wellpi_wells = action->wellpi_wells(schedule_.wellMatcher(reportStep), matches);
+        const auto wellpi = fetchWellPI<Scalar>(this->wellModel_, wellpi_wells, this->comm_);
 
-        const auto sim_update = this->schedule_
-            .applyAction(reportStep, *action, matching_wells, wellpi);
+        const auto sim_update_current = this->schedule_
+            .applyAction(reportStep, *action, matches, wellpi);
 
-        this->applySimulatorUpdate(reportStep, sim_update, commit_wellstate, transUp);
-        actionState_.add_run(*action, simTime, std::move(actionResult));
+        sim_update.append(sim_update_current);
+        this->actionState_.add_run(*action, simTime, actionResult);
     }
 
     if (non_triggered > 0) {
         logInactiveActions(non_triggered, ts);
     }
 
+    bool commit_wellstate = false;
+    this->applySimulatorUpdate(reportStep, sim_update, transUp, commit_wellstate);
     // The well state has been stored in a previous object when the time
     // step has completed successfully, the action process might have
     // modified the well state, and to be certain that is not overwritten
@@ -192,8 +269,8 @@ template<class Scalar>
 void ActionHandler<Scalar>::
 applySimulatorUpdate(const int report_step,
                      const SimulatorUpdate& sim_update,
-                     bool& commit_wellstate,
-                     const TransFunc& updateTrans)
+                     const TransFunc& updateTrans,
+                     bool& commit_wellstate)
 {
     OPM_TIMEBLOCK(applySimulatorUpdate);
 
@@ -213,66 +290,17 @@ applySimulatorUpdate(const int report_step,
     }
 }
 
-template<class Scalar>
-std::unordered_map<std::string, Scalar>
-ActionHandler<Scalar>::
-fetchWellPI(const int reportStep,
-            const Action::ActionX& action,
-            const std::vector<std::string>& matching_wells) const
-{
-  auto wellpi_wells = action.wellpi_wells(WellMatcher(schedule_[reportStep].well_order(),
-                                                      schedule_[reportStep].wlist_manager()),
-                                          matching_wells);
-
-  if (wellpi_wells.empty())
-      return {};
-
-  const auto num_wells = schedule_[reportStep].well_order().size();
-  std::vector<Scalar> wellpi_vector(num_wells);
-  for (const auto& wname : wellpi_wells) {
-      if (this->wellModel_.hasWell(wname)) {
-          const auto& well = schedule_.getWell( wname, reportStep );
-          wellpi_vector[well.seqIndex()] = this->wellModel_.wellPI(wname);
-      }
-  }
-
-  if (comm_.size() > 1) {
-      std::vector<Scalar> wellpi_buffer(num_wells * comm_.size());
-      comm_.gather( wellpi_vector.data(), wellpi_buffer.data(), num_wells, 0 );
-      if (comm_.rank() == 0) {
-          for (int rank = 1; rank < comm_.size(); rank++) {
-              for (std::size_t well_index=0; well_index < num_wells; well_index++) {
-                  const auto global_index = rank*num_wells + well_index;
-                  const auto value = wellpi_buffer[global_index];
-                  if (value != 0)
-                      wellpi_vector[well_index] = value;
-              }
-          }
-      }
-      comm_.broadcast(wellpi_vector.data(), wellpi_vector.size(), 0);
-  }
-
-  std::unordered_map<std::string, Scalar> wellpi;
-  for (const auto& wname : wellpi_wells) {
-      const auto& well = schedule_.getWell( wname, reportStep );
-      wellpi[wname] = wellpi_vector[ well.seqIndex() ];
-  }
-  return wellpi;
-}
-
 template<class Scalar>
 void ActionHandler<Scalar>::
 evalUDQAssignments(const unsigned episodeIdx,
                    UDQState& udq_state)
 {
-    const auto& udq = schedule_[episodeIdx].udq();
-
-    udq.eval_assign(episodeIdx,
-                    this->schedule_,
-                    this->schedule_.wellMatcher(episodeIdx),
-                    this->schedule_.segmentMatcherFactory(episodeIdx),
-                    this->summaryState_,
-                    udq_state);
+    this->schedule_[episodeIdx].udq()
+        .eval_assign(this->schedule_.wellMatcher(episodeIdx),
+                     this->schedule_[episodeIdx].group_order(),
+                     this->schedule_.segmentMatcherFactory(episodeIdx),
+                     this->summaryState_,
+                     udq_state);
 }
 
 template class ActionHandler<double>;
