@@ -34,10 +34,12 @@
 #include <opm/common/OpmLog/OpmLog.hpp>
 
 #include <opm/grid/CpGrid.hpp>
+#include <opm/grid/cpgrid/LevelCartesianIndexMapper.hpp>
 
 #include <opm/models/common/multiphasebaseproperties.hh>
 
 #include <opm/simulators/flow/AluGridCartesianIndexMapper.hpp>
+#include <opm/simulators/flow/AluGridLevelCartesianIndexMapper.hpp>
 #include <opm/simulators/flow/FlowBaseVanguard.hpp>
 #include <opm/simulators/flow/Transmissibility.hpp>
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
@@ -106,6 +108,7 @@ public:
     using EquilGrid = GetPropType<TypeTag, Properties::EquilGrid>;
     using GridView = GetPropType<TypeTag, Properties::GridView>;        
     using CartesianIndexMapper = Dune::CartesianIndexMapper<Grid>;
+    using LevelCartesianIndexMapper = Opm::LevelCartesianIndexMapper<Grid>;
     using EquilCartesianIndexMapper = Dune::CartesianIndexMapper<EquilGrid>;
     using TransmissibilityType = Transmissibility<Grid, GridView, ElementMapper, CartesianIndexMapper, Scalar>;
     using Factory = Dune::FromToGridFactory<Grid>;
@@ -113,7 +116,7 @@ public:
     static constexpr int dimension = Grid::dimension;
     static constexpr int dimensionworld = Grid::dimensionworld;
 
-    AluGridVanguard(Simulator& simulator)
+    explicit AluGridVanguard(Simulator& simulator)
         : FlowBaseVanguard<TypeTag>(simulator)
     {
       this->mpiRank = FlowGenericVanguard::comm().rank();
@@ -193,7 +196,12 @@ public:
             globalTrans_->update(false, TransmissibilityType::TransUpdateQuantities::Trans,
                                  [&](unsigned int i) { return gridEquilIdxToGridIdx(i);});
         }
-        
+
+    }
+
+    void addLgrs()
+    {
+    // do nothing: AluGrid with LGRs not supported yet!
     }
 
     template<class DataHandle>
@@ -232,6 +240,14 @@ public:
     const CartesianIndexMapper& cartesianIndexMapper() const
     { return *cartesianIndexMapper_; }
 
+    /*!
+     * \brief Returns the object which maps a global element index of the simulation grid
+     *        to the corresponding element index of the level logically Cartesian index.
+     *        No refinement is supported for AluGrid so it coincides with CartesianIndexMapper.
+     */
+    const LevelCartesianIndexMapper levelCartesianIndexMapper() const
+    { return LevelCartesianIndexMapper(*cartesianIndexMapper_); }
+
     /*!
      * \brief Returns mapper from compressed to cartesian indices for the EQUIL grid
      */
