@@ -66,12 +66,14 @@ BlackoilModelParameters<Scalar>::BlackoilModelParameters()
     min_strict_cnv_iter_ = Parameters::Get<Parameters::MinStrictCnvIter>();
     min_strict_mb_iter_ = Parameters::Get<Parameters::MinStrictMbIter>();
     solve_welleq_initially_ = Parameters::Get<Parameters::SolveWelleqInitially>();
+    pre_solve_network_ = Parameters::Get<Parameters::PreSolveNetwork>();
     update_equations_scaling_ = Parameters::Get<Parameters::UpdateEquationsScaling>();
     use_update_stabilization_ = Parameters::Get<Parameters::UseUpdateStabilization>();
     matrix_add_well_contributions_ = Parameters::Get<Parameters::MatrixAddWellContributions>();
     check_well_operability_ = Parameters::Get<Parameters::EnableWellOperabilityCheck>();
     check_well_operability_iter_ = Parameters::Get<Parameters::EnableWellOperabilityCheckIter>();
     max_number_of_well_switches_ = Parameters::Get<Parameters::MaximumNumberOfWellSwitches>();
+    max_number_of_group_switches_ = Parameters::Get<Parameters::MaximumNumberOfGroupSwitches>();
     use_average_density_ms_wells_ = Parameters::Get<Parameters::UseAverageDensityMsWells>();
     local_well_solver_control_switching_ = Parameters::Get<Parameters::LocalWellSolveControlSwitching>();
     use_implicit_ipr_ = Parameters::Get<Parameters::UseImplicitIpr>();
@@ -91,17 +93,23 @@ BlackoilModelParameters<Scalar>::BlackoilModelParameters()
     local_tolerance_scaling_cnv_ = Parameters::Get<Parameters::LocalToleranceScalingCnv<Scalar>>();
     nldd_num_initial_newton_iter_ = Parameters::Get<Parameters::NlddNumInitialNewtonIter>();
     num_local_domains_ = Parameters::Get<Parameters::NumLocalDomains>();
-    local_domain_partition_imbalance_ = std::max(Scalar{1.0}, Parameters::Get<Parameters::LocalDomainsPartitioningImbalance<Scalar>>());
-    local_domain_partition_method_ = Parameters::Get<Parameters::LocalDomainsPartitioningMethod>();
+    local_domains_partition_imbalance_ = std::max(Scalar{1.0}, Parameters::Get<Parameters::LocalDomainsPartitioningImbalance<Scalar>>());
+    local_domains_partition_method_ = Parameters::Get<Parameters::LocalDomainsPartitioningMethod>();
+    local_domains_partition_well_neighbor_levels_ = Parameters::Get<Parameters::LocalDomainsPartitionWellNeighborLevels>();
     deck_file_name_ = Parameters::Get<Parameters::EclDeckFileName>();
-    network_max_strict_iterations_ = Parameters::Get<Parameters::NetworkMaxStrictIterations>();
-    network_max_iterations_ = Parameters::Get<Parameters::NetworkMaxIterations>();
-    local_domain_ordering_ = domainOrderingMeasureFromString(Parameters::Get<Parameters::LocalDomainsOrderingMeasure>());
+    network_max_strict_outer_iterations_ = Parameters::Get<Parameters::NetworkMaxStrictOuterIterations>();
+    network_max_outer_iterations_ = Parameters::Get<Parameters::NetworkMaxOuterIterations>();
+    network_max_sub_iterations_ = Parameters::Get<Parameters::NetworkMaxSubIterations>();
+    network_pressure_update_damping_factor_ = Parameters::Get<Parameters::NetworkPressureUpdateDampingFactor<Scalar>>();
+    network_max_pressure_update_in_bars_ = Parameters::Get<Parameters::NetworkMaxPressureUpdateInBars<Scalar>>();
+    local_domains_ordering_ = domainOrderingMeasureFromString(Parameters::Get<Parameters::LocalDomainsOrderingMeasure>());
     write_partitions_ = Parameters::Get<Parameters::DebugEmitCellPartition>();
 
-    convergence_monitoring_ = Parameters::Get<Parameters::ConvergenceMonitoring>();
-    convergence_monitoring_cutoff_ = Parameters::Get<Parameters::ConvergenceMonitoringCutOff>();
-    convergence_monitoring_decay_factor_ = Parameters::Get<Parameters::ConvergenceMonitoringDecayFactor<Scalar>>();
+    monitor_params_.enabled_ = Parameters::Get<Parameters::ConvergenceMonitoring>();
+    monitor_params_.cutoff_ = Parameters::Get<Parameters::ConvergenceMonitoringCutOff>();
+    monitor_params_.decay_factor_ = Parameters::Get<Parameters::ConvergenceMonitoringDecayFactor<Scalar>>();
+
+    nupcol_group_rate_tolerance_ = Parameters::Get<Parameters::NupcolGroupRateTolerance<Scalar>>();
 }
 
 template<class Scalar>
@@ -190,6 +198,8 @@ void BlackoilModelParameters<Scalar>::registerParameters()
          "number of Newton iterations are reached.");
     Parameters::Register<Parameters::SolveWelleqInitially>
         ("Fully solve the well equations before each iteration of the reservoir model");
+    Parameters::Register<Parameters::PreSolveNetwork>
+        ("Pre solve and iterate the network model at start-up");
     Parameters::Register<Parameters::UpdateEquationsScaling>
         ("Update scaling factors for mass balance equations during the run");
     Parameters::Register<Parameters::UseUpdateStabilization>
@@ -203,6 +213,8 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Enable the well operability checking during iterations");
     Parameters::Register<Parameters::MaximumNumberOfWellSwitches>
         ("Maximum number of times a well can switch to the same control");
+    Parameters::Register<Parameters::MaximumNumberOfGroupSwitches>
+        ("Maximum number of times a group can switch to the same control");
     Parameters::Register<Parameters::UseAverageDensityMsWells>
         ("Approximate segment densitities by averaging over segment and its outlet");
     Parameters::Register<Parameters::LocalWellSolveControlSwitching>
@@ -211,10 +223,16 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Compute implict IPR for stability checks and stable solution search");
     Parameters::Register<Parameters::CheckGroupConstraintsInnerWellIterations>
         ("Allow checking of group constraints during inner well iterations");        
-    Parameters::Register<Parameters::NetworkMaxStrictIterations>
-        ("Maximum iterations in network solver before relaxing tolerance");
-    Parameters::Register<Parameters::NetworkMaxIterations>
-        ("Maximum number of iterations in the network solver before giving up");
+    Parameters::Register<Parameters::NetworkMaxStrictOuterIterations>
+        ("Maximum outer iterations in network solver before relaxing tolerance");
+    Parameters::Register<Parameters::NetworkMaxOuterIterations>
+        ("Maximum outer number of iterations in the network solver before giving up");
+    Parameters::Register<Parameters::NetworkMaxSubIterations>
+        ("Maximum number of sub-iterations to update network pressures (within a single well/group control update)");
+    Parameters::Register<Parameters::NetworkPressureUpdateDampingFactor<Scalar>>
+        ("Damping factor in the inner network pressure update iterations");
+    Parameters::Register<Parameters::NetworkMaxPressureUpdateInBars<Scalar>>
+        ("Maximum pressure update in the inner network pressure update iterations");
     Parameters::Register<Parameters::NonlinearSolver>
         ("Choose nonlinear solver. Valid choices are newton or nldd.");
     Parameters::Register<Parameters::LocalSolveApproach>
@@ -236,6 +254,8 @@ void BlackoilModelParameters<Scalar>::registerParameters()
          "'zoltan', "
          "'simple', "
          "and the name of a partition file ending with '.partition'.");
+    Parameters::Register<Parameters::LocalDomainsPartitionWellNeighborLevels>
+        ("Number of neighbor levels around wells to include in the same domain during NLDD partitioning");
     Parameters::Register<Parameters::LocalDomainsOrderingMeasure>
         ("Subdomain ordering measure. Allowed values are "
          "'maxpressure', "
@@ -251,11 +271,14 @@ void BlackoilModelParameters<Scalar>::registerParameters()
     Parameters::Register<Parameters::ConvergenceMonitoringDecayFactor<Scalar>>
         ("Decay factor for convergence monitoring");
 
+    Parameters::Register<Parameters::NupcolGroupRateTolerance<Scalar>>
+        ("Tolerance for acceptable changes in VREP/RAIN group rates");
+
     Parameters::Hide<Parameters::DebugEmitCellPartition>();
 
-    // if openMP is available, determine the number threads per process automatically.
+    // if openMP is available, use two threads per mpi rank by default
 #if _OPENMP
-    Parameters::SetDefault<Parameters::ThreadsPerProcess>(-1);
+    Parameters::SetDefault<Parameters::ThreadsPerProcess>(2);
 #endif
 }
 
