@@ -33,10 +33,12 @@
 #include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
 
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
+#include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
@@ -173,7 +175,7 @@ struct EclWriteTasklet : public Opm::TaskletInterface
     {}
 
     // callback to eclIO serial writeTimeStep method
-    void run()
+    void run() override
     {
         this->eclIO_.writeTimeStep(this->actionState_,
                                    this->wtestState_,
@@ -631,7 +633,7 @@ evalSummary(const int                                            reportStepNum,
             const std::map<std::string, double>&                 miscSummaryData,
             const std::map<std::string, std::vector<double>>&    regionData,
             const Inplace&                                       inplace,
-            const Inplace&                                       initialInPlace,
+            const std::optional<Inplace>&                        initialInPlace,
             const InterRegFlowMap&                               interRegFlows,
             SummaryState&                                        summaryState,
             UDQState&                                            udqState)
@@ -675,10 +677,10 @@ evalSummary(const int                                            reportStepNum,
         // step we are currently on.
         const auto udq_step = reportStepNum - 1;
 
-        this->schedule_.getUDQConfig(udq_step)
+        this->schedule_[udq_step].udq()
             .eval(udq_step,
-                  this->schedule_,
                   this->schedule_.wellMatcher(udq_step),
+                  this->schedule_[udq_step].group_order(),
                   this->schedule_.segmentMatcherFactory(udq_step),
                   [es = std::cref(this->eclState_)]() {
                       return std::make_unique<RegionSetMatcher>
