@@ -30,23 +30,30 @@
 
 #include <dune/grid/common/partitionset.hh>
 
+#include <opm/common/TimingMacros.hpp> // OPM_TIMEBLOCK
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
+#include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
 
+#include <opm/output/eclipse/Inplace.hpp>
 #include <opm/output/eclipse/RestartValue.hpp>
 
+#include <opm/models/blackoil/blackoilproperties.hh> // Properties::EnableMech, EnableTemperature, EnableSolvent
+#include <opm/models/common/multiphasebaseproperties.hh> // Properties::FluidSystem
+
 #include <opm/simulators/flow/CollectDataOnIORank.hpp>
 #include <opm/simulators/flow/countGlobalCells.hpp>
 #include <opm/simulators/flow/EclGenericWriter.hpp>
 #include <opm/simulators/flow/FlowBaseVanguard.hpp>
-#include <opm/simulators/flow/OutputBlackoilModule.hpp>
 #include <opm/simulators/timestepping/SimulatorTimer.hpp>
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/utils/ParallelRestart.hpp>
 #include <opm/simulators/utils/ParallelSerialization.hpp>
 
+#include <boost/date_time/posix_time/posix_time.hpp>
+
 #include <limits>
 #include <map>
 #include <memory>
@@ -58,9 +65,6 @@
 
 namespace Opm::Parameters {
 
-// enable the ECL output by default
-struct EnableEclOutput { static constexpr bool value = true; };
-
 // If available, write the ECL output in a non-blocking manner
 struct EnableAsyncEclOutput { static constexpr bool value = true; };
 
@@ -101,7 +105,7 @@ namespace Opm {
  * - This class requires to use the black oil model with the element
  *   centered finite volume discretization.
  */
-template <class TypeTag>
+template <class TypeTag, class OutputModule>
 class EclWriter : public EclGenericWriter<GetPropType<TypeTag, Properties::Grid>,
                                           GetPropType<TypeTag, Properties::EquilGrid>,
                                           GetPropType<TypeTag, Properties::GridView>,
@@ -120,7 +124,7 @@ class EclWriter : public EclGenericWriter<GetPropType<TypeTag, Properties::Grid>
     using ElementMapper = GetPropType<TypeTag, Properties::ElementMapper>;
     using ElementIterator = typename GridView::template Codim<0>::Iterator;
     using BaseType = EclGenericWriter<Grid,EquilGrid,GridView,ElementMapper,Scalar>;
-    
+
     typedef Dune::MultipleCodimMultipleGeomTypeMapper< GridView > VertexMapper;
 
     enum { enableEnergy = getPropValue<TypeTag, Properties::EnableEnergy>() };
@@ -132,7 +136,7 @@ public:
 
     static void registerParameters()
     {
-        OutputBlackOilModule<TypeTag>::registerParameters();
+        OutputModule::registerParameters();
 
         Parameters::Register<Parameters::EnableAsyncEclOutput>
             ("Write the ECL-formated results in a non-blocking way "
@@ -144,7 +148,7 @@ public:
     // The Simulator object should preferably have been const - the
     // only reason that is not the case is due to the SummaryState
     // object owned deep down by the vanguard.
-    EclWriter(Simulator& simulator)
+    explicit EclWriter(Simulator& simulator)
         : BaseType(simulator.vanguard().schedule(),
                    simulator.vanguard().eclState(),
                    simulator.vanguard().summaryConfig(),
@@ -169,13 +173,13 @@ public:
 
             eclBroadcast(this->simulator_.vanguard().grid().comm(), smryCfg);
 
-            this->outputModule_ = std::make_unique<OutputBlackOilModule<TypeTag>>
+            this->outputModule_ = std::make_unique<OutputModule>
                 (simulator, smryCfg, this->collectOnIORank_);
         }
         else
 #endif
         {
-            this->outputModule_ = std::make_unique<OutputBlackOilModule<TypeTag>>
+            this->outputModule_ = std::make_unique<OutputModule>
                 (simulator, this->eclIO_->finalSummaryConfig(), this->collectOnIORank_);
         }
 
@@ -199,7 +203,7 @@ public:
     {
         OPM_TIMEBLOCK(evalSummaryState);
         const int reportStepNum = simulator_.episodeIndex() + 1;
-        
+
         /*
           The summary data is not evaluated for timestep 0, that is
           implemented with a:
@@ -217,7 +221,7 @@ public:
           "Correct" in this context means unchanged behavior, might very
           well be more correct to actually remove this if test.
         */
-        
+
         if (reportStepNum == 0)
             return;
 
@@ -243,8 +247,10 @@ public:
         if (this->collectOnIORank_.isParallel()) {
             OPM_BEGIN_PARALLEL_TRY_CATCH()
 
+            std::map<std::pair<std::string,int>,double> dummy;
             this->collectOnIORank_.collect({},
                                            outputModule_->getBlockData(),
+                                           dummy,
                                            localWellData,
                                            localWBP,
                                            localGroupAndNetworkData,
@@ -270,22 +276,22 @@ public:
             OPM_END_PARALLEL_TRY_CATCH("Collect to I/O rank: ",
                                        this->simulator_.vanguard().grid().comm());
         }
-        
+
 
         std::map<std::string, double> miscSummaryData;
         std::map<std::string, std::vector<double>> regionData;
         Inplace inplace;
-        
+
         {
             OPM_TIMEBLOCK(outputFipLogAndFipresvLog);
 
             inplace = outputModule_->calc_inplace(miscSummaryData, regionData, simulator_.gridView().comm());
-            
+
             if (this->collectOnIORank_.isIORank()){
                 inplace_ = inplace;
             }
         }
-        
+
         // Add TCPU
         if (totalCpuTime != 0.0) {
             miscSummaryData["TCPU"] = totalCpuTime;
@@ -305,7 +311,7 @@ public:
         if (this->sub_step_report_.max_linear_iterations != 0) {
             miscSummaryData["NLINSMAX"] = this->sub_step_report_.max_linear_iterations;
         }
-        if (this->simulation_report_.success.total_newton_iterations != 0) {
+        if (this->simulation_report_.success.total_linear_iterations != 0) {
             miscSummaryData["MSUMLINS"] = this->simulation_report_.success.total_linear_iterations;
         }
         if (this->simulation_report_.success.total_newton_iterations != 0) {
@@ -343,12 +349,6 @@ public:
     //! \brief Writes the initial FIP report as configured in RPTSOL.
     void writeInitialFIPReport()
     {
-        const auto& fip = simulator_.vanguard().eclState().getEclipseConfig().fip();
-        if (!fip.output(FIPConfig::OutputField::FIELD) &&
-            !fip.output(FIPConfig::OutputField::RESV)) {
-            return;
-        }
-
         const auto& gridView = simulator_.vanguard().gridView();
         const int num_interior = detail::
             countLocalInteriorCellsGridView(gridView);
@@ -366,44 +366,70 @@ public:
             this->outputModule_->updateFluidInPlace(dofIdx, intQuants, totVolume);
         }
 
-        std::map<std::string, double> miscSummaryData;
-        std::map<std::string, std::vector<double>> regionData;
-        Inplace inplace;
+        // We always calculate the initial fip values as it may be used by various
+        // keywords in the Schedule, e.g. FIP=2 in RPTSCHED but no FIP in RPTSOL
+        outputModule_->calc_initial_inplace(simulator_.gridView().comm());
+
+        // check if RPTSOL entry has FIP output
+        const auto& fip = simulator_.vanguard().eclState().getEclipseConfig().fip();
+        if (fip.output(FIPConfig::OutputField::FIELD) ||
+            fip.output(FIPConfig::OutputField::RESV))
         {
             OPM_TIMEBLOCK(outputFipLogAndFipresvLog);
-            
-            boost::posix_time::ptime start_time = boost::posix_time::from_time_t(simulator_.vanguard().schedule().getStartTime());
+            boost::posix_time::ptime start_time =
+                boost::posix_time::from_time_t(simulator_.vanguard().schedule().getStartTime());
 
-            inplace = outputModule_->calc_inplace(miscSummaryData, regionData, simulator_.gridView().comm());
-
-            if (this->collectOnIORank_.isIORank()){
-                inplace_ = inplace;
-                
+            if (this->collectOnIORank_.isIORank()) {
+                inplace_ = outputModule_->initialInplace().value();
                 outputModule_->outputFipAndResvLog(inplace_, 0, 0.0, start_time,
                                                   false, simulator_.gridView().comm());
             }
         }
     }
 
-    void writeReports(const SimulatorTimer& timer) {
-        auto rstep = timer.reportStepNum();
-
-        if ((rstep > 0) && (this->collectOnIORank_.isIORank())){
-
-            const auto& rpt = this->schedule_[rstep-1].rpt_config.get();
-            if (rpt.contains("WELLS") && rpt.at("WELLS") > 0) {
-                outputModule_->outputTimeStamp("WELLS", timer.simulationTimeElapsed(), rstep, timer.currentDateTime());
-                outputModule_->outputProdLog(rstep-1);
-                outputModule_->outputInjLog(rstep-1);
-                outputModule_->outputCumLog(rstep-1);
-            }
-
-            outputModule_->outputFipAndResvLog(inplace_, rstep, timer.simulationTimeElapsed(),
-                                               timer.currentDateTime(), false, simulator_.gridView().comm());
-
-
-            OpmLog::note("");   // Blank line after all reports.
+    void writeReports(const SimulatorTimer& timer)
+    {
+        if (! this->collectOnIORank_.isIORank()) {
+            return;
         }
+
+        // SimulatorTimer::reportStepNum() is the simulator's zero-based
+        // "episode index".  This is generally the index value needed to
+        // look up objects in the Schedule container.  That said, function
+        // writeReports() is invoked at the *beginning* of a report
+        // step/episode which means we typically need the objects from the
+        // *previous* report step/episode.  We therefore need special case
+        // handling for reportStepNum() == 0 in base runs and
+        // reportStepNum() <= restart step in restarted runs.
+        const auto firstStep = this->initialStep();
+        const auto simStep =
+            std::max(timer.reportStepNum() - 1, firstStep);
+
+        const auto& rpt = this->schedule_[simStep].rpt_config();
+
+        if (rpt.contains("WELSPECS") && (rpt.at("WELSPECS") > 0)) {
+            // Requesting a well specification report is valid at all times,
+            // including reportStepNum() == initialStep().
+            this->writeWellspecReport(timer);
+        }
+
+        if (timer.reportStepNum() == firstStep) {
+            // No dynamic flows at the beginning of the initialStep().
+            return;
+        }
+
+        if (rpt.contains("WELLS") && rpt.at("WELLS") > 0) {
+            this->writeWellflowReport(timer, simStep, rpt.at("WELLS"));
+        }
+
+        this->outputModule_->outputFipAndResvLog(this->inplace_,
+                                                 timer.reportStepNum(),
+                                                 timer.simulationTimeElapsed(),
+                                                 timer.currentDateTime(),
+                                                 /* isSubstep = */ false,
+                                                 simulator_.gridView().comm());
+
+        OpmLog::note("");   // Blank line after all reports.
     }
 
     void writeOutput(data::Solution&& localCellData, bool isSubStep)
@@ -422,13 +448,12 @@ public:
         auto localAquiferData = simulator_.problem().aquiferModel().aquiferData();
         auto localWellTestState = simulator_.problem().wellModel().wellTestState();
 
-        const bool isFlowsn = this->outputModule_->hasFlowsn();
-        auto flowsn = this->outputModule_->getFlowsn();
+        const bool isFlowsn = this->outputModule_->getFlows().hasFlowsn();
+        auto flowsn = this->outputModule_->getFlows().getFlowsn();
 
-        const bool isFloresn = this->outputModule_->hasFloresn();
-        auto floresn = this->outputModule_->getFloresn();
+        const bool isFloresn = this->outputModule_->getFlows().hasFloresn();
+        auto floresn = this->outputModule_->getFlows().getFloresn();
 
-        // data::Solution localCellData = {};
         if (! isSubStep || Parameters::Get<Parameters::EnableWriteAllSolutions>()) {
 
             if (localCellData.empty()) {
@@ -436,7 +461,9 @@ public:
             }
 
             // Add cell data to perforations for RFT output
-            this->outputModule_->addRftDataToWells(localWellData, reportStepNum);
+            this->outputModule_->addRftDataToWells(localWellData,
+                                                   reportStepNum,
+                                                   simulator_.gridView().comm());
         }
 
         if (this->collectOnIORank_.isParallel() ||
@@ -449,6 +476,7 @@ public:
 
             this->collectOnIORank_.collect(localCellData,
                                            this->outputModule_->getBlockData(),
+                                           this->outputModule_->getExtraBlockData(),
                                            localWellData,
                                            /* wbpData = */ {},
                                            localGroupAndNetworkData,
@@ -467,7 +495,7 @@ public:
         if (this->collectOnIORank_.isIORank()) {
             const Scalar curTime = simulator_.time() + simulator_.timeStepSize();
             const Scalar nextStepSize = simulator_.problem().nextTimeStepSize();
-            std::optional<int> timeStepIdx; 
+            std::optional<int> timeStepIdx;
             if (Parameters::Get<Parameters::EnableWriteAllSolutions>()) {
                 timeStepIdx = simulator_.timeStepIndex();
             }
@@ -546,6 +574,37 @@ public:
         const auto& gridView = this->simulator_.vanguard().gridView();
         const auto numElements = gridView.size(/*codim=*/0);
 
+        // Try to load restart step 0 to calculate initial FIP
+        {
+            this->outputModule_->allocBuffers(numElements,
+                                              0,
+                                              /*isSubStep = */false,
+                                              /*log = */      false,
+                                              /*isRestart = */true);
+
+            const auto restartSolution =
+                loadParallelRestartSolution(this->eclIO_.get(),
+                                            solutionKeys, gridView.comm(), 0);
+
+            if (!restartSolution.empty()) {
+                for (auto elemIdx = 0*numElements; elemIdx < numElements; ++elemIdx) {
+                    const auto globalIdx = this->collectOnIORank_.localIdxToGlobalIdx(elemIdx);
+                    this->outputModule_->setRestart(restartSolution, elemIdx, globalIdx);
+                }
+
+                this->simulator_.problem().readSolutionFromOutputModule(0, true);
+                ElementContext elemCtx(this->simulator_);
+                for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
+                    elemCtx.updatePrimaryStencil(elem);
+                    elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
+
+                    this->outputModule_->updateFluidInPlace(elemCtx);
+                }
+
+                this->outputModule_->calc_initial_inplace(this->simulator_.gridView().comm());
+            }
+        }
+
         {
             // The episodeIndex is rewound one step back before calling
             // beginRestart() and cannot be used here.  We just ask the
@@ -636,33 +695,22 @@ public:
 
     void endRestart()
     {
-        // We need these objects to satisfy the interface requirements of
-        // member function calc_inplace(), but the objects are otherwise
-        // unused and intentionally so.
-        auto miscSummaryData = std::map<std::string, double>{};
-        auto regionData = std::map<std::string, std::vector<double>>{};
-
-        // Note: calc_inplace() *also* assigns the output module's
-        // "initialInplace_" data member.  This is, semantically speaking,
-        // very wrong, as the run's intial volumes then correspond to the
-        // volumes at the restart time instead of the start of the base run.
-        // Nevertheless, this is how Flow has "always" done it.
-        //
-        // See GenericOutputBlackoilModule::accumulateRegionSums() for
-        // additional comments.
-        auto inplace = this->outputModule_
-            ->calc_inplace(miscSummaryData, regionData,
-                           this->simulator_.gridView().comm());
+        // Calculate initial in-place volumes.
+        // Does nothing if they have already been calculated,
+        // e.g. from restart data at T=0.
+        this->outputModule_->calc_initial_inplace(this->simulator_.gridView().comm());
 
         if (this->collectOnIORank_.isIORank()) {
-            this->inplace_ = std::move(inplace);
+            if (this->outputModule_->initialInplace().has_value()) {
+                this->inplace_ = this->outputModule_->initialInplace().value();
+            }
         }
     }
 
-    const OutputBlackOilModule<TypeTag>& outputModule() const
+    const OutputModule& outputModule() const
     { return *outputModule_; }
 
-    OutputBlackOilModule<TypeTag>& mutableOutputModule() const
+    OutputModule& mutableOutputModule() const
     { return *outputModule_; }
 
     Scalar restartTimeStepSize() const
@@ -723,46 +771,17 @@ private:
             OPM_TIMEBLOCK(prepareCellBasedData);
 
             this->outputModule_->prepareDensityAccumulation();
-
+            this->outputModule_->setupExtractors(isSubStep, reportStepNum);
             for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
                 elemCtx.updatePrimaryStencil(elem);
                 elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
 
                 this->outputModule_->processElement(elemCtx);
-            }
-
-            this->outputModule_->accumulateDensityParallel();
-        }
-
-        if constexpr (enableMech) {
-            if (simulator_.vanguard().eclState().runspec().mech()) {
-                OPM_TIMEBLOCK(prepareMechData);
-                for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
-                    elemCtx.updatePrimaryStencil(elem);
-                    elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
-                    outputModule_->processElementMech(elemCtx);
-                }
-            }
-        }
-
-        if (! this->simulator_.model().linearizer().getFlowsInfo().empty()) {
-            OPM_TIMEBLOCK(prepareFlowsData);
-            for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
-                elemCtx.updatePrimaryStencil(elem);
-                elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
-
-                this->outputModule_->processElementFlows(elemCtx);
-            }
-        }
-
-        {
-            OPM_TIMEBLOCK(prepareBlockData);
-            for (const auto& elem : elements(gridView, Dune::Partitions::interior)) {
-                elemCtx.updatePrimaryStencil(elem);
-                elemCtx.updatePrimaryIntensiveQuantities(/*timeIdx=*/0);
-
                 this->outputModule_->processElementBlockData(elemCtx);
             }
+            this->outputModule_->clearExtractors();
+
+            this->outputModule_->accumulateDensityParallel();
         }
 
         {
@@ -823,8 +842,49 @@ private:
         this->outputModule_->finalizeFluxData();
     }
 
+    void writeWellspecReport(const SimulatorTimer& timer) const
+    {
+        const auto changedWells = this->schedule_
+            .changed_wells(timer.reportStepNum(), this->initialStep());
+
+        if (changedWells.empty()) {
+            return;
+        }
+
+        this->outputModule_->outputWellspecReport(changedWells,
+                                                  timer.reportStepNum(),
+                                                  timer.simulationTimeElapsed(),
+                                                  timer.currentDateTime());
+    }
+
+    void writeWellflowReport(const SimulatorTimer& timer,
+                             const int             simStep,
+                             const int             wellsRequest) const
+    {
+        this->outputModule_->outputTimeStamp("WELLS",
+                                             timer.simulationTimeElapsed(),
+                                             timer.reportStepNum(),
+                                             timer.currentDateTime());
+
+        const auto wantConnData = wellsRequest > 1;
+
+        this->outputModule_->outputProdLog(simStep, wantConnData);
+        this->outputModule_->outputInjLog(simStep, wantConnData);
+        this->outputModule_->outputCumLog(simStep, wantConnData);
+        this->outputModule_->outputMSWLog(simStep);
+    }
+
+    int initialStep() const
+    {
+        const auto& initConfig = this->eclState().cfg().init();
+
+        return initConfig.restartRequested()
+            ? initConfig.getRestartStep()
+            : 0;
+    }
+
     Simulator& simulator_;
-    std::unique_ptr<OutputBlackOilModule<TypeTag> > outputModule_;
+    std::unique_ptr<OutputModule> outputModule_;
     Scalar restartTimeStepSize_;
     int rank_ ;
     Inplace inplace_;
