@@ -28,7 +28,6 @@
 #ifndef OPM_FLOW_GENERIC_PROBLEM_HPP
 #define OPM_FLOW_GENERIC_PROBLEM_HPP
 
-
 #include <opm/material/common/UniformXTabulated2DFunction.hpp>
 #include <opm/material/common/Tabulated1DFunction.hpp>
 
@@ -37,7 +36,6 @@
 #include <array>
 #include <cstddef>
 #include <functional>
-#include <set>
 #include <string>
 #include <vector>
 
@@ -167,15 +165,10 @@ public:
      */
     Scalar solventRsw(unsigned elemIdx) const;
 
-    /*!
-     * \brief Returns the dynamic drsdt convective mixing value
-     */
-    Scalar drsdtcon(unsigned elemIdx, int episodeIdx) const;
-
     /*!
      * \brief Returns the initial polymer concentration for a given a cell index
      */
-    Scalar  polymerConcentration(unsigned elemIdx) const;
+    Scalar polymerConcentration(unsigned elemIdx) const;
 
     /*!
     * \brief Returns the polymer molecule weight for a given cell index
@@ -186,27 +179,27 @@ public:
     /*!
      * \brief Returns the initial microbial concentration for a given a cell index
      */
-    Scalar  microbialConcentration(unsigned elemIdx) const;
+    Scalar microbialConcentration(unsigned elemIdx) const;
 
     /*!
      * \brief Returns the initial oxygen concentration for a given a cell index
      */
-    Scalar  oxygenConcentration(unsigned elemIdx) const;
+    Scalar oxygenConcentration(unsigned elemIdx) const;
 
     /*!
      * \brief Returns the initial urea concentration for a given a cell index
      */
-    Scalar  ureaConcentration(unsigned elemIdx) const;
+    Scalar ureaConcentration(unsigned elemIdx) const;
 
     /*!
      * \brief Returns the initial biofilm concentration for a given a cell index
      */
-    Scalar  biofilmConcentration(unsigned elemIdx) const;
+    Scalar biofilmConcentration(unsigned elemIdx) const;
 
     /*!
      * \brief Returns the initial calcite concentration for a given a cell index
      */
-    Scalar  calciteConcentration(unsigned elemIdx) const;
+    Scalar calciteConcentration(unsigned elemIdx) const;
 
     /*!
      * \brief Returns the index the relevant PVT region given a cell index
@@ -245,15 +238,6 @@ public:
      */
     Scalar rockCompressibility(unsigned globalSpaceIdx) const;
 
-    /*!
-     * Direct access to rock reference pressure.
-     *
-     * While the above overload could be implemented in terms of this method,
-     * that would require always looking up the global space index, which
-     * is not always needed.
-     */
-    Scalar rockReferencePressure(unsigned globalSpaceIdx) const;
-
     /*!
      * \brief Direct indexed access to the porosity.
      *
@@ -364,6 +348,9 @@ protected:
     // equilibration parameters
     int numPressurePointsEquil_;
 
+    bool enableDriftCompensation_;
+    bool explicitRockCompaction_;
+
     // To lookup origin cell indices
     using Grid = std::remove_cv_t< typename std::remove_reference<decltype(gridView_.grid())>::type>;
     using LookUpData = Opm::LookUpData<Grid,GridView>;
