@@ -23,6 +23,11 @@
 #ifndef OPM_FLOW_GENERIC_PROBLEM_IMPL_HPP
 #define OPM_FLOW_GENERIC_PROBLEM_IMPL_HPP
 
+#ifndef OPM_FLOW_GENERIC_PROBLEM_HPP
+#include <config.h>
+#include <opm/simulators/flow/FlowGenericProblem.hpp>
+#endif
+
 #include <dune/common/parametertree.hh>
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
@@ -31,9 +36,15 @@
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Units/Units.hpp>
 
-#include <opm/simulators/flow/FlowGenericProblem.hpp>
+#include <opm/models/discretization/common/fvbaseparameters.hh>
+#include <opm/models/utils/basicparameters.hh>
+#include <opm/models/utils/parametersystem.hpp>
+
+#include <opm/simulators/flow/FlowProblemParameters.hpp>
 #include <opm/simulators/flow/SolutionContainers.hpp>
 
+#include <opm/simulators/timestepping/EclTimeSteppingParams.hpp>
+
 #include <boost/date_time.hpp>
 
 #include <fmt/format.h>
@@ -54,6 +65,24 @@ FlowGenericProblem(const EclipseState& eclState,
     , gridView_(gridView)
     , lookUpData_(gridView)
 {
+    enableTuning_ = Parameters::Get<Parameters::EnableTuning>();
+    enableDriftCompensation_ = Parameters::Get<Parameters::EnableDriftCompensation>();
+    initialTimeStepSize_ = Parameters::Get<Parameters::InitialTimeStepSize<Scalar>>();
+    maxTimeStepAfterWellEvent_ = unit::convert::from
+        (Parameters::Get<Parameters::TimeStepAfterEventInDays<Scalar>>(), unit::day);
+
+    // The value N for this parameter is defined in the following order of precedence:
+    //
+    // 1. Command line value (--num-pressure-points-equil=N)
+    //
+    // 2. EQLDIMS item 2.  Default value from
+    //    opm-common/opm/input/eclipse/share/keywords/000_Eclipse100/E/EQLDIMS
+
+    numPressurePointsEquil_ = Parameters::IsSet<Parameters::NumPressurePointsEquil>()
+        ? Parameters::Get<Parameters::NumPressurePointsEquil>()
+        : eclState.getTableManager().getEqldims().getNumDepthNodesP();
+
+    explicitRockCompaction_ = Parameters::Get<Parameters::ExplicitRockCompaction>();
 }
 
 template<class GridView, class FluidSystem>
@@ -116,6 +145,11 @@ readRockParameters_(const std::vector<Scalar>& cellCenterDepths,
         }
     }
 
+    // Warn that ROCK and ROCKOPTS item 2 = STORE is used together
+    if (rock_config.store()) {
+        OpmLog::warning("ROCKOPTS item 2 set to STORE, ROCK item 1 replaced with initial (equilibrated) pressures");
+    }
+
     // read the parameters for water-induced rock compaction
     readRockCompactionParameters_();
 
@@ -152,7 +186,7 @@ readRockParameters_(const std::vector<Scalar>& cellCenterDepths,
 
     // Store overburden pressure pr element
     const auto& overburdTables = eclState_.getTableManager().getOverburdTables();
-    if (!overburdTables.empty()) {
+    if (!overburdTables.empty() && !rock_config.store()) {
         overburdenPressure_.resize(numElem,0.0);
         std::size_t numRocktabTables = rock_config.num_rock_tables();
 
@@ -174,6 +208,9 @@ readRockParameters_(const std::vector<Scalar>& cellCenterDepths,
                 overburdenTables[tableIdx].eval(cellCenterDepths[elemIdx], /*extrapolation=*/true);
         }
     }
+    else if (!overburdTables.empty() && rock_config.store()) {
+        OpmLog::warning("ROCKOPTS item 2 set to STORE, OVERBURD ignored!");
+    } 
 }
 
 template<class GridView, class FluidSystem>
@@ -284,21 +321,6 @@ rockCompressibility(unsigned globalSpaceIdx) const
     return this->rockParams_[tableIdx].compressibility;
 }
 
-template<class GridView, class FluidSystem>
-typename FlowGenericProblem<GridView,FluidSystem>::Scalar
-FlowGenericProblem<GridView,FluidSystem>::
-rockReferencePressure(unsigned globalSpaceIdx) const
-{
-    if (this->rockParams_.empty())
-        return 1e5;
-
-    unsigned tableIdx = 0;
-    if (!this->rockTableIdx_.empty()) {
-        tableIdx = this->rockTableIdx_[globalSpaceIdx];
-    }
-    return this->rockParams_[tableIdx].referencePressure;
-}
-
 template<class GridView, class FluidSystem>
 typename FlowGenericProblem<GridView,FluidSystem>::Scalar
 FlowGenericProblem<GridView,FluidSystem>::
