@@ -49,11 +49,13 @@
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
 #include <opm/input/eclipse/Schedule/Group/GroupEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
+#include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/Network/Balance.hpp>
 #include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
 #include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 #include <opm/input/eclipse/Schedule/OilVaporizationProperties.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/ReservoirCouplingInfo.hpp>
 #include <opm/input/eclipse/Schedule/RFTConfig.hpp>
 #include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
@@ -106,6 +108,39 @@ FlowGenericVanguard::FlowGenericVanguard(SimulationModelParams&& params)
     : python(std::make_shared<Python>())
 {
     defineSimulationModel(std::move(params));
+
+    fileName_ = Parameters::Get<Parameters::EclDeckFileName>();
+    edgeWeightsMethod_   = Dune::EdgeWeightMethod(Parameters::Get<Parameters::EdgeWeightsMethod>());
+
+#if HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
+    numJacobiBlocks_ = Parameters::Get<Parameters::NumJacobiBlocks>();
+#endif // HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
+
+    ownersFirst_ = Parameters::Get<Parameters::OwnerCellsFirst>();
+
+#if HAVE_MPI
+    numOverlap_ = Parameters::Get<Parameters::NumOverlap>();
+    addCorners_ = Parameters::Get<Parameters::AddCorners>();
+    partitionMethod_ = Dune::PartitionMethod(Parameters::Get<Parameters::PartitionMethod>());
+    serialPartitioning_ = Parameters::Get<Parameters::SerialPartitioning>();
+    zoltanParams_ = Parameters::Get<Parameters::ZoltanParams>();
+    zoltanPhgEdgeSizeThreshold_ = Parameters::Get<Parameters::ZoltanPhgEdgeSizeThreshold>();
+    metisParams_ = Parameters::Get<Parameters::MetisParams>();
+
+    externalPartitionFile_ = Parameters::Get<Parameters::ExternalPartition>();
+#endif // HAVE_MPI
+
+    enableDistributedWells_ = Parameters::Get<Parameters::AllowDistributedWells>();
+    enableEclOutput_ = Parameters::Get<Parameters::EnableEclOutput>();
+    allow_splitting_inactive_wells_ = Parameters::Get<Parameters::AllowSplittingInactiveWells>();
+    ignoredKeywords_ = Parameters::Get<Parameters::IgnoreKeywords>();
+
+    const int output_param = Parameters::Get<Parameters::EclOutputInterval>();
+    if (output_param >= 0) {
+        outputInterval_ = output_param;
+    }
+
+    useMultisegmentWell_ = Parameters::Get<Parameters::UseMultisegmentWell>();
 }
 
 FlowGenericVanguard::SimulationModelParams
@@ -149,7 +184,7 @@ void FlowGenericVanguard::readDeck(const std::string& filename)
                   modelParams_.actionState_,
                   modelParams_.wtestState_,
                   modelParams_.eclSummaryConfig_,
-                  nullptr, "normal", "normal", "100", false, false, false, {});
+                  nullptr, "normal", "normal", "100", false, false, false, {}, /*slaveMode=*/false);
     modelParams_.setupTime_ = setupTimer.stop();
 }
 
@@ -320,15 +355,12 @@ void FlowGenericVanguard::init()
         {
             if (comm.rank() == 0)
             {
-                std::string message =
-                        std::string("Option --allow-distributed-wells=true is only allowed if model\n")
-                        + "only has only standard wells. You need to provide option \n"
-                        + " with --enable-multisegement-wells=false to treat existing \n"
-                        + "multisegment wells as standard wells.";
-                OpmLog::error(message);
+                OpmLog::info("Option --allow-distributed-wells=true in a model with\n"
+                             "multisegment wells. This feature is still experimental. You can\n"
+                             "set --use-multisegment-well=false to treat the existing\n"
+                             "multisegment wells as standard wells.");
             }
             comm.barrier();
-            OPM_THROW(std::invalid_argument, "All wells need to be standard wells!");
         }
     }
 }
@@ -390,6 +422,8 @@ void FlowGenericVanguard::registerParameters_()
         ("The number of report steps that ought to be skipped between two writes of ECL results");
     Parameters::Register<Parameters::EnableDryRun>
         ("Specify if the simulation ought to be actually run, or just pretended to be");
+    Parameters::Register<Parameters::EnableEclOutput>
+        ("Write binary output which is compatible with the commercial Eclipse simulator");
     Parameters::Register<Parameters::EnableOpmRstFile>
         ("Include OPM-specific keywords in the ECL restart file to "
          "enable restart of OPM simulators from these files");
@@ -419,31 +453,41 @@ void FlowGenericVanguard::registerParameters_()
 #if HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
     Parameters::Register<Parameters::NumJacobiBlocks>
         ("Number of blocks to be created for the Block-Jacobi preconditioner.");
-#endif
+#endif // HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
 
     Parameters::Register<Parameters::OwnerCellsFirst>
         ("Order cells owned by rank before ghost/overlap cells.");
 #if HAVE_MPI
+    Parameters::Register<Parameters::AddCorners>
+        ("Add corners to partition.");
+    Parameters::Register<Parameters::NumOverlap>
+        ("Numbers of layers overlap in parallel partition");
     Parameters::Register<Parameters::PartitionMethod>
-        ("Choose partitioning strategy: 0=simple, 1=Zoltan, 2=METIS.");
+        ("Choose partitioning strategy: 0=simple, 1=Zoltan, 2=METIS, "
+         "3=Zoltan with all cells of well represented by one vertex.");
     Parameters::Register<Parameters::SerialPartitioning>
         ("Perform partitioning for parallel runs on a single process.");
     Parameters::Register<Parameters::ZoltanImbalanceTol<Scalar>>
-        ("Tolerable imbalance of the loadbalancing provided by Zoltan. DEPRECATED: Use --imbalance-tol instead");
+        ("Tolerable imbalance of the loadbalancing provided by Zoltan. "
+         "DEPRECATED: Use --imbalance-tol instead.");
     Parameters::Register<Parameters::ZoltanParams>
         ("Configuration of Zoltan partitioner. "
          "Valid options are: graph, hypergraph or scotch. "
          "Alternatively, you can request a configuration to be read "
-         "from a JSON file by giving the filename here, ending with '.json.' "
+         "from a JSON file by giving the filename here, with extension '.json'. "
          "See https://sandialabs.github.io/Zoltan/ug_html/ug.html "
          "for available Zoltan options.");
     Parameters::Register<Parameters::ImbalanceTol<Scalar>>
-        ("Tolerable imbalance of the loadbalancing (default: 1.1).");
+        ("Tolerable imbalance of the loadbalancing.");
+    Parameters::Register<Parameters::ZoltanPhgEdgeSizeThreshold>
+        ("Low-level threshold fraction in the range [0,1] controlling "
+         "which hypergraph edge to omit. Used if --zoltan-params=\"graph\" "
+         "or if --zoltan-params=\"hypergraph\".");
     Parameters::Register<Parameters::MetisParams>
         ("Configuration of Metis partitioner. "
          "You can request a configuration to be read "
-         "from a JSON file by giving the filename here, ending with '.json.' "
-         "See http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf"
+         "from a JSON file by giving the filename here, with extension '.json'. "
+         "See http://glaros.dtc.umn.edu/gkhome/fetch/sw/metis/manual.pdf "
          "for available METIS options.");
     Parameters::Register<Parameters::ExternalPartition>
         ("Name of file from which to load an externally generated "
@@ -454,9 +498,13 @@ void FlowGenericVanguard::registerParameters_()
 
     Parameters::Hide<Parameters::ZoltanImbalanceTol<Scalar>>();
     Parameters::Hide<Parameters::ZoltanParams>();
-#endif
+    Parameters::Hide<Parameters::ZoltanPhgEdgeSizeThreshold>();
+#endif // HAVE_MPI
+
     Parameters::Register<Parameters::AllowDistributedWells>
         ("Allow the perforations of a well to be distributed to interior of multiple processes");
+    Parameters::Register<Parameters::AllowSplittingInactiveWells>
+        ("Allow inactive (never non-shut) wells to be split across multiple domains");
     // register here for the use in the tests without BlackoilModelParameters
     Parameters::Register<Parameters::UseMultisegmentWell>
         ("Use the well model for multi-segment wells instead of the one for single-segment wells");
@@ -466,6 +514,6 @@ template void FlowGenericVanguard::registerParameters_<double>();
 
 #if FLOW_INSTANTIATE_FLOAT
 template void FlowGenericVanguard::registerParameters_<float>();
-#endif
+#endif // FLOW_INSTANTIATE_FLOAT
 
 } // namespace Opm
