@@ -30,8 +30,12 @@
 
 #include <opm/simulators/flow/Banners.hpp>
 #include <opm/simulators/flow/FlowUtils.hpp>
+#include <opm/simulators/flow/NlddReporting.hpp>
 #include <opm/simulators/flow/SimulatorFullyImplicitBlackoil.hpp>
 
+#if HAVE_MPI
+#define RESERVOIR_COUPLING_ENABLED
+#endif
 #if HAVE_DUNE_FEM
 #include <dune/fem/misc/mpimanager.hh>
 #else
@@ -50,7 +54,6 @@ namespace Opm::Parameters {
 
 // Do not merge parallel output files or warn about them
 struct EnableLoggingFalloutWarning { static constexpr bool value = false; };
-
 struct OutputInterval { static constexpr int value = 1; };
 
 } // namespace Opm::Parameters
@@ -101,12 +104,11 @@ namespace Opm {
                 ("Developer option to see whether logging was on non-root processors. "
                  "In that case it will be appended to the *.DBG or *.PRT files");
 
-            ThreadManager::registerParameters();
-            Simulator::registerParameters();
-
             // register the base parameters
             registerAllParameters_<TypeTag>(/*finalizeRegistration=*/false);
 
+            Simulator::registerParameters();
+
             detail::hideUnusedParameters<Scalar>();
 
             Parameters::endRegistration();
@@ -156,6 +158,9 @@ namespace Opm {
                 }
             }
 
+            // set the maximum limit on OMP threads
+            setMaxThreads();
+
             return status;
         }
 
@@ -278,12 +283,18 @@ namespace Opm {
             mpi_rank_ = comm.rank();
             mpi_size_ = comm.size();
 
+            setMaxThreads();
+        }
+
+        static void setMaxThreads()
+        {
 #if _OPENMP
             // If openMP is available, default to 2 threads per process unless
             // OMP_NUM_THREADS is set or command line --threads-per-process used.
             // Issue a warning if both OMP_NUM_THREADS and --threads-per-process are set,
             // but let the environment variable take precedence.
             constexpr int default_threads = 2;
+            const bool isSet = Parameters::IsSet<Parameters::ThreadsPerProcess>();
             const int requested_threads = Parameters::Get<Parameters::ThreadsPerProcess>();
             int threads = requested_threads > 0 ? requested_threads : default_threads;
 
@@ -294,19 +305,19 @@ namespace Opm {
                 if (result.ec == std::errc() && omp_num_threads > 0) {
                     // Set threads to omp_num_threads if it was successfully parsed and is positive
                     threads = omp_num_threads;
-                    // Warning in 'Main.hpp', where this code is duplicated
-                    // if (requested_threads > 0) {
-                    //     OpmLog::warning("Environment variable OMP_NUM_THREADS takes precedence over the --threads-per-process cmdline argument.");
-                    // }
+                    if (isSet) {
+                        OpmLog::warning("Environment variable OMP_NUM_THREADS takes precedence over the --threads-per-process cmdline argument.");
+                    }
                 } else {
                     OpmLog::warning("Invalid value for OMP_NUM_THREADS environment variable.");
                 }
             }
 
-            // We are not limiting this to the number of processes
-            // reported by OpenMP as on some hardware (and some OpenMPI
-            // versions) this will be 1 when run with mpirun
-            omp_set_num_threads(threads);
+            // Requesting -1 thread will let OMP automatically deduce the number
+            // but setting OMP_NUM_THREADS takes precedence.
+            if (env_var || !(isSet && requested_threads == -1)) {
+                omp_set_num_threads(threads);
+            }
 #endif
 
             using TM = GetPropType<TypeTag, Properties::ThreadManager>;
@@ -358,7 +369,11 @@ namespace Opm {
         // Callback that will be called from runSimulatorInitOrRun_().
         int runSimulatorRunCallback_()
         {
+#ifdef RESERVOIR_COUPLING_ENABLED
+            SimulatorReport report = simulator_->run(*simtimer_, this->argc_, this->argv_);
+#else
             SimulatorReport report = simulator_->run(*simtimer_);
+#endif
             runSimulatorAfterSim_(report);
             return report.success.exit_status;
         }
@@ -366,13 +381,28 @@ namespace Opm {
         // Callback that will be called from runSimulatorInitOrRun_().
         int runSimulatorInitCallback_()
         {
+#ifdef RESERVOIR_COUPLING_ENABLED
+            simulator_->init(*simtimer_, this->argc_, this->argv_);
+#else
             simulator_->init(*simtimer_);
+#endif
             return EXIT_SUCCESS;
         }
 
         // Output summary after simulation has completed
         void runSimulatorAfterSim_(SimulatorReport &report)
         {
+            if (simulator_->model().hasNlddSolver()) {
+                const auto& odir = eclState().getIOConfig().getOutputDir();
+                // Write the number of nonlinear iterations per cell to a file in ResInsight compatible format
+                simulator_->model().writeNonlinearIterationsPerCell(odir);
+                // Write the NLDD statistics to the DBG file
+                reportNlddStatistics(simulator_->model().domainAccumulatedReports(),
+                                     simulator_->model().localAccumulatedReports(),
+                                     this->output_cout_,
+                                     FlowGenericVanguard::comm());
+            }
+
             if (! this->output_cout_) {
                 return;
             }
@@ -384,7 +414,7 @@ namespace Opm {
                 = omp_get_max_threads();
 #endif
 
-            printFlowTrailer(mpi_size_, threads, total_setup_time_, deck_read_time_, report, simulator_->model().localAccumulatedReports());
+            printFlowTrailer(mpi_size_, threads, total_setup_time_, deck_read_time_, report);
 
             detail::handleExtraConvergenceOutput(report,
                                                  Parameters::Get<Parameters::OutputExtraConvergenceInfo>(),
