@@ -37,8 +37,8 @@
 #include <opm/common/utility/TimeService.hpp>
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/E.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
+#include <opm/input/eclipse/Units/Units.hpp>
 
 #include <opm/material/common/ConditionalStorage.hpp>
 #include <opm/material/common/Valgrind.hpp>
@@ -72,8 +72,10 @@
 #include <opm/utility/CopyablePtr.hpp>
 
 #include <algorithm>
+#include <cstddef>
 #include <functional>
 #include <set>
+#include <stdexcept>
 #include <string>
 #include <vector>
 
@@ -170,7 +172,6 @@ public:
     using BaseType::shouldWriteOutput;
     using BaseType::shouldWriteRestartFile;
     using BaseType::rockCompressibility;
-    using BaseType::rockReferencePressure;
     using BaseType::porosity;
 
     /*!
@@ -183,7 +184,6 @@ public:
         registerFlowProblemParameters<Scalar>();
     }
 
-
     /*!
      * \copydoc FvBaseProblem::handlePositionalParameter
      */
@@ -224,38 +224,20 @@ public:
         , pffDofData_(simulator.gridView(), this->elementMapper())
         , tracerModel_(simulator)
     {
-        const auto& vanguard = simulator.vanguard();
-
-        enableDriftCompensation_ = Parameters::Get<Parameters::EnableDriftCompensation>();
-
-        enableVtkOutput_ = Parameters::Get<Parameters::EnableVtkOutput>();
-
-        this->enableTuning_ = Parameters::Get<Parameters::EnableTuning>();
-        this->initialTimeStepSize_ = Parameters::Get<Parameters::InitialTimeStepSize<Scalar>>();
-        this->maxTimeStepAfterWellEvent_ = Parameters::Get<Parameters::TimeStepAfterEventInDays<Scalar>>() * 24 * 60 * 60;
-
-        // The value N for this parameter is defined in the following order of presedence:
-        // 1. Command line value (--num-pressure-points-equil=N)
-        // 2. EQLDIMS item 2
-        // Default value is defined in opm-common/src/opm/input/eclipse/share/keywords/000_Eclipse100/E/EQLDIMS
-        if (Parameters::IsSet<Parameters::NumPressurePointsEquil>())
-        {
-            this->numPressurePointsEquil_ = Parameters::Get<Parameters::NumPressurePointsEquil>();
-        } else {
-            this->numPressurePointsEquil_ = simulator.vanguard().eclState().getTableManager().getEqldims().getNumDepthNodesP();
+        if (! Parameters::Get<Parameters::CheckSatfuncConsistency>()) {
+            // User did not enable the "new" saturation function consistency
+            // check module.  Run the original checker instead.  This is a
+            // temporary measure.
+            RelpermDiagnostics relpermDiagnostics{};
+            relpermDiagnostics.diagnosis(simulator.vanguard().eclState(),
+                                         simulator.vanguard().levelCartesianIndexMapper());
         }
-
-        explicitRockCompaction_ = Parameters::Get<Parameters::ExplicitRockCompaction>();
-
-
-        RelpermDiagnostics relpermDiagnostics;
-        relpermDiagnostics.diagnosis(vanguard.eclState(), vanguard.cartesianIndexMapper());
     }
 
     virtual ~FlowProblem() = default;
 
     void prefetch(const Element& elem) const
-    { pffDofData_.prefetch(elem); }
+    { this->pffDofData_.prefetch(elem); }
 
     /*!
      * \brief This method restores the complete state of the problem and its sub-objects
@@ -488,7 +470,7 @@ public:
      * \brief Write the requested quantities of the current solution into the output
      *        files.
      */
-    void writeOutput(bool verbose = true)
+    virtual void writeOutput(bool verbose)
     {
         OPM_TIMEBLOCK(problemWriteOutput);
         // use the generic code to prepare the output fields and to
@@ -714,7 +696,28 @@ public:
     Scalar rockReferencePressure(const Context& context, unsigned spaceIdx, unsigned timeIdx) const
     {
         unsigned globalSpaceIdx = context.globalSpaceIndex(spaceIdx, timeIdx);
-        return this->rockReferencePressure(globalSpaceIdx);
+        return rockReferencePressure(globalSpaceIdx);
+    }
+
+    /*!
+    * \copydoc BlackoilProblem::rockReferencePressure
+    */
+    Scalar rockReferencePressure(unsigned globalSpaceIdx) const
+    {
+        const auto& rock_config = this->simulator().vanguard().eclState().getSimulationConfig().rock_config();
+        if (rock_config.store()) {
+            return asImp_().initialFluidState(globalSpaceIdx).pressure(refPressurePhaseIdx_());
+        }
+        else {
+            if (this->rockParams_.empty())
+                return 1e5;
+
+            unsigned tableIdx = 0;
+            if (!this->rockTableIdx_.empty()) {
+                tableIdx = this->rockTableIdx_[globalSpaceIdx];
+            }
+            return this->rockParams_[tableIdx].referencePressure;
+        }
     }
 
     /*!
@@ -1058,6 +1061,7 @@ public:
 
         const auto& fs = intQuants.fluidState();
         LhsEval effectivePressure = decay<LhsEval>(fs.pressure(refPressurePhaseIdx_()));
+        const auto& rock_config = this->simulator().vanguard().eclState().getSimulationConfig().rock_config();
         if (!this->minRefPressure_.empty())
             // The pore space change is irreversible
             effectivePressure =
@@ -1067,6 +1071,9 @@ public:
         if (!this->overburdenPressure_.empty())
             effectivePressure -= this->overburdenPressure_[elementIdx];
 
+        if (rock_config.store()) {
+            effectivePressure -= asImp_().initialFluidState(elementIdx).pressure(refPressurePhaseIdx_());
+        }
 
         if (!this->rockCompPoroMult_.empty()) {
             return this->rockCompPoroMult_[tableIdx].eval(effectivePressure, /*extrapolation=*/true);
@@ -1095,7 +1102,7 @@ public:
 
 
     /*!
-     * \brief Return the well transmissibility multiplier due to rock changues.
+     * \brief Return the well transmissibility multiplier due to rock changes.
      */
     template <class LhsEval>
     LhsEval wellTransMultiplier(const IntensiveQuantities& intQuants, unsigned elementIdx) const
@@ -1105,7 +1112,7 @@ public:
         const bool implicit = !this->explicitRockCompaction_;
         double trans_mult = implicit ? this->simulator().problem().template computeRockCompTransMultiplier_<double>(intQuants, elementIdx)
                                      : this->simulator().problem().getRockCompTransMultVal(elementIdx);
-        trans_mult *= this->simulator().problem().template permFactTransMultiplier<double>(intQuants);
+        trans_mult *= this->simulator().problem().template permFactTransMultiplier<double>(intQuants, elementIdx);
     
         return trans_mult;
     }
@@ -1146,6 +1153,15 @@ public:
         case BCComponent::POLYMER:
             this->handlePolymerBC(bc, rate);
             break;
+        case BCComponent::MICR:
+            this->handleMicrBC(bc, rate);
+            break;
+        case BCComponent::OXYG:
+            this->handleOxygBC(bc, rate);
+            break;
+        case BCComponent::UREA:
+            this->handleUreaBC(bc, rate);
+            break;
         case BCComponent::NONE:
             throw std::logic_error("you need to specify the component when RATE type is set in BC");
             break;
@@ -1623,7 +1639,7 @@ protected:
 
         const auto& fs = intQuants.fluidState();
         LhsEval effectivePressure = decay<LhsEval>(fs.pressure(refPressurePhaseIdx_()));
-
+        const auto& rock_config = this->simulator().vanguard().eclState().getSimulationConfig().rock_config();
         if (!this->minRefPressure_.empty())
             // The pore space change is irreversible
             effectivePressure =
@@ -1632,6 +1648,10 @@ protected:
 
         if (!this->overburdenPressure_.empty())
             effectivePressure -= this->overburdenPressure_[elementIdx];
+        
+        if (rock_config.store()) {
+            effectivePressure -= asImp_().initialFluidState(elementIdx).pressure(refPressurePhaseIdx_());
+        }
 
         if (!this->rockCompTransMult_.empty())
             return this->rockCompTransMult_[tableIdx].eval(effectivePressure, /*extrapolation=*/true);
@@ -1649,15 +1669,11 @@ protected:
     std::shared_ptr<EclMaterialLawManager> materialLawManager_;
     std::shared_ptr<EclThermalLawManager> thermalLawManager_;
 
-    bool enableDriftCompensation_;
     GlobalEqVector drift_;
 
     WellModel wellModel_;
     AquiferModel aquiferModel_;
 
-    bool enableVtkOutput_;
-
-
     PffGridVector<GridView, Stencil, PffDofData_, DofMapper> pffDofData_;
     TracerModel tracerModel_;
 
@@ -1694,11 +1710,15 @@ protected:
 
     virtual void handlePolymerBC(const BCProp::BCFace&, RateVector&) const = 0;
 
+    virtual void handleMicrBC(const BCProp::BCFace&, RateVector&) const = 0;
+
+    virtual void handleOxygBC(const BCProp::BCFace&, RateVector&) const = 0;
+
+    virtual void handleUreaBC(const BCProp::BCFace&, RateVector&) const = 0;
+
     BCData<int> bcindex_;
     bool nonTrivialBoundaryConditions_ = false;
-    bool explicitRockCompaction_ = false;
     bool first_step_ = true;
-
 };
 
 } // namespace Opm
