@@ -81,7 +81,7 @@ void mergeParallelLogFiles(std::string_view output_dir,
                                              enableLoggingFalloutWarning));
 }
 
-void handleExtraConvergenceOutput(SimulatorReport& report,
+void handleExtraConvergenceOutput(const SimulatorReport& report,
                                   std::string_view option,
                                   std::string_view optionName,
                                   std::string_view output_dir,
@@ -262,6 +262,12 @@ void hideUnusedParameters()
     Parameters::Hide<Parameters::UseAverageDensityMsWells>();
 }
 
+namespace {
+    bool isEmptyString(const std::string& str) {
+        return str.size()==0 || str=="\"\"" || str=="''";
+    }
+}
+
 int eclPositionalParameter(std::function<void(const std::string&, const std::string&)> addKey,
                            std::set<std::string>& seenParams,
                            std::string& errorMsg,
@@ -283,6 +289,9 @@ int eclPositionalParameter(std::function<void(const std::string&, const std::str
         return 0;
     }
 
+    if (isEmptyString(argv[paramIdx]))
+        return 1;
+
     if (seenParams.count("EclDeckFileName") > 0) {
         errorMsg =
             "Parameter 'EclDeckFileName' specified multiple times"
