@@ -35,6 +35,7 @@
 
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/LgrCollection.hpp>
 
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
@@ -48,9 +49,6 @@
 
 #if HAVE_DUNE_FEM
 #include <dune/fem/gridpart/adaptiveleafgridpart.hh>
-#if !DUNE_VERSION_GTE(DUNE_FEM, 2, 9)
-#include <dune/fem/gridpart/common/gridpart2gridview.hh>
-#endif
 #include <opm/simulators/flow/FemCpGridCompat.hpp>
 #endif //HAVE_DUNE_FEM
 
@@ -146,20 +144,34 @@ template<class ElementMapper, class GridView, class Scalar>
 void GenericCpGridVanguard<ElementMapper, GridView, Scalar>::
 doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
                const bool                               ownersFirst,
+               const bool                               addCorners,
+               const int                                numOverlap,
                const Dune::PartitionMethod              partitionMethod,
                const bool                               serialPartitioning,
                const bool                               enableDistributedWells,
+               const bool                               allowSplittingInactiveWells,
                const double                             imbalanceTol,
                const GridView&                          gridView,
                const Schedule&                          schedule,
                EclipseState&                            eclState1,
                FlowGenericVanguard::ParallelWellStruct& parallelWells,
-               const int                                numJacobiBlocks)
+               const int                                numJacobiBlocks,
+               const bool                               enableEclOutput)
 {
-    if (partitionMethod == Dune::PartitionMethod::zoltan && !this->zoltanParams().empty())
-        this->grid_->setPartitioningParams(setupZoltanParams(this->zoltanParams()));
-    if (partitionMethod == Dune::PartitionMethod::metis && !this->metisParams().empty())
-        this->grid_->setPartitioningParams(setupMetisParams(this->metisParams()));
+    if (((partitionMethod == Dune::PartitionMethod::zoltan) ||
+         (partitionMethod == Dune::PartitionMethod::zoltanGoG)) &&
+        !this->zoltanParams().empty())
+    {
+        this->grid_->setPartitioningParams
+            (setupZoltanParams(this->zoltanParams(),
+                               this->zoltanPhgEdgeSizeThreshold()));
+    }
+    else if ((partitionMethod == Dune::PartitionMethod::metis) &&
+        !this->metisParams().empty())
+    {
+        this->grid_->setPartitioningParams
+            (setupMetisParams(this->metisParams()));
+    }
 
     const auto mpiSize = this->grid_->comm().size();
 
@@ -167,16 +179,6 @@ doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
         (numJacobiBlocks > 1) && (mpiSize == 1);
 
     if ((mpiSize > 1) || (numJacobiBlocks > 1)) {
-        if (this->grid_->size(0) > 0) {
-            // Generally needed in parallel runs both when there is and when
-            // there is not an externally defined load-balancing function.
-            // In addition to being used in CpGrid::loadBalance(), the
-            // transmissibilities are also output to the .INIT file.  Thus,
-            // transmissiblity values must exist on the I/O rank for derived
-            // classes such as EclCpGridVanguard<>.
-            this->allocTrans();
-        }
-
         // CpGrid's loadBalance() method uses transmissibilities as edge
         // weights.  This is arguably a layering violation and extracting
         // the per-face transmissibilities as a linear array is relatively
@@ -185,6 +187,14 @@ doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
         auto loadBalancerSet = static_cast<int>(externalLoadBalancer.has_value());
         this->grid_->comm().broadcast(&loadBalancerSet, 1, 0);
 
+        if ((this->grid_->size(0) > 0) &&
+            (enableEclOutput ||
+             (loadBalancerSet == 0) ||
+             partitionJacobiBlocks))
+        {
+            this->allocTrans();
+        }
+
         std::vector<double> faceTrans;
         {
             OPM_TIMEBLOCK(extractTrans);
@@ -193,13 +203,20 @@ doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
             }
         }
 
+        // Skipping inactive wells in partitioning currently does not play nice with restart..
+        const bool restart = eclState1.getInitConfig().restartRequested();
+        const bool split_inactive = (!restart && allowSplittingInactiveWells);
         const auto wells = ((mpiSize > 1) || partitionJacobiBlocks)
-            ? schedule.getWellsatEnd()
+            ? split_inactive
+                ? schedule.getActiveWellsAtEnd()
+                : schedule.getWellsatEnd()
             : std::vector<Well>{};
+
         const auto& possibleFutureConnections = schedule.getPossibleFutureConnections();
         // Distribute the grid and switch to the distributed view.
         if (mpiSize > 1) {
-            this->distributeGrid(edgeWeightsMethod, ownersFirst, partitionMethod,
+            this->distributeGrid(edgeWeightsMethod, ownersFirst,
+                                 addCorners, numOverlap, partitionMethod,
                                  serialPartitioning, enableDistributedWells,
                                  imbalanceTol, loadBalancerSet != 0,
                                  faceTrans, wells,
@@ -207,6 +224,48 @@ doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
                                  eclState1, parallelWells);
         }
 
+        // Add inactive wells to all ranks with connections (not solved, so OK even without distributed wells)
+        if (split_inactive) {
+            std::unordered_set<unsigned> cellOnRank;
+            const auto& global_cells = this->grid_->globalCell();
+            for (const auto cell : global_cells) cellOnRank.insert(cell);
+            const auto inactive_well_names = schedule.getInactiveWellNamesAtEnd();
+            std::size_t num_wells = inactive_well_names.size();
+            std::vector<int> well_on_rank(num_wells, 0);
+            std::size_t well_idx = 0;
+            for (const auto& well_name : inactive_well_names) {
+                const auto& well = schedule.getWell(well_name, schedule.size()-1);
+                for (const auto& conn: well.getConnections()) {
+                    if (cellOnRank.count(conn.global_index()) > 0) {
+                        well_on_rank[well_idx] = 1;
+                        parallelWells.emplace_back(well_name, true);
+                        break;
+                    }
+                }
+                if (!well_on_rank[well_idx]) parallelWells.emplace_back(well_name, false);
+                ++well_idx;
+            }
+
+            // Provide information message
+            const auto& comm = this->grid_->comm();
+            const auto nranks = comm.size();
+            // // values from rank i will be at indices i*num_wells, i*num_wells + 1, ..., (i+1) * num_wells -1
+            std::vector<int> well_on_rank_global(num_wells * nranks, 0);
+            comm.allgather(well_on_rank.data(), static_cast<int>(num_wells), well_on_rank_global.data());
+            if (comm.rank() == 0) {
+                well_idx = 0;
+                for (const auto& well_name : inactive_well_names) {
+                    std::string msg = fmt::format("Well {} is inactive, with perforations on ranks: ", well_name);
+                    for (int i=0; i<nranks; ++i) {
+                        if (well_on_rank_global[i*num_wells + well_idx]) msg += fmt::format("{} ", i);
+                    }
+                    OpmLog::info(msg);
+                    ++well_idx;
+                }
+            }
+        }
+        std::sort(parallelWells.begin(), parallelWells.end());
+
         // Calling Schedule::filterConnections would remove any perforated
         // cells that exist only on other ranks even in the case of
         // distributed wells.  But we need all connections to figure out the
@@ -285,6 +344,8 @@ void
 GenericCpGridVanguard<ElementMapper, GridView, Scalar>::
 distributeGrid(const Dune::EdgeWeightMethod                          edgeWeightsMethod,
                const bool                                            ownersFirst,
+               const bool                                            addCorners,
+               const int                                             numOverlap,
                const Dune::PartitionMethod                           partitionMethod,
                const bool                                            serialPartitioning,
                const bool                                            enableDistributedWells,
@@ -299,10 +360,12 @@ distributeGrid(const Dune::EdgeWeightMethod                          edgeWeights
     if (auto* eclState = dynamic_cast<ParallelEclipseState*>(&eclState1);
         eclState != nullptr)
     {
-        this->distributeGrid(edgeWeightsMethod, ownersFirst, partitionMethod,
+        this->distributeGrid(edgeWeightsMethod, ownersFirst, addCorners,
+                             numOverlap, partitionMethod,
                              serialPartitioning, enableDistributedWells,
                              imbalanceTol, loadBalancerSet, faceTrans,
-                             wells, possibleFutureConnections, eclState, parallelWells);
+                             wells, possibleFutureConnections,
+                             eclState, parallelWells);
     }
     else {
         const auto message = std::string {
@@ -323,6 +386,8 @@ void
 GenericCpGridVanguard<ElementMapper, GridView, Scalar>::
 distributeGrid(const Dune::EdgeWeightMethod                          edgeWeightsMethod,
                const bool                                            ownersFirst,
+               const bool                                            addCorners,
+               const int                                             numOverlap,
                const Dune::PartitionMethod                           partitionMethod,
                const bool                                            serialPartitioning,
                const bool                                            enableDistributedWells,
@@ -341,27 +406,29 @@ distributeGrid(const Dune::EdgeWeightMethod                          edgeWeights
         *this->grid_, *eclState
     };
 
-    const auto addCornerCells = false;
-    const auto overlapLayers = 1;
+    const auto addCornerCells = addCorners;
+    const auto overlapLayers = numOverlap;
 
     if (loadBalancerSet) {
         auto parts = isIORank
             ? (*externalLoadBalancer)(*this->grid_)
             : std::vector<int>{};
-        //For this case, simple partitioning is selected automatically
-        parallelWells =
-            std::get<1>(this->grid_->loadBalance(handle, parts, &wells, possibleFutureConnections, ownersFirst,
-                                                 addCornerCells, overlapLayers));
+
+        // For this case, simple partitioning is selected automatically.
+        parallelWells = std::get<1>
+            (this->grid_->loadBalance(handle, parts, &wells,
+                                      possibleFutureConnections, ownersFirst,
+                                      addCornerCells, overlapLayers));
     }
     else {
-        parallelWells =
-            std::get<1>(this->grid_->loadBalance(handle, edgeWeightsMethod,
-                                                 &wells, possibleFutureConnections,
-                                                 serialPartitioning,
-                                                 faceTrans.data(), ownersFirst,
-                                                 addCornerCells, overlapLayers,
-                                                 partitionMethod, imbalanceTol,
-                                                 enableDistributedWells));
+        parallelWells = std::get<1>
+            (this->grid_->loadBalance(handle, edgeWeightsMethod,
+                                      &wells, possibleFutureConnections,
+                                      serialPartitioning,
+                                      faceTrans.data(), ownersFirst,
+                                      addCornerCells, overlapLayers,
+                                      partitionMethod, imbalanceTol,
+                                      enableDistributedWells));
     }
 }
 
@@ -370,12 +437,14 @@ distributeGrid(const Dune::EdgeWeightMethod                          edgeWeights
 template<class ElementMapper, class GridView, class Scalar>
 void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(EclipseState& eclState)
 {
-    const EclipseGrid* input_grid = nullptr;
-    std::vector<double> global_porv;
+    const auto isRoot = this->mpiRank == 0;
+
     // At this stage the ParallelEclipseState instance is still in global
     // view; on rank 0 we have undistributed data for the entire grid, on
     // the other ranks the EclipseState is empty.
-    if (mpiRank == 0) {
+    const EclipseGrid* input_grid = nullptr;
+    std::vector<double> global_porv;
+    if (isRoot) {
         input_grid = &eclState.getInputGrid();
         global_porv = eclState.fieldProps().porv(true);
         OpmLog::info("\nProcessing grid");
@@ -383,6 +452,7 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
 
     // --- Create grid ---
     OPM_TIMEBLOCK(createGrids);
+
 #if HAVE_MPI
     this->grid_ = std::make_unique<Dune::CpGrid>(FlowGenericVanguard::comm());
 #else
@@ -390,14 +460,14 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
 #endif
 
     // Note: removed_cells is guaranteed to be empty on ranks other than 0.
-    auto removed_cells =
-        this->grid_->processEclipseFormat(input_grid,
-                                          &eclState,
-                                          /*isPeriodic=*/false,
-                                          /*flipNormals=*/false,
-                                          /*clipZ=*/false);
+    auto removed_cells = this->grid_
+        ->processEclipseFormat(input_grid,
+                               &eclState,
+                               /* isPeriodic = */ false,
+                               /* flipNormals = */ false,
+                               /* clipZ = */ false);
 
-    if (mpiRank == 0) {
+    if (isRoot) {
         const auto& active_porv = eclState.fieldProps().porv(false);
         const auto& unit_system = eclState.getUnits();
         const auto& volume_unit = unit_system.name( UnitSystem::measure::volume);
@@ -420,53 +490,37 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
 
     cartesianIndexMapper_ = std::make_unique<CartesianIndexMapper>(*grid_);
 
-    // --- Add LGRs and update Leaf Grid View ---
-    // Check if input file contains Lgrs.
-    const auto& lgrs = eclState.getLgrs();
-    const auto lgrsSize = lgrs.size();
-    // If there are lgrs, create the grid with them, and update the leaf grid view.
-    if (lgrsSize)
-    {
-        OpmLog::info("\nAdding LGRs to the grid and updating its leaf grid view");
-        this->addLgrsUpdateLeafView(lgrs, lgrsSize);
-    }
-
 #if HAVE_MPI
-    {
-        const bool has_numerical_aquifer = eclState.aquifer().hasNumericalAquifer();
-        int mpiSize = 1;
-        MPI_Comm_size(grid_->comm(), &mpiSize);
+    if (this->grid_->comm().size() > 1) {
+        // Numerical aquifers generate new NNCs during grid processing.  We
+        // need to distribute these NNCs from the root process to all other
+        // MPI processes.
+        if (eclState.aquifer().hasNumericalAquifer()) {
+            auto nnc_input = eclState.getInputNNC();
+            Parallel::MpiSerializer ser(this->grid_->comm());
+            ser.broadcast(nnc_input);
 
-        // when there is numerical aquifers, new NNC are generated during
-        // grid processing we need to pass the NNC from root process to
-        // other processes
-        if ( mpiSize > 1)
-        {
-            if (has_numerical_aquifer) {
-                auto nnc_input = eclState.getInputNNC();
-                Parallel::MpiSerializer ser(grid_->comm());
-                ser.broadcast(nnc_input);
-                if (mpiRank > 0) {
-                    eclState.setInputNNC(nnc_input);
-                }
+            if (! isRoot) {
+                eclState.setInputNNC(nnc_input);
             }
-            bool hasPinchNnc = eclState.hasPinchNNC();
-            grid_->comm().broadcast(&hasPinchNnc, 1, 0);
+        }
 
-            if(hasPinchNnc)
-            {
-                auto pinch_nnc = eclState.getPinchNNC();
-                Parallel::MpiSerializer ser(grid_->comm());
-                ser.broadcast(pinch_nnc);
-                if (mpiRank > 0) {
-                    eclState.setPinchNNC(std::move(pinch_nnc));
-                }
+        bool hasPinchNnc = eclState.hasPinchNNC();
+        grid_->comm().broadcast(&hasPinchNnc, 1, 0);
+
+        if (hasPinchNnc) {
+            auto pinch_nnc = eclState.getPinchNNC();
+            Parallel::MpiSerializer ser(this->grid_->comm());
+            ser.broadcast(pinch_nnc);
+
+            if (! isRoot) {
+                eclState.setPinchNNC(std::move(pinch_nnc));
             }
         }
     }
 #endif
 
-    // --- Copy grid with LGRs to equilGrid_ ---
+    // --- Copy grid to equilGrid_ ---
     // We use separate grid objects: one for the calculation of the initial
     // condition via EQUIL and one for the actual simulation. The reason is
     // that the EQUIL code is allergic to distributed grids and the
@@ -475,21 +529,20 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
     //
     // After loadbalance, grid_ will contain a global and distribute view.
     // equilGrid_ being a shallow copy only the global view.
-    if (mpiRank == 0)
-    {
-        equilGrid_.reset(new Dune::CpGrid(*grid_));
-        equilCartesianIndexMapper_ = std::make_unique<CartesianIndexMapper>(*equilGrid_);
+    if (isRoot) {
+        this->equilGrid_ = std::make_unique<Dune::CpGrid>(*this->grid_);
+        this->equilCartesianIndexMapper_ =
+            std::make_unique<CartesianIndexMapper>(*this->equilGrid_);
 
-        eclState.reset_actnum(UgGridHelpers::createACTNUM(*grid_));
+        eclState.reset_actnum(UgGridHelpers::createACTNUM(*this->grid_));
         eclState.set_active_indices(this->grid_->globalCell());
     }
 
     {
         auto size = removed_cells.size();
-
         this->grid_->comm().broadcast(&size, 1, 0);
 
-        if (mpiRank != 0) {
+        if (! isRoot) {
             removed_cells.resize(size);
         }
 
@@ -502,7 +555,9 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
 }
 
 template<class ElementMapper, class GridView, class Scalar>
-void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::addLgrsUpdateLeafView(const LgrCollection& lgrCollection, const int lgrsSize)
+void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::addLgrsUpdateLeafView(const LgrCollection& lgrCollection,
+                                                                                 const int lgrsSize,
+                                                                                 Dune::CpGrid& grid)
 {
     std::vector<std::array<int,3>> cells_per_dim_vec;
     std::vector<std::array<int,3>> startIJK_vec;
@@ -521,7 +576,7 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::addLgrsUpdateLeafView
         endIJK_vec.push_back({lgrCarfin.I2()+1, lgrCarfin.J2()+1, lgrCarfin.K2()+1});
         lgrName_vec.emplace_back(lgrCarfin.NAME());
     }
-    this->grid_->addLgrsUpdateLeafView(cells_per_dim_vec, startIJK_vec, endIJK_vec, lgrName_vec);
+    grid.addLgrsUpdateLeafView(cells_per_dim_vec, startIJK_vec, endIJK_vec, lgrName_vec);
 };
 
 template<class ElementMapper, class GridView, class Scalar>
@@ -570,6 +625,13 @@ GenericCpGridVanguard<ElementMapper,GridView,Scalar>::cartesianIndexMapper() con
     return *cartesianIndexMapper_;
 }
 
+template<class ElementMapper, class GridView, class Scalar>
+const LevelCartesianIndexMapper<Dune::CpGrid>
+GenericCpGridVanguard<ElementMapper,GridView,Scalar>::levelCartesianIndexMapper() const
+{
+    return LevelCartesianIndexMapper(*grid_);
+}
+
 template<class ElementMapper, class GridView, class Scalar>
 const Dune::CartesianIndexMapper<Dune::CpGrid>&
 GenericCpGridVanguard<ElementMapper,GridView,Scalar>::equilCartesianIndexMapper() const
@@ -620,49 +682,16 @@ INSTANTIATE_TYPE(float)
 #endif
 
 #if HAVE_DUNE_FEM
-#if DUNE_VERSION_GTE(DUNE_FEM, 2, 9)
 using GV = Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid,
                                            (Dune::PartitionIteratorType)4,
                                            false>;
-#define INSTANTIATE_FEM_TYPE(T)                                                       \
-  template class GenericCpGridVanguard<Dune::MultipleCodimMultipleGeomTypeMapper<GV>, \
-                                       GV,                                            \
-                                       T>;
-#else
-#define INSTANTIATE_FEM_TYPE(T)                         \
-    template class GenericCpGridVanguard<               \
-        Dune::MultipleCodimMultipleGeomTypeMapper<      \
-            Dune::GridView<                             \
-                Dune::Fem::GridPart2GridViewTraits<     \
-                    Dune::Fem::AdaptiveLeafGridPart<    \
-                        Dune::CpGrid,                   \
-                        Dune::PartitionIteratorType(4), \
-                        false>>>>,                      \
-        Dune::GridView<                                 \
-            Dune::Fem::GridPart2GridViewTraits<         \
-                Dune::Fem::AdaptiveLeafGridPart<        \
-                    Dune::CpGrid,                       \
-                    Dune::PartitionIteratorType(4),     \
-                    false>>>,                           \
-        T>;                                             \
-    template class GenericCpGridVanguard<               \
-        Dune::MultipleCodimMultipleGeomTypeMapper<      \
-            Dune::Fem::GridPart2GridViewImpl<           \
-                Dune::Fem::AdaptiveLeafGridPart<        \
-                    Dune::CpGrid,                       \
-                    Dune::PartitionIteratorType(4),     \
-                    false>>>,                           \
-        Dune::Fem::GridPart2GridViewImpl<               \
-            Dune::Fem::AdaptiveLeafGridPart<            \
-                Dune::CpGrid,                           \
-                Dune::PartitionIteratorType(4),         \
-                false> >,                               \
-        T>;
-#endif
-
-INSTANTIATE_FEM_TYPE(double)
+template class GenericCpGridVanguard<Dune::MultipleCodimMultipleGeomTypeMapper<GV>,
+                                     GV,
+                                     double>;
 #if FLOW_INSTANTIATE_FLOAT
-INSTANTIATE_FEM_TYPE(float)
+template class GenericCpGridVanguard<Dune::MultipleCodimMultipleGeomTypeMapper<GV>,
+                                     GV,
+                                     float>;
 #endif
 
 #endif // HAVE_DUNE_FEM
