@@ -28,6 +28,7 @@
 #define OPM_GENERIC_CPGRID_VANGUARD_HPP
 
 #include <opm/grid/CpGrid.hpp>
+#include <opm/grid/cpgrid/LevelCartesianIndexMapper.hpp>
 
 #include <opm/simulators/flow/FlowGenericVanguard.hpp>
 
@@ -78,6 +79,7 @@ template<class ElementMapper, class GridView, class Scalar>
 class GenericCpGridVanguard {
 protected:
     using CartesianIndexMapper = Dune::CartesianIndexMapper<Dune::CpGrid>;
+    using LevelCartesianIndexMapper = Opm::LevelCartesianIndexMapper<Dune::CpGrid>;
     using Element = typename GridView::template Codim<0>::Entity;
 
 public:
@@ -131,6 +133,12 @@ public:
      */
     const CartesianIndexMapper& cartesianIndexMapper() const;
 
+    /*!
+     * \brief Returns the object which maps a global element index of the simulation grid
+     *        to the corresponding element index of the level logically Cartesian index.
+     */
+    const LevelCartesianIndexMapper levelCartesianIndexMapper() const;
+
     /*!
      * \brief Returns mapper from compressed to cartesian indices for the EQUIL grid
      */
@@ -150,15 +158,19 @@ protected:
 #if HAVE_MPI
     void doLoadBalance_(const Dune::EdgeWeightMethod             edgeWeightsMethod,
                         const bool                               ownersFirst,
+                        const bool                               addCorners,
+                        const int                                numOverlap,
                         const Dune::PartitionMethod              partitionMethod,
                         const bool                               serialPartitioning,
                         const bool                               enableDistributedWells,
+                        const bool                               allowSplittingInactiveWells,
                         const double                             imbalanceTol,
                         const GridView&                          gridView,
                         const Schedule&                          schedule,
                         EclipseState&                            eclState,
                         FlowGenericVanguard::ParallelWellStruct& parallelWells,
-                        const int                                numJacobiBlocks);
+                        const int                                numJacobiBlocks,
+                        const bool                               enableEclOutput);
 
     void distributeFieldProps_(EclipseState& eclState);
 
@@ -167,6 +179,8 @@ private:
 
     void distributeGrid(const Dune::EdgeWeightMethod                          edgeWeightsMethod,
                         const bool                                            ownersFirst,
+                        const bool                                            addCorners,
+                        const int                                             numOverlap,
                         const Dune::PartitionMethod                           partitionMethod,
                         const bool                                            serialPartitioning,
                         const bool                                            enableDistributedWells,
@@ -180,6 +194,8 @@ private:
 
     void distributeGrid(const Dune::EdgeWeightMethod                          edgeWeightsMethod,
                         const bool                                            ownersFirst,
+                        const bool                                            addCorners,
+                        const int                                             numOverlap,
                         const Dune::PartitionMethod                           partitionMethod,
                         const bool                                            serialPartitioning,
                         const bool                                            enableDistributedWells,
@@ -193,6 +209,7 @@ private:
 
 protected:
     virtual const std::string& zoltanParams() const = 0;
+    virtual double zoltanPhgEdgeSizeThreshold() const = 0;
     virtual const std::string& metisParams() const = 0;
 
 #endif  // HAVE_MPI
@@ -200,7 +217,9 @@ protected:
     void allocCartMapper();
 
     void doCreateGrids_(EclipseState& eclState);
-    void addLgrsUpdateLeafView(const LgrCollection& lgrCollection, const int lgrsSize);
+    void addLgrsUpdateLeafView(const LgrCollection& lgrCollection,
+                               const int lgrsSize,
+                               Dune::CpGrid& grid);
 
     virtual void allocTrans() = 0;
     virtual double getTransmissibility(unsigned I, unsigned J) const = 0;
