@@ -32,10 +32,17 @@
 #include <opm/output/data/Wells.hpp>
 #include <opm/output/eclipse/Inplace.hpp>
 
-#include <opm/simulators/flow/FlowsData.hpp>
+#include <opm/simulators/flow/ExtboContainer.hpp>
+#include <opm/simulators/flow/FIPContainer.hpp>
+#include <opm/simulators/flow/FlowsContainer.hpp>
 #include <opm/simulators/flow/InterRegFlows.hpp>
 #include <opm/simulators/flow/LogOutputHelper.hpp>
+#include <opm/simulators/flow/MechContainer.hpp>
+#include <opm/simulators/flow/MICPContainer.hpp>
 #include <opm/simulators/flow/RegionPhasePVAverage.hpp>
+#include <opm/simulators/flow/RFTContainer.hpp>
+#include <opm/simulators/flow/RSTConv.hpp>
+#include <opm/simulators/flow/TracerContainer.hpp>
 
 #include <opm/simulators/utils/ParallelCommunication.hpp>
 
@@ -48,9 +55,17 @@
 #include <utility>
 #include <vector>
 
+namespace Opm::Parameters {
+
+struct ForceDisableFluidInPlaceOutput { static constexpr bool value = false; };
+struct ForceDisableResvFluidInPlaceOutput { static constexpr bool value = false; };
+
+} // namespace Opm::Parameters
+
 namespace Opm {
 
 namespace data { class Solution; }
+class EclHysteresisConfig;
 class EclipseState;
 class Schedule;
 class SummaryConfig;
@@ -65,13 +80,10 @@ public:
     // Virtual destructor for safer inheritance.
     virtual ~GenericOutputBlackoilModule();
 
-     Scalar* getPRESSURE_ptr(void) {
-        return (this->fluidPressure_.data()) ;
-    };
-
-    int  getPRESSURE_size( void ) {
-        return (this->fluidPressure_.size()) ;
-    };
+    /*!
+     * \brief Register all run-time parameters for the Vtk output module.
+     */
+    static void registerParameters();
 
     void outputTimeStamp(const std::string& lbl,
                          double elapsed,
@@ -87,30 +99,39 @@ public:
     void accumulateDensityParallel();
 
     // write cumulative production and injection reports to output
-    void outputCumLog(std::size_t reportStepNum);
+    void outputCumLog(std::size_t reportStepNum,
+                      const bool connData);
 
     // write production report to output
-    void outputProdLog(std::size_t reportStepNum);
+    void outputProdLog(std::size_t reportStepNum,
+                       const bool connData);
 
     // write injection report to output
-    void outputInjLog(std::size_t reportStepNum);
+    void outputInjLog(std::size_t reportStepNum,
+                      const bool connData);
+
+    // write msw report to output
+    void outputMSWLog(std::size_t reportStepNum);
+
+    // calculate Initial Fluid In Place
+    void calc_initial_inplace(const Parallel::Communication& comm);
 
     // calculate Fluid In Place
     Inplace calc_inplace(std::map<std::string, double>& miscSummaryData,
                          std::map<std::string, std::vector<double>>& regionData,
                          const Parallel::Communication& comm);
 
-    void outputFipAndResvLog(const Inplace& inplace,
-                         const std::size_t reportStepNum,
-                         double elapsed,
-                         boost::posix_time::ptime currentDate,
-                         const bool substep,
-                         const Parallel::Communication& comm);
+    void outputWellspecReport(const std::vector<std::string>& changedWells,
+                              const std::size_t reportStepNum,
+                              const double elapsed,
+                              boost::posix_time::ptime currentDate) const;
 
     void outputErrorLog(const Parallel::Communication& comm) const;
 
     void addRftDataToWells(data::Wells& wellDatas,
-                           std::size_t reportStepNum);
+                           std::size_t reportStepNum,
+                           const Parallel::Communication& comm)
+    { this->rftC_.addToWells(wellDatas, reportStepNum, comm); }
 
     /*!
      * \brief Move all buffers to data::Solution.
@@ -177,90 +198,14 @@ public:
         return 0;
     }
 
-    Scalar getMicrobialConcentration(unsigned elemIdx) const
-    {
-        if (cMicrobes_.size() > elemIdx)
-            return cMicrobes_[elemIdx];
+    const std::vector<Scalar>& getFluidPressure() const
+    { return fluidPressure_; }
 
-        return 0;
-    }
+    const MICPContainer<Scalar>& getMICP() const
+    { return this->micpC_; }
 
-    Scalar getOxygenConcentration(unsigned elemIdx) const
-    {
-        if (cOxygen_.size() > elemIdx)
-            return cOxygen_[elemIdx];
-
-        return 0;
-    }
-
-    Scalar getUreaConcentration(unsigned elemIdx) const
-    {
-        if (cUrea_.size() > elemIdx)
-            return cUrea_[elemIdx];
-
-        return 0;
-    }
-
-    Scalar getBiofilmConcentration(unsigned elemIdx) const
-    {
-        if (cBiofilm_.size() > elemIdx)
-            return cBiofilm_[elemIdx];
-
-        return 0;
-    }
-
-    Scalar getCalciteConcentration(unsigned elemIdx) const
-    {
-        if (cCalcite_.size() > elemIdx)
-            return cCalcite_[elemIdx];
-
-        return 0;
-    }
-
-    const std::array<FlowsData<double>, 3>& getFlowsn() const
-    {
-        return this->flowsn_;
-    }
-
-    bool hasFlowsn() const
-    {
-        return enableFlowsn_;
-    }
-
-    bool hasFlows() const
-    {
-        return enableFlows_;
-    }
-
-    bool hasBlockFlows() const
-    {
-        return blockFlows_;
-    }
-
-    bool anyFlows() const
-    {
-        return anyFlows_;
-    }
-
-    const std::array<FlowsData<double>, 3>& getFloresn() const
-    {
-        return this->floresn_;
-    }
-
-    bool hasFloresn() const
-    {
-        return enableFloresn_;
-    }
-
-    bool hasFlores() const
-    {
-        return enableFlores_;
-    }
-
-    bool anyFlores() const
-    {
-        return anyFlores_;
-    }
+    const FlowsContainer<FluidSystem>& getFlows() const
+    { return this->flowsC_; }
 
     bool needInterfaceFluxes([[maybe_unused]] const bool isSubStep) const
     {
@@ -272,9 +217,14 @@ public:
         return blockData_;
     }
 
-    const Inplace& initialInplace() const
+    std::map<std::pair<std::string, int>, double>& getExtraBlockData()
     {
-        return this->initialInplace_.value();
+        return extraBlockData_;
+    }
+
+    const std::optional<Inplace>& initialInplace() const
+    {
+        return this->initialInplace_;
     }
 
     bool localDataValid() const{
@@ -289,17 +239,18 @@ public:
         local_data_valid_ = true;
     }
 
-    void setCnvData(const std::vector<std::vector<int>>& data)
-    {
-        cnvData_ = data;
-    }
-
     template<class Serializer>
     void serializeOp(Serializer& serializer)
     {
         serializer(initialInplace_);
     }
 
+    RSTConv& getConv()
+    { return this->rst_conv_; }
+
+    const RSTConv& getConv() const
+    { return this->rst_conv_; }
+
     //! \brief Assign fields that are in global numbering to the solution.
     //! \detail This is used to add fields that for some reason cannot be collected
     //!         using the regular collect mechanism to the solution. In particular this
@@ -311,9 +262,9 @@ protected:
     using StringBuffer = std::vector<std::string>;
     enum { numPhases = FluidSystem::numPhases };
     enum { numComponents = FluidSystem::numComponents };
-    enum { gasPhaseIdx = FluidSystem::gasPhaseIdx };
-    enum { oilPhaseIdx = FluidSystem::oilPhaseIdx };
-    enum { waterPhaseIdx = FluidSystem::waterPhaseIdx };
+    static constexpr int gasPhaseIdx = FluidSystem::gasPhaseIdx;
+    static constexpr int oilPhaseIdx = FluidSystem::oilPhaseIdx;
+    static constexpr int waterPhaseIdx = FluidSystem::waterPhaseIdx;
     enum { gasCompIdx = FluidSystem::gasCompIdx };
     enum { oilCompIdx = FluidSystem::oilCompIdx };
     enum { waterCompIdx = FluidSystem::waterCompIdx };
@@ -324,6 +275,8 @@ protected:
                                 const SummaryConfig& summaryConfig,
                                 const SummaryState& summaryState,
                                 const std::string& moduleVersionName,
+                                RSTConv::LocalToGlobalCellFunc globalCell,
+                                const Parallel::Communication& comm,
                                 bool enableEnergy,
                                 bool enableTemperature,
                                 bool enableMech,
@@ -340,13 +293,9 @@ protected:
                         const bool substep,
                         const bool log,
                         const bool isRestart,
-                        const bool vapparsActive,
-                        const bool enablePCHysteresis,
-                        const bool enableNonWettingHysteresis,
-                        const bool enableWettingHysteresis,
-                        unsigned numTracers,
-                        const std::vector<bool>& enableSolTracers,
-                        unsigned numOutputNnc);
+                        const EclHysteresisConfig* hysteresisConfig,
+                        unsigned numOutputNnc = 0,
+                        std::map<std::string, int> rstKeywords = {});
 
     void makeRegionSum(Inplace& inplace,
                        const std::string& region_name,
@@ -378,6 +327,8 @@ protected:
     static Scalar sum(const ScalarBuffer& v);
 
     void setupBlockData(std::function<bool(int)> isCartIdxOnThisRank);
+    void setupExtraBlockData(const std::size_t        reportStepNum,
+                             std::function<bool(int)> isCartIdxOnThisRank);
 
     virtual bool isDefunctParallelWell(std::string wname) const = 0;
 
@@ -406,39 +357,7 @@ protected:
     bool forceDisableFipresvOutput_{false};
     bool computeFip_{false};
 
-    struct OutputFIPRestart {
-        /// Whether or not run requests (surface condition) fluid-in-place
-        /// restart file output using the 'FIP' mnemonic.
-        bool noPrefix {false};
-
-        /// Whether or not run requests surface condition fluid-in-place
-        /// restart file output using the 'SFIP' mnemonic.
-        bool surface {false};
-
-        /// Whether or not run requests reservoir condition fluid-in-place
-        /// restart file output using the 'RFIP' mnemonic.
-        bool reservoir {false};
-
-        void clearBits()
-        {
-            this->noPrefix = this->surface = this->reservoir = false;
-        }
-
-        explicit operator bool() const
-        {
-            return this->noPrefix || this->surface || this->reservoir;
-        }
-    } outputFipRestart_{};
-
-    bool anyFlows_{false};
-    bool anyFlores_{false};
-    bool blockFlows_{false};
-    bool enableFlows_{false};
-    bool enableFlores_{false};
-    bool enableFlowsn_{false};
-    bool enableFloresn_{false};
-
-    std::unordered_map<Inplace::Phase, ScalarBuffer> fip_;
+    FIPContainer<FluidSystem> fipC_;
     std::unordered_map<std::string, std::vector<int>> regions_;
     std::unordered_map<Inplace::Phase, std::vector<SummaryConfigNode>> regionNodes_;
 
@@ -470,12 +389,7 @@ protected:
     ScalarBuffer cSalt_;
     ScalarBuffer pSalt_;
     ScalarBuffer permFact_;
-    ScalarBuffer extboX_;
-    ScalarBuffer extboY_;
-    ScalarBuffer extboZ_;
-    ScalarBuffer mFracOil_;
-    ScalarBuffer mFracGas_;
-    ScalarBuffer mFracCo2_;
+    ExtboContainer<Scalar> extboC_;
     ScalarBuffer soMax_;
     ScalarBuffer swMax_;
     ScalarBuffer sgmax_;
@@ -493,41 +407,13 @@ protected:
     ScalarBuffer minimumOilPressure_;
     ScalarBuffer saturatedOilFormationVolumeFactor_;
     ScalarBuffer rockCompTransMultiplier_;
-    ScalarBuffer cMicrobes_;
-    ScalarBuffer cOxygen_;
-    ScalarBuffer cUrea_;
-    ScalarBuffer cBiofilm_;
-    ScalarBuffer cCalcite_;
+    MICPContainer<Scalar> micpC_;
     ScalarBuffer pcgw_;
     ScalarBuffer pcow_;
     ScalarBuffer pcog_;
 
     // buffers for mechanical output
-    ScalarBuffer mechPotentialForce_;
-    ScalarBuffer mechPotentialPressForce_;
-    ScalarBuffer mechPotentialTempForce_;
-
-    ScalarBuffer dispX_;
-    ScalarBuffer dispY_;
-    ScalarBuffer dispZ_;
-    ScalarBuffer stressXX_;
-    ScalarBuffer stressYY_;
-    ScalarBuffer stressZZ_;
-    ScalarBuffer stressXY_;
-    ScalarBuffer stressXZ_;
-    ScalarBuffer stressYZ_;
-    ScalarBuffer delstressXX_;
-    ScalarBuffer delstressYY_;
-    ScalarBuffer delstressZZ_;
-    ScalarBuffer delstressXY_;
-    ScalarBuffer delstressXZ_;
-    ScalarBuffer delstressYZ_;
-    ScalarBuffer strainXX_;
-    ScalarBuffer strainYY_;
-    ScalarBuffer strainZZ_;
-    ScalarBuffer strainXY_;
-    ScalarBuffer strainXZ_;
-    ScalarBuffer strainYZ_;
+    MechContainer<Scalar> mech_;
 
     std::array<ScalarBuffer, numPhases> saturation_;
     std::array<ScalarBuffer, numPhases> invB_;
@@ -535,23 +421,19 @@ protected:
     std::array<ScalarBuffer, numPhases> viscosity_;
     std::array<ScalarBuffer, numPhases> relativePermeability_;
 
-    std::vector<ScalarBuffer> freeTracerConcentrations_;
-    std::vector<ScalarBuffer> solTracerConcentrations_;
+    TracerContainer<FluidSystem> tracerC_;
 
     std::array<ScalarBuffer, numPhases> residual_;
 
-    std::array<std::array<ScalarBuffer, numPhases>, 6> flows_;
-    std::array<std::array<ScalarBuffer, numPhases>, 6> flores_;
+    FlowsContainer<FluidSystem> flowsC_;
 
-    std::array<FlowsData<double>, 3> floresn_;
-    std::array<FlowsData<double>, 3> flowsn_;
+    RFTContainer<FluidSystem> rftC_;
+    RSTConv rst_conv_; //!< Helper class for RPTRST CONV
 
-    std::map<std::size_t, Scalar> oilConnectionPressures_;
-    std::map<std::size_t, Scalar> waterConnectionSaturations_;
-    std::map<std::size_t, Scalar> gasConnectionSaturations_;
     std::map<std::pair<std::string, int>, double> blockData_;
-
-    std::vector<std::vector<int>> cnvData_; //!< Data for CNV_xxx arrays
+    // Extra block data required for non-summary output reasons
+    // Example is the block pressures for RPTSCHED WELLS=2
+    std::map<std::pair<std::string, int>, double> extraBlockData_;
 
     std::optional<Inplace> initialInplace_;
     bool local_data_valid_{false};
