@@ -32,18 +32,19 @@
 
 #include <opm/grid/common/CartesianIndexMapper.hpp>
 
+#include <opm/input/eclipse/EclipseState/Phase.hpp>
+
 #include <opm/models/blackoil/blackoilmodel.hh>
 
 #include <opm/simulators/linalg/matrixblock.hh>
-
-#include <opm/input/eclipse/EclipseState/Phase.hpp>
+#include <opm/simulators/wells/WellTracerRate.hpp>
 
 #include <array>
 #include <cstddef>
 #include <functional>
-#include <map>
 #include <memory>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 namespace Opm {
@@ -88,13 +89,19 @@ public:
     /*!
     * \brief Return well tracer rates
     */
-    const std::map<std::pair<std::string, std::string>, Scalar>&
-    getWellTracerRates() const {return wellTracerRate_;}
-    const std::map<std::pair<std::string, std::string>, Scalar>&
-    getWellFreeTracerRates() const {return wellFreeTracerRate_;}
-    const std::map<std::pair<std::string, std::string>, Scalar>&
-    getWellSolTracerRates() const {return wellSolTracerRate_;}
-    const std::map<std::tuple<std::string, std::string, std::size_t>, Scalar>&
+    const std::unordered_map<int, std::vector<WellTracerRate<Scalar>>>&
+    getWellTracerRates() const
+    { return wellTracerRate_; }
+
+    const std::unordered_map<int, std::vector<WellTracerRate<Scalar>>>&
+    getWellFreeTracerRates() const
+    { return wellFreeTracerRate_; }
+
+    const std::unordered_map<int, std::vector<WellTracerRate<Scalar>>>&
+    getWellSolTracerRates() const
+    { return wellSolTracerRate_; }
+
+    const std::unordered_map<int, std::vector<MSWellTracerRate<Scalar>>>&
     getMswTracerRates() const {return mSwTracerRate_;}
 
     template<class Serializer>
@@ -133,6 +140,12 @@ protected:
 
     Scalar currentConcentration_(const Well& eclWell, const std::string& name) const;
 
+    //! \brief Tracer type index
+    enum TracerTypeIdx {
+        Free     = 0,
+        Solution = 1,
+    };
+
     const GridView& gridView_;
     const EclipseState& eclState_;
     const CartesianIndexMapper& cartMapper_;
@@ -145,13 +158,12 @@ protected:
     std::vector<TracerVectorSingle> freeTracerConcentration_;
     std::vector<TracerVectorSingle> solTracerConcentration_;
 
-    // <wellName, tracerName> -> wellRate
-    std::map<std::pair<std::string, std::string>, Scalar> wellTracerRate_;
-    std::map<std::pair<std::string, std::string>, Scalar> wellFreeTracerRate_;
-    std::map<std::pair<std::string, std::string>, Scalar> wellSolTracerRate_;
+    // well_index -> tracer rates
+    std::unordered_map<int, std::vector<WellTracerRate<Scalar>>> wellTracerRate_;
+    std::unordered_map<int, std::vector<WellTracerRate<Scalar>>> wellFreeTracerRate_;
+    std::unordered_map<int, std::vector<WellTracerRate<Scalar>>> wellSolTracerRate_;
 
-    // <wellName, tracerName, segNum> -> wellRate
-    std::map<std::tuple<std::string, std::string, std::size_t>, Scalar> mSwTracerRate_;
+    std::unordered_map<int, std::vector<MSWellTracerRate<Scalar>>> mSwTracerRate_;
 
     /// \brief Function returning the cell centers
     std::function<std::array<double,dimWorld>(int)> centroids_;
