@@ -23,37 +23,6 @@
 #ifndef OPM_MAIN_HEADER_INCLUDED
 #define OPM_MAIN_HEADER_INCLUDED
 
-#include <flow/flow_blackoil.hpp>
-#include <flow/flow_blackoil_legacyassembly.hpp>
-
-#include <flow/flow_gasoil.hpp>
-#include <flow/flow_gasoildiffuse.hpp>
-#include <flow/flow_gasoil_energy.hpp>
-#include <flow/flow_oilwater.hpp>
-#include <flow/flow_gaswater.hpp>
-#include <flow/flow_gaswater_solvent.hpp>
-#include <flow/flow_solvent.hpp>
-#include <flow/flow_solvent_foam.hpp>
-#include <flow/flow_polymer.hpp>
-#include <flow/flow_extbo.hpp>
-#include <flow/flow_foam.hpp>
-#include <flow/flow_brine.hpp>
-#include <flow/flow_brine_saltprecipitation.hpp>
-#include <flow/flow_gaswater_saltprec_vapwat.hpp>
-#include <flow/flow_gaswater_saltprec_energy.hpp>
-#include <flow/flow_brine_precsalt_vapwat.hpp>
-#include <flow/flow_onephase.hpp>
-#include <flow/flow_onephase_energy.hpp>
-#include <flow/flow_oilwater_brine.hpp>
-#include <flow/flow_gaswater_brine.hpp>
-#include <flow/flow_gaswater_energy.hpp>
-#include <flow/flow_gaswater_dissolution.hpp>
-#include <flow/flow_gaswater_dissolution_diffuse.hpp>
-#include <flow/flow_energy.hpp>
-#include <flow/flow_oilwater_polymer.hpp>
-#include <flow/flow_oilwater_polymer_injectivity.hpp>
-#include <flow/flow_micp.hpp>
-
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 
 #include <opm/models/utils/propertysystem.hh>
@@ -61,6 +30,7 @@
 
 #include <opm/simulators/flow/Banners.hpp>
 #include <opm/simulators/flow/FlowMain.hpp>
+#include <opm/simulators/flow/FlowProblemBlackoil.hpp>
 
 #if HAVE_DUNE_FEM
 #include <dune/fem/misc/mpimanager.hh>
@@ -72,6 +42,10 @@
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
 #endif
 
+#if HAVE_CUDA
+#include <opm/simulators/linalg/gpuistl/device_management.hpp>
+#endif
+
 #if HAVE_DAMARIS
 #include <opm/simulators/utils/DamarisKeywords.hpp>
 #endif
@@ -131,7 +105,7 @@ public:
     Main(int argc, char** argv, bool ownMPI = true);
 
     // This constructor can be called from Python
-    Main(const std::string& filename, bool mpi_init = true, bool mpi_finalize = true);
+    explicit Main(const std::string& filename, bool mpi_init = true, bool mpi_finalize = true);
 
     // This constructor can be called from Python when Python has
     // already parsed a deck
@@ -145,9 +119,16 @@ public:
     ~Main();
 
     void setArgvArgc_(const std::string& filename);
-
+    void maybeSaveReservoirCouplingSlaveLogFilename_();
+    void maybeRedirectReservoirCouplingSlaveOutput_();
     void initMPI();
 
+    /// Run simulation.
+    ///
+    /// Selects an appropriate simulator based on runtime information in the
+    /// input deck.
+    ///
+    /// \return Simulation's status/exit code.
     int runDynamic()
     {
         int exitCode = EXIT_SUCCESS;
@@ -161,6 +142,13 @@ public:
         return exitCode;
     }
 
+    /// Run simulation.
+    ///
+    /// Uses staticially configured simulator defined at call site.
+    ///
+    /// \tparam TypeTag Simulation type's statically configured properties.
+    ///
+    /// \return Simulation's status/exit code.
     template <class TypeTag>
     int runStatic()
     {
@@ -182,93 +170,6 @@ public:
         return exitCode;
     }
 
-private:
-    int dispatchDynamic_()
-    {
-        const auto& rspec = this->eclipseState_->runspec();
-        const auto& phases = rspec.phases();
-
-        this->setupVanguard();
-
-        // run the actual simulator
-        //
-        // TODO: make sure that no illegal combinations like thermal and
-        //       twophase are requested.
-        const bool thermal = eclipseState_->getSimulationConfig().isThermal();
-
-        // Single-phase case
-        if (rspec.micp()) {
-            return this->runMICP(phases);
-        }
-
-        // water-only case
-        else if (phases.size() == 1 && phases.active(Phase::WATER) && !thermal) {
-            return this->runWaterOnly(phases);
-        }
-
-        // water-only case with energy
-        else if (phases.size() == 2 && phases.active(Phase::WATER) && thermal) {
-            return this->runWaterOnlyEnergy(phases);
-        }
-
-        // Twophase cases
-        else if (phases.size() == 2 && !thermal) {
-            return this->runTwoPhase(phases);
-        }
-
-        // Polymer case
-        else if (phases.active(Phase::POLYMER)) {
-            return this->runPolymer(phases);
-        }
-
-        // Foam case
-        else if (phases.active(Phase::FOAM) && !phases.active(Phase::SOLVENT)) {
-            return this->runFoam();
-        }
-
-        // Solvent case
-        else if (phases.active(Phase::SOLVENT)) {
-            return this->runSolvent(phases);
-        }
-
-        // Brine case
-        else if (phases.active(Phase::BRINE) && !thermal) {
-            return this->runBrine(phases);
-        }
-
-        // Extended BO case
-        else if (phases.active(Phase::ZFRACTION)) {
-            return this->runExtendedBlackOil();
-        }
-
-        // Energy case
-        else if (thermal) {
-            return this->runThermal(phases);
-        }
-
-        // Blackoil case
-        else if (phases.size() == 3) {
-            return this->runBlackOil();
-        }
-
-        else {
-            if (outputCout_) {
-                std::cerr << "No suitable configuration found, valid are "
-                          << "Twophase, polymer, foam, brine, solvent, "
-                          << "energy, and blackoil.\n";
-            }
-
-            return EXIT_FAILURE;
-        }
-    }
-
-    template <class TypeTag>
-    int dispatchStatic_()
-    {
-        this->setupVanguard();
-        return flowMain<TypeTag>(argc_, argv_, outputCout_, outputFiles_);
-    }
-
 protected:
     /// \brief Initialize
     /// \param exitCode The exitCode of the program.
@@ -408,6 +309,7 @@ protected:
                            keepKeywords,
                            getNumThreads(),
                            Parameters::Get<Parameters::EclOutputInterval>(),
+                           Parameters::Get<Parameters::Slave>(),
                            cmdline_params,
                            Opm::moduleVersion(),
                            Opm::compileTimestamp());
@@ -426,6 +328,10 @@ protected:
             return false;
         }
 
+#if HAVE_CUDA
+    Opm::gpuistl::printDevice();
+#endif
+
         exitCode = EXIT_SUCCESS;
         return true;
     }
@@ -451,231 +357,128 @@ private:
     // use the parameter system instead.
     void handleTestSplitCommunicatorCmdLine_();
 
-    int runMICP(const Phases& phases)
+    /// Dispatch to actual simulation functions based on input deck's setup.
+    ///
+    /// Called from runDynamic()
+    ///
+    /// \return Simulation's status/exit code.
+    int dispatchDynamic_();
+
+    /// Dispatch to actual simulation function based on statically
+    /// configured simulation type.
+    ///
+    /// Called from runStatic()
+    ///
+    /// \tparam TypeTag Simulation type's statically configured properties.
+    ///
+    /// \return Simulation's status/exit code.
+    template <class TypeTag>
+    int dispatchStatic_()
     {
-        if (!phases.active(Phase::WATER) || (phases.size() > 2)) {
-            if (outputCout_) {
-                std::cerr << "No valid configuration is found for MICP simulation, "
-                          << "the only valid option is water + MICP\n";
-            }
-
-            return EXIT_FAILURE;
-        }
-
-        return flowMICPMain(this->argc_,
-                            this->argv_,
-                            this->outputCout_,
-                            this->outputFiles_);
+        this->setupVanguard();
+        return flowMain<TypeTag>(argc_, argv_, outputCout_, outputFiles_);
     }
 
-    int runTwoPhase(const Phases& phases)
-    {
-        const bool diffusive = eclipseState_->getSimulationConfig().isDiffusive();
-        const bool disgasw = eclipseState_->getSimulationConfig().hasDISGASW();
-        const bool vapwat = eclipseState_->getSimulationConfig().hasVAPWAT();
+    /// Run a simulation with MICP effects.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runMICP(const Phases& phases);
 
-        // oil-gas
-        if (phases.active( Phase::OIL ) && phases.active( Phase::GAS )) {
-            if (diffusive) {
-                return flowGasOilDiffuseMain(argc_, argv_, outputCout_, outputFiles_);
-            } else {
-                return flowGasOilMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-        }
+    /// Run a simulation with two active phases.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runTwoPhase(const Phases& phases);
 
-        // oil-water
-        else if ( phases.active( Phase::OIL ) && phases.active( Phase::WATER ) ) {
-            if (diffusive) {
-                if (outputCout_) {
-                    std::cerr << "The DIFFUSE option is not available for the two-phase water/oil model." << std::endl;
-                }
-                return EXIT_FAILURE;
-            }
-            return flowOilWaterMain(argc_, argv_, outputCout_, outputFiles_);
-        }
+    /// Run a simulation with polymers.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runPolymer(const Phases& phases);
 
-        // gas-water
-        else if ( phases.active( Phase::GAS ) && phases.active( Phase::WATER ) ) {
-            if (disgasw || vapwat) {
-                if (diffusive) {
-                    return flowGasWaterDissolutionDiffuseMain(argc_, argv_, outputCout_, outputFiles_);
-                }
-                return flowGasWaterDissolutionMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-            if (diffusive) {
-                if (outputCout_) {
-                    std::cerr << "The DIFFUSE option is not available for the two-phase gas/water model without disgasw or vapwat." << std::endl;
-                }
-                return EXIT_FAILURE;
-            }
+    /// Run a simulation with foam.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \return Simulation's status/exit code.
+    int runFoam();
 
-            return flowGasWaterMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-        else {
-            if (outputCout_) {
-                std::cerr << "No suitable configuration found, valid are Twophase (oilwater, oilgas and gaswater), polymer, solvent, or blackoil" << std::endl;
-            }
+    /// Run a single phase, water-only simulation.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runWaterOnly(const Phases& phases);
 
-            return EXIT_FAILURE;
-        }
-    }
+    /// Run a single phase, water-only simulation with themal/energy effects.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runWaterOnlyEnergy(const Phases& phases);
 
-    int runPolymer(const Phases& phases)
-    {
-        if (! phases.active(Phase::WATER)) {
-            if (outputCout_)
-                std::cerr << "No valid configuration is found for polymer simulation, valid options include "
-                          << "oilwater + polymer and blackoil + polymer" << std::endl;
+    /// Run a simulation with brine, typically in a CCS workflow
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runBrine(const Phases& phases);
 
-            return EXIT_FAILURE;
-        }
+    /// Run a simulation with solvents.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runSolvent(const Phases& phases);
 
-        // Need to track the polymer molecular weight
-        // for the injectivity study
-        if (phases.active(Phase::POLYMW)) {
-            // only oil water two phase for now
-            assert (phases.size() == 4);
-            return flowOilWaterPolymerInjectivityMain(argc_, argv_, outputCout_, outputFiles_);
-        }
+    /// Run a simulation with the extended black-oil model.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \return Simulation's status/exit code.
+    int runExtendedBlackOil();
 
-        if (phases.size() == 3) { // oil water polymer case
-            return flowOilWaterPolymerMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-        else {
-            return flowPolymerMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-    }
+    /// Run a three-phase simulation with thermal effects.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \param[in] phases Run's active phases.  Needed to determine whether
+    /// or not the run's phase setup is supported.
+    ///
+    /// \return Simulation's status/exit code.
+    int runThermal(const Phases& phases);
 
-    int runFoam()
-    {
-        return flowFoamMain(argc_, argv_, outputCout_, outputFiles_);
-    }
-
-    int runWaterOnly(const Phases& phases)
-    {
-        if (!phases.active(Phase::WATER) || phases.size() != 1) {
-            if (outputCout_)
-                std::cerr << "No valid configuration is found for water-only simulation, valid options include "
-                          << "water, water + thermal" << std::endl;
-
-            return EXIT_FAILURE;
-        }
-
-        return flowWaterOnlyMain(argc_, argv_, outputCout_, outputFiles_);
-    }
-
-    int runWaterOnlyEnergy(const Phases& phases)
-    {
-        if (!phases.active(Phase::WATER) || phases.size() != 2) {
-            if (outputCout_)
-                std::cerr << "No valid configuration is found for water-only simulation, valid options include "
-                          << "water, water + thermal" << std::endl;
-
-            return EXIT_FAILURE;
-        }
-
-        return flowWaterOnlyEnergyMain(argc_, argv_, outputCout_, outputFiles_);
-    }
-
-    int runBrine(const Phases& phases)
-    {
-        if (! phases.active(Phase::WATER) || phases.size() == 2) {
-            if (outputCout_)
-                std::cerr << "No valid configuration is found for brine simulation, valid options include "
-                          << "oilwater + brine, gaswater + brine and blackoil + brine" << std::endl;
-
-            return EXIT_FAILURE;
-        }
-
-        if (phases.size() == 3) {
-
-            if (phases.active(Phase::OIL)){ // oil water brine case
-                return flowOilWaterBrineMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-            if (phases.active(Phase::GAS)){ // gas water brine case
-                if (eclipseState_->getSimulationConfig().hasPRECSALT() &&
-                    eclipseState_->getSimulationConfig().hasVAPWAT()) {
-                    //case with water vaporization into gas phase and salt precipitation
-                    return flowGasWaterSaltprecVapwatMain(argc_, argv_, outputCout_, outputFiles_);
-                }
-                else {
-                    return flowGasWaterBrineMain(argc_, argv_, outputCout_, outputFiles_);
-                }
-            }
-        }
-        else if (eclipseState_->getSimulationConfig().hasPRECSALT()) {
-            if (eclipseState_->getSimulationConfig().hasVAPWAT()) {
-                    //case with water vaporization into gas phase and salt precipitation
-                    return flowBrinePrecsaltVapwatMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-            else {
-                return flowBrineSaltPrecipitationMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-        }
-        else {
-            return flowBrineMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-
-        return EXIT_FAILURE;
-    }
-
-    int runSolvent(const Phases& phases)
-    {
-        if (phases.active(Phase::FOAM)) {
-            return flowSolventFoamMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-        // solvent + gas + water
-        if (!phases.active( Phase::OIL ) && phases.active( Phase::WATER ) && phases.active( Phase::GAS )) {
-            return flowGasWaterSolventMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-
-        // solvent + gas + water + oil
-        if (phases.active( Phase::OIL ) && phases.active( Phase::WATER ) && phases.active( Phase::GAS )) {
-            return flowSolventMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-
-        if (outputCout_)
-            std::cerr << "No valid configuration is found for solvent simulation, valid options include "
-                      << "gas + water + solvent and gas + oil + water + solvent" << std::endl;
-
-        return EXIT_FAILURE;
-    }
-
-    int runExtendedBlackOil()
-    {
-        return flowExtboMain(argc_, argv_, outputCout_, outputFiles_);
-    }
-
-    int runThermal(const Phases& phases)
-    {
-        // oil-gas-thermal
-        if (!phases.active( Phase::WATER ) && phases.active( Phase::OIL ) && phases.active( Phase::GAS )) {
-            return flowGasOilEnergyMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-
-        // water-gas-thermal
-        if (!phases.active( Phase::OIL ) && phases.active( Phase::WATER ) && phases.active( Phase::GAS )) {
-
-            if (phases.active(Phase::BRINE)){
-                return flowGasWaterSaltprecEnergyMain(argc_, argv_, outputCout_, outputFiles_);
-            }
-            return flowGasWaterEnergyMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-
-        return flowEnergyMain(argc_, argv_, outputCout_, outputFiles_);
-    }
-
-    int runBlackOil()
-    {
-        const bool diffusive = eclipseState_->getSimulationConfig().isDiffusive();
-        if (diffusive) {
-            // Use the traditional linearizer, as the TpfaLinearizer does not
-            // support the diffusion module yet.
-            return flowBlackoilMain(argc_, argv_, outputCout_, outputFiles_);
-        } else {
-            return flowBlackoilTpfaMain(argc_, argv_, outputCout_, outputFiles_);
-        }
-    }
+    /// Run a regular three-phase simulation without thermal effects.
+    ///
+    /// Called from dispatchDynamic_()
+    ///
+    /// \return Simulation's status/exit code.
+    int runBlackOil();
 
     void readDeck(const std::string& deckFilename,
                   const std::string& outputDir,
@@ -688,48 +491,18 @@ private:
                   const bool keepKeywords,
                   const std::size_t numThreads,
                   const int output_param,
+                  const bool slaveMode,
                   const std::string& parameters,
                   std::string_view moduleVersion,
                   std::string_view compileTimestamp);
 
     static int getNumThreads()
     {
-
-        int threads;
-
 #ifdef _OPENMP
-        // This function is called before the parallel OpenMP stuff gets initialized.
-        // That initialization happens after the deck is read and we want this message.
-        // Hence we duplicate the code of setupParallelism to get the number of threads.
-        static bool first_time = true;
-        constexpr int default_threads = 2;
-        const int requested_threads = Parameters::Get<Parameters::ThreadsPerProcess>();
-        threads = requested_threads > 0 ? requested_threads : default_threads;
-
-        const char* env_var = getenv("OMP_NUM_THREADS");
-        if (env_var) {
-            int omp_num_threads = -1;
-            auto result = std::from_chars(env_var, env_var + std::strlen(env_var), omp_num_threads);
-            const bool can_output = first_time && FlowGenericVanguard::comm().rank() == 0;
-            if (result.ec == std::errc() && omp_num_threads > 0) {
-                // Set threads to omp_num_threads if it was successfully parsed and is positive
-                threads = omp_num_threads;
-                if (can_output && requested_threads > 0) {
-                    std::cout << "Warning: Environment variable OMP_NUM_THREADS takes precedence over the --threads-per-process cmdline argument."
-                              << std::endl;
-                }
-            } else {
-                if (can_output) {
-                    std::cout << ("Warning: Invalid value for OMP_NUM_THREADS environment variable.") << std::endl;
-                }
-            }
-        }
-
-        first_time = false;
+        return omp_get_max_threads();
 #else
-        threads = 1;
+        return 1;
 #endif
-        return threads;
     }
 
 #if HAVE_DAMARIS
@@ -762,6 +535,9 @@ private:
     // To demonstrate run with non_world_comm
     bool test_split_comm_ = false;
     bool isSimulationRank_ = true;
+#if HAVE_MPI
+    std::string reservoirCouplingSlaveOutputFilename_{};
+#endif
 #if HAVE_DAMARIS
     bool enableDamarisOutput_ = false;
 #endif
