@@ -24,6 +24,18 @@
 
 #include <opm/common/ErrorMacros.hpp>
 
+#if HAVE_MPI
+#define RESERVOIR_COUPLING_ENABLED
+#endif
+#ifdef RESERVOIR_COUPLING_ENABLED
+#include <opm/input/eclipse/Schedule/ResCoup/ReservoirCouplingInfo.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/MasterGroup.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/Slaves.hpp>
+#include <opm/simulators/flow/ReservoirCouplingMaster.hpp>
+#include <opm/simulators/flow/ReservoirCouplingSlave.hpp>
+#include <opm/common/Exceptions.hpp>
+#endif
+
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 #include <opm/grid/utility/StopWatch.hpp>
@@ -34,6 +46,7 @@
 #include <opm/simulators/flow/ConvergenceOutputConfiguration.hpp>
 #include <opm/simulators/flow/ExtraConvergenceOutputThread.hpp>
 #include <opm/simulators/flow/NonlinearSolver.hpp>
+#include <opm/simulators/flow/SimulatorConvergenceOutput.hpp>
 #include <opm/simulators/flow/SimulatorReportBanners.hpp>
 #include <opm/simulators/flow/SimulatorSerializer.hpp>
 #include <opm/simulators/timestepping/AdaptiveTimeStepping.hpp>
@@ -45,18 +58,15 @@
 #include <opm/simulators/utils/HDF5Serializer.hpp>
 #endif
 
-#include <fmt/format.h>
-
-#include <cstddef>
 #include <filesystem>
 #include <memory>
-#include <optional>
 #include <string>
 #include <string_view>
-#include <thread>
 #include <utility>
 #include <vector>
 
+#include <fmt/format.h>
+
 namespace Opm::Parameters {
 
 struct EnableAdaptiveTimeStepping { static constexpr bool value = true; };
@@ -65,15 +75,24 @@ struct SaveStep { static constexpr auto* value = ""; };
 struct SaveFile { static constexpr auto* value = ""; };
 struct LoadFile { static constexpr auto* value = ""; };
 struct LoadStep { static constexpr int value = -1; };
+struct Slave { static constexpr bool value = false; };
 
 } // namespace Opm::Parameters
 
+namespace Opm::detail {
+
+void registerSimulatorParameters();
+
+}
+
 namespace Opm {
 
 /// a simulator for the blackoil model
 template<class TypeTag>
 class SimulatorFullyImplicitBlackoil : private SerializableSim
 {
+protected:
+    struct MPI_Comm_Deleter;
 public:
     using Simulator = GetPropType<TypeTag, Properties::Simulator>;
     using Grid = GetPropType<TypeTag, Properties::Grid>;
@@ -85,12 +104,13 @@ public:
     using SolutionVector = GetPropType<TypeTag, Properties::SolutionVector>;
     using MaterialLawParams = GetPropType<TypeTag, Properties::MaterialLawParams>;
     using AquiferModel = GetPropType<TypeTag, Properties::AquiferModel>;
+    using Model = GetPropType<TypeTag, Properties::NonlinearSystem>;
 
     using TimeStepper = AdaptiveTimeStepping<TypeTag>;
     using PolymerModule = BlackOilPolymerModule<TypeTag>;
     using MICPModule = BlackOilMICPModule<TypeTag>;
 
-    using Model = BlackoilModel<TypeTag>;
+
     using Solver = NonlinearSolver<TypeTag, Model>;
     using ModelParameters = typename Model::ModelParameters;
     using SolverParameters = typename Solver::SolverParameters;
@@ -117,7 +137,7 @@ public:
     /// \param[in] eclipse_state the object which represents an internalized ECL deck
     /// \param[in] output_writer
     /// \param[in] threshold_pressures_by_face   if nonempty, threshold pressures that inhibit flow
-    SimulatorFullyImplicitBlackoil(Simulator& simulator)
+    explicit SimulatorFullyImplicitBlackoil(Simulator& simulator)
         : simulator_(simulator)
         , serializer_(*this,
                       FlowGenericVanguard::comm(),
@@ -134,15 +154,23 @@ public:
         if (this->grid().comm().rank() == 0) {
             this->terminalOutput_ = Parameters::Get<Parameters::EnableTerminalOutput>();
 
-            this->startConvergenceOutputThread(Parameters::Get<Parameters::OutputExtraConvergenceInfo>(),
-                                               R"(OutputExtraConvergenceInfo (--output-extra-convergence-info))");
+            auto getPhaseName = ConvergenceOutputThread::ComponentToPhaseName {
+                [compNames = typename Model::ComponentName{}](const int compIdx)
+                { return std::string_view { compNames.name(compIdx) }; }
+            };
+
+            this->convergence_output_.
+                startThread(this->simulator_.vanguard().eclState(),
+                            Parameters::Get<Parameters::OutputExtraConvergenceInfo>(),
+                            R"(OutputExtraConvergenceInfo (--output-extra-convergence-info))",
+                            getPhaseName);
         }
     }
 
-    ~SimulatorFullyImplicitBlackoil()
+    ~SimulatorFullyImplicitBlackoil() override
     {
         // Safe to call on all ranks, not just the I/O rank.
-        this->endConvergenceOutputThread();
+        convergence_output_.endThread();
     }
 
     static void registerParameters()
@@ -150,39 +178,7 @@ public:
         ModelParameters::registerParameters();
         SolverParameters::registerParameters();
         TimeStepper::registerParameters();
-
-        Parameters::Register<Parameters::EnableTerminalOutput>
-            ("Print high-level information about the simulation's progress to the terminal");
-        Parameters::Register<Parameters::EnableAdaptiveTimeStepping>
-            ("Use adaptive time stepping between report steps");
-        Parameters::Register<Parameters::OutputExtraConvergenceInfo>
-            ("Provide additional convergence output "
-             "files for diagnostic purposes. "
-             "\"none\" gives no extra output and "
-             "overrides all other options, "
-             "\"steps\" generates an INFOSTEP file, "
-             "\"iterations\" generates an INFOITER file. "
-             "Combine options with commas, e.g., "
-             "\"steps,iterations\" for multiple outputs.");
-        Parameters::Register<Parameters::SaveStep>
-            ("Save serialized state to .OPMRST file. "
-             "Either a specific report step, \"all\" to save "
-             "all report steps or \":x\" to save every x'th step."
-             "Use negative values of \"x\" to keep only the last "
-             "written step, or \"last\" to save every step, keeping "
-             "only the last.");
-        Parameters::Register<Parameters::LoadStep>
-            ("Load serialized state from .OPMRST file. "
-             "Either a specific report step, or 0 to load last "
-             "stored report step.");
-        Parameters::Register<Parameters::SaveFile>
-            ("FileName for .OPMRST file used for saving serialized state. "
-             "If empty, CASENAME.OPMRST is used.");
-        Parameters::Hide<Parameters::SaveFile>();
-        Parameters::Register<Parameters::LoadFile>
-            ("FileName for .OPMRST file used to load serialized state. "
-             "If empty, CASENAME.OPMRST is used.");
-        Parameters::Hide<Parameters::LoadFile>();
+        detail::registerSimulatorParameters();
     }
 
     /// Run the simulation.
@@ -191,9 +187,15 @@ public:
     /// \param[in,out] timer       governs the requested reporting timesteps
     /// \param[in,out] state       state of reservoir: pressure, fluxes
     /// \return                    simulation report, with timing data
+#ifdef RESERVOIR_COUPLING_ENABLED
+    SimulatorReport run(SimulatorTimer& timer, int argc, char** argv)
+    {
+        init(timer, argc, argv);
+#else
     SimulatorReport run(SimulatorTimer& timer)
     {
         init(timer);
+#endif
         // Make cache up to date. No need for updating it in elementCtx.
         // NB! Need to be at the correct step in case of restart
         simulator_.setEpisodeIndex(timer.currentStepNum());
@@ -208,8 +210,69 @@ public:
         return finalize();
     }
 
-    void init(SimulatorTimer &timer)
+#ifdef RESERVOIR_COUPLING_ENABLED
+    // This method should only be called if slave mode (i.e. Parameters::Get<Parameters::Slave>())
+    // is false. We try to determine if this is a normal flow simulation or a reservoir
+    // coupling master. It is a normal flow simulation if the schedule does not contain
+    // any SLAVES and GRUPMAST keywords.
+    bool checkRunningAsReservoirCouplingMaster()
     {
+        for (std::size_t report_step = 0; report_step < this->schedule().size(); ++report_step) {
+            auto rescoup = this->schedule()[report_step].rescoup();
+            auto slave_count = rescoup.slaveCount();
+            auto master_group_count = rescoup.masterGroupCount();
+            // - GRUPMAST and SLAVES keywords need to be specified at the same report step
+            // - They can only occur once in the schedule
+            if (slave_count > 0 && master_group_count > 0) {
+                return true;
+            }
+            else if (slave_count > 0 && master_group_count == 0) {
+                throw ReservoirCouplingError(
+                    "Inconsistent reservoir coupling master schedule: "
+                    "Slave count is greater than 0 but master group count is 0"
+                );
+            }
+            else if (slave_count == 0 && master_group_count > 0) {
+                throw ReservoirCouplingError(
+                    "Inconsistent reservoir coupling master schedule: "
+                    "Master group count is greater than 0 but slave count is 0"
+                );
+            }
+        }
+        return false;
+    }
+#endif
+
+#ifdef RESERVOIR_COUPLING_ENABLED
+    // NOTE: The argc and argv will be used when launching a slave process
+    void init(const SimulatorTimer& timer, int argc, char** argv)
+    {
+        auto slave_mode = Parameters::Get<Parameters::Slave>();
+        if (slave_mode) {
+            this->reservoirCouplingSlave_ =
+                std::make_unique<ReservoirCouplingSlave>(
+                    FlowGenericVanguard::comm(),
+                    this->schedule(), timer
+                );
+            this->reservoirCouplingSlave_->sendActivationDateToMasterProcess();
+            this->reservoirCouplingSlave_->sendSimulationStartDateToMasterProcess();
+            this->reservoirCouplingSlave_->receiveMasterGroupNamesFromMasterProcess();
+        }
+        else {
+            auto master_mode = checkRunningAsReservoirCouplingMaster();
+            if (master_mode) {
+                this->reservoirCouplingMaster_ =
+                    std::make_unique<ReservoirCouplingMaster>(
+                        FlowGenericVanguard::comm(),
+                        this->schedule(),
+                        argc, argv
+                    );
+            }
+        }
+#else
+    void init(const SimulatorTimer& timer)
+    {
+#endif
         simulator_.setEpisodeIndex(-1);
 
         // Create timers and file for writing timing info.
@@ -227,12 +290,19 @@ public:
             if (enableTUNING) {
                 adaptiveTimeStepping_ = std::make_unique<TimeStepper>(max_next_tstep,
                                                                       sched_state.tuning(),
-                                                                      unitSystem, terminalOutput_);
+                                                                      unitSystem, report_, terminalOutput_);
             }
             else {
-                adaptiveTimeStepping_ = std::make_unique<TimeStepper>(unitSystem, max_next_tstep, terminalOutput_);
+                adaptiveTimeStepping_ = std::make_unique<TimeStepper>(unitSystem, report_, max_next_tstep, terminalOutput_);
             }
-
+#ifdef RESERVOIR_COUPLING_ENABLED
+            if (this->reservoirCouplingSlave_) {
+                adaptiveTimeStepping_->setReservoirCouplingSlave(this->reservoirCouplingSlave_.get());
+            }
+            else if (this->reservoirCouplingMaster_) {
+                adaptiveTimeStepping_->setReservoirCouplingMaster(this->reservoirCouplingMaster_.get());
+            }
+#endif
             if (isRestart()) {
                 // For restarts the simulator may have gotten some information
                 // about the next timestep size from the OPMEXTRA field
@@ -268,9 +338,6 @@ public:
             std::ostringstream ss;
             timer.report(ss);
             OpmLog::debug(ss.str());
-        }
-
-        if (terminalOutput_) {
             details::outputReportStep(timer);
         }
 
@@ -283,7 +350,7 @@ public:
             simulator_.setEpisodeLength(0.0);
             simulator_.setTimeStepSize(0.0);
             wellModel_().beginReportStep(timer.currentStepNum());
-            simulator_.problem().writeOutput();
+            simulator_.problem().writeOutput(true);
 
             report_.success.output_write_time += perfTimer.stop();
         }
@@ -317,11 +384,14 @@ public:
         // \Note: The report steps are met in any case
         // \Note: The sub stepping will require a copy of the state variables
         if (adaptiveTimeStepping_) {
-            auto tuningUpdater = [enableTUNING, this, reportStep = timer.currentStepNum()]()
+            auto tuningUpdater = [enableTUNING, this,
+                                  reportStep = timer.currentStepNum()](const double curr_time,
+                                                                       double dt, const int timeStep)
             {
                 auto& schedule = this->simulator_.vanguard().schedule();
                 auto& events = this->schedule()[reportStep].events();
 
+                bool result = false;
                 if (events.hasEvent(ScheduleEvents::TUNING_CHANGE)) {
                     // Unset the event to not trigger it again on the next sub step
                     schedule.clear_event(ScheduleEvents::TUNING_CHANGE, reportStep);
@@ -335,14 +405,56 @@ public:
                         // \Note: Need to update both solver (model) and simulator since solver is re-created each report step.
                         solver_->model().updateTUNING(tuning);
                         this->updateTUNING(tuning);
+                        dt = this->adaptiveTimeStepping_->suggestedNextStep();
                     } else {
+                        dt = max_next_tstep;
                         this->adaptiveTimeStepping_->updateNEXTSTEP(max_next_tstep);
                     }
-                    return max_next_tstep >0;
+                    result = max_next_tstep > 0;
                 }
-                return false;
+
+                const auto& wcycle = schedule[reportStep].wcycle.get();
+                if (wcycle.empty()) {
+                    return result;
+                }
+
+                const auto& wmatcher = schedule.wellMatcher(reportStep);
+                double wcycle_time_step =
+                    wcycle.nextTimeStep(curr_time,
+                                        dt,
+                                        wmatcher,
+                                        this->wellModel_().wellOpenTimes(),
+                                        this->wellModel_().wellCloseTimes(),
+                                        [timeStep,
+                                         &wg_events = this->wellModel_().reportStepStartEvents()]
+                                        (const std::string& name)
+                                        {
+                                            if (timeStep != 0) {
+                                                return false;
+                                            }
+                                            return wg_events.hasEvent(name, ScheduleEvents::REQUEST_OPEN_WELL);
+                                        });
+
+                wcycle_time_step = this->grid().comm().min(wcycle_time_step);
+                if (dt != wcycle_time_step) {
+                    this->adaptiveTimeStepping_->updateNEXTSTEP(wcycle_time_step);
+                    return true;
+                }
+
+                return result;
             };
-            tuningUpdater();
+
+            tuningUpdater(timer.simulationTimeElapsed(),
+                          this->adaptiveTimeStepping_->suggestedNextStep(), 0);
+
+#ifdef RESERVOIR_COUPLING_ENABLED
+            if (this->reservoirCouplingMaster_) {
+                this->reservoirCouplingMaster_->maybeSpawnSlaveProcesses(timer.currentStepNum());
+            }
+            else if (this->reservoirCouplingSlave_) {
+                this->reservoirCouplingSlave_->maybeActivate(timer.currentStepNum());
+            }
+#endif
             const auto& events = schedule()[timer.currentStepNum()].events();
             bool event = events.hasEvent(ScheduleEvents::NEW_WELL) ||
                 events.hasEvent(ScheduleEvents::INJECTION_TYPE_CHANGED) ||
@@ -350,7 +462,7 @@ public:
                 events.hasEvent(ScheduleEvents::PRODUCTION_UPDATE) ||
                 events.hasEvent(ScheduleEvents::INJECTION_UPDATE) ||
                 events.hasEvent(ScheduleEvents::WELL_STATUS_CHANGE);
-            auto stepReport = adaptiveTimeStepping_->step(timer, *solver_, event, nullptr, tuningUpdater);
+            auto stepReport = adaptiveTimeStepping_->step(timer, *solver_, event, tuningUpdater);
             report_ += stepReport;
             //Pass simulation report to eclwriter for summary output
             simulator_.problem().setSimulationReport(report_);
@@ -370,7 +482,7 @@ public:
         perfTimer.start();
         const double nextstep = adaptiveTimeStepping_ ? adaptiveTimeStepping_->suggestedNextStep() : -1.0;
         simulator_.problem().setNextTimeStepSize(nextstep);
-        simulator_.problem().writeOutput();
+        simulator_.problem().writeOutput(true);
         report_.success.output_write_time += perfTimer.stop();
 
         solver_->model().endReportStep();
@@ -385,14 +497,7 @@ public:
             // Grab the step convergence reports that are new since last we
             // were here.
             const auto& reps = this->solver_->model().stepReports();
-
-            auto reports = std::vector<StepReport> {
-                reps.begin() + this->already_reported_steps_, reps.end()
-            };
-
-            this->writeConvergenceOutput(std::move(reports));
-
-            this->already_reported_steps_ = reps.size();
+            convergence_output_.write(reps);
         }
 
         // Increment timer, remember well state.
@@ -480,7 +585,6 @@ protected:
         return simulator_.vanguard().globalCell();
     }
 
-
     std::unique_ptr<Solver> createSolver(WellModel& wellModel)
     {
         auto model = std::make_unique<Model>(simulator_,
@@ -528,65 +632,6 @@ protected:
     const WellModel& wellModel_() const
     { return simulator_.problem().wellModel(); }
 
-    void startConvergenceOutputThread(std::string_view convOutputOptions,
-                                      std::string_view optionName)
-    {
-        const auto config = ConvergenceOutputConfiguration {
-            convOutputOptions, optionName
-        };
-        if (! config.want(ConvergenceOutputConfiguration::Option::Iterations)) {
-            return;
-        }
-
-        auto getPhaseName = ConvergenceOutputThread::ComponentToPhaseName {
-            [compNames = typename Model::ComponentName{}](const int compIdx)
-            { return std::string_view { compNames.name(compIdx) }; }
-        };
-
-        auto convertTime = ConvergenceOutputThread::ConvertToTimeUnits {
-            [usys = this->eclState().getUnits()](const double time)
-            { return usys.from_si(UnitSystem::measure::time, time); }
-        };
-
-        this->convergenceOutputQueue_.emplace();
-        this->convergenceOutputObject_.emplace
-            (this->eclState().getIOConfig().getOutputDir(),
-             this->eclState().getIOConfig().getBaseName(),
-             std::move(getPhaseName),
-             std::move(convertTime),
-             config, *this->convergenceOutputQueue_);
-
-        this->convergenceOutputThread_
-            .emplace(&ConvergenceOutputThread::writeASynchronous,
-                     &this->convergenceOutputObject_.value());
-    }
-
-    void writeConvergenceOutput(std::vector<StepReport>&& reports)
-    {
-        if (! this->convergenceOutputThread_.has_value()) {
-            return;
-        }
-
-        auto requests = std::vector<ConvergenceReportQueue::OutputRequest>{};
-        requests.reserve(reports.size());
-
-        for (auto&& report : reports) {
-            requests.push_back({ report.report_step, report.current_step, std::move(report.report) });
-        }
-
-        this->convergenceOutputQueue_->enqueue(std::move(requests));
-    }
-
-    void endConvergenceOutputThread()
-    {
-        if (! this->convergenceOutputThread_.has_value()) {
-            return;
-        }
-
-        this->convergenceOutputQueue_->signalLastOutputRequest();
-        this->convergenceOutputThread_->join();
-    }
-
     // Data.
     Simulator& simulator_;
 
@@ -601,14 +646,17 @@ protected:
     bool terminalOutput_;
 
     SimulatorReport report_;
-    std::size_t already_reported_steps_ = 0;
     std::unique_ptr<time::StopWatch> solverTimer_;
     std::unique_ptr<time::StopWatch> totalTimer_;
     std::unique_ptr<TimeStepper> adaptiveTimeStepping_;
 
-    std::optional<ConvergenceReportQueue> convergenceOutputQueue_{};
-    std::optional<ConvergenceOutputThread> convergenceOutputObject_{};
-    std::optional<std::thread> convergenceOutputThread_{};
+    SimulatorConvergenceOutput convergence_output_{};
+
+#ifdef RESERVOIR_COUPLING_ENABLED
+    bool slaveMode_{false};
+    std::unique_ptr<ReservoirCouplingMaster> reservoirCouplingMaster_{nullptr};
+    std::unique_ptr<ReservoirCouplingSlave> reservoirCouplingSlave_{nullptr};
+#endif
 
     SimulatorSerializer serializer_;
 };
