@@ -21,15 +21,24 @@
 #define OPM_PY_BLACKOIL_SIMULATOR_HEADER_INCLUDED
 
 #include <python/simulators/PyMain.hpp>
-#include <opm/simulators/flow/FlowMain.hpp>
-#include <opm/models/utils/propertysystem.hh>
+
 #include <opm/models/utils/parametersystem.hpp>
-#include <opm/simulators/flow/python/Pybind11Exporter.hpp>
+#include <opm/models/utils/propertysystem.hh>
+
+#include <opm/simulators/flow/FlowMain.hpp>
+#include <opm/simulators/flow/TTagFlowProblemTPFA.hpp>
 #include <opm/simulators/flow/python/PyFluidState.hpp>
 #include <opm/simulators/flow/python/PyMaterialState.hpp>
+#include <opm/simulators/flow/python/Pybind11Exporter.hpp>
+
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
+#include <opm/input/eclipse/Schedule/Schedule.hpp>
+
+#include <map>
+#include <memory>
+#include <string>
+#include <vector>
 
 namespace Opm::Pybind {
 
@@ -40,7 +49,8 @@ private:
     using Simulator = Opm::GetPropType<TypeTag, Opm::Properties::Simulator>;
 
 public:
-    PyBlackOilSimulator( const std::string& deckFilename);
+    PyBlackOilSimulator(const std::string& deckFilename,
+                        const std::vector<std::string>& args);
     PyBlackOilSimulator(
         std::shared_ptr<Opm::Deck> deck,
         std::shared_ptr<Opm::EclipseState> state,
@@ -92,6 +102,7 @@ private:
     std::shared_ptr<Opm::EclipseState> eclipse_state_;
     std::shared_ptr<Opm::Schedule> schedule_;
     std::shared_ptr<Opm::SummaryConfig> summary_config_;
+    std::vector<std::string> args_;
 };
 
 } // namespace Opm::Pybind
