@@ -31,6 +31,8 @@
 #include <opm/simulators/linalg/GraphColoring.hpp>
 #include <opm/simulators/linalg/matrixblock.hh>
 
+#include <cassert>
+
 namespace Opm
 {
 namespace detail
@@ -41,6 +43,7 @@ template<class M>
 void ghost_last_bilu0_decomposition (M& A, std::size_t interiorSize)
 {
     // iterator types
+    assert(interiorSize <= A.N());
     using rowiterator = typename M::RowIterator;
     using coliterator = typename M::ColIterator;
     using block = typename M::block_type;
@@ -311,6 +314,7 @@ ParallelOverlappingILU0(const Matrix& A,
 {
     // BlockMatrix is a Subclass of FieldMatrix that just adds
     // methods. Therefore this cast should be safe.
+    assert(interiorSize <= A_->N());
     update( );
 }
 
@@ -443,6 +447,7 @@ update()
 
             if (comm_) {
                 interiorSize_ = detail::set_interiorSize(A_->N(), interiorSize_, *comm_);
+                assert(interiorSize_ <= A_->N());
             }
 
             // create ILU-0 decomposition
@@ -511,11 +516,7 @@ update()
                 break;
             default:
                 if (interiorSize_ == A_->N())
-#if DUNE_VERSION_LT(DUNE_GRID, 2, 8)
-                    bilu0_decomposition( *ILU_ );
-#else
                     Dune::ILU::blockILU0Decomposition( *ILU_ );
-#endif
                 else
                     detail::ghost_last_bilu0_decomposition(*ILU_, interiorSize_);
                 break;
