@@ -33,9 +33,10 @@ namespace Opm
     SimulatorReportSingle SimulatorReportSingle::serializationTestObject()
     {
         return SimulatorReportSingle{1.0, 2.0, 3.0, 4.0, 5.0, 6.0,
-                                     7.0, 8.0, 9.0, 10.0, 11.0,
-                                     12, 13, 14, 15, 16, 17,
-                                     true, false, 18, 19.0, 20.0};
+                                     7.0, 8.0, 9.0, 10.0, 11.0, 12.0,
+                                     13, 14, 15, 16, 17, 18,
+                                     true, false, 19, 20.0, 21.0,
+                                     22, 23, 24, 25, 26, 27, 28};
     }
 
     bool SimulatorReportSingle::operator==(const SimulatorReportSingle& rhs) const
@@ -49,6 +50,7 @@ namespace Opm
                this->assemble_time_well == rhs.assemble_time_well &&
                this->linear_solve_setup_time == rhs.linear_solve_setup_time &&
                this->linear_solve_time == rhs.linear_solve_time &&
+               this->local_solve_time == rhs.local_solve_time &&
                this->update_time == rhs.update_time &&
                this->output_write_time == rhs.output_write_time &&
                this->total_well_iterations == rhs.total_well_iterations &&
@@ -61,7 +63,14 @@ namespace Opm
                this->well_group_control_changed == rhs.well_group_control_changed &&
                this->exit_status == rhs.exit_status &&
                this->global_time == rhs.global_time &&
-               this->timestep_length == rhs.timestep_length;
+               this->timestep_length == rhs.timestep_length &&
+               this->num_domains == rhs.num_domains &&
+               this->num_wells == rhs.num_wells &&
+               this->num_overlap_cells == rhs.num_overlap_cells &&
+               this->num_owned_cells == rhs.num_owned_cells &&
+               this->converged_domains == rhs.converged_domains &&
+               this->unconverged_domains == rhs.unconverged_domains &&
+               this->accepted_unconverged_domains == rhs.accepted_unconverged_domains;
     }
 
     void SimulatorReportSingle::operator+=(const SimulatorReportSingle& sr)
@@ -70,6 +79,7 @@ namespace Opm
         transport_time += sr.transport_time;
         linear_solve_setup_time += sr.linear_solve_setup_time;
         linear_solve_time += sr.linear_solve_time;
+        local_solve_time += sr.local_solve_time;
         solver_time += sr.solver_time;
         assemble_time += sr.assemble_time;
         pre_post_time += sr.pre_post_time;
@@ -86,6 +96,10 @@ namespace Opm
         }
         max_linear_iterations = std::max(max_linear_iterations, sr.total_linear_iterations);
 
+        converged_domains += sr.converged_domains;
+        unconverged_domains += sr.unconverged_domains;
+        accepted_unconverged_domains += sr.accepted_unconverged_domains;
+
         // It makes no sense adding time points. Therefore, do not 
         // overwrite the value of global_time which gets set in 
         // NonlinearSolver.hpp by the line:
@@ -106,14 +120,16 @@ namespace Opm
                           linear_solve_time);
     }
 
+    // Helper lambda to avoid division by zero
+    auto noZero = [](auto val)
+    {
+        if (val == decltype(val){0})
+            return decltype(val){1};
+        return val;
+    };
+
     void SimulatorReportSingle::reportFullyImplicit(std::ostream& os, const SimulatorReportSingle* failureReport) const
     {
-        auto noZero = [](auto val)
-        {
-            if (val == decltype(val){0})
-                return decltype(val){1};
-            return val;
-        };
         // Disabling this output as it is redundant with the solver_time, now
         // output as "Simulation time". Usually very small difference between them.
         // os << fmt::format("Total time:                 {:9.2f} s\n", total_time);
@@ -160,6 +176,17 @@ namespace Opm
             }
             os << std::endl;
 
+            if (local_solve_time > 0.0) {
+                t = local_solve_time + (failureReport ? failureReport->local_solve_time : 0.0);
+                os << fmt::format("  Local solve time:           {:7.2f} s", t);
+                if (failureReport) {
+                  os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                                    failureReport->local_solve_time,
+                                    100*failureReport->local_solve_time/noZero(t));
+                }
+                os << std::endl;
+            }
+
             t = update_time + (failureReport ? failureReport->update_time : 0.0);
             os << fmt::format("  Props/update time:          {:7.2f} s", t);
             if (failureReport) {
@@ -210,6 +237,112 @@ namespace Opm
         os << std::endl;
     }
 
+
+    void SimulatorReportSingle::reportNLDD(std::ostream& os, const SimulatorReportSingle* failureReport) const
+    {
+        os << fmt::format("Owned + overlap cells:       {:7}\n", num_owned_cells + num_overlap_cells);
+        os << fmt::format("Number of wells:             {:7}\n", num_wells);
+        os << fmt::format("Number of domains:           {:7}\n", num_domains);
+        os << fmt::format("-------------------------------------------------------\n");
+        double t = total_time + (failureReport ? failureReport->total_time : 0.0);
+        os << fmt::format("Total time:                   {:9.2f} s\n", t);
+
+        t = pre_post_time + (failureReport ? failureReport->pre_post_time : 0.0);
+        os << fmt::format("  Pre/post/wait time:           {:7.2f} s\n", t);
+
+        t = solver_time + (failureReport ? failureReport->solver_time : 0.0);
+        os << fmt::format("  Solver time:                {:9.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->solver_time,
+                            100*failureReport->solver_time/noZero(t));
+        }
+        os << std::endl;
+
+        t = assemble_time + (failureReport ? failureReport->assemble_time : 0.0);
+        os << fmt::format("    Assembly time:            {:9.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->assemble_time,
+                            100*failureReport->assemble_time/noZero(t));
+            }
+        os << std::endl;
+
+        t = assemble_time_well + (failureReport ? failureReport->assemble_time_well : 0.0);
+        os << fmt::format("      Well assembly:            {:7.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->assemble_time_well,
+                            100*failureReport->assemble_time_well/noZero(t));
+        }
+        os << std::endl;
+
+        t = linear_solve_time + (failureReport ? failureReport->linear_solve_time : 0.0);
+        os << fmt::format("    Linear solve time:        {:9.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->linear_solve_time,
+                            100*failureReport->linear_solve_time/noZero(t));
+        }
+        os << std::endl;
+
+        t = linear_solve_setup_time + (failureReport ? failureReport->linear_solve_setup_time : 0.0);
+        os << fmt::format("      Linear setup:             {:7.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->linear_solve_setup_time,
+                            100*failureReport->linear_solve_setup_time/noZero(t));
+        }
+        os << std::endl;
+
+        t = update_time + (failureReport ? failureReport->update_time : 0.0);
+        os << fmt::format("    Props/update time:          {:7.2f} s", t);
+        if (failureReport) {
+            os << fmt::format(" (Wasted: {:2.1f} s; {:2.1f}%)",
+                            failureReport->update_time,
+                            100*failureReport->update_time/noZero(t));
+        }
+        os << std::endl;
+
+        int n = total_linearizations + (failureReport ? failureReport->total_linearizations : 0);
+        os << fmt::format("Overall Linearizations:       {:7}", n);
+        if (failureReport) {
+          os << fmt::format("   (Wasted: {:5}; {:2.1f}%)",
+                            failureReport->total_linearizations,
+                            100.0*failureReport->total_linearizations/noZero(n));
+        }
+        os << std::endl;
+
+        n = total_newton_iterations + (failureReport ? failureReport->total_newton_iterations : 0);
+        os << fmt::format("Overall Nonlinear Iterations: {:7}", n);
+        if (failureReport) {
+          os << fmt::format("   (Wasted: {:5}; {:2.1f}%)",
+                            failureReport->total_newton_iterations,
+                            100.0*failureReport->total_newton_iterations/noZero(n));
+        }
+        os << std::endl;
+
+        n = total_linear_iterations + (failureReport ? failureReport->total_linear_iterations : 0);
+        os << fmt::format("Overall Linear Iterations:    {:7}", n);
+        if (failureReport) {
+          os << fmt::format("   (Wasted: {:5}; {:2.1f}%)",
+                            failureReport->total_linear_iterations,
+                            100.0*failureReport->total_linear_iterations/noZero(n));
+        }
+        os << std::endl;
+        os << fmt::format("-------------------------------------------------------\n");
+        n = converged_domains + (failureReport ? failureReport->converged_domains : 0);
+        os << fmt::format("Converged domain solves:     {:7}", n);
+        os << std::endl;
+        n = accepted_unconverged_domains + (failureReport ? failureReport->accepted_unconverged_domains : 0);
+        os << fmt::format("  Accepted with relaxed tol: {:7}", n);
+        os << std::endl;
+        n = unconverged_domains + (failureReport ? failureReport->unconverged_domains : 0);
+        os << fmt::format("Unconverged domain solves:   {:7}", n);
+        os << std::endl << std::endl;
+
+    }
+
     SimulatorReport SimulatorReport::serializationTestObject()
     {
         return SimulatorReport{SimulatorReportSingle::serializationTestObject(),
@@ -247,9 +380,14 @@ namespace Opm
         success.reportFullyImplicit(os, &failure);
     }
 
+    void SimulatorReport::reportNLDD(std::ostream& os) const
+    {
+        success.reportNLDD(os, &failure);
+    }
+
     void SimulatorReport::fullReports(std::ostream& os) const
     {
-        os << "  Time(day)  TStep(day)  Assembly    LSetup    LSolve    Update    Output WellIt Lins NewtIt LinIt Conv\n";
+        os << "  Time(day)  TStep(day)  Assembly    LSetup    LSolve    LocSol    Update    Output WellIt Lins NewtIt LinIt Conv\n";
         for (std::size_t i = 0; i < this->stepreports.size(); ++i) {
             const SimulatorReportSingle& sr = this->stepreports[i];
             os.precision(10);
@@ -261,6 +399,7 @@ namespace Opm
             os << std::setw(9) << sr.assemble_time << " ";
             os << std::setw(9) << sr.linear_solve_setup_time << " ";
             os << std::setw(9) << sr.linear_solve_time << " ";
+            os << std::setw(9) << sr.local_solve_time << " ";
             os << std::setw(9) << sr.update_time << " ";
             os << std::setw(9) << sr.output_write_time << " ";
             os.precision(6);
