@@ -22,6 +22,7 @@
 #define OPM_TIMESTEPCONTROL_HEADER_INCLUDED
 
 #include <opm/simulators/timestepping/TimeStepControlInterface.hpp>
+#include <opm/simulators/timestepping/AdaptiveSimulatorTimer.hpp>
 
 #include <string>
 #include <vector>
@@ -32,7 +33,8 @@ namespace Opm
       SimpleIterationCount,
       PID,
       PIDAndIterationCount,
-      HardCodedTimeStep
+      HardCodedTimeStep,
+      General3rdOrder,
     };
 
     ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -59,7 +61,10 @@ namespace Opm
         static SimpleIterationCountTimeStepControl serializationTestObject();
 
         /// \brief \copydoc TimeStepControlInterface::computeTimeStepSize
-        double computeTimeStepSize( const double dt, const int iterations, const RelativeChangeInterface& /* relativeChange */, const double /*simulationTimeElapsed */ ) const;
+        double computeTimeStepSize(const double dt,
+                                   const int iterations,
+                                   const RelativeChangeInterface& /* relativeChange */,
+                                   const AdaptiveSimulatorTimer& /* substepTimer */ ) const override;
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
@@ -101,13 +106,16 @@ namespace Opm
         /// \param tol      tolerance for the relative changes of the numerical solution to be accepted
         ///                 in one time step (default is 1e-3)
         /// \param verbose  if true get some output (default = false)
-        PIDTimeStepControl( const double tol = 1e-3,
-                            const bool verbose = false );
+        explicit PIDTimeStepControl(const double tol = 1e-3,
+                                    const bool verbose = false);
 
         static PIDTimeStepControl serializationTestObject();
 
         /// \brief \copydoc TimeStepControlInterface::computeTimeStepSize
-        double computeTimeStepSize( const double dt, const int /* iterations */, const RelativeChangeInterface& relativeChange, const double /*simulationTimeElapsed */ ) const;
+        double computeTimeStepSize(const double dt,
+                                   const int /* iterations */,
+                                   const RelativeChangeInterface& relativeChange,
+                                   const AdaptiveSimulatorTimer& /* substepTimer */ ) const override;
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
@@ -143,17 +151,20 @@ namespace Opm
         /// \param tol        tolerance for the relative changes of the numerical solution to be accepted
         ///                   in one time step (default is 1e-3)
         /// \param verbose    if true get some output (default = false)
-        PIDAndIterationCountTimeStepControl( const int target_iterations = 20,
-                                             const double decayDampingFactor = 1.0,
-                                             const double growthDampingFactor = 1.0/1.2,
-                                             const double tol = 1e-3,
-                                             const double minTimeStepBasedOnIterations = 0.,
-                                             const bool verbose = false);
+        explicit PIDAndIterationCountTimeStepControl(const int target_iterations = 20,
+                                                     const double decayDampingFactor = 1.0,
+                                                     const double growthDampingFactor = 1.0/1.2,
+                                                     const double tol = 1e-3,
+                                                     const double minTimeStepBasedOnIterations = 0.,
+                                                     const bool verbose = false);
 
         static PIDAndIterationCountTimeStepControl serializationTestObject();
 
         /// \brief \copydoc TimeStepControlInterface::computeTimeStepSize
-        double computeTimeStepSize( const double dt, const int iterations, const RelativeChangeInterface& relativeChange, const double /*simulationTimeElapsed */ ) const;
+        double computeTimeStepSize(const double dt,
+                                   const int iterations,
+                                   const RelativeChangeInterface& relativeChange,
+                                   const AdaptiveSimulatorTimer& /* substepTimer */ ) const override;
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
@@ -174,6 +185,49 @@ namespace Opm
         const double  minTimeStepBasedOnIterations_;
     };
 
+    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
+    ///
+    ///  General 3rd order controller
+    ///
+    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
+    class General3rdOrderController : public TimeStepControlInterface
+    {
+    public:
+        static constexpr TimeStepControlType Type = TimeStepControlType::General3rdOrder;
+
+        General3rdOrderController( const double tolerance = 1e-3,
+                                   const double safetyFactor = 0.8,
+                                   const bool verbose = false );
+
+        static General3rdOrderController serializationTestObject();
+
+        double computeTimeStepSize(const double dt,
+                                   const int /*iterations */,
+                                   const RelativeChangeInterface& relativeChange,
+                                   const AdaptiveSimulatorTimer& substepTimer) const override;
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(tolerance_);
+            serializer(safetyFactor_);
+            serializer(errors_);
+            serializer(timeSteps_);
+            serializer(verbose_);
+        }
+
+        bool operator==(const General3rdOrderController&) const;
+
+
+    protected:
+        const double tolerance_ = 1e-3;
+        const double safetyFactor_ = 0.8;
+        mutable std::vector<double> errors_{};
+        mutable std::vector<double> timeSteps_{};
+        mutable int counterSinceFailure_ = 0;
+        const bool verbose_ = false;
+    };
+
     ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
     ///
     ///  HardcodedTimeStepControl
@@ -191,12 +245,15 @@ namespace Opm
 
         /// \brief constructor
         /// \param filename   filename contaning the timesteps
-        explicit HardcodedTimeStepControl( const std::string& filename);
+        explicit HardcodedTimeStepControl(const std::string& filename);
 
         static HardcodedTimeStepControl serializationTestObject();
 
         /// \brief \copydoc TimeStepControlInterface::computeTimeStepSize
-        double computeTimeStepSize( const double dt, const int /* iterations */, const RelativeChangeInterface& /*relativeChange */, const double simulationTimeElapsed) const;
+        double computeTimeStepSize(const double dt,
+                                   const int /* iterations */,
+                                   const RelativeChangeInterface& /*relativeChange */,
+                                   const AdaptiveSimulatorTimer& substepTimer) const override;
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
