@@ -34,6 +34,9 @@ namespace Opm::Mpi {
 
 namespace detail {
 
+template <typename T>
+constexpr bool is_pod_v = std::is_standard_layout_v<T> && std::is_trivial_v<T>;
+
 std::size_t mpi_buffer_size(const std::size_t bufsize, const std::size_t position);
 
 //! \brief Abstract struct for packing which is (partially) specialized for specific types.
@@ -189,7 +192,7 @@ struct Packer
 {
     //! \brief Constructor.
     //! \param comm The communicator to use
-    Packer(Parallel::Communication comm)
+    explicit Packer(Parallel::Communication comm)
         : m_comm(comm)
     {}
 
@@ -199,7 +202,7 @@ struct Packer
     template<class T>
     std::size_t packSize(const T& data) const
     {
-        return detail::Packing<std::is_pod_v<T>,T>::packSize(data, m_comm);
+        return detail::Packing<detail::is_pod_v<T>,T>::packSize(data, m_comm);
     }
 
     //! \brief Calculates the pack size for an array.
@@ -209,7 +212,7 @@ struct Packer
     template<class T>
     std::size_t packSize(const T* data, std::size_t n) const
     {
-        static_assert(std::is_pod_v<T>, "Array packing not supported for non-pod data");
+        static_assert(detail::is_pod_v<T>, "Array packing not supported for non-pod data");
         return detail::Packing<true,T>::packSize(data, n, m_comm);
     }
 
@@ -223,7 +226,7 @@ struct Packer
               std::vector<char>& buffer,
               std::size_t& position) const
     {
-        detail::Packing<std::is_pod_v<T>,T>::pack(data, buffer, position, m_comm);
+        detail::Packing<detail::is_pod_v<T>,T>::pack(data, buffer, position, m_comm);
     }
 
     //! \brief Pack an array.
@@ -238,7 +241,7 @@ struct Packer
               std::vector<char>& buffer,
               std::size_t& position) const
     {
-        static_assert(std::is_pod_v<T>, "Array packing not supported for non-pod data");
+        static_assert(detail::is_pod_v<T>, "Array packing not supported for non-pod data");
         detail::Packing<true,T>::pack(data, n, buffer, position, m_comm);
     }
 
@@ -252,7 +255,7 @@ struct Packer
                 const std::vector<char>& buffer,
                 std::size_t& position) const
     {
-        detail::Packing<std::is_pod_v<T>,T>::unpack(data, buffer, position, m_comm);
+        detail::Packing<detail::is_pod_v<T>,T>::unpack(data, buffer, position, m_comm);
     }
 
     //! \brief Unpack an array.
@@ -267,7 +270,7 @@ struct Packer
                 const std::vector<char>& buffer,
                 std::size_t& position) const
     {
-        static_assert(std::is_pod_v<T>, "Array packing not supported for non-pod data");
+        static_assert(detail::is_pod_v<T>, "Array packing not supported for non-pod data");
         detail::Packing<true,T>::unpack(data, n, buffer, position, m_comm);
     }
 
