@@ -16,13 +16,30 @@
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
+#include "opm/input/eclipse/EclipseState/Grid/FieldProps.hpp"
 #include <config.h>
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
 
+#include <opm/input/eclipse/EclipseState/Grid/FIPRegionStatistics.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldData.hpp>
+#include <opm/input/eclipse/EclipseState/Runspec.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
 
+#include <cstddef>
+#include <map>
+#include <regex>
+#include <stdexcept>
+#include <string>
+#include <vector>
+
+namespace {
+    bool is_FIP(const std::string& keyword)
+    {
+        return std::regex_match(keyword, std::regex { "FIP[A-Z0-9]{1,5}" });
+    }
+}
+
 namespace Opm {
 
 
@@ -63,7 +80,7 @@ std::vector<double> ParallelFieldPropsManager::porv(bool global) const
     if (m_comm.rank() == 0)
         global_porv = m_manager.porv(true);
 
-    size_t size = global_porv.size();
+    std::size_t size = global_porv.size();
     m_comm.broadcast(&size, 1, 0);
     global_porv.resize(size);
     m_comm.broadcast(global_porv.data(), size, 0);
@@ -106,12 +123,16 @@ std::vector<int> ParallelFieldPropsManager::get_global_int(const std::string& ke
     std::vector<int> result;
     int exceptionThrown{};
 
-    if (m_comm.rank() == 0)
-    {
-        try
-        {
-            result = m_manager.get_global_int(keyword);
-        }catch(std::exception& e) {
+    if (m_comm.rank() == 0) {
+        try {
+            // Recall: FIP* keywords are special.  We care only about the
+            // first three characters of the name following the initial
+            // three-character "FIP" prefix, hence "substr(0, 6)".
+            result = is_FIP(keyword)
+                ? this->m_manager.get_global_int(keyword.substr(0, 6))
+                : this->m_manager.get_global_int(keyword);
+        }
+        catch (std::exception& e) {
             exceptionThrown = 1;
             OpmLog::error("No integer property field: " + keyword + " ("+e.what()+")");
             m_comm.broadcast(&exceptionThrown, 1, 0);
@@ -124,7 +145,7 @@ std::vector<int> ParallelFieldPropsManager::get_global_int(const std::string& ke
     if (exceptionThrown)
         OPM_THROW_NOLOG(std::runtime_error, "No integer property field: " + keyword);
 
-    size_t size = result.size();
+    std::size_t size = result.size();
     m_comm.broadcast(&size, 1, 0);
     result.resize(size);
     m_comm.broadcast(result.data(), size, 0);
@@ -178,7 +199,7 @@ std::vector<double> ParallelFieldPropsManager::get_global_double(const std::stri
     if (exceptionThrown)
         OPM_THROW_NOLOG(std::runtime_error, "No double property field: " + keyword);
 
-    size_t size = result.size();
+    std::size_t size = result.size();
     m_comm.broadcast(&size, 1, 0);
     result.resize(size);
     m_comm.broadcast(result.data(), size, 0);
@@ -210,6 +231,19 @@ bool ParallelFieldPropsManager::has_double(const std::string& keyword) const
     return it != m_doubleProps.end();
 }
 
+std::vector<std::string> ParallelFieldPropsManager::fip_regions() const
+{
+    constexpr auto maxchar = std::string::size_type{6};
+
+    std::vector<std::string> result;
+    for (const auto& key : m_intProps) {
+        if (Fieldprops::keywords::isFipxxx(key.first)) {
+            result.push_back(key.first.substr(0, maxchar));
+        }
+    }
+    return result;
+}
+
 
 ParallelEclipseState::ParallelEclipseState(Parallel::Communication comm)
     : m_fieldProps(field_props, comm)
@@ -251,6 +285,20 @@ const FieldPropsManager& ParallelEclipseState::globalFieldProps() const
 }
 
 
+void ParallelEclipseState::computeFipRegionStatistics()
+{
+    if (! this->fipRegionStatistics_.has_value()) {
+        this->fipRegionStatistics_
+            .emplace(declaredMaxRegionID(this->runspec()),
+                     this->fieldProps(),
+                     [this](std::vector<int>& maxRegionID)
+                     {
+                         this->m_comm.max(maxRegionID.data(), maxRegionID.size());
+                     });
+    }
+}
+
+
 const EclipseGrid& ParallelEclipseState::getInputGrid() const
 {
     if (m_comm.rank() != 0)
