@@ -253,7 +253,6 @@ const KeywordValidation::UnsupportedKeywords& unsupportedKeywords()
         {"GTADD", {true, std::nullopt}},
         {"GTMULT", {true, std::nullopt}},
         {"GUIDECAL", {true, std::nullopt}},
-        {"GSATPROD", {true, std::nullopt}},
         {"GUPFREQ", {true, std::nullopt}},
         {"GWRTWCV", {true, std::nullopt}},
         {"HALFTRAN", {true, std::nullopt}},
@@ -398,7 +397,6 @@ const KeywordValidation::UnsupportedKeywords& unsupportedKeywords()
         {"NARROW", {true, std::nullopt}},
         {"NCONSUMP", {true, std::nullopt}},
         {"NCOMPS", {false, std::nullopt}},
-        {"NEFAC", {true, std::nullopt}},
         {"NETCOMPA", {true, std::nullopt}},
         {"NEXT", {false, std::nullopt}},
         {"NEXTSTPL", {true, std::nullopt}},
@@ -486,6 +484,7 @@ const KeywordValidation::UnsupportedKeywords& unsupportedKeywords()
         {"PLYATEMP", {true, std::nullopt}},
         {"PLYCAMAX", {true, std::nullopt}},
         {"PLYDHFLF", {true, std::nullopt}},
+        {"PLYSHEAR", {true, std::nullopt}},
         {"PRORDER", {true, std::nullopt}},
         {"PRVD", {true, std::nullopt}},
         {"PVTGWO", {true, std::nullopt}},
@@ -557,7 +556,6 @@ const KeywordValidation::UnsupportedKeywords& unsupportedKeywords()
         {"SEPVALS", {true, std::nullopt}},
         {"SFOAM", {true, std::nullopt}},
         {"SGF32D", {true, std::nullopt}},
-        {"SHRATE", {true, std::string{"See the PLYSHEAR keyword instead"}}},
         {"SIGMA", {true, std::nullopt}},
         {"SIGMAGD", {true, std::nullopt}},
         {"SIGMAGDV", {true, std::nullopt}},
@@ -672,7 +670,6 @@ const KeywordValidation::UnsupportedKeywords& unsupportedKeywords()
         {"WCONINJP", {true, std::nullopt}},
         {"WCUTBACK", {true, std::nullopt}},
         {"WCUTBACT", {true, std::nullopt}},
-        {"WCYCLE", {true, std::nullopt}},
         {"WDRILTIM", {true, std::nullopt}},
         {"WDRILPRI", {true, std::nullopt}},
         {"WDRILRES", {true, std::nullopt}},
