@@ -54,8 +54,11 @@
 
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
 #include <opm/input/eclipse/Schedule/ArrayDimChecker.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/ReservoirCouplingInfo.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/MasterGroup.hpp>
+#include <opm/input/eclipse/Schedule/ResCoup/Slaves.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/input/eclipse/Schedule/SummaryState.hpp>
+
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
@@ -64,6 +67,7 @@
 
 #include <opm/simulators/flow/KeywordValidation.hpp>
 #include <opm/simulators/flow/ValidationFunctions.hpp>
+#include <opm/simulators/utils/FullySupportedFlowKeywords.hpp>
 #include <opm/simulators/utils/ParallelEclipseState.hpp>
 #include <opm/simulators/utils/ParallelSerialization.hpp>
 #include <opm/simulators/utils/PartiallySupportedFlowKeywords.hpp>
@@ -74,12 +78,10 @@
 #include <cstdlib>
 #include <cstdint>
 #include <filesystem>
-#include <functional>
 #include <memory>
 #include <regex>
 #include <sstream>
 #include <stdexcept>
-#include <unordered_map>
 #include <utility>
 
 namespace {
@@ -105,6 +107,8 @@ namespace {
                                 const Opm::ParseContext&             parseContext,
                                 const bool                           initFromRestart,
                                 const std::optional<int>&            outputInterval,
+                                const bool                           lowActionParsingStrictness,
+                                const bool                           keepKeywords,
                                 Opm::EclipseState&                   eclipseState,
                                 std::shared_ptr<Opm::Python>         python,
                                 std::shared_ptr<Opm::Schedule>&      schedule,
@@ -152,7 +156,8 @@ namespace {
         if (schedule == nullptr) {
             schedule = std::make_shared<Opm::Schedule>
                 (deck, eclipseState, parseContext, errorGuard,
-                 std::move(python), outputInterval, init_state);
+                 std::move(python), lowActionParsingStrictness,  /*slaveMode=*/false,
+                 keepKeywords, outputInterval, init_state);
         }
 
         // Read network pressures from restart
@@ -173,19 +178,21 @@ namespace {
     void createNonRestartDynamicObjects(const Opm::Deck&                     deck,
                                         const Opm::EclipseState&             eclipseState,
                                         const Opm::ParseContext&             parseContext,
+                                        const bool                           lowActionParsingStrictness,
+                                        const bool                           keepKeywords,
                                         std::shared_ptr<Opm::Python>         python,
                                         std::shared_ptr<Opm::Schedule>&      schedule,
                                         std::unique_ptr<Opm::UDQState>&      udqState,
                                         std::unique_ptr<Opm::Action::State>& actionState,
                                         std::unique_ptr<Opm::WellTestState>& wtestState,
-                                        Opm::ErrorGuard&                     errorGuard)
+                                        Opm::ErrorGuard&                     errorGuard,
+                                        const bool                           slaveMode)
     {
         if (schedule == nullptr) {
             schedule = std::make_shared<Opm::Schedule>
                 (deck, eclipseState, parseContext,
-                 errorGuard, std::move(python));
+                 errorGuard, std::move(python), lowActionParsingStrictness, slaveMode, keepKeywords);
         }
-
         udqState = std::make_unique<Opm::UDQState>
             ((*schedule)[0].udq().params().undefinedValue());
 
@@ -203,11 +210,22 @@ namespace {
     {
         Opm::Deck deck(parser.parseFile(deckFilename, parseContext, errorGuard));
 
-        auto keyword_validator = Opm::KeywordValidation::KeywordValidator {
-            Opm::FlowKeywordValidation::unsupportedKeywords(),
+        Opm::KeywordValidation::SupportedKeywords partiallySupported  {
             Opm::FlowKeywordValidation::partiallySupported<std::string>(),
             Opm::FlowKeywordValidation::partiallySupported<int>(),
-            Opm::FlowKeywordValidation::partiallySupported<double>(),
+            Opm::FlowKeywordValidation::partiallySupported<double>()
+        };
+
+        Opm::KeywordValidation::SupportedKeywords fullySupported  {
+            Opm::FlowKeywordValidation::fullySupported<std::string>(),
+            Opm::FlowKeywordValidation::fullySupported<int>(),
+            Opm::FlowKeywordValidation::fullySupported<double>()
+        };
+
+        auto keyword_validator = Opm::KeywordValidation::KeywordValidator {
+            Opm::FlowKeywordValidation::unsupportedKeywords(),
+            partiallySupported,
+            fullySupported,
             Opm::KeywordValidation::specialValidation()
         };
 
@@ -231,6 +249,24 @@ namespace {
 #endif
     }
 
+    void inconsistentScheduleError(const std::string& message)
+    {
+        OPM_THROW(std::logic_error,
+                  fmt::format("Inconsistent SCHEDULE section: {}", message));
+    }
+
+    void checkScheduleKeywordConsistency(const Opm::Schedule& schedule)
+    {
+        const auto& final_state = schedule.back();
+        const auto& rescoup = final_state.rescoup();
+        if (rescoup.slaveCount() > 0 && rescoup.masterGroupCount() == 0) {
+            inconsistentScheduleError("SLAVES keyword without GRUPMAST keyword");
+        }
+        if (rescoup.slaveCount() == 0 && rescoup.masterGroupCount() > 0) {
+            inconsistentScheduleError("GRUPMAST keyword without SLAVES keyword");
+        }
+    }
+
     void readOnIORank(Opm::Parallel::Communication         comm,
                       const std::string&                   deckFilename,
                       const Opm::ParseContext*             parseContext,
@@ -244,8 +280,11 @@ namespace {
                       const bool                           initFromRestart,
                       const bool                           checkDeck,
                       const bool                           treatCriticalAsNonCritical,
+                      const bool                           lowActionParsingStrictness,
+                      const bool                           keepKeywords,
                       const std::optional<int>&            outputInterval,
-                      Opm::ErrorGuard&                     errorGuard)
+                      Opm::ErrorGuard&                     errorGuard,
+                      const bool                           slaveMode)
     {
         OPM_TIMEBLOCK(readDeck);
         if (((schedule == nullptr) || (summaryConfig == nullptr)) &&
@@ -268,17 +307,20 @@ namespace {
         if (eclipseState->getInitConfig().restartRequested()) {
             loadObjectsFromRestart(deck, parser, *parseContext,
                                    initFromRestart, outputInterval,
+                                   lowActionParsingStrictness, keepKeywords,
                                    *eclipseState, std::move(python),
                                    schedule, udqState, actionState, wtestState,
                                    errorGuard);
         }
         else {
-            createNonRestartDynamicObjects(deck, *eclipseState,
-                                           *parseContext, std::move(python),
+            createNonRestartDynamicObjects(deck, *eclipseState, *parseContext,
+                                           lowActionParsingStrictness, keepKeywords,
+                                           std::move(python),
                                            schedule, udqState, actionState, wtestState,
-                                           errorGuard);
+                                           errorGuard, slaveMode);
         }
 
+        checkScheduleKeywordConsistency(*schedule);
         eclipseState->appendAqufluxSchedule(schedule->getAquiferFluxSchedule());
 
         if (Opm::OpmLog::hasBackend("STDOUT_LOGGER")) {
@@ -405,10 +447,10 @@ void Opm::ensureOutputDirExists(const std::string& cmdline_output_dir)
 
 void Opm::prepareResultOutputDirectory(const std::string&           baseName,
                                        const std::filesystem::path& outputDir)  
-{   
+{
     //Loop over all files in the output directory and subdirectories and delete them if their name is baseName + a correct extension
     std::regex r(baseName + R"(\.(F?(DBG|E?GRID|INIT|PRT|RFT|SMSPEC|UNSMRY|UNRST)|([ABCFGHSTUXYZ]\d{4})|(INFOSTEP|INFOITER|OPMRST)))");
-    for (auto& file : std::filesystem::recursive_directory_iterator(outputDir)) {
+    for (auto& file : std::filesystem::directory_iterator(outputDir)) {
         std::string fileName = file.path().filename();
         if (std::regex_match(fileName, r)) {
             std::filesystem::remove(file);
@@ -474,7 +516,6 @@ Opm::setupLogging(Parallel::Communication& comm,
     }
     logFileStream << ".PRT";
     debugFileStream << ".DBG";
-
     FileOutputMode output;
     {
         static std::map<std::string, FileOutputMode> stringToOutputMode =
@@ -512,15 +553,7 @@ Opm::setupLogging(Parallel::Communication& comm,
     }
 
     if (comm.rank() == 0) {
-        std::shared_ptr<Opm::StreamLog> streamLog = std::make_shared<Opm::StreamLog>(std::cout, Opm::Log::StdoutMessageTypes);
-        Opm::OpmLog::addBackend(stdout_log_id, streamLog);
-        // Set a tag limit of 10 (no category limit). Will later in
-        // the run be replaced by calling setupMessageLimiter(), after
-        // the deck is read and the (possibly user-set) category
-        // limits are known.
-        streamLog->setMessageLimiter(std::make_shared<Opm::MessageLimiter>(10));
-        bool use_color_coding = OpmLog::stdoutIsTerminal();
-        streamLog->setMessageFormatter(std::make_shared<Opm::SimpleMessageFormatter>(use_color_coding));
+        setupStreamLogging(stdout_log_id);
     }
 
     return output;
@@ -536,12 +569,15 @@ void Opm::readDeck(Opm::Parallel::Communication    comm,
                    std::shared_ptr<SummaryConfig>& summaryConfig,
                    std::shared_ptr<Python>         python,
                    const std::string&              parsingStrictness,
+                   const std::string&              actionParsingStrictness,
+                   const std::string&              inputSkipMode,
                    const bool                      initFromRestart,
                    const bool                      checkDeck,
-                   const std::optional<int>&       outputInterval)
+                   const bool                      keepKeywords,
+                   const std::optional<int>&       outputInterval,
+                   const bool                      slaveMode)
 {
     auto errorGuard = std::make_unique<ErrorGuard>();
-
     int parseSuccess = 1; // > 0 is success
     std::string failureMessage;
 
@@ -549,19 +585,30 @@ void Opm::readDeck(Opm::Parallel::Communication    comm,
         OPM_THROW(std::runtime_error,
                   fmt::format("Incorrect value {} for parameter ParsingStrictness, must be 'high', 'normal', or 'low'", parsingStrictness));
     }
+    if (actionParsingStrictness != "normal" && actionParsingStrictness != "low") {
+        OPM_THROW(std::runtime_error,
+                  fmt::format("Incorrect value {} for parameter ActionParsingStrictness, must be 'normal', or 'low'", actionParsingStrictness));
+    }
+    if (inputSkipMode != "100" && inputSkipMode != "300" && inputSkipMode != "all") {
+        OPM_THROW(std::runtime_error,
+                  fmt::format("Incorrect value {} for parameter InputSkipMode, must be '100', '300', or 'all'", inputSkipMode));
+    }
 
     if (comm.rank() == 0) { // Always true when !HAVE_MPI
         const bool exitOnAllErrors = (parsingStrictness == "high");
         const bool treatCriticalAsNonCritical = (parsingStrictness == "low");
+        const bool lowActionParsingStrictness = (actionParsingStrictness == "low");
         try {
             auto parseContext = setupParseContext(exitOnAllErrors);
             if (treatCriticalAsNonCritical) { // Continue with invalid names if parsing strictness is set to low
                 parseContext->update(ParseContext::SCHEDULE_INVALID_NAME, InputErrorAction::WARN);
             }
+            parseContext->setInputSkipMode(inputSkipMode);
             readOnIORank(comm, deckFilename, parseContext.get(),
                          eclipseState, schedule, udqState, actionState, wtestState,
                          summaryConfig, std::move(python), initFromRestart,
-                         checkDeck, treatCriticalAsNonCritical, outputInterval, *errorGuard);
+                         checkDeck, treatCriticalAsNonCritical, lowActionParsingStrictness,
+                         keepKeywords, outputInterval, *errorGuard, slaveMode);
 
             // Update schedule so that re-parsing after actions use same strictness
             assert(schedule);
@@ -605,7 +652,7 @@ void Opm::readDeck(Opm::Parallel::Communication    comm,
 
     if (*errorGuard) { // errors encountered
         parseSuccess = 0;
-        errorGuard->dump();
+        failureMessage += errorGuard->formattedErrors();
         errorGuard->clear();
     }
 
@@ -613,7 +660,7 @@ void Opm::readDeck(Opm::Parallel::Communication    comm,
 
     if (! parseSuccess) {
         if (comm.rank() == 0) {
-            OpmLog::error(fmt::format("Unrecoverable errors while loading input: {}", failureMessage));
+            OpmLog::error(fmt::format("Unrecoverable errors while loading input:\n{}", failureMessage));
         }
 
 #if HAVE_MPI
@@ -648,3 +695,16 @@ std::unique_ptr<Opm::ParseContext> Opm::setupParseContext(const bool strictParsi
 
     return parseContext;
 }
+
+void Opm::setupStreamLogging(const std::string& stdout_log_id)
+{
+    std::shared_ptr<Opm::StreamLog> streamLog = std::make_shared<Opm::StreamLog>(std::cout, Opm::Log::StdoutMessageTypes);
+    Opm::OpmLog::addBackend(stdout_log_id, streamLog);
+    // Set a tag limit of 10 (no category limit). Will later in
+    // the run be replaced by calling setupMessageLimiter(), after
+    // the deck is read and the (possibly user-set) category
+    // limits are known.
+    streamLog->setMessageLimiter(std::make_shared<Opm::MessageLimiter>(10));
+    const bool use_color_coding = OpmLog::stdoutIsTerminal();
+    streamLog->setMessageFormatter(std::make_shared<Opm::SimpleMessageFormatter>(use_color_coding));
+}
