@@ -45,10 +45,13 @@
 
 #include <opm/grid/CpGrid.hpp>
 #include <opm/grid/polyhedralgrid.hh>
+#include <opm/grid/cpgrid/LevelCartesianIndexMapper.hpp>
+#include <opm/grid/polyhedralgrid/levelcartesianindexmapper.hh>
 
 #ifdef HAVE_DUNE_ALUGRID
 #include <dune/alugrid/grid.hh>
 #include <dune/alugrid/3d/gridview.hh>
+#include <opm/simulators/flow/AluGridLevelCartesianIndexMapper.hpp>
 #endif // HAVE_DUNE_ALUGRID
 
 namespace Opm {
@@ -74,32 +77,32 @@ namespace Opm {
         }
 
         if (!hasWater && !hasGas && hasOil && !hasSolvent) {
-            const std::string msg = "System:  Single phase Oil system. Nothing to check";
+            const std::string msg = "System: Single phase Oil system. Nothing to check";
             OpmLog::info(msg);
             return false;
         }
         if (hasWater && hasGas && !hasOil && !hasSolvent) {
-            const std::string msg = "System:  Water-Gas system.";
+            const std::string msg = "System: Water-Gas system.";
             OpmLog::info(msg);
             fluidSystem_ = FluidSystem::WaterGas;
         }
         if (hasWater && hasOil && !hasGas && !hasSolvent) {
-            const std::string msg = "System:  Oil-Water system.";
+            const std::string msg = "System: Oil-Water system.";
             OpmLog::info(msg);
             fluidSystem_ = FluidSystem::OilWater;
         }
         if (hasOil && hasGas && !hasWater && !hasSolvent) {
-            const std::string msg = "System:  Oil-Gas system.";
+            const std::string msg = "System: Oil-Gas system.";
             OpmLog::info(msg);
             fluidSystem_ = FluidSystem::OilGas;
         }
         if (hasOil && hasWater && hasGas && !hasSolvent) {
-            const std::string msg = "System:  Black-oil system.";
+            const std::string msg = "System: Black-oil system.";
             OpmLog::info(msg);
             fluidSystem_ = FluidSystem::BlackOil;
         }
         if (hasSolvent) {
-            const std::string msg = "System:  Solvent model.";
+            const std::string msg = "System: Solvent model.";
             OpmLog::info(msg);
             fluidSystem_ = FluidSystem::Solvent;
         }
@@ -822,9 +825,9 @@ namespace Opm {
         }
     }
 
-    template <class CartesianIndexMapper>
+    template <class LevelCartesianIndexMapper>
     void RelpermDiagnostics::diagnosis(const EclipseState& eclState,
-                                       const CartesianIndexMapper& cartesianIndexMapper)
+                                       const LevelCartesianIndexMapper& levelCartesianIndexMapper)
     {
         OpmLog::info("\n===============Saturation Functions Diagnostics===============\n");
         bool doDiagnostics = phaseCheck_(eclState);
@@ -833,16 +836,16 @@ namespace Opm {
         satFamilyCheck_(eclState);
         tableCheck_(eclState);
         unscaledEndPointsCheck_(eclState);
-        scaledEndPointsCheck_(eclState, cartesianIndexMapper);
+        scaledEndPointsCheck_(eclState, levelCartesianIndexMapper);
     }
 
-    template <class CartesianIndexMapper>
+    template <class LevelCartesianIndexMapper>
     void RelpermDiagnostics::scaledEndPointsCheck_(const EclipseState& eclState,
-                                                   const CartesianIndexMapper& cartesianIndexMapper)
+                                                   const LevelCartesianIndexMapper& levelCartesianIndexMapper)
     {
         // All end points are subject to round-off errors, checks should account for it
         const float tolerance = 1e-6;
-        const int nc = cartesianIndexMapper.compressedLevelZeroSize();
+        const int nc = levelCartesianIndexMapper.compressedSize(0);
         const bool threepoint = eclState.runspec().endpointScaling().threepoint();
         scaledEpsInfo_.resize(nc);
         EclEpsGridProperties epsGridProperties(eclState, false);
@@ -852,7 +855,7 @@ namespace Opm {
             std::string cellIdx;
             {
                 std::array<int, 3> ijk;
-                cartesianIndexMapper.cartesianCoordinateLevel(c, ijk, 0);
+                levelCartesianIndexMapper.cartesianCoordinate(c, ijk, 0);
                 cellIdx = "(" + std::to_string(ijk[0]) + ", " +
                     std::to_string(ijk[1]) + ", " +
                     std::to_string(ijk[2]) + ")";
@@ -887,8 +890,8 @@ namespace Opm {
     }
 
 #define INSTANCE_DIAGNOSIS(...) \
-    template void RelpermDiagnostics::diagnosis<Dune::CartesianIndexMapper<__VA_ARGS__>>(const EclipseState&, const Dune::CartesianIndexMapper<__VA_ARGS__>&); \
-    template void RelpermDiagnostics::scaledEndPointsCheck_<Dune::CartesianIndexMapper<__VA_ARGS__>>(const EclipseState&, const Dune::CartesianIndexMapper<__VA_ARGS__>&);
+    template void RelpermDiagnostics::diagnosis<Opm::LevelCartesianIndexMapper<__VA_ARGS__>>(const EclipseState&, const Opm::LevelCartesianIndexMapper<__VA_ARGS__>&); \
+    template void RelpermDiagnostics::scaledEndPointsCheck_<Opm::LevelCartesianIndexMapper<__VA_ARGS__>>(const EclipseState&, const Opm::LevelCartesianIndexMapper<__VA_ARGS__>&);
 
     INSTANCE_DIAGNOSIS(Dune::CpGrid)
     INSTANCE_DIAGNOSIS(Dune::PolyhedralGrid<3,3>)
