@@ -24,30 +24,30 @@
 #ifndef OPM_BLACKOILWELLMODEL_HEADER_INCLUDED
 #define OPM_BLACKOILWELLMODEL_HEADER_INCLUDED
 
+#include <dune/common/fmatrix.hh>
+#include <dune/istl/bcrsmatrix.hh>
+#include <dune/istl/matrixmatrix.hh>
+
 #include <opm/common/OpmLog/OpmLog.hpp>
 
-#include <cstddef>
-#include <map>
-#include <memory>
-#include <optional>
-#include <string>
-#include <vector>
-
-#include <opm/grid/utility/SparseTable.hpp>
-
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRate.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 
-#include <opm/models/discretization/common/baseauxiliarymodule.hh>
+#include <opm/material/densead/Math.hpp>
 
 #include <opm/simulators/flow/countGlobalCells.hpp>
 #include <opm/simulators/flow/FlowBaseVanguard.hpp>
-#include <opm/simulators/flow/SubDomain.hpp>
 
 #include <opm/simulators/linalg/matrixblock.hh>
 
+#include <opm/simulators/timestepping/SimulatorReport.hpp>
+#include <opm/simulators/timestepping/gatherConvergenceReport.hpp>
+
+#include <opm/simulators/utils/DeferredLogger.hpp>
+
+#include <opm/simulators/wells/BlackoilWellModelGasLift.hpp>
 #include <opm/simulators/wells/BlackoilWellModelGeneric.hpp>
 #include <opm/simulators/wells/BlackoilWellModelGuideRates.hpp>
 #include <opm/simulators/wells/GasLiftGroupInfo.hpp>
@@ -63,38 +63,32 @@
 #include <opm/simulators/wells/StandardWell.hpp>
 #include <opm/simulators/wells/VFPInjProperties.hpp>
 #include <opm/simulators/wells/VFPProdProperties.hpp>
-#include <opm/simulators/wells/WGState.hpp>
+#include <opm/simulators/wells/WellConnectionAuxiliaryModule.hpp>
 #include <opm/simulators/wells/WellGroupHelpers.hpp>
 #include <opm/simulators/wells/WellInterface.hpp>
 #include <opm/simulators/wells/WellProdIndexCalculator.hpp>
 #include <opm/simulators/wells/WellState.hpp>
+#include <opm/simulators/wells/WGState.hpp>
 
-#include <opm/simulators/timestepping/SimulatorReport.hpp>
-#include <opm/simulators/timestepping/gatherConvergenceReport.hpp>
-
-#include <dune/common/fmatrix.hh>
-#include <dune/istl/bcrsmatrix.hh>
-#include <dune/istl/matrixmatrix.hh>
-
-#include <opm/material/densead/Math.hpp>
-
-#include <opm/simulators/utils/DeferredLogger.hpp>
-
-namespace Opm::Parameters {
-
-struct EnableTerminalOutput { static constexpr bool value = true; };
-
-} // namespace Opm::Parameters
+#include <cstddef>
+#include <map>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <vector>
 
 namespace Opm {
 
-#if COMPILE_BDA_BRIDGE
+template<class Scalar> class BlackoilWellModelNldd;
+template<class T> class SparseTable;
+
+#if COMPILE_GPU_BRIDGE
 template<class Scalar> class WellContributions;
 #endif
 
         /// Class for handling the blackoil well model.
         template<typename TypeTag>
-        class BlackoilWellModel : public BaseAuxiliaryModule<TypeTag>
+        class BlackoilWellModel : public WellConnectionAuxiliaryModule<TypeTag, BlackoilWellModel<TypeTag>>
                                 , public BlackoilWellModelGeneric<GetPropType<TypeTag,
                                                                               Properties::Scalar>>
         {
@@ -110,18 +104,11 @@ template<class Scalar> class WellContributions;
             using RateVector = GetPropType<TypeTag, Properties::RateVector>;
             using GlobalEqVector = GetPropType<TypeTag, Properties::GlobalEqVector>;
             using SparseMatrixAdapter = GetPropType<TypeTag, Properties::SparseMatrixAdapter>;
-            using GasLiftSingleWell = typename WellInterface<TypeTag>::GasLiftSingleWell;
-            using GLiftOptWells = typename BlackoilWellModelGeneric<Scalar>::GLiftOptWells;
-            using GLiftProdWells = typename BlackoilWellModelGeneric<Scalar>::GLiftProdWells;
-            using GLiftWellStateMap =
-                typename BlackoilWellModelGeneric<Scalar>::GLiftWellStateMap;
-            using GLiftEclWells = typename GasLiftGroupInfo<Scalar>::GLiftEclWells;
-            using GLiftSyncGroups = typename GasLiftSingleWellGeneric<Scalar>::GLiftSyncGroups;
-
             using ModelParameters = BlackoilModelParameters<Scalar>;
 
+            using WellConnectionModule = WellConnectionAuxiliaryModule<TypeTag, BlackoilWellModel<TypeTag>>;
+
             constexpr static std::size_t pressureVarIndex = GetPropType<TypeTag, Properties::Indices>::pressureSwitchIdx;
-            typedef typename BaseAuxiliaryModule<TypeTag>::NeighborSet NeighborSet;
 
             static const int numEq = Indices::numEq;
             static const int solventSaturationIdx = Indices::solventSaturationIdx;
@@ -132,13 +119,13 @@ template<class Scalar> class WellContributions;
 
             // TODO: where we should put these types, WellInterface or Well Model?
             // or there is some other strategy, like TypeTag
-            typedef Dune::FieldVector<Scalar, numEq    > VectorBlockType;
-            typedef Dune::BlockVector<VectorBlockType> BVector;
+            using VectorBlockType = Dune::FieldVector<Scalar, numEq>;
+            using BVector = Dune::BlockVector<VectorBlockType>;
 
-            typedef BlackOilPolymerModule<TypeTag> PolymerModule;
-            typedef BlackOilMICPModule<TypeTag> MICPModule;
+            using PolymerModule = BlackOilPolymerModule<TypeTag>;
+            using MICPModule = BlackOilMICPModule<TypeTag>;
 
-            // For the conversion between the surface volume rate and resrevoir voidage rate
+            // For the conversion between the surface volume rate and reservoir voidage rate
             using RateConverterType = RateConverter::
                 SurfaceToReservoirVoidage<FluidSystem, std::vector<int> >;
 
@@ -146,58 +133,11 @@ template<class Scalar> class WellContributions;
             using AverageRegionalPressureType = RegionAverageCalculator::
                 AverageRegionalPressure<FluidSystem, std::vector<int> >;
 
-            using Domain = SubDomain<Grid>;
-
-            BlackoilWellModel(Simulator& simulator);
+            explicit BlackoilWellModel(Simulator& simulator);
 
             void init();
             void initWellContainer(const int reportStepIdx) override;
 
-            /////////////
-            // <eWoms auxiliary module stuff>
-            /////////////
-            unsigned numDofs() const override
-            // No extra dofs are inserted for wells. (we use a Schur complement.)
-            { return 0; }
-
-            void addNeighbors(std::vector<NeighborSet>& neighbors) const override;
-
-            void applyInitial() override
-            {}
-
-            void linearize(SparseMatrixAdapter& jacobian, GlobalEqVector& res) override;
-            void linearizeDomain(const Domain& domain, SparseMatrixAdapter& jacobian, GlobalEqVector& res);
-
-            void postSolve(GlobalEqVector& deltaX) override
-            {
-                recoverWellSolutionAndUpdateWellState(deltaX);
-            }
-
-            void postSolveDomain(GlobalEqVector& deltaX, const Domain& domain)
-            {
-                recoverWellSolutionAndUpdateWellStateDomain(deltaX, domain);
-            }
-
-            /////////////
-            // </ eWoms auxiliary module stuff>
-            /////////////
-
-            template <class Restarter>
-            void deserialize(Restarter& /* res */)
-            {
-                // TODO (?)
-            }
-
-            /*!
-             * \brief This method writes the complete state of the well
-             *        to the harddisk.
-             */
-            template <class Restarter>
-            void serialize(Restarter& /* res*/)
-            {
-                // TODO (?)
-            }
-
             void beginEpisode()
             {
                 OPM_TIMEBLOCK(beginEpsiode);
@@ -274,10 +214,10 @@ template<class Scalar> class WellContributions;
                     const auto& mswTracerRates = simulator_.problem()
                         .tracerModel().getMswTracerRates();
 
-                    this->assignWellTracerRates(wsrpt, tracerRates);
-                    this->assignWellTracerRates(wsrpt, freeTracerRates);
-                    this->assignWellTracerRates(wsrpt, solTracerRates);
-                    this->assignMswTracerRates(wsrpt, mswTracerRates);
+                    this->assignWellTracerRates(wsrpt, tracerRates, this->reportStepIndex());
+                    this->assignWellTracerRates(wsrpt, freeTracerRates, this->reportStepIndex());
+                    this->assignWellTracerRates(wsrpt, solTracerRates, this->reportStepIndex());
+                    this->assignMswTracerRates(wsrpt, mswTracerRates, this->reportStepIndex());
                 }
 
                 BlackoilWellModelGuideRates(*this)
@@ -285,10 +225,10 @@ template<class Scalar> class WellContributions;
 
                 this->assignWellTargets(wsrpt);
                 this->assignShutConnections(wsrpt, this->reportStepIndex());
-                // only used to compute gas injection mass rates for CO2STORE runs
+                // only used to compute gas injection mass rates for CO2STORE/H2STORE runs
                 // The gas reference density is thus the same for all pvt regions
                 // We therefore for simplicity use 0 here 
-                if (eclState().runspec().co2Storage()) {
+                if (eclState().runspec().co2Storage() || eclState().runspec().h2Storage()) {
                     this->assignMassGasRate(wsrpt, FluidSystem::referenceDensity(FluidSystem::gasPhaseIdx, 0));
                 }
                 return wsrpt;
@@ -296,45 +236,30 @@ template<class Scalar> class WellContributions;
 
             data::WellBlockAveragePressures wellBlockAveragePressures() const
             {
-                return this->computeWellBlockAveragePressures();
+                return this->wbp_.computeWellBlockAveragePressures(this->gravity_);
             }
 
-            // subtract B*inv(D)*C * x from A*x
-            void apply(const BVector& x, BVector& Ax) const;
-
-            void applyDomain(const BVector& x, BVector& Ax, const int domainIndex) const;
-
-#if COMPILE_BDA_BRIDGE
+#if COMPILE_GPU_BRIDGE
             // accumulate the contributions of all Wells in the WellContributions object
             void getWellContributions(WellContributions<Scalar>& x) const;
 #endif
 
-            // apply well model with scaling of alpha
-            void applyScaleAdd(const Scalar alpha, const BVector& x, BVector& Ax) const;
-
-            void applyScaleAddDomain(const Scalar alpha, const BVector& x, BVector& Ax, const int domainIndex) const;
-
             // Check if well equations is converged.
             ConvergenceReport getWellConvergence(const std::vector<Scalar>& B_avg, const bool checkWellGroupControls = false) const;
 
-            // Check if well equations are converged locally.
-            ConvergenceReport getDomainWellConvergence(const Domain& domain,
-                                                       const std::vector<Scalar>& B_avg,
-                                                       DeferredLogger& local_deferredLogger) const;
-
             const SimulatorReportSingle& lastReport() const;
 
             void addWellContributions(SparseMatrixAdapter& jacobian) const;
 
             // add source from wells to the reservoir matrix
             void addReservoirSourceTerms(GlobalEqVector& residual,
-                                         std::vector<typename SparseMatrixAdapter::MatrixBlock*>& diagMatAddress) const;
+                                         const std::vector<typename SparseMatrixAdapter::MatrixBlock*>& diagMatAddress) const;
 
             // called at the beginning of a report step
             void beginReportStep(const int time_step);
 
-            // it should be able to go to prepareTimeStep(), however, the updateWellControls() and initPrimaryVariablesEvaluation()
-            // makes it a little more difficult. unless we introduce if (iterationIdx != 0) to avoid doing the above functions
+            // it should be able to go to prepareTimeStep(), however, the updateWellControls()
+            // makes it a little more difficult. unless we introduce if (iterationIdx != 0) to avoid doing the above function
             // twice at the beginning of the time step
             /// Calculating the explict quantities used in the well calculation. By explicit, we mean they are cacluated
             /// at the beginning of the time step and no derivatives are included in these quantities
@@ -342,10 +267,8 @@ template<class Scalar> class WellContributions;
             // some preparation work, mostly related to group control and RESV,
             // at the beginning of each time step (Not report step)
             void prepareTimeStep(DeferredLogger& deferred_logger);
-            void initPrimaryVariablesEvaluation() const;
-            void initPrimaryVariablesEvaluationDomain(const Domain& domain) const;
 
-            std::pair<bool, bool>
+            std::tuple<bool, bool, Scalar>
             updateWellControls(const bool mandatory_network_balance, DeferredLogger& deferred_logger, const bool relax_network_tolerance = false);
 
             void updateAndCommunicate(const int reportStepIdx,
@@ -358,21 +281,26 @@ template<class Scalar> class WellContributions;
                                     const int iterationIdx);
 
             WellInterfacePtr getWell(const std::string& well_name) const;
-            bool hasWell(const std::string& well_name) const;
 
             using PressureMatrix = Dune::BCRSMatrix<Opm::MatrixBlock<Scalar, 1, 1>>;
 
-            void addWellPressureEquations(PressureMatrix& jacobian, const BVector& weights,const bool use_well_weights) const;
-
-            void addWellPressureEquationsDomain([[maybe_unused]] PressureMatrix& jacobian,
-                                                [[maybe_unused]] const BVector& weights,
-                                                [[maybe_unused]] const bool use_well_weights,
-                                                [[maybe_unused]] const int domainIndex) const;
-
-
+            void addWellPressureEquations(PressureMatrix& jacobian,
+                                          const BVector& weights,
+                                          const bool use_well_weights) const;
             void addWellPressureEquationsStruct(PressureMatrix& jacobian) const;
-
-            void initGliftEclWellMap(GLiftEclWells &ecl_well_map);
+            void addWellPressureEquationsDomain(PressureMatrix& jacobian,
+                                                const BVector& weights,
+                                                const bool use_well_weights,
+                                                const int domainIndex) const
+            {
+                if (!nldd_) {
+                    OPM_THROW(std::logic_error, "Attempt to access NLDD data without a NLDD solver");
+                }
+                return nldd_->addWellPressureEquations(jacobian,
+                                                       weights,
+                                                       use_well_weights,
+                                                       domainIndex);
+            }
 
             /// \brief Get list of local nonshut wells
             const std::vector<WellInterfacePtr>& localNonshutWells() const
@@ -380,18 +308,55 @@ template<class Scalar> class WellContributions;
                 return well_container_;
             }
 
-            // prototype for assemble function for ASPIN solveLocal()
-            // will try to merge back to assemble() when done prototyping
-            void assembleDomain(const int iterationIdx,
-                                const double dt,
-                                const Domain& domain);
-            void updateWellControlsDomain(DeferredLogger& deferred_logger, const Domain& domain);
+            const SparseTable<int>& well_local_cells() const
+            {
+                if (!nldd_) {
+                    OPM_THROW(std::logic_error, "Attempt to access NLDD data without a NLDD solver");
+                }
+                return nldd_->well_local_cells();
+            }
 
-            void logPrimaryVars() const;
-            std::vector<Scalar> getPrimaryVarsDomain(const Domain& domain) const;
-            void setPrimaryVarsDomain(const Domain& domain, const std::vector<Scalar>& vars);
+            const std::map<std::string, int>& well_domain() const
+            {
+                if (!nldd_) {
+                    OPM_THROW(std::logic_error, "Attempt to access NLDD data without a NLDD solver");
+                }
 
-            void setupDomains(const std::vector<Domain>& domains);
+                return nldd_->well_domain();
+            }
+
+            auto begin() const { return well_container_.begin(); }
+            auto end() const { return well_container_.end(); }
+            bool empty() const { return well_container_.empty(); }
+
+            bool addMatrixContributions() const
+            { return param_.matrix_add_well_contributions_; }
+
+            int numStrictIterations() const
+            { return param_.strict_outer_iter_wells_; }
+
+            int compressedIndexForInterior(int cartesian_cell_idx) const override
+            {
+                return simulator_.vanguard().compressedIndexForInterior(cartesian_cell_idx);
+            }
+
+            // using the solution x to recover the solution xw for wells and applying
+            // xw to update Well State
+            void recoverWellSolutionAndUpdateWellState(const BVector& x);
+
+            // using the solution x to recover the solution xw for wells and applying
+            // xw to update Well State
+            void recoverWellSolutionAndUpdateWellStateDomain(const BVector& x,
+                                                             const int domainIdx);
+
+            const Grid& grid() const
+            { return simulator_.vanguard().grid(); }
+
+            const Simulator& simulator() const
+            { return simulator_; }
+
+            void setNlddAdapter(BlackoilWellModelNldd<TypeTag>* mod)
+            { nldd_ = mod; }
 
         protected:
             Simulator& simulator_;
@@ -415,8 +380,9 @@ template<class Scalar> class WellContributions;
             createTypedWellPointer(const int wellID,
                                    const int time_step) const;
 
-            WellInterfacePtr createWellForWellTest(const std::string& well_name, const int report_step, DeferredLogger& deferred_logger) const;
-
+            WellInterfacePtr createWellForWellTest(const std::string& well_name,
+                                                   const int report_step,
+                                                   DeferredLogger& deferred_logger) const;
 
             const ModelParameters param_;
             std::size_t global_num_cells_{};
@@ -429,33 +395,13 @@ template<class Scalar> class WellContributions;
             std::unique_ptr<RateConverterType> rateConverter_{};
             std::map<std::string, std::unique_ptr<AverageRegionalPressureType>> regionalAveragePressureCalculator_{};
 
-            struct WBPCalcID
-            {
-                std::optional<typename std::vector<WellInterfacePtr>::size_type> openWellIdx_{};
-                std::size_t wbpCalcIdx_{};
-            };
-
-            std::vector<WBPCalcID> wbpCalcMap_{};
-
             SimulatorReportSingle last_report_{};
 
             // Pre-step network solve at static reservoir conditions (group and well states might be updated)
             void doPreStepNetworkRebalance(DeferredLogger& deferred_logger);
 
-            // used to better efficiency of calcuation
-            mutable BVector scaleAddRes_{};
-
             std::vector<Scalar> B_avg_{};
 
-            // Keep track of the domain of each well, if using subdomains.
-            std::map<std::string, int> well_domain_;
-
-            // Store the local index of the wells perforated cells in the domain, if using sumdomains
-            SparseTable<int> well_local_cells_;
-
-            const Grid& grid() const
-            { return simulator_.vanguard().grid(); }
-
             const EquilGrid& equilGrid() const
             { return simulator_.vanguard().equilGrid(); }
 
@@ -471,8 +417,9 @@ template<class Scalar> class WellContributions;
             // the function handles one iteration of updating well controls and network pressures.
             // it is possible to decouple the update of well controls and network pressures further.
             // the returned two booleans are {continue_due_to_network, well_group_control_changed}, respectively
-            std::pair<bool, bool> updateWellControlsAndNetworkIteration(const bool mandatory_network_balance,
+            std::tuple<bool, bool, Scalar> updateWellControlsAndNetworkIteration(const bool mandatory_network_balance,
                                                                         const bool relax_network_tolerance,
+                                                                        const bool optimize_gas_lift,
                                                                         const double dt,
                                                                         DeferredLogger& local_deferredLogger);
 
@@ -480,7 +427,7 @@ template<class Scalar> class WellContributions;
                                               const double dt,
                                               DeferredLogger& local_deferredLogger);
 
-            void computeWellGroupThp(const double dt, DeferredLogger& local_deferredLogger);
+            bool computeWellGroupThp(const double dt, DeferredLogger& local_deferredLogger);
 
             /// Update rank's notion of intersecting wells and their
             /// associate solution variables.
@@ -504,27 +451,9 @@ template<class Scalar> class WellContributions;
             // called at the end of a report step
             void endReportStep();
 
-            // using the solution x to recover the solution xw for wells and applying
-            // xw to update Well State
-            void recoverWellSolutionAndUpdateWellState(const BVector& x);
-
-            // using the solution x to recover the solution xw for wells and applying
-            // xw to update Well State
-            void recoverWellSolutionAndUpdateWellStateDomain(const BVector& x, const Domain& domain);
-
             // setting the well_solutions_ based on well_state.
             void updatePrimaryVariables(DeferredLogger& deferred_logger);
 
-            void initializeWBPCalculationService();
-
-            data::WellBlockAveragePressures
-            computeWellBlockAveragePressures() const;
-
-            typename ParallelWBPCalculation<Scalar>::EvaluatorFactory
-            makeWellSourceEvaluatorFactory(const std::vector<Well>::size_type wellIdx) const;
-
-            void registerOpenWellsForWBPCalculation();
-
             void updateAverageFormationFactor();
 
             void computePotentials(const std::size_t widx,
@@ -546,30 +475,12 @@ template<class Scalar> class WellContributions;
             int reportStepIndex() const;
 
             void assembleWellEq(const double dt, DeferredLogger& deferred_logger);
-            void assembleWellEqDomain(const double dt, const Domain& domain, DeferredLogger& deferred_logger);
 
             void prepareWellsBeforeAssembling(const double dt, DeferredLogger& deferred_logger);
 
             // TODO: finding a better naming
             void assembleWellEqWithoutIteration(const double dt, DeferredLogger& deferred_logger);
 
-            bool maybeDoGasLiftOptimize(DeferredLogger& deferred_logger);
-
-            void gasLiftOptimizationStage1(DeferredLogger& deferred_logger,
-                                           GLiftProdWells& prod_wells,
-                                           GLiftOptWells& glift_wells,
-                                           GasLiftGroupInfo<Scalar>& group_info,
-                                           GLiftWellStateMap& state_map);
-
-            // cannot be const since it accesses the non-const WellState
-            void gasLiftOptimizationStage1SingleWell(WellInterface<TypeTag>* well,
-                                                     DeferredLogger& deferred_logger,
-                                                     GLiftProdWells& prod_wells,
-                                                     GLiftOptWells& glift_wells,
-                                                     GasLiftGroupInfo<Scalar>& group_info,
-                                                     GLiftWellStateMap& state_map,
-                                                     GLiftSyncGroups& groups_to_sync);
-
             void extractLegacyCellPvtRegionIndex_();
 
             void extractLegacyDepth_();
@@ -590,28 +501,22 @@ template<class Scalar> class WellContributions;
 
             void computeWellTemperature();
 
-            int compressedIndexForInterior(int cartesian_cell_idx) const override {
-                return simulator_.vanguard().compressedIndexForInterior(cartesian_cell_idx);
-            }
-
         private:
             BlackoilWellModel(Simulator& simulator, const PhaseUsage& pu);
 
+            BlackoilWellModelGasLift<TypeTag> gaslift_;
+            BlackoilWellModelNldd<TypeTag>* nldd_ = nullptr; //!< NLDD well model adapter (not owned)
+
             // These members are used to avoid reallocation in specific functions
-            // (e.g., apply, applyDomain) instead of using local variables.
+            // instead of using local variables.
             // Their state is not relevant between function calls, so they can
             // (and must) be mutable, as the functions using them are const.
             mutable BVector x_local_;
-            mutable BVector Ax_local_;
-            mutable BVector res_local_;
-            mutable GlobalEqVector linearize_res_local_;
         };
 
 
 } // namespace Opm
 
-#ifndef OPM_BLACKOILWELLMODEL_IMPL_HEADER_INCLUDED
 #include "BlackoilWellModel_impl.hpp"
-#endif
 
 #endif
