@@ -37,19 +37,6 @@
 
 namespace Opm {
 
-template<class Scalar>
-bool BlackoilWellModelConstraints<Scalar>::
-hasTHPConstraints() const
-{
-    int local_result = false;
-    for (const auto& well : wellModel_.genericWells()) {
-        if (well->wellHasTHPConstraints(wellModel_.summaryState())) {
-            local_result = true;
-        }
-    }
-    return wellModel_.comm().max(local_result);
-}
-
 template<class Scalar>
 std::pair<Group::InjectionCMode, Scalar>
 BlackoilWellModelConstraints<Scalar>::
@@ -476,10 +463,25 @@ actionOnBrokenConstraints(const Group& group,
                           DeferredLogger& deferred_logger) const
 {
 
-    bool changed = false;
     const Group::ProductionCMode oldControl =
         wellModel_.groupState().production_control(group.name());
 
+    // We switch to higher groups independently of the given group limit action in GCONPROD item 7
+    if (newControl == Group::ProductionCMode::FLD && oldControl != Group::ProductionCMode::FLD) {
+        // If newControl is FLD, the group should be subject to higher order controls
+        assert(group.productionGroupControlAvailable());
+        group_state.production_control(group.name(), newControl);
+        if (wellModel_.comm().rank() == 0) {
+            const std::string message = fmt::format("Switching production control mode for group {} from {} to {}",
+            group.name(),
+            Group::ProductionCMode2String(oldControl),
+            Group::ProductionCMode2String(newControl));
+            deferred_logger.debug(message);
+        }
+        return true;
+    }
+
+    bool changed = false;
     std::string ss;
     switch (group_limit_action.allRates) {
     case Group::ExceedAction::NONE: {
@@ -561,8 +563,9 @@ template<class Scalar>
 bool BlackoilWellModelConstraints<Scalar>::
 updateGroupIndividualControl(const Group& group,
                              const int reportStepIdx,
-                             std::map<std::pair<std::string,Phase>,std::string>& switched_inj,
-                             std::map<std::string, std::string>& switched_prod,
+                             const int max_number_of_group_switch,
+                             std::map<std::string, std::array<std::vector<Group::InjectionCMode>, 3>>& switched_inj,
+                             std::map<std::string, std::vector<Group::ProductionCMode>>& switched_prod,
                              std::map<std::string, std::pair<std::string, std::string>>& closed_offending_wells,
                              GroupState<Scalar>& group_state,
                              WellState<Scalar>& well_state,
@@ -576,13 +579,44 @@ updateGroupIndividualControl(const Group& group,
             if (!group.hasInjectionControl(phase)) {
                 continue;
             }
+            bool group_is_oscillating = false;
+            if (auto groupPos = switched_inj.find(group.name()); groupPos != switched_inj.end()) {
+                auto& ctrls = groupPos->second[static_cast<std::underlying_type_t<Phase>>(phase)];
+                for (const auto& ctrl : ctrls) {
+                    if (std::count(ctrls.begin(), ctrls.end(), ctrl) <= max_number_of_group_switch) {
+                        continue;
+                    }
+
+                    if (ctrls.back() != *(ctrls.end() - 2)) {
+                        if (wellModel_.comm().rank() == 0 ) {
+                            std::ostringstream os;
+                            os << phase;
+                            const std::string msg =
+                                fmt::format("Group control for {} injector group {} is oscillating. Group control kept at {}.",
+                                            std::move(os).str(),
+                                            group.name(),
+                                            Group::InjectionCMode2String(ctrl));
+                            deferred_logger.info(msg);
+                        }
+                        ctrls.push_back(ctrl);
+                    }
+                    group_is_oscillating = true;
+                    break;
+                }
+            }
+
+            if (group_is_oscillating) {
+                continue;
+            }
+
             const auto& changed_this = this->checkGroupInjectionConstraints(group,
                                                                             reportStepIdx,
                                                                             phase);
             if (changed_this.first != Group::InjectionCMode::NONE)
             {
-                switched_inj.insert_or_assign({group.name(), phase},
-                                     Group::InjectionCMode2String(changed_this.first));
+                switched_inj[group.name()][static_cast<std::underlying_type_t<Phase>>(phase)].push_back(
+                             changed_this.first);
+
                 this->actionOnBrokenConstraints(group, changed_this.first, phase,
                                                 group_state, deferred_logger);
                 WellGroupHelpers<Scalar>::updateWellRatesFromGroupTargetScale(changed_this.second,
@@ -597,6 +631,28 @@ updateGroupIndividualControl(const Group& group,
         }
     }
     if (group.isProductionGroup()) {
+
+        if (auto groupPos = switched_prod.find(group.name()); groupPos != switched_prod.end()) {
+            auto& ctrls = groupPos->second;
+            for (const auto& ctrl : ctrls) {
+                if (std::count(ctrls.begin(), ctrls.end(), ctrl) <= max_number_of_group_switch) {
+                    continue;
+                }
+
+                if (ctrls.back() != *(ctrls.end() - 2)) {
+                    if (wellModel_.comm().rank() == 0) {
+                        const std::string msg =
+                        fmt::format("Group control for production group {} is oscillating. Group control kept at {}.",
+                                    group.name(),
+                                    Group::ProductionCMode2String(ctrl));
+                        deferred_logger.info(msg);
+                    }
+                    ctrls.push_back(ctrl);
+                }
+                return false;
+            }
+        }
+
         const auto& changed_this = this->checkGroupProductionConstraints(group,
                                                                          reportStepIdx,
                                                                          deferred_logger);
@@ -612,8 +668,7 @@ updateGroupIndividualControl(const Group& group,
                                             group_state, deferred_logger);
 
             if(changed) {
-                switched_prod.insert_or_assign(group.name(),
-                                    Group::ProductionCMode2String(changed_this.first));
+                switched_prod[group.name()].push_back(changed_this.first);
                 WellGroupHelpers<Scalar>::updateWellRatesFromGroupTargetScale(changed_this.second,
                                                                               group,
                                                                               wellModel_.schedule(),
