@@ -32,12 +32,14 @@
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
-
+#include <opm/simulators/wells/BlackoilWellModelWBP.hpp>
+#include <opm/simulators/wells/ConnectionIndexMap.hpp>
 #include <opm/simulators/wells/ParallelPAvgDynamicSourceData.hpp>
 #include <opm/simulators/wells/ParallelWBPCalculation.hpp>
 #include <opm/simulators/wells/PerforationData.hpp>
 #include <opm/simulators/wells/WellFilterCake.hpp>
 #include <opm/simulators/wells/WellProdIndexCalculator.hpp>
+#include <opm/simulators/wells/WellTracerRate.hpp>
 #include <opm/simulators/wells/WGState.hpp>
 
 #include <cstddef>
@@ -53,6 +55,7 @@
 namespace Opm {
     class DeferredLogger;
     class EclipseState;
+    template<class Scalar> class BlackoilWellModelGasLiftGeneric;
     template<class Scalar> class GasLiftGroupInfo;
     template<class Scalar> class GasLiftSingleWellGeneric;
     template<class Scalar> class GasLiftWellState;
@@ -75,6 +78,12 @@ namespace Opm { namespace data {
     struct NodeData;
 }} // namespace Opm::data
 
+namespace Opm::Parameters {
+
+struct EnableTerminalOutput { static constexpr bool value = true; };
+
+} // namespace Opm::Parameters
+
 namespace Opm {
 
 /// Class for handling the blackoil well model.
@@ -82,12 +91,8 @@ template<class Scalar>
 class BlackoilWellModelGeneric
 {
 public:
-    // ---------      Types      ---------
-    using GLiftOptWells = std::map<std::string, std::unique_ptr<GasLiftSingleWellGeneric<Scalar>>>;
-    using GLiftProdWells = std::map<std::string, const WellInterfaceGeneric<Scalar>*>;
-    using GLiftWellStateMap = std::map<std::string, std::unique_ptr<GasLiftWellState<Scalar>>>;
-
     BlackoilWellModelGeneric(Schedule& schedule,
+                             BlackoilWellModelGasLiftGeneric<Scalar>& gaslift,
                              const SummaryState& summaryState,
                              const EclipseState& eclState,
                              const PhaseUsage& phase_usage,
@@ -102,7 +107,12 @@ public:
 
     /// return true if wells are available in the reservoir
     bool wellsActive() const;
-    bool hasWell(const std::string& wname) const;
+
+    //! \brief Returns true if well is defined and has connections on current rank.
+    bool hasLocalWell(const std::string& wname) const;
+
+    //! \brief Returns true if well is defined, open and has connections on current rank.
+    bool hasOpenLocalWell(const std::string& well_name) const;
 
     /// return true if network is active (at least one network well in prediction mode)
     bool networkActive() const;
@@ -110,6 +120,12 @@ public:
     // whether there exists any multisegment well open on this process
     bool anyMSWellOpenLocal() const;
 
+    const std::vector<Well>& eclWells() const
+    { return wells_ecl_; }
+
+    bool terminalOutput() const
+    { return terminal_output_; }
+
     const Well& getWellEcl(const std::string& well_name) const;
     std::vector<Well> getLocalWells(const int timeStepIdx) const;
     const Schedule& schedule() const { return schedule_; }
@@ -118,6 +134,9 @@ public:
     std::vector<const WellInterfaceGeneric<Scalar>*> genericWells() const
     { return {well_container_generic_.begin(), well_container_generic_.end()}; }
 
+    std::vector<WellInterfaceGeneric<Scalar>*> genericWells()
+    { return well_container_generic_; }
+
     /*
       Immutable version of the currently active wellstate.
     */
@@ -135,8 +154,8 @@ public:
     }
 
     /*
-      Will return the currently active nupcolWellState; must initialize
-      the internal nupcol wellstate with initNupcolWellState() first.
+      Will return the currently active nupcolWellState; must update
+      the internal nupcol wellstate with updateNupcolWGState() first.
     */
     const WellState<Scalar>& nupcolWellState() const
     {
@@ -177,9 +196,6 @@ public:
 
     data::GroupAndNetworkValues groupAndNetworkData(const int reportStepIdx) const;
 
-    /// Return true if any well has a THP constraint.
-    bool hasTHPConstraints() const;
-
     /// Checks if network is active (at least one network well on prediction).
     void updateNetworkActiveState(const int report_step);
 
@@ -191,7 +207,8 @@ public:
     /// Shut down any single well
     /// Returns true if the well was actually found and shut.
     bool forceShutWellByName(const std::string& wellname,
-                             const double simulation_time);
+                             const double simulation_time,
+                             const bool dont_shut_grup_wells);
 
     const std::vector<PerforationData<Scalar>>& perfData(const int well_idx) const
     { return well_perf_data_[well_idx]; }
@@ -204,6 +221,12 @@ public:
 
     const GuideRate& guideRate() const { return guideRate_; }
 
+    const std::map<std::string, double>& wellOpenTimes() const { return well_open_times_; }
+    const std::map<std::string, double>& wellCloseTimes() const { return well_close_times_; }
+    const WellGroupEvents& reportStepStartEvents() const { return report_step_start_events_; }
+
+    std::vector<int> getCellsForConnections(const Well& well) const;
+
     bool reportStepStarts() const { return report_step_starts_; }
 
     bool shouldBalanceNetwork(const int reportStepIndex,
@@ -215,6 +238,8 @@ public:
     }
     bool wasDynamicallyShutThisTimeStep(const std::string& well_name) const;
 
+    void logPrimaryVars() const;
+
     template<class Serializer>
     void serializeOp(Serializer& serializer)
     {
@@ -229,29 +254,20 @@ public:
         serializer(active_wgstate_);
         serializer(last_valid_wgstate_);
         serializer(nupcol_wgstate_);
-        serializer(last_glift_opt_time_);
         serializer(switched_prod_groups_);
         serializer(switched_inj_groups_);
         serializer(closed_offending_wells_);
+        serializer(gen_gaslift_);
     }
 
-    bool operator==(const BlackoilWellModelGeneric& rhs) const
-    {
-        return this->initial_step_ == rhs.initial_step_ &&
-               this->report_step_starts_ == rhs.report_step_starts_ &&
-               this->last_run_wellpi_ == rhs.last_run_wellpi_ &&
-               this->local_shut_wells_ == rhs.local_shut_wells_ &&
-               this->closed_this_step_ == rhs.closed_this_step_ &&
-               this->node_pressures_ == rhs.node_pressures_ &&
-               this->prev_inj_multipliers_ == rhs.prev_inj_multipliers_ &&
-               this->active_wgstate_ == rhs.active_wgstate_ &&
-               this->last_valid_wgstate_ == rhs.last_valid_wgstate_ &&
-               this->nupcol_wgstate_ == rhs.nupcol_wgstate_ &&
-               this->last_glift_opt_time_ == rhs.last_glift_opt_time_ &&
-               this->switched_prod_groups_ == rhs.switched_prod_groups_ &&
-               this->switched_inj_groups_ == rhs.switched_inj_groups_ &&
-               this->closed_offending_wells_ == rhs.closed_offending_wells_;            
-    }
+    bool operator==(const BlackoilWellModelGeneric& rhs) const;
+
+    const ParallelWellInfo<Scalar>&
+    parallelWellInfo(const std::size_t idx) const
+    { return local_parallel_well_info_[idx].get(); }
+
+    const ConnectionIndexMap& connectionIndexMap(const std::size_t idx)
+    { return conn_idx_map_[idx]; }
 
 protected:
     /*
@@ -266,19 +282,19 @@ protected:
           try again with a smaller timestep we need to recover the last
           valid wellstate. This is maintained with the
           last_valid_well_state_ member and the functions
-          commitWellState() and resetWellState().
+          commitWGState() and resetWellState().
 
         3. For the NUPCOL functionality we should either use the
            currently active wellstate or a wellstate frozen at max
            nupcol iterations. This is handled with the member
-           nupcol_well_state_ and the initNupcolWellState() function.
+           nupcol_well_state_ and the updateNupcolWGState() function.
     */
 
     /*
       Will return the last good wellstate. This is typcially used when
       initializing a new report step where the Schedule object might
       have introduced new wells. The wellstate returned by
-      prevWellState() must have been stored with the commitWellState()
+      prevWellState() must have been stored with the commitWGState()
       function first.
     */
     const WellState<Scalar>& prevWellState() const
@@ -321,6 +337,8 @@ protected:
         this->nupcol_wgstate_ = this->active_wgstate_;
     }
 
+    void reportGroupSwitching(DeferredLogger& local_deferredLogger) const;
+
     /// \brief Create the parallel well information
     /// \param localWells The local wells from ECL schedule
     std::vector<std::reference_wrapper<ParallelWellInfo<Scalar>>>
@@ -331,7 +349,9 @@ protected:
 
     bool wasDynamicallyShutThisTimeStep(const int well_index) const;
 
-    Scalar updateNetworkPressures(const int reportStepIdx);
+    Scalar updateNetworkPressures(const int reportStepIdx,
+                                  const Scalar damping_factor,
+                                  const Scalar update_upper_bound);
 
     void updateWsolvent(const Group& group,
                         const int reportStepIdx,
@@ -373,27 +393,18 @@ protected:
 
     bool checkGroupHigherConstraints(const Group& group,
                                      DeferredLogger& deferred_logger,
-                                     const int reportStepIdx);
+                                     const int reportStepIdx,
+                                     const int max_number_of_group_switch);
 
     void updateAndCommunicateGroupData(const int reportStepIdx,
-                                       const int iterationIdx);
+                                       const int iterationIdx,
+                                       const Scalar tol_nupcol,
+                                       DeferredLogger& deferred_logger);
 
     void inferLocalShutWells();
 
     void setRepRadiusPerfLength();
 
-    void gliftDebug(const std::string& msg,
-                    DeferredLogger& deferred_logger) const;
-
-    void gliftDebugShowALQ(DeferredLogger& deferred_logger);
-
-    void gasLiftOptimizationStage2(DeferredLogger& deferred_logger,
-                                   GLiftProdWells& prod_wells,
-                                   GLiftOptWells& glift_wells,
-                                   GasLiftGroupInfo<Scalar>& group_info,
-                                   GLiftWellStateMap& map,
-                                   const int episodeIndex);
-
     virtual void computePotentials(const std::size_t widx,
                                    const WellState<Scalar>& well_state_copy,
                                    std::string& exc_msg,
@@ -431,18 +442,21 @@ protected:
     /// \brief get compressed index for interior cells (-1, otherwise
     virtual int compressedIndexForInterior(int cartesian_cell_idx) const = 0;
 
-    std::vector<int> getCellsForConnections(const Well& well) const;
     std::vector<std::vector<int>> getMaxWellConnections() const;
 
     std::vector<std::string> getWellsForTesting(const int timeStepIdx,
                                                 const double simulationTime);
 
-    using WellTracerRates = std::map<std::pair<std::string, std::string>, Scalar>;
+    using WellTracerRates = std::unordered_map<int, std::vector<WellTracerRate<Scalar>>>;
     void assignWellTracerRates(data::Wells& wsrpt,
-                               const WellTracerRates& wellTracerRates) const;
-    using MswTracerRates = std::map<std::tuple<std::string, std::string, std::size_t>, Scalar>;
+                               const WellTracerRates& wellTracerRates,
+                               const unsigned reportStep) const;
+
+    using MswTracerRates = std::unordered_map<int, std::vector<MSWellTracerRate<Scalar>>>;
     void assignMswTracerRates(data::Wells& wsrpt,
-                              const MswTracerRates& mswTracerRates) const;
+                              const MswTracerRates& mswTracerRates,
+                              const unsigned reportStep) const;
+
     void assignMassGasRate(data::Wells& wsrpt,
                            const Scalar& gasDensity) const;
 
@@ -450,6 +464,8 @@ protected:
     const SummaryState& summaryState_;
     const EclipseState& eclState_;
     const Parallel::Communication& comm_;
+    BlackoilWellModelGasLiftGeneric<Scalar>& gen_gaslift_;
+    BlackoilWellModelWBP<Scalar> wbp_;
 
     PhaseUsage phase_usage_;
     bool terminal_output_{false};
@@ -463,91 +479,11 @@ protected:
     std::vector<Well> wells_ecl_;
     std::vector<std::vector<PerforationData<Scalar>>> well_perf_data_;
 
-    /// Connection index mappings
-    class ConnectionIndexMap
-    {
-    public:
-        /// Constructor.
-        ///
-        /// \param[in] numConns Total number of well connections, both open
-        ///   and closed/shut.  Typically \code WellConnections::size() \endcode.
-        explicit ConnectionIndexMap(const std::size_t numConns)
-            : local_(numConns, -1)
-        {
-            this->global_.reserve(numConns);
-            this->open_.reserve(numConns);
-        }
+    // Times at which wells were opened (for WCYCLE)
+    std::map<std::string, double> well_open_times_;
 
-        /// Enumerate/map new active connection.
-        ///
-        /// \param[in] connIdx Global well connection index.  Must be an
-        ///   integer in the range 0..numConns-1.
-        ///
-        /// \param[in] connIsOpen Whether or not the connection is
-        ///   open/flowing.
-        void addActiveConnection(const int  connIdx,
-                                 const bool connIsOpen)
-        {
-            this->local_[connIdx] =
-                static_cast<int>(this->global_.size());
-
-            this->global_.push_back(connIdx);
-
-            const auto open_conn_idx = connIsOpen
-                ? this->num_open_conns_++
-                : -1;
-
-            this->open_.push_back(open_conn_idx);
-        }
-
-        /// Get local connection IDs/indices of every existing well
-        /// connection.
-        ///
-        /// Negative value (-1) for connections that don't intersect the
-        /// current rank.
-        const std::vector<int>& local() const
-        {
-            return this->local_;
-        }
-
-        /// Get global connection ID of local (on-rank) connection.
-        ///
-        /// \param[in] connIdx Local connection index.
-        ///
-        /// \return Global connection ID of \p connIdx.
-        int global(const int connIdx) const
-        {
-            return this->global_[connIdx];
-        }
-
-        /// Get open connection ID of local (on-rank) connection.
-        ///
-        /// \param[in] connIdx Local connection index.
-        ///
-        /// \return Open connection ID of \p connIdx.  Integer in the range
-        ///   0..#open connections - 1 if the connection is open or negative
-        ///   value (-1) otherwise.
-        int open(const int connIdx) const
-        {
-            return this->open_[connIdx];
-        }
-
-    private:
-        /// Local connection IDs/indices of every existing well connection.
-        /// Negative value (-1) for connections that don't intersect the
-        /// current rank.
-        std::vector<int> local_{};
-
-        /// Global connection index of each on-rank reservoir connection.
-        /// Reverse/transpose mapping of \c local_.
-        std::vector<int> global_{};
-
-        /// Open connection index of each on-rank reservoir connection.
-        std::vector<int> open_{};
-
-        /// Number of open connections on this rank.
-        int num_open_conns_{0};
-    };
+    // Times at which wells were shut (for WCYCLE)
+    std::map<std::string, double> well_close_times_;
 
     std::vector<ConnectionIndexMap> conn_idx_map_{};
     std::function<bool(const Well&)> not_on_process_{};
@@ -561,7 +497,6 @@ protected:
     std::vector<std::reference_wrapper<ParallelWellInfo<Scalar>>> local_parallel_well_info_;
 
     std::vector<WellProdIndexCalculator<Scalar>> prod_index_calc_;
-    mutable ParallelWBPCalculation<Scalar> wbpCalculationService_;
 
     std::vector<int> pvt_region_idx_;
 
@@ -581,21 +516,18 @@ protected:
       The various wellState members should be accessed and modified
       through the accessor functions wellState(), prevWellState(),
       commitWellState(), resetWellState(), nupcolWellState() and
-      updateNupcolWellState().
+      updateNupcolWGState().
     */
     WGState<Scalar> active_wgstate_;
     WGState<Scalar> last_valid_wgstate_;
     WGState<Scalar> nupcol_wgstate_;
-
-    bool glift_debug = false;
-
-    double last_glift_opt_time_ = -1.0;
+    WellGroupEvents report_step_start_events_; //!< Well group events at start of report step
 
     bool wellStructureChangedDynamically_{false};
 
     // Store maps of group name and new group controls for output
-    std::map<std::string, std::string> switched_prod_groups_;
-    std::map<std::pair<std::string, Phase>, std::string> switched_inj_groups_;
+    std::map<std::string, std::vector<Group::ProductionCMode>> switched_prod_groups_;
+    std::map<std::string, std::array<std::vector<Group::InjectionCMode>, 3>> switched_inj_groups_;
     // Store map of group name and close offending well for output
     std::map<std::string, std::pair<std::string, std::string>> closed_offending_wells_;
 
