@@ -192,7 +192,9 @@ loadRestartGuideRates(const int                    report_step,
                       GuideRate&                   guide_rate) const
 {
     for (const auto& [well_name, rst_well] : rst_wells) {
-        if (!wellModel_.hasWell(well_name) || wellModel_.getWellEcl(well_name).isInjector()) {
+        if (!wellModel_.hasLocalWell(well_name) ||
+            wellModel_.getWellEcl(well_name).isInjector())
+        {
             continue;
         }
 
@@ -256,7 +258,7 @@ loadRestartData(const data::Wells&                 rst_wells,
     {
         const auto& well_name = well_state.name(well_index);
 
-        this->loadRestartWellData(well_name, handle_ms_well, phs,
+        this->loadRestartWellData(well_name, handle_ms_well && !well_state.is_permanently_inactive_well(well_name), phs,
                                   rst_wells.at(well_name),
                                   wellModel_.perfData(well_index),
                                   well_state.well(well_index));
