@@ -418,10 +418,10 @@ checkNewtonIterationIdxOk_(const std::string& well_name)
                 "LIFTOPT item4 == YES, it = {}, nupcol = {} -->  GLIFT optimize = {}",
                 this->iteration_idx_,
                 nupcol,
-                ((this->iteration_idx_ <= nupcol) ? "TRUE" : "FALSE"));
+                ((this->iteration_idx_ < nupcol) ? "TRUE" : "FALSE"));
             displayDebugMessage_(msg, well_name);
         }
-        return this->iteration_idx_ <= nupcol;
+        return this->iteration_idx_ < nupcol;
     }
     else {
         if (this->debug) {
@@ -532,22 +532,29 @@ getProducerWellRates_(const Well* well, int well_index)
 
     const auto controls = well->productionControls(this->summary_state_);
     Scalar oil_rate = oil_pot;
-    if (controls.hasControl(Well::ProducerCMode::ORAT)) {
-        oil_rate = std::min(static_cast<Scalar>(controls.oil_rate), oil_rate);
-    }
+    Scalar water_rate = water_pot;
     Scalar gas_rate = gas_pot;
+
+    if (controls.hasControl(Well::ProducerCMode::ORAT) && oil_rate > static_cast<Scalar>(controls.oil_rate)) {
+        water_rate *= (static_cast<Scalar>(controls.oil_rate) / oil_rate); 
+        oil_rate = static_cast<Scalar>(controls.oil_rate);
+    }
+    
     if (controls.hasControl(Well::ProducerCMode::GRAT)) {
         gas_rate = std::min(static_cast<Scalar>(controls.gas_rate), gas_rate);
     }
-    Scalar water_rate = water_pot;
-    if (controls.hasControl(Well::ProducerCMode::WRAT)) {
-        water_rate = std::min(static_cast<Scalar>(controls.water_rate), water_rate);
+    
+    if (controls.hasControl(Well::ProducerCMode::WRAT) && water_rate > static_cast<Scalar>(controls.water_rate)) {
+        oil_rate *= (static_cast<Scalar>(controls.water_rate) / water_rate); 
+        water_rate = static_cast<Scalar>(controls.water_rate);
     }
     if (controls.hasControl(Well::ProducerCMode::LRAT)) {
         Scalar liquid_rate = oil_rate + water_rate;
-        Scalar liquid_rate_lim = std::min(static_cast<Scalar>(controls.liquid_rate), liquid_rate);
-        water_rate = water_rate / liquid_rate * liquid_rate_lim;
-        oil_rate = oil_rate / liquid_rate * liquid_rate_lim;
+        Scalar liquid_rate_lim = static_cast<Scalar>(controls.liquid_rate);
+        if (liquid_rate > liquid_rate_lim) {
+            water_rate = water_rate / liquid_rate * liquid_rate_lim;
+            oil_rate = oil_rate / liquid_rate * liquid_rate_lim;
+        }
     }
 
     return {oil_rate, gas_rate, water_rate, oil_pot, gas_pot, water_pot};
@@ -577,7 +584,8 @@ initializeGroupRatesRecursive_(const Group& group)
                 if (well->isProducer()) {
                     auto [sw_oil_rate, sw_gas_rate, sw_water_rate, sw_oil_pot, sw_gas_pot, sw_water_pot] = getProducerWellRates_(well, index);
                     auto sw_alq = this->well_state_.getALQ(well_name);
-                    Scalar factor = well->getEfficiencyFactor();
+                    const Scalar factor = well->getEfficiencyFactor() *
+                                          this->well_state_.getGlobalEfficiencyScalingFactor(well_name);
                     oil_rate += (factor * sw_oil_rate);
                     gas_rate += (factor * sw_gas_rate);
                     water_rate += (factor * sw_water_rate);
@@ -639,19 +647,6 @@ initializeGroupRatesRecursive_(const Group& group)
     }
     if (oil_target || liquid_target || water_target || gas_target || max_total_gas || max_alq) {
         updateGroupIdxMap_(group.name());
-        if (oil_target)
-            oil_rate = std::min(oil_rate, *oil_target);
-        if (gas_target)
-            gas_rate = std::min(gas_rate, *gas_target);
-        if (water_target)
-            water_rate = std::min(water_rate, *water_target);
-        if (liquid_target) {
-            Scalar liquid_rate = oil_rate + water_rate;
-            Scalar liquid_rate_limited = std::min(liquid_rate, *liquid_target);
-            oil_rate = oil_rate / liquid_rate * liquid_rate_limited;
-            water_rate = water_rate / liquid_rate * liquid_rate_limited;
-        }
-
         this->group_rate_map_.try_emplace(group.name(),
             oil_rate, gas_rate, water_rate, alq,
             oil_potential, gas_potential, water_potential,
@@ -660,6 +655,25 @@ initializeGroupRatesRecursive_(const Group& group)
             debugDisplayUpdatedGroupRates(
                 group.name(), oil_rate, gas_rate, water_rate, alq);
         }
+
+        if (oil_target && oil_rate > *oil_target)  {
+            water_rate *= (*oil_target/oil_rate);
+            oil_rate = *oil_target;
+        }
+        if (gas_target && gas_rate > *gas_target)
+            gas_rate = *gas_target;
+        if (water_target && water_rate > *water_target) {
+            oil_rate *= (*water_target/water_rate); 
+            water_rate = *water_target;
+        }
+        if (liquid_target) {
+            Scalar liquid_rate = oil_rate + water_rate;
+            Scalar liquid_rate_limited = *liquid_target;
+            if (liquid_rate > liquid_rate_limited) {
+                oil_rate = oil_rate / liquid_rate * liquid_rate_limited;
+                water_rate = water_rate / liquid_rate * liquid_rate_limited;
+            }
+        }
     }
     return std::make_tuple(oil_rate, gas_rate, water_rate, oil_potential, gas_potential, water_potential, alq);
 }
@@ -687,10 +701,13 @@ initializeWell2GroupMapRecursive_(const Group& group,
             // TODO: can the same well be memember of two different groups
             //  (on the same recursion level) ?
             assert(this->well_group_map_.count(well_name) == 0);
-            if (checkDoGasLiftOptimization_(well_name)) {
+            bool checkDoGasLift = checkDoGasLiftOptimization_(well_name);
+            checkDoGasLift = this->comm_.max(checkDoGasLift);
+            if (checkDoGasLift) {
                 const auto &well = this->schedule_.getWell(
                     well_name, this->report_step_idx_);
-                Scalar wfac = well.getEfficiencyFactor();
+                const Scalar wfac = well.getEfficiencyFactor() *
+                                    this->well_state_.getGlobalEfficiencyScalingFactor(well_name);
                 auto [itr, success] = this->well_group_map_.insert(
                       {well_name, /*empty vector*/ {}});
                 assert(success);
