@@ -18,6 +18,7 @@
 */
 
 #include <config.h>
+#include <opm/common/TimingMacros.hpp>
 #include <opm/simulators/wells/GasLiftSingleWellGeneric.hpp>
 
 #include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
@@ -138,6 +139,7 @@ std::unique_ptr<GasLiftWellState<Scalar>>
 GasLiftSingleWellGeneric<Scalar>::
 runOptimize(const int iteration_idx)
 {
+    OPM_TIMEFUNCTION();
     std::unique_ptr<GasLiftWellState<Scalar>> state;
     if (this->optimize_) {
         if (this->debug_limit_increase_decrease_) {
@@ -170,6 +172,74 @@ runOptimize(const int iteration_idx)
     return state;
 }
 
+template<class Scalar>
+std::pair<Scalar, bool>
+GasLiftSingleWellGeneric<Scalar>::
+wellTestALQ()
+{
+    // If WLIFTOPT item 2 is NO we don't optimize
+    if (!this->optimize_) {
+        return {0.0, false};
+    }
+
+    Scalar temp_alq = std::max(this->min_alq_, Scalar(0.0));
+    auto cur_alq = temp_alq;
+    auto init_rates = computeLimitedWellRatesWithALQ_(temp_alq);
+    LimitedRates new_rates = *init_rates;
+    std::optional<LimitedRates> rates;
+    Scalar old_gradient = 0.0;
+    bool alq_is_limited = false;
+    bool increase = true;
+    OptimizeState state {*this, increase};
+    bool success = false;
+
+    while (!state.stop_iteration && (++state.it <= this->max_iterations_)) {
+        if (state.checkAlqOutsideLimits(temp_alq, new_rates.oil))
+            break;
+        std::optional<Scalar> alq_opt;
+        std::tie(alq_opt, alq_is_limited) = state.addOrSubtractAlqIncrement(temp_alq);
+        if (!alq_opt)
+            break;
+
+        temp_alq = *alq_opt;
+        if (this->debug)
+            state.debugShowIterationInfo(temp_alq);
+        rates = new_rates;
+        auto temp_rates = computeLimitedWellRatesWithALQ_(temp_alq);
+        if (!temp_rates)
+            temp_rates->oil = 0.0;
+        if (temp_rates->bhp_is_limited)
+            state.stop_iteration = true;
+
+        auto gradient = state.calcEcoGradient(rates->oil, temp_rates->oil, rates->gas, temp_rates->gas);
+        if (this->debug)
+            debugCheckNegativeGradient_(gradient,
+                                        cur_alq,
+                                        temp_alq,
+                                        rates->oil,
+                                        temp_rates->oil,
+                                        rates->gas,
+                                        temp_rates->gas,
+                                        increase);
+        if (!success && gradient != old_gradient) {
+            success = true;
+        }
+        if (success && gradient <= old_gradient) {
+            break;
+        }
+        cur_alq = temp_alq;
+        new_rates = *temp_rates;
+        old_gradient = gradient;
+    }
+    if (state.it > this->max_iterations_) {
+        warnMaxIterationsExceeded_();
+    }
+    if (success) {
+        this->well_state_.gliftUpdateAlqIncreaseCount(this->well_name_, increase);
+    }
+    return {cur_alq, success};
+}
+
 /****************************************
  * Protected methods in alphabetical order
  ****************************************/
@@ -204,7 +274,7 @@ addOrSubtractAlqIncrement_(Scalar alq, bool increase) const
                 limited = true;
             }
         } else {
-            if (alq < 0) {
+            if (alq < 1e-12) {
                 alq = 0.0;
                 limited = true;
             }
@@ -261,7 +331,7 @@ checkGroupTargetsViolated(const BasicRates& rates,
             auto target_opt = this->group_info_.getTarget(rate_type, group_name);
             if (target_opt) {
                 auto delta_rate = new_rates[rate_type] - rates[rate_type];
-                auto new_group_rate = this->group_info_.getPotential(rate_type, group_name) + efficiency * delta_rate;
+                auto new_group_rate = this->group_info_.getRate(rate_type, group_name) + efficiency * delta_rate;
                 if (new_group_rate > *target_opt) {
                     if (this->debug) {
                         const std::string msg
@@ -362,6 +432,7 @@ std::optional<typename GasLiftSingleWellGeneric<Scalar>::BasicRates>
 GasLiftSingleWellGeneric<Scalar>::
 computeWellRatesWithALQ_(Scalar alq) const
 {
+    OPM_TIMEFUNCTION();
     std::optional<BasicRates> rates;
     auto bhp_opt = computeBhpAtThpLimit_(alq);
     if (bhp_opt) {
@@ -421,8 +492,8 @@ void GasLiftSingleWellGeneric<Scalar>::
 debugShowBhpAlqTable_()
 {
     Scalar alq = 0.0;
-    const std::string fmt_fmt1 {"{:^12s} {:^12s} {:^12s} {:^12s}"};
-    const std::string fmt_fmt2 {"{:>12.5g} {:>12.5g} {:>12.5g} {:>12.5g}"};
+    constexpr std::string_view fmt_fmt1 {"{:^12s} {:^12s} {:^12s} {:^12s}"};
+    constexpr std::string_view fmt_fmt2 {"{:>12.5g} {:>12.5g} {:>12.5g} {:>12.5g}"};
     const std::string header = fmt::format(fmt_fmt1, "ALQ", "BHP", "oil", "gas");
     displayDebugMessage_(header);
     auto max_it = 50;
@@ -666,6 +737,14 @@ getRateWithLimit_(Rate rate_type, const BasicRates& rates) const
     //   for why the rate was limited.
     std::optional<Rate> target_type;
 
+    // we also need to limit the other rate (currently only for water and oil and not gas)
+    Scalar rate2 = 0.0;
+    if (rate_type == Rate::oil) {
+        rate2 = getRate_(Rate::water, rates);
+    } else if (rate_type == Rate::water) {
+        rate2 = getRate_(Rate::oil, rates);
+    }
+
     if (hasProductionControl_(rate_type)) {
         auto target = getProductionTarget_(rate_type);
         if (new_rate > target) {
@@ -675,42 +754,73 @@ getRateWithLimit_(Rate rate_type, const BasicRates& rates) const
                                                 new_rate,
                                                 target);
             displayDebugMessage_(msg);
+            rate2 *= target/new_rate;
             new_rate = target;
-            target_type = rate_type;
+            target_type = rate_type; 
         }
     }
-    if (((rate_type == Rate::oil) || (rate_type == Rate::water)) && hasProductionControl_(Rate::liquid)) {
-        Scalar rate2;
+    if (((rate_type == Rate::oil) || (rate_type == Rate::water))) {
         if (rate_type == Rate::oil) {
-            rate2 = getRate_(Rate::water, rates);
+            if(hasProductionControl_(Rate::water)) {
+                auto water_target = getProductionTarget_(Rate::water);
+                if (rate2 > water_target) {
+                    new_rate *= (water_target / rate2);
+                    target_type = Rate::water;
+                    rate2 = water_target;
+                    const std::string msg = fmt::format("limiting {} rate to {} due to WRAT target: "
+                                                        "computed WRAT: {}, target WRAT: {}",
+                                                        GasLiftGroupInfo<Scalar>::rateToString(rate_type),
+                                                        new_rate,
+                                                        rate2,
+                                                        water_target);
+                    displayDebugMessage_(msg);
+                }
+            }
         } else {
-            rate2 = getRate_(Rate::oil, rates);
+            if(hasProductionControl_(Rate::oil)) {
+                auto oil_target = getProductionTarget_(Rate::oil);
+                if (rate2 > oil_target) {
+                    new_rate *= (oil_target / rate2);
+                    target_type = Rate::oil;
+                    rate2 = oil_target;
+                    const std::string msg = fmt::format("limiting {} rate to {} due to ORAT target: "
+                                                        "computed ORAT: {}, target ORAT: {}",
+                                                        GasLiftGroupInfo<Scalar>::rateToString(rate_type),
+                                                        new_rate,
+                                                        rate2,
+                                                        oil_target);
+                    displayDebugMessage_(msg);
+                }
+            }
         }
-        // Note: Since "new_rate" was first updated for ORAT or WRAT, see first "if"
-        //   statement in the method, the rate is limited due to LRAT only if
-        //   it becomes less than the rate limited by a WRAT or ORAT target..
-        Scalar liq_rate = new_rate + rate2;
 
-        auto liq_target = getProductionTarget_(Rate::liquid);
-        if (liq_rate > liq_target) {
-            Scalar fraction = new_rate / liq_rate;
-            // NOTE: since
-            //      fraction * liq_rate = new_rate,
-            //  we must have
-            //      fraction * liq_target < new_rate
-            //  since
-            //      liq_target < liq_rate
-            //  therefore new_rate will become less than it original was and
-            //  limited = true.
-            new_rate = fraction * liq_target;
-            target_type = Rate::liquid;
-            const std::string msg = fmt::format("limiting {} rate to {} due to LRAT target: "
-                                                "computed LRAT: {}, target LRAT: {}",
-                                                GasLiftGroupInfo<Scalar>::rateToString(rate_type),
-                                                new_rate,
-                                                liq_rate,
-                                                liq_target);
-            displayDebugMessage_(msg);
+        if(hasProductionControl_(Rate::liquid)) {
+            // Note: Since "new_rate" was first updated for ORAT or WRAT, see first "if"
+            //   statement in the method, the rate is limited due to LRAT only if
+            //   it becomes less than the rate limited by a WRAT or ORAT target..
+            Scalar liq_rate = new_rate + rate2;
+
+            auto liq_target = getProductionTarget_(Rate::liquid);
+            if (liq_rate > liq_target) {
+                Scalar fraction = new_rate / liq_rate;
+                // NOTE: since
+                //      fraction * liq_rate = new_rate,
+                //  we must have
+                //      fraction * liq_target < new_rate
+                //  since
+                //      liq_target < liq_rate
+                //  therefore new_rate will become less than it original was and
+                //  limited = true.
+                new_rate = fraction * liq_target;
+                target_type = Rate::liquid;
+                const std::string msg = fmt::format("limiting {} rate to {} due to LRAT target: "
+                                                    "computed LRAT: {}, target LRAT: {}",
+                                                    GasLiftGroupInfo<Scalar>::rateToString(rate_type),
+                                                    new_rate,
+                                                    liq_rate,
+                                                    liq_target);
+                displayDebugMessage_(msg);
+            }
         }
     }
     // TODO: Also check RESV target?
@@ -768,25 +878,7 @@ getLiquidRateWithGroupLimit_(const Scalar new_oil_rate,
         = getRateWithGroupLimit_(Rate::liquid, new_liquid_rate, liquid_rate, gr_name_dont_limit);
     bool limited = group_name != nullptr;
     if (limited) {
-        // the oil, gas, and water cases can be handled directly by
-        //  getRateWithGroupLimit_() above. However, for the liquid case
-        //  we must do some postprocessing. I chose to include it here
-        //  instead of cluttering up getRateWithGroupLimit_() with this
-        //  special case.
-        Scalar delta_water = new_water_rate - water_rate;
-        Scalar delta_oil = new_oil_rate - oil_rate;
-
-        Scalar gr_water_rate = this->group_info_.waterRate(*group_name);
-        Scalar gr_oil_rate = this->group_info_.oilRate(*group_name);
-
-        // NOTE: these rates are too large according to the limited liquid rate
-        //  but it does not matter since we are only using them to calculate
-        //  the fraction of the liquid corresponding to the oil phase
-        Scalar new_gr_water_rate = gr_water_rate + efficiency * delta_water;
-        Scalar new_gr_oil_rate = gr_oil_rate + efficiency * delta_oil;
-        Scalar new_gr_liquid_rate = new_gr_water_rate + new_gr_oil_rate;
-
-        Scalar oil_fraction = new_gr_oil_rate / new_gr_liquid_rate;
+        Scalar oil_fraction = oil_rate / liquid_rate;
         Scalar delta_liquid = liquid_rate_limited - liquid_rate;
         auto limited_oil_rate = oil_rate + oil_fraction * delta_liquid;
         auto limited_water_rate = water_rate + (1.0 - oil_fraction) * delta_liquid;
@@ -880,6 +972,7 @@ getInitialRatesWithLimit_() const
         auto temp_rates = getLimitedRatesFromRates_(*rates);
         BasicRates old_rates = getWellStateRates_();
         limited_rates = updateRatesToGroupLimits_(old_rates, temp_rates);
+
         initial_alq = alq;
     }
     return {limited_rates, initial_alq};
@@ -891,9 +984,10 @@ GasLiftSingleWellGeneric<Scalar>::
 getLimitedRatesFromRates_(const BasicRates& rates) const
 {
     auto [oil_rate, oil_limiting_target] = getOilRateWithLimit2_(rates);
+    bool oil_is_limited = oil_limiting_target.has_value();
     auto [gas_rate, gas_is_limited] = getGasRateWithLimit_(rates);
     auto [water_rate, water_limiting_target] = getWaterRateWithLimit2_(rates);
-    bool oil_is_limited = oil_limiting_target.has_value();
+   
     bool water_is_limited = water_limiting_target.has_value();
     return LimitedRates {oil_rate,
                          gas_rate,
@@ -929,7 +1023,7 @@ getWellStateRates_() const
                                             water_rate);
         displayDebugMessage_(msg);
     }
-    return BasicRates {oil_rate, water_rate, gas_rate, /*bhp_is_limited=*/false};
+    return BasicRates {oil_rate, gas_rate, water_rate, /*bhp_is_limited=*/false};
 }
 
 template<class Scalar>
@@ -957,11 +1051,10 @@ GasLiftSingleWellGeneric<Scalar>::
 increaseALQtoPositiveOilRate_(Scalar alq,
                               const LimitedRates& orig_rates) const
 {
-    bool stop_iteration = false;
     Scalar temp_alq = alq;
     // use the copy constructor to only copy the rates
     BasicRates rates = orig_rates;
-    while (!stop_iteration) {
+    while (true) {
         temp_alq += this->increment_;
         if (temp_alq > this->max_alq_)
             break;
@@ -987,10 +1080,9 @@ increaseALQtoMinALQ_(const Scalar orig_alq,
     assert(min_alq >= 0);
     assert(orig_alq < min_alq);
     assert(min_alq < this->max_alq_);
-    bool stop_iteration = false;
     Scalar alq = orig_alq;
     LimitedRates rates = orig_rates;
-    while (!stop_iteration) {
+    while (true) {
         Scalar temp_alq = alq + this->increment_;
 
         alq = temp_alq;
@@ -1018,7 +1110,7 @@ logSuccess_(Scalar alq, const int iteration_idx)
                                             ((alq > this->orig_alq_) ? "increased" : "decreased"),
                                             this->orig_alq_,
                                             alq);
-    this->deferred_logger_.info(message);
+    this->deferred_logger_.debug(message);
 }
 
 template<class Scalar>
@@ -1071,8 +1163,6 @@ maybeAdjustALQbeforeOptimizeLoop_(const LimitedRates& orig_rates,
             const std::string msg = fmt::format("adjusted ALQ to: {}", alq);
             displayDebugMessage_(msg);
         }
-        Scalar delta_alq = alq - orig_alq;
-        updateGroupRates_(orig_rates, rates, delta_alq);
     }
     return {rates, alq};
 }
@@ -1086,11 +1176,10 @@ GasLiftSingleWellGeneric<Scalar>::
 reduceALQtoGroupAlqLimits_(const Scalar orig_alq,
                            const LimitedRates& orig_rates) const
 {
-    bool stop_this_iteration = false;
     Scalar alq = orig_alq;
     BasicRates rates {orig_rates};
     Scalar temp_alq = orig_alq;
-    while (!stop_this_iteration) {
+    while (true) {
         if (temp_alq == 0)
             break;
         temp_alq -= this->increment_;
@@ -1179,8 +1268,7 @@ reduceALQtoWellTarget_(const Scalar orig_alq,
     Scalar alq = orig_alq;
     Scalar temp_alq = alq;
     std::optional<LimitedRates> new_rates;
-    bool stop_iteration = false;
-    while (!stop_iteration) {
+    while (true) {
         if (temp_alq == 0)
             break;
         temp_alq -= this->increment_;
@@ -1237,6 +1325,7 @@ std::unique_ptr<GasLiftWellState<Scalar>>
 GasLiftSingleWellGeneric<Scalar>::
 runOptimizeLoop_(bool increase)
 {
+    OPM_TIMEFUNCTION();
     if (this->debug)
         debugShowProducerControlMode();
     std::unique_ptr<GasLiftWellState<Scalar>> ret_value; // nullptr initially
@@ -1275,6 +1364,7 @@ runOptimizeLoop_(bool increase)
         state.stop_iteration = true;
     }
     while (!state.stop_iteration && (++state.it <= this->max_iterations_)) {
+        OPM_TIMEBLOCK(GasliftOptimizationLoopStage1);
         if (state.checkRatesViolated(new_rates))
             break;
         if (state.checkAlqOutsideLimits(temp_alq, new_rates.oil))
@@ -1354,6 +1444,7 @@ template<class Scalar>
 std::unique_ptr<GasLiftWellState<Scalar>>
 GasLiftSingleWellGeneric<Scalar>::runOptimize1_()
 {
+    OPM_TIMEFUNCTION();
     std::unique_ptr<GasLiftWellState<Scalar>> state;
     int inc_count = this->well_state_.gliftGetAlqIncreaseCount(this->well_name_);
     int dec_count = this->well_state_.gliftGetAlqDecreaseCount(this->well_name_);
@@ -1376,6 +1467,7 @@ template<class Scalar>
 std::unique_ptr<GasLiftWellState<Scalar>>
 GasLiftSingleWellGeneric<Scalar>::runOptimize2_()
 {
+    OPM_TIMEFUNCTION();
     std::unique_ptr<GasLiftWellState<Scalar>> state;
     state = tryIncreaseLiftGas_();
     if (!state || !(state->alqChanged())) {
@@ -1465,13 +1557,16 @@ updateRatesToGroupLimits_(const BasicRates& old_rates,
 {
     LimitedRates new_rates = rates;
     auto [new_oil_rate, oil_is_limited] = getOilRateWithGroupLimit_(new_rates.oil, old_rates.oil, gr_name);
+    auto mod_water_rate = new_rates.water;
     if (oil_is_limited) {
         new_rates.oil_limiting_target = Rate::oil;
+        mod_water_rate *=  (new_oil_rate / new_rates.oil);
     }
     auto [new_gas_rate, gas_is_limited] = getGasRateWithGroupLimit_(new_rates.gas, old_rates.gas, gr_name);
-    auto [new_water_rate, water_is_limited] = getWaterRateWithGroupLimit_(new_rates.water, old_rates.water, gr_name);
+    auto [new_water_rate, water_is_limited] = getWaterRateWithGroupLimit_(mod_water_rate, old_rates.water, gr_name);
     if (water_is_limited) {
         new_rates.water_limiting_target = Rate::water;
+        new_oil_rate *=  (new_water_rate / new_rates.water);
     }
     auto [new_oil_rate2, new_water_rate2, oil_is_limited2, water_is_limited2]
         = getLiquidRateWithGroupLimit_(new_oil_rate, old_rates.oil, new_water_rate, old_rates.water, gr_name);
@@ -1498,7 +1593,7 @@ template<class Scalar>
 void GasLiftSingleWellGeneric<Scalar>::
 updateWellStateAlqFixedValue_(const GasLiftWell& well)
 {
-    auto& max_alq_optional = well.max_rate();
+    const auto& max_alq_optional = well.max_rate();
     if (max_alq_optional) {
         // According to WLIFTOPT, item 3:
         // If item 2 is NO, then item 3 is regarded as the fixed
