@@ -18,6 +18,7 @@
 */
 
 #include <config.h>
+#include <opm/common/TimingMacros.hpp>
 #include <opm/simulators/wells/GasLiftStage2.hpp>
 
 #include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
@@ -82,6 +83,7 @@ GasLiftStage2<Scalar>::GasLiftStage2(const int report_step_idx,
 template<class Scalar>
 void GasLiftStage2<Scalar>::runOptimize()
 {
+    OPM_TIMEFUNCTION();
     const auto& group = this->schedule_.getGroup("FIELD", this->report_step_idx_);
     optimizeGroupsRecursive_(group);
 }
@@ -117,11 +119,6 @@ addOrRemoveALQincrement_(GradMap &grad_map,
             well_name, (add ? "adding" : "subtracting"), old_alq, new_alq);
         this->displayDebugMessage_(msg);
     }
-    state.update(gi.new_oil_rate, gi.oil_is_limited,
-                 gi.new_gas_rate, gi.gas_is_limited,
-                 gi.alq, gi.alq_is_limited,
-                 gi.new_water_rate, gi.water_is_limited, add);
-
     this->well_state_.setALQ(well_name, gi.alq);
     const auto& pu = this->well_state_.phaseUsage();
     std::vector<Scalar> well_pot(pu.num_phases, 0.0);
@@ -424,9 +421,10 @@ template<class Scalar>
 void GasLiftStage2<Scalar>::
 optimizeGroup_(const Group& group)
 {
+    OPM_TIMEFUNCTION();
     const auto& group_name = group.name();
     const auto prod_control = this->group_state_.production_control(group_name);
-    if (this->glo_.has_group(group.name()) || ((prod_control != Group::ProductionCMode::NONE) && (prod_control != Group::ProductionCMode::FLD)))
+    if ((prod_control != Group::ProductionCMode::NONE) && (prod_control != Group::ProductionCMode::FLD))
     {
         if (this->debug) {
             const std::string msg = fmt::format("optimizing (control = {})", Group::ProductionCMode2String(prod_control));
@@ -436,7 +434,7 @@ optimizeGroup_(const Group& group)
         std::vector<GradPair> inc_grads;
         std::vector<GradPair> dec_grads;
         redistributeALQ_(wells, group, inc_grads, dec_grads);
-        removeSurplusALQ_(group, inc_grads, dec_grads);
+        removeSurplusALQ_(group, dec_grads);
     }
     else {
         if (this->debug) {
@@ -450,6 +448,7 @@ template<class Scalar>
 void GasLiftStage2<Scalar>::
 optimizeGroupsRecursive_(const Group& group)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& group_name : group.groups()) {
         if(!this->schedule_.back().groups.has(group_name))
             continue;
@@ -479,7 +478,7 @@ recalculateGradientAndUpdateData_(GradPairItr& grad_itr,
     // only applies to wells in the well_state_map (i.e. wells on this rank)
     // the grads and other grads are synchronized later
     if(this->stage1_wells_.count(name) > 0) {
-        GasLiftSingleWell &gs_well = *(this->stage1_wells_.at(name).get());
+        const GasLiftSingleWell& gs_well = *(this->stage1_wells_.at(name).get());
         {
             auto grad = calcIncOrDecGrad_(name, gs_well, gr_name_dont_limit, increase);
             if (grad) {
@@ -493,7 +492,7 @@ recalculateGradientAndUpdateData_(GradPairItr& grad_itr,
         }
         // If the old incremental/decremental gradient was defined, it becomes the new
         //   decremental/incremental gradient
-        if (old_grad) {
+        if (old_grad && !other_grads.empty()) {
             // NOTE: Either creates a new item or reassigns
             // The old incremental gradient becomes the new decremental gradient
             //   or the old decremental gradient becomes the new incremental gradient
@@ -509,11 +508,14 @@ recalculateGradientAndUpdateData_(GradPairItr& grad_itr,
                 updateGradVector_(name, other_grads, grad->grad);
             }
             else {
-                for (auto it = other_grads.begin(); it != other_grads.end(); it++) {
-                    if (it->first == name) {
-                        other_grads.erase(it);
-                        deleteGrad_(name, !increase);
-                    }
+                auto new_end = std::remove_if(other_grads.begin(), other_grads.end(), [&name](const GradPair& pair)
+                                                                        {
+                                                                            return (name==pair.first);
+                                                                        }
+                            );
+                if (new_end != other_grads.end()) {
+                    this->deleteGrad_(name, !increase);
+                    other_grads.erase(new_end, other_grads.end());
                 }
             }
         }
@@ -570,6 +572,7 @@ redistributeALQ_(std::vector<GasLiftSingleWell*>& wells,
                  std::vector<GradPair>& inc_grads,
                  std::vector<GradPair>& dec_grads)
 {
+    OPM_TIMEFUNCTION();
     OptimizeState state {*this, group};
     if (this->comm_.size() == 1) {
         // NOTE: 'inc_grads' and 'dec_grads' can never grow larger than wells.size()
@@ -634,13 +637,14 @@ redistributeALQ_(std::vector<GasLiftSingleWell*>& wells,
 template<class Scalar>
 void GasLiftStage2<Scalar>::
 removeSurplusALQ_(const Group& group,
-                  std::vector<GradPair>& inc_grads,
                   std::vector<GradPair>& dec_grads)
 {
+    OPM_TIMEFUNCTION();
     if (dec_grads.empty()) {
         displayDebugMessage_("no wells to remove ALQ from. Skipping");
         return;
     }
+    std::vector<GradPair> empty_vector;
     assert(!dec_grads.empty());
     const auto max_glift = getGroupMaxALQ_(group);
     const auto max_totalgas = getGroupMaxTotalGas_(group);
@@ -660,13 +664,13 @@ removeSurplusALQ_(const Group& group,
             alq, max_glift_str);
         displayDebugMessage_(msg);
     }
-    SurplusState state {*this, group, oil_rate, gas_rate, water_rate, alq,
+    SurplusState state {*this, group, this->well_state_, oil_rate, gas_rate, water_rate, alq,
                         static_cast<Scalar>(min_eco_grad),
                         static_cast<Scalar>(controls.oil_target),
                         static_cast<Scalar>(controls.gas_target),
                         static_cast<Scalar>(controls.water_target),
                         static_cast<Scalar>(controls.liquid_target),
-                        max_glift, max_totalgas };
+                        max_glift, max_totalgas};
 
     while (!stop_iteration) {
         if (dec_grads.size() >= 2) {
@@ -697,14 +701,14 @@ removeSurplusALQ_(const Group& group,
             state.addOrRemoveALQincrement( this->dec_grads_, well_name, /*add=*/false);
             // We pass the current group rate in order to avoid limiting the rates
             // and gaslift based on the current group limits. In other words we want to reduce
-            // the gasslift as much as possible as long as we are able to produce the group
+            // the gaslift as much as possible as long as we are able to produce the group
             // targets
+            // Note: empty_vector passed to avoid adding gradients to inc_grads
             recalculateGradientAndUpdateData_(
-                        dec_grad_itr, group.name(), /*increase=*/false, dec_grads, inc_grads);
+                        dec_grad_itr, group.name(), /*increase=*/false, dec_grads, empty_vector);
 
-            // The dec_grads and inc_grads needs to be syncronized across ranks
+            // The dec_grads needs to be syncronized across ranks
             mpiSyncGlobalGradVector_(dec_grads);
-            mpiSyncGlobalGradVector_(inc_grads);
             // NOTE: recalculateGradientAndUpdateData_() will remove the current gradient
             //   from dec_grads if it cannot calculate a new decremental gradient.
             //   This will invalidate dec_grad_itr and well_name
@@ -798,6 +802,60 @@ updateGradVector_(const std::string& name,
     //   later in getEcoGradients()
 }
 
+
+template<class Scalar>
+void
+GasLiftStage2<Scalar>::
+updateGroupInfo(const std::string& well_name, bool add)
+{
+    const auto delta = computeDelta(well_name, add);
+    const auto& [delta_oil, delta_gas, delta_water, delta_alq] = delta;
+    if (this->group_info_.hasWell(well_name)) {
+        const auto& pairs = this->group_info_.getWellGroups(well_name);
+       for (const auto& [group_name, efficiency] : pairs) {
+            this->group_info_.update(group_name,
+                                     efficiency * delta_oil,
+                                     efficiency * delta_gas,
+                                     efficiency * delta_water,
+                                     efficiency * delta_alq);
+        }
+    }
+}
+
+template<class Scalar>
+std::array<Scalar, 4>
+GasLiftStage2<Scalar>::
+computeDelta(const std::string& well_name, bool add)
+{
+    std::array<Scalar, 4> delta = {0.0, 0.0, 0.0, 0.0};
+    // compute the delta on wells on own rank
+    if (this->well_state_map_.count(well_name) > 0) {
+        const GradInfo& gi = add? this->inc_grads_.at(well_name) : this->dec_grads_.at(well_name);
+        GasLiftWellState<Scalar>& state = *(this->well_state_map_.at(well_name).get());
+        GasLiftSingleWell& gs_well = *(this->stage1_wells_.at(well_name).get());
+        const WellInterfaceGeneric<Scalar>& well = gs_well.getWell();
+        // only get deltas for wells owned by this rank
+        if (this->well_state_.wellIsOwned(well.indexOfWell(), well_name)) {
+            const auto& well_ecl = well.wellEcl();
+            Scalar factor = well_ecl.getEfficiencyFactor() *
+                            this->well_state_[well_name].efficiency_scaling_factor;
+            auto& [delta_oil, delta_gas, delta_water, delta_alq] = delta;
+            delta_oil = factor * (gi.new_oil_rate - state.oilRate());
+            delta_gas = factor * (gi.new_gas_rate - state.gasRate());
+            delta_water = factor * (gi.new_water_rate - state.waterRate());
+            delta_alq = factor * (gi.alq - state.alq());
+        }
+        state.update(gi.new_oil_rate, gi.oil_is_limited,
+                gi.new_gas_rate, gi.gas_is_limited,
+                gi.alq, gi.alq_is_limited,
+                gi.new_water_rate, gi.water_is_limited, add);
+    }
+
+    // and communicate the results
+    this->comm_.sum(delta.data(), delta.size());
+
+    return delta;
+}
 /***********************************************
  * Public methods declared in OptimizeState
  ***********************************************/
@@ -930,6 +988,9 @@ redistributeALQ(GradPairItr& min_dec_grad,
         this->parent.dec_grads_, /*well_name=*/min_dec_grad->first, /*add=*/false);
     this->parent.addOrRemoveALQincrement_(
         this->parent.inc_grads_, /*well_name=*/max_inc_grad->first, /*add=*/true);
+
+    this->parent.updateGroupInfo(min_dec_grad->first, /*add=*/false);
+    this->parent.updateGroupInfo(max_inc_grad->first, /*add=*/true);
 }
 
 /**********************************************
@@ -966,6 +1027,7 @@ addOrRemoveALQincrement(GradMap& grad_map,
         this->parent.displayDebugMessage_(msg);
     }
     this->parent.addOrRemoveALQincrement_(grad_map, well_name, add);
+    this->parent.updateGroupInfo(well_name, add);
 }
 
 template<class Scalar>
@@ -1026,7 +1088,7 @@ checkGasTarget(Scalar delta_gas)
         // the change in gas rate is added to the gas rate to make sure the
         // group still can produce its target
         // i.e. we want to find the solution that optimize gas lift while still
-        // producing the given group limit     
+        // producing the given group limit
         if (this->gas_target < (this->gas_rate + delta_gas) ) {
             if (this->parent.debug) {
                 const std::string msg = fmt::format("group: {} : "
@@ -1122,7 +1184,8 @@ computeDelta(const std::string& well_name)
         // only get deltas for wells owned by this rank
         if (this->parent.well_state_.wellIsOwned(well.indexOfWell(), well_name)) {
             const auto& well_ecl = well.wellEcl();
-            Scalar factor = well_ecl.getEfficiencyFactor();
+            Scalar factor = well_ecl.getEfficiencyFactor() *
+                            this->well_state[well_name].efficiency_scaling_factor;
             auto& [delta_oil, delta_gas, delta_water, delta_alq] = delta;
             delta_oil = factor * (gi.new_oil_rate - state.oilRate());
             delta_gas = factor * (gi.new_gas_rate - state.gasRate());
