@@ -31,9 +31,14 @@
 
 #include <map>
 #include <vector>
+#include <utility>
 
 namespace Opm {
 
+    class GConSump;
+    class Schedule;
+    class SummaryState;
+
 template<class Scalar>
 class GroupState {
 public:
@@ -47,10 +52,14 @@ public:
     bool has_production_rates(const std::string& gname) const;
     void update_production_rates(const std::string& gname,
                                  const std::vector<Scalar>& rates);
+    void update_network_leaf_node_production_rates(const std::string& gname,
+                                 const std::vector<Scalar>& rates);
     const std::vector<Scalar>& production_rates(const std::string& gname) const;
+    const std::vector<Scalar>& network_leaf_node_production_rates(const std::string& gname) const;
 
     void update_well_group_thp(const std::string& gname, const double& thp);
     Scalar well_group_thp(const std::string& gname) const;
+    bool is_autochoke_group(const std::string& gname) const;
 
     bool has_production_reduction_rates(const std::string& gname) const;
     void update_production_reduction_rates(const std::string& gname,
@@ -95,6 +104,10 @@ public:
     void injection_control(const std::string& gname, Phase phase, Group::InjectionCMode cmode);
     Group::InjectionCMode injection_control(const std::string& gname, Phase phase) const;
 
+    void update_gconsump(const Schedule& schedule, const int report_step, const SummaryState& summary_state);
+    const std::pair<Scalar, Scalar>& gconsump_rates(const std::string& gname) const;
+
+
     std::size_t data_size() const;
     std::size_t collect(Scalar* data) const;
     std::size_t distribute(const Scalar* data);
@@ -111,7 +124,7 @@ public:
         // Create a function that calls some function
         // for all the individual data items to simplify
         // the further code.
-        auto iterateContainer = [](auto& container, auto& func) {
+        auto iterateContainer = [](auto& container, const auto& func) {
             for (auto& x : container) {
                 func(x.second);
             }
@@ -120,6 +133,7 @@ public:
 
         auto forAllGroupData = [&](auto& func) {
             iterateContainer(m_production_rates, func);
+            iterateContainer(m_network_leaf_node_production_rates, func);
             iterateContainer(prod_red_rates, func);
             iterateContainer(inj_red_rates, func);
             iterateContainer(inj_resv_rates, func);
@@ -177,6 +191,7 @@ public:
     {
         serializer(num_phases);
         serializer(m_production_rates);
+        serializer(m_network_leaf_node_production_rates);
         serializer(production_controls);
         serializer(group_thp);
         serializer(prod_red_rates);
@@ -189,11 +204,13 @@ public:
         serializer(m_gpmaint_target);
         serializer(injection_controls);
         serializer(gpmaint_state);
+        serializer(m_gconsump_rates);
     }
 
 private:
     std::size_t num_phases{};
     std::map<std::string, std::vector<Scalar>> m_production_rates;
+    std::map<std::string, std::vector<Scalar>> m_network_leaf_node_production_rates;
     std::map<std::string, Group::ProductionCMode> production_controls;
     std::map<std::string, std::vector<Scalar>> prod_red_rates;
     std::map<std::string, std::vector<Scalar>> inj_red_rates;
@@ -207,6 +224,8 @@ private:
 
     std::map<std::pair<Phase, std::string>, Group::InjectionCMode> injection_controls;
     WellContainer<GPMaint::State> gpmaint_state;
+    std::map<std::string, std::pair<Scalar, Scalar>> m_gconsump_rates; // Pair with {consumption_rate, import_rate} for each group
+    static constexpr std::pair<Scalar, Scalar> zero_pair = {0.0, 0.0};
 };
 
 }
