@@ -24,9 +24,11 @@
 #include <opm/simulators/wells/MSWellHelpers.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
+#include <opm/common/TimingMacros.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
 
 #include <opm/input/eclipse/Schedule/MSW/SICD.hpp>
+#include <opm/input/eclipse/Schedule/MSW/Segment.hpp>
 
 #include <opm/material/densead/Evaluation.hpp>
 #include <opm/material/densead/Math.hpp>
@@ -34,36 +36,42 @@
 #include <opm/simulators/utils/DeferredLogger.hpp>
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 
+#include <opm/simulators/wells/ParallelWellInfo.hpp>
+
 #include <dune/istl/preconditioners.hh>
 #include <dune/istl/solvers.hh>
 
-#if HAVE_UMFPACK
+#if HAVE_SUITESPARSE_UMFPACK
 #include <dune/istl/umfpack.hh>
-#endif // HAVE_UMFPACK
+#endif // HAVE_SUITESPARSE_UMFPACK
 
 #include <cmath>
+#include <cstddef>
 
 namespace {
 
-template <typename ValueType>
+template <typename ValueType, typename Scalar>
 ValueType haalandFormular(const ValueType& re,
-                          const double diameter,
-                          const double roughness)
+                          const Scalar diameter,
+                          const Scalar roughness)
 {
-    const ValueType value = -3.6 * log10(6.9 / re + std::pow(roughness / (3.7 * diameter), 10. / 9.) );
-
-    // sqrt(1/f) should be non-positive
-    assert(value >= 0.0);
-
-    return 1. / (value * value);
+    // We now guard for high roughness values (possible singularity) during input handling.
+    // Keepeing a check here in case of unforeseen future usage.
+    assert( re >= 4000. );
+    const Scalar rel_roughness = roughness/diameter;
+    if (rel_roughness > Opm::Segment::MAX_REL_ROUGHNESS) {
+        throw std::invalid_argument("Too large relative roughness in Haaland friction factor calculations.");
+    }
+    const ValueType value = -3.6 * log10(6.9 / re + std::pow(rel_roughness / 3.7, 10. / 9.) );
+    return 1.0 / (value*value);
 }
 
 // water in oil emulsion viscosity
 // TODO: maybe it should be two different ValueTypes. When we calculate the viscosity for transitional zone
-template <typename ValueType>
+template <typename ValueType, typename Scalar>
 ValueType WIOEmulsionViscosity(const ValueType& oil_viscosity,
                                const ValueType& water_liquid_fraction,
-                               const double max_visco_ratio)
+                               const Scalar max_visco_ratio)
 {
     const ValueType temp_value = 1. / (1. - (0.8415 / 0.7480 * water_liquid_fraction) );
     const ValueType viscosity_ratio = pow(temp_value, 2.5);
@@ -76,10 +84,10 @@ ValueType WIOEmulsionViscosity(const ValueType& oil_viscosity,
 }
 
 // oil in water emulsion viscosity
-template <typename ValueType>
+template <typename ValueType, typename Scalar>
 ValueType OIWEmulsionViscosity(const ValueType& water_viscosity,
                                const ValueType& water_liquid_fraction,
-                               const double max_visco_ratio)
+                               const Scalar max_visco_ratio)
 {
     const ValueType temp_value = 1. / (1. - (0.6019 / 0.6410) * (1. - water_liquid_fraction) );
     const ValueType viscosity_ratio = pow(temp_value, 2.5);
@@ -93,10 +101,52 @@ ValueType OIWEmulsionViscosity(const ValueType& water_viscosity,
 
 }
 
-namespace Opm {
+namespace Opm::mswellhelpers {
 
-namespace mswellhelpers
+template<class MatrixType>
+ParallellMSWellB<MatrixType>::
+ParallellMSWellB(const MatrixType& B,
+                 const ParallelWellInfo<Scalar>& parallel_well_info)
+    : B_(B), parallel_well_info_(parallel_well_info)
+{}
+
+template<class MatrixType>
+template<class X, class Y>
+void ParallellMSWellB<MatrixType>::
+mv (const X& x, Y& y) const
 {
+    // x.size() == 0 indicates that there are no active perforations on this process.
+    // Then all contributions come from the communication below.
+    if (x.size() > 0) {
+        B_.mv(x, y);
+    } else {
+        y = 0;
+    }
+
+    if (this->parallel_well_info_.communication().size() > 1)
+    {
+        // Communicate here to get the contributions from all segments
+        this->parallel_well_info_.communication().sum(y.data(), y.size());
+    }
+}
+
+template<class MatrixType>
+template<class X, class Y>
+void ParallellMSWellB<MatrixType>::
+mmv (const X& x, Y& y) const
+{
+    if (this->parallel_well_info_.communication().size() == 1)
+    {
+        // Do the same thing as before. The else branch
+        // produces different rounding errors and results
+        // slightly different iteration counts / well curves
+        B_.mmv(x, y);
+    } else {
+        Y temp(y.size(), 0);
+        mv(x, temp); // includes parallel reduction
+        y -= temp;
+    }
+}
 
     /// Applies umfpack and checks for singularity
 template <typename MatrixType, typename VectorType>
@@ -104,25 +154,28 @@ VectorType
 applyUMFPack(Dune::UMFPack<MatrixType>& linsolver,
              VectorType x)
 {
-#if HAVE_UMFPACK
+#if HAVE_SUITESPARSE_UMFPACK
     // The copy of x seems mandatory for calling UMFPack!
     VectorType y(x.size());
     y = 0.;
 
     // Object storing some statistics about the solving process
     Dune::InverseOperatorResult res;
+    if constexpr (std::is_same_v<typename VectorType::field_type,float>) {
+        OPM_THROW(std::runtime_error, "Cannot use applyUMFPack() with floats.");
+    } else {
+        // Solve
+        linsolver.apply(y, x, res);
 
-    // Solve
-    linsolver.apply(y, x, res);
-
-    // Checking if there is any inf or nan in y
-    // it will be the solution before we find a way to catch the singularity of the matrix
-    for (size_t i_block = 0; i_block < y.size(); ++i_block) {
-        for (size_t i_elem = 0; i_elem < y[i_block].size(); ++i_elem) {
-            if (std::isinf(y[i_block][i_elem]) || std::isnan(y[i_block][i_elem]) ) {
-                const std::string msg{"nan or inf value found after UMFPack solve due to singular matrix"};
-                OpmLog::debug(msg);
-                OPM_THROW_NOLOG(NumericalProblem, msg);
+        // Checking if there is any inf or nan in y
+        // it will be the solution before we find a way to catch the singularity of the matrix
+        for (std::size_t i_block = 0; i_block < y.size(); ++i_block) {
+            for (std::size_t i_elem = 0; i_elem < y[i_block].size(); ++i_elem) {
+                if (std::isinf(y[i_block][i_elem]) || std::isnan(y[i_block][i_elem]) ) {
+                    const std::string msg{"nan or inf value found after UMFPack solve due to singular matrix"};
+                    OpmLog::debug(msg);
+                    OPM_THROW_NOLOG(NumericalProblem, msg);
+                }
             }
         }
     }
@@ -131,7 +184,7 @@ applyUMFPack(Dune::UMFPack<MatrixType>& linsolver,
     // this is not thread safe
     OPM_THROW(std::runtime_error, "Cannot use applyUMFPack() without UMFPACK. "
               "Reconfigure opm-simulators with SuiteSparse/UMFPACK support and recompile.");
-#endif // HAVE_UMFPACK
+#endif // HAVE_SUITESPARSE_UMFPACK
 }
 
 template <typename VectorType, typename MatrixType>
@@ -140,24 +193,28 @@ invertWithUMFPack(const int size,
                   const int bsize,
                   Dune::UMFPack<MatrixType>& linsolver)
 {
-#if HAVE_UMFPACK
+#if HAVE_SUITESPARSE_UMFPACK
     VectorType e(size);
     e = 0.0;
 
     // Make a full block matrix.
     Dune::Matrix<typename MatrixType::block_type> inv(size, size);
 
-    // Create inverse by passing basis vectors to the solver.
-    for (int ii = 0; ii < size; ++ii) {
-        for (int jj = 0; jj < bsize; ++jj) {
-            e[ii][jj] = 1.0;
-            auto col = applyUMFPack(linsolver, e);
-            for (int cc = 0; cc < size; ++cc) {
-                for (int dd = 0; dd < bsize; ++dd) {
-                    inv[cc][ii][dd][jj] = col[cc][dd];
+    if constexpr (std::is_same_v<typename VectorType::field_type,float>) {
+        OPM_THROW(std::runtime_error, "Cannot use invertWithUMFPack() with floats.");
+    } else {
+        // Create inverse by passing basis vectors to the solver.
+        for (int ii = 0; ii < size; ++ii) {
+            for (int jj = 0; jj < bsize; ++jj) {
+                e[ii][jj] = 1.0;
+                auto col = applyUMFPack(linsolver, e);
+                for (int cc = 0; cc < size; ++cc) {
+                    for (int dd = 0; dd < bsize; ++dd) {
+                        inv[cc][ii][dd][jj] = col[cc][dd];
+                    }
                 }
+                e[ii][jj] = 0.0;
             }
-            e[ii][jj] = 0.0;
         }
     }
 
@@ -166,7 +223,7 @@ invertWithUMFPack(const int size,
     // this is not thread safe
     OPM_THROW(std::runtime_error, "Cannot use invertWithUMFPack() without UMFPACK. "
               "Reconfigure opm-simulators with SuiteSparse/UMFPACK support and recompile.");
-#endif // HAVE_UMFPACK
+#endif // HAVE_SUITESPARSE_UMFPACK
 }
 
 template <typename MatrixType, typename VectorType>
@@ -204,23 +261,23 @@ invDX(const MatrixType& D, VectorType x, DeferredLogger& deferred_logger)
     linsolver.apply(y, x, res);
 
     if ( !res.converged ) {
-        OPM_DEFLOG_THROW(NumericalProblem, "the invDX did not converge ", deferred_logger);
+        OPM_DEFLOG_PROBLEM(NumericalProblem, "the invDX did not converge ", deferred_logger);
     }
 
     return y;
 }
 
-template <typename ValueType>
-ValueType frictionPressureLoss(const double l, const double diameter,
-                               const double area, const double roughness,
+template <typename ValueType, typename Scalar>
+ValueType frictionPressureLoss(const Scalar l, const Scalar diameter,
+                               const Scalar area, const Scalar roughness,
                                const ValueType& density,
                                const ValueType& w, const ValueType& mu)
 {
     // Reynolds number
     const ValueType re = abs( diameter * w / (area * mu));
 
-    constexpr double re_value1 = 2000.;
-    constexpr double re_value2 = 4000.;
+    constexpr Scalar re_value1 = 2000.;
+    constexpr Scalar re_value2 = 4000.;
 
     if (re < re_value1) {
         // not using the formula directly because of the division with very small w
@@ -232,7 +289,7 @@ ValueType frictionPressureLoss(const double l, const double diameter,
     if (re > re_value2) {
         f = haalandFormular(re, diameter, roughness);
     } else { // in between
-        constexpr double f1 = 16. / re_value1;
+        constexpr Scalar f1 = 16. / re_value1;
         const ValueType f2 = haalandFormular(re_value2, diameter, roughness);
         f = (f2 - f1) / (re_value2 - re_value1) * (re - re_value1) + f1;
     }
@@ -240,19 +297,19 @@ ValueType frictionPressureLoss(const double l, const double diameter,
     return 2. * f * l * w * w / (area * area * diameter * density);
 }
 
-template <typename ValueType>
+template <typename ValueType, typename Scalar>
 ValueType valveContrictionPressureLoss(const ValueType& mass_rate,
                                        const ValueType& density,
-                                       const double area_con, const double cv)
+                                       const Scalar area_con, const Scalar cv)
 {
     // the formulation is adjusted a little bit for convinience
     // velocity = mass_rate / (density * area) is applied to the original formulation
-    const double area = (area_con > 1.e-10 ? area_con : 1.e-10);
+    const Scalar area = (area_con > 1.e-10 ? area_con : 1.e-10);
     return mass_rate * mass_rate / (2. * density * cv * cv * area * area);
 }
 
-template <typename ValueType>
-ValueType velocityHead(const double area, const ValueType& mass_rate,
+template <typename ValueType, typename Scalar>
+ValueType velocityHead(const Scalar area, const ValueType& mass_rate,
                        const ValueType& density)
 {
     // \Note: a factor of 2 is added to the formulation in order to match results from the
@@ -260,21 +317,21 @@ ValueType velocityHead(const double area, const ValueType& mass_rate,
     return (mass_rate * mass_rate / (area * area * density));
 }
 
-template <typename ValueType>
+template <typename ValueType, typename Scalar>
 ValueType emulsionViscosity(const ValueType& water_fraction,
                             const ValueType& water_viscosity,
                             const ValueType& oil_fraction,
                             const ValueType& oil_viscosity,
                             const SICD& sicd)
 {
-    const double width_transition = sicd.widthTransitionRegion();
+    const Scalar width_transition = sicd.widthTransitionRegion();
 
     // it is just for now, we should be able to treat it.
     if (width_transition <= 0.) {
         OPM_THROW(std::runtime_error, "Not handling non-positive transition width now");
     }
 
-    const double critical_value = sicd.criticalValue();
+    const Scalar critical_value = sicd.criticalValue();
     const ValueType transition_start_value = critical_value - width_transition / 2.0;
     const ValueType transition_end_value = critical_value + width_transition / 2.0;
 
@@ -286,7 +343,7 @@ ValueType emulsionViscosity(const ValueType& water_fraction,
 
     const ValueType water_liquid_fraction = water_fraction / liquid_fraction;
 
-    const double max_visco_ratio = sicd.maxViscosityRatio();
+    const Scalar max_visco_ratio = sicd.maxViscosityRatio();
     if (water_liquid_fraction <= transition_start_value) {
         return WIOEmulsionViscosity(oil_viscosity, water_liquid_fraction, max_visco_ratio);
     } else if (water_liquid_fraction >= transition_end_value) {
@@ -300,54 +357,76 @@ ValueType emulsionViscosity(const ValueType& water_fraction,
     }
 }
 
-template<int Dim>
-using Vec = Dune::BlockVector<Dune::FieldVector<double,Dim>>;
-template<int Dim>
-using Mat = Dune::BCRSMatrix<Dune::FieldMatrix<double,Dim,Dim>>;
+template<class Scalar, int Dim>
+using Vec = Dune::BlockVector<Dune::FieldVector<Scalar,Dim>>;
+template<class Scalar, int M, int N = M>
+using Mat = Dune::BCRSMatrix<Dune::FieldMatrix<Scalar,M,N>>;
 
-#define INSTANCE_UMF(Dim) \
-    template Vec<Dim> applyUMFPack<Mat<Dim>,Vec<Dim>>(Dune::UMFPack<Mat<Dim>>&, \
-                                                      Vec<Dim>); \
-    template Dune::Matrix<typename Mat<Dim>::block_type> \
-    invertWithUMFPack<Vec<Dim>,Mat<Dim>>(const int, const int, Dune::UMFPack<Mat<Dim>>&);
+#define INSTANTIATE_PARALLELLMSWELLB(T, M, N)                                                                                                                                                                                                                                      \
+    template class ParallellMSWellB<Mat<T,M,N>>;                                                                                                                                                                                                 \
+    template void ParallellMSWellB<Mat<T,M,N>>::mv(Vec<T,N> const&,Vec<T,M>& ) const; \
+    template void ParallellMSWellB<Mat<T,M,N>>::mmv(Vec<T,N> const&,Vec<T,M>& ) const;
 
-INSTANCE_UMF(2)
-INSTANCE_UMF(3)
-INSTANCE_UMF(4)
+#define INSTANTIATE_ALL_PARALLELLMSWELLB(T) \
+    INSTANTIATE_PARALLELLMSWELLB(T, 2, 1)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 2, 2)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 2, 6)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 3, 2)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 3, 3)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 3, 4)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 4, 3)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 4, 4)   \
+    INSTANTIATE_PARALLELLMSWELLB(T, 4, 5)
 
-#define INSTANCE_IMPL(...) \
-    template __VA_ARGS__ \
-    frictionPressureLoss<__VA_ARGS__>(const double, \
-                                      const double, \
-                                      const double, \
-                                      const double, \
-                                      const __VA_ARGS__&, \
-                                      const __VA_ARGS__&, \
-                                      const __VA_ARGS__&); \
-    template  __VA_ARGS__ \
-    valveContrictionPressureLoss<__VA_ARGS__>(const __VA_ARGS__& mass_rate, \
-                                              const __VA_ARGS__& density, \
-                                              const double, const double); \
-    template __VA_ARGS__ \
-    velocityHead<__VA_ARGS__>(const double, const __VA_ARGS__&, const __VA_ARGS__&); \
-    template __VA_ARGS__ \
-    emulsionViscosity<__VA_ARGS__>(const __VA_ARGS__&,  \
-                                   const __VA_ARGS__&, \
-                                   const __VA_ARGS__&, \
-                                   const __VA_ARGS__&, \
-                                   const SICD&);
+#define INSTANTIATE_UMF(T,Dim)                                             \
+    template Vec<T,Dim> applyUMFPack(Dune::UMFPack<Mat<T,Dim>>&,           \
+                                     Vec<T,Dim>);                          \
+    template Dune::Matrix<typename Mat<T,Dim>::block_type>                 \
+    invertWithUMFPack<Vec<T,Dim>,Mat<T,Dim>>(const int, const int,         \
+                                             Dune::UMFPack<Mat<T,Dim>>&);
 
-#define INSTANCE_EVAL(Dim) \
-    INSTANCE_IMPL(DenseAd::Evaluation<double,Dim>)
+#define INSTANTIATE_IMPL(T,...)                                    \
+    template __VA_ARGS__                                           \
+    frictionPressureLoss(const T,                                  \
+                         const T,                                  \
+                         const T,                                  \
+                         const T,                                  \
+                         const __VA_ARGS__&,                       \
+                         const __VA_ARGS__&,                       \
+                         const __VA_ARGS__&);                      \
+    template  __VA_ARGS__                                          \
+    valveContrictionPressureLoss(const __VA_ARGS__& mass_rate,     \
+                                 const __VA_ARGS__& density,       \
+                                 const T, const T);                \
+    template __VA_ARGS__                                           \
+    velocityHead(const T, const __VA_ARGS__&, const __VA_ARGS__&); \
+    template __VA_ARGS__                                           \
+    emulsionViscosity<__VA_ARGS__,T>(const __VA_ARGS__&,           \
+                                     const __VA_ARGS__&,           \
+                                     const __VA_ARGS__&,           \
+                                     const __VA_ARGS__&,           \
+                                     const SICD&);
 
-INSTANCE_EVAL(3)
-INSTANCE_EVAL(4)
-INSTANCE_EVAL(5)
-INSTANCE_EVAL(6)
-INSTANCE_EVAL(7)
-INSTANCE_EVAL(8)
-INSTANCE_EVAL(9)
+#define INSTANTIATE_EVAL(T,Dim) \
+    INSTANTIATE_IMPL(T, DenseAd::Evaluation<T,Dim>)
 
-} // namespace mswellhelpers
+#define INSTANTIATE_TYPE(T) \
+    INSTANTIATE_UMF(T,2)    \
+    INSTANTIATE_UMF(T,3)    \
+    INSTANTIATE_UMF(T,4)    \
+    INSTANTIATE_EVAL(T,3)   \
+    INSTANTIATE_EVAL(T,4)   \
+    INSTANTIATE_EVAL(T,5)   \
+    INSTANTIATE_EVAL(T,6)   \
+    INSTANTIATE_EVAL(T,7)   \
+    INSTANTIATE_EVAL(T,8)   \
+    INSTANTIATE_EVAL(T,9)   \
+    INSTANTIATE_ALL_PARALLELLMSWELLB(T)
 
-} // namespace Opm
+INSTANTIATE_TYPE(double)
+
+#if FLOW_INSTANTIATE_FLOAT
+INSTANTIATE_TYPE(float)
+#endif
+
+} // namespace Opm::mswellhelpers
