@@ -22,6 +22,8 @@
 #ifndef OPM_MULTISEGMENTWELL_HEADER_INCLUDED
 #define OPM_MULTISEGMENTWELL_HEADER_INCLUDED
 
+#include <opm/models/common/multiphasebaseproperties.hh>
+
 #include <opm/simulators/wells/WellInterface.hpp>
 #include <opm/simulators/wells/MultisegmentWellEval.hpp>
 
@@ -84,8 +86,6 @@ namespace Opm {
                   const std::vector<Scalar>& B_avg,
                   const bool changed_to_open_this_step) override;
 
-        void initPrimaryVariablesEvaluation() override;
-
         /// updating the well state based the current control mode
         void updateWellStateWithTarget(const Simulator& simulator,
                                        const GroupState<Scalar>& group_state,
@@ -157,7 +157,8 @@ namespace Opm {
         computeBhpAtThpLimitProdWithAlq(const Simulator& simulator,
                                         const SummaryState& summary_state,
                                         const Scalar alq_value,
-                                        DeferredLogger& deferred_logger) const override;
+                                        DeferredLogger& deferred_logger,
+                                        bool iterate_if_no_solution) const override;
 
         std::vector<Scalar> getPrimaryVars() const override;
 
@@ -223,7 +224,7 @@ namespace Opm {
         // get the mobility for specific perforation
         template<class Value>
         void getMobility(const Simulator& simulator,
-                         const int perf,
+                         const int local_perf_index,
                          std::vector<Value>& mob,
                          DeferredLogger& deferred_logger) const;
 
@@ -247,6 +248,7 @@ namespace Opm {
                                     DeferredLogger& deferred_logger) const;
 
         bool computeWellPotentialsImplicit(const Simulator& simulator,
+                                           const WellState<Scalar>& well_state,
                                            std::vector<Scalar>& well_potentials,
                                            DeferredLogger& deferred_logger) const;
 
@@ -323,8 +325,6 @@ namespace Opm {
 
 }
 
-#ifndef OPM_MULTISEGMENTWELL_IMPL_HEADER_INCLUDED
 #include "MultisegmentWell_impl.hpp"
-#endif
 
 #endif // OPM_MULTISEGMENTWELL_HEADER_INCLUDED
