@@ -46,7 +46,7 @@ template<class Scalar, int numWellEq, int numEq>
 class MultisegmentWellEquationAccess {
 public:
     //! \brief Constructor initializes reference to the equation system.
-    MultisegmentWellEquationAccess(MultisegmentWellEquations<Scalar,numWellEq,numEq>& eqns)
+    explicit MultisegmentWellEquationAccess(MultisegmentWellEquations<Scalar,numWellEq,numEq>& eqns)
         : eqns_(eqns)
     {}
 
@@ -96,6 +96,10 @@ assembleControlEq(const WellState<Scalar>& well_state,
                   const bool stopped_or_zero_target,
                   DeferredLogger& deferred_logger) const
 {
+    /*
+        This function assembles the control equation, similar as for StandardWells.
+        It does *not* need communication.
+    */
     static constexpr int Gas = BlackoilPhases::Vapour;
     static constexpr int Oil = BlackoilPhases::Liquid;
     static constexpr int Water = BlackoilPhases::Aqua;
@@ -212,6 +216,12 @@ assembleAccelerationTerm(const int seg_target,
     // acceleration term shold be
     //  * velocity head for seg_target if seg = seg_target
     //  * negative velocity head for seg if seg != seg_target   
+    
+    /*
+        This method is called in MultisegmentWellEval::assembleAccelerationPressureLoss.
+        It does *not* need communication.
+    */
+
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg_target][SPres] -= accelerationTerm.value();
     eqns.D()[seg_target][seg][SPres][SPres] -= accelerationTerm.derivative(SPres + Indices::numEq);
@@ -231,6 +241,11 @@ assembleHydroPressureLoss(const int seg,
                           const EvalWell& hydro_pressure_drop_seg,
                           Equations& eqns1) const
 {
+    /*
+        This method is called in MultisegmentWellEval::assembleAccelerationAndHydroPressureLosses.
+        It does *not* need communication.
+    */
+
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][SPres] -= hydro_pressure_drop_seg.value();
     for (int pv_idx = 0; pv_idx < numWellEq; ++pv_idx) {
@@ -246,6 +261,9 @@ assemblePressureEqExtraDerivatives(const int seg,
                                    const EvalWell& extra_derivatives,
                                    Equations& eqns1) const
 {
+    /*
+        This method does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     // disregard residual
     // Frac - derivatives are zero (they belong to upwind^2)
@@ -261,18 +279,19 @@ assemblePressureEq(const int seg,
                    const int outlet_segment_index,
                    const EvalWell& pressure_equation,
                    const EvalWell& outlet_pressure,
-                   Equations& eqns1,
-                   bool wfrac,
-                   bool gfrac) const
+                   Equations& eqns1) const
 {
+    /*
+        This method does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][SPres] += pressure_equation.value();
     eqns.D()[seg][seg][SPres][SPres] += pressure_equation.derivative(SPres + Indices::numEq);
     eqns.D()[seg][seg][SPres][WQTotal] += pressure_equation.derivative(WQTotal + Indices::numEq);
-    if (wfrac) {
+    if constexpr (has_wfrac_variable) {
         eqns.D()[seg][seg_upwind][SPres][WFrac] += pressure_equation.derivative(WFrac + Indices::numEq);
     }
-    if (gfrac) {
+    if constexpr (has_gfrac_variable) {
         eqns.D()[seg][seg_upwind][SPres][GFrac] += pressure_equation.derivative(GFrac + Indices::numEq);
     }
 
@@ -289,6 +308,13 @@ assembleTrivialEq(const int seg,
                   const Scalar value,
                   Equations& eqns1) const
 {
+    /*
+        This method is called from MultisegmentWellEval::assembleICDPressureEq,
+        which is called from MultisegmentWellEval::assemblePressureEq.
+        This is the counterpart to assembleControlEquation, where assembleControlEquation is responsible for the top segment
+        and assembleICDPressureEq is responsible for the remaining segments.
+        This method does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][SPres] = value;
     eqns.D()[seg][seg][SPres][WQTotal] = 1.;
@@ -301,6 +327,10 @@ assembleAccumulationTerm(const int seg,
                          const EvalWell& accumulation_term,
                          Equations& eqns1) const
 {
+    /*
+        This method is called from MultisegmentWell::assembleWellEqWithoutIteration.
+        It only assembles on the diagonal of D and it does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][comp_idx] += accumulation_term.value();
     for (int pv_idx = 0; pv_idx < numWellEq; ++pv_idx) {
@@ -316,13 +346,17 @@ assembleOutflowTerm(const int seg,
                     const EvalWell& segment_rate,
                     Equations& eqns1) const
 {
+    /*
+        This method is called from MultisegmentWell::assembleWellEqWithoutIteration.
+        It does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][comp_idx] -= segment_rate.value();
     eqns.D()[seg][seg][comp_idx][WQTotal] -= segment_rate.derivative(WQTotal + Indices::numEq);
-    if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+    if constexpr (has_wfrac_variable) {
         eqns.D()[seg][seg_upwind][comp_idx][WFrac] -= segment_rate.derivative(WFrac + Indices::numEq);
     }
-    if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+    if constexpr (has_gfrac_variable) {
         eqns.D()[seg][seg_upwind][comp_idx][GFrac] -= segment_rate.derivative(GFrac + Indices::numEq);
     }
     // pressure derivative should be zero
@@ -337,13 +371,17 @@ assembleInflowTerm(const int seg,
                    const EvalWell& inlet_rate,
                    Equations& eqns1) const
  {
+    /*
+        This method is called from MultisegmentWell::assembleWellEqWithoutIteration.
+        It does *not* need communication.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     eqns.residual()[seg][comp_idx] += inlet_rate.value();
     eqns.D()[seg][inlet][comp_idx][WQTotal] += inlet_rate.derivative(WQTotal + Indices::numEq);
-    if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+    if constexpr (has_wfrac_variable) {
         eqns.D()[seg][inlet_upwind][comp_idx][WFrac] += inlet_rate.derivative(WFrac + Indices::numEq);
     }
-    if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+    if constexpr (has_gfrac_variable) {
         eqns.D()[seg][inlet_upwind][comp_idx][GFrac] += inlet_rate.derivative(GFrac + Indices::numEq);
     }
     // pressure derivative should be zero
@@ -352,11 +390,17 @@ assembleInflowTerm(const int seg,
 template<class FluidSystem, class Indices>
 void MultisegmentWellAssemble<FluidSystem,Indices>::
 assemblePerforationEq(const int seg,
-                      const int cell_idx,
+                      const int local_perf_index,
                       const int comp_idx,
                       const EvalWell& cq_s_effective,
                       Equations& eqns1) const
 {
+    /*
+        This method is called from MultisegmentWell::assembleWellEqWithoutIteration.
+        It *does* need communication, i.e. this method only assembles the parts of the matrix this process is responsible for
+        and after calling this function, the diagonal of the matrix D and the residual need to be combined by calling
+        the function MultisegmentWellEquations::sumDistributed.
+    */
     MultisegmentWellEquationAccess<Scalar,numWellEq,Indices::numEq> eqns(eqns1);
     // subtract sum of phase fluxes in the well equations.
     eqns.residual()[seg][comp_idx] += cq_s_effective.value();
@@ -364,7 +408,7 @@ assemblePerforationEq(const int seg,
     // assemble the jacobians
     for (int pv_idx = 0; pv_idx < numWellEq; ++pv_idx) {
         // also need to consider the efficiency factor when manipulating the jacobians.
-        eqns.C()[seg][cell_idx][pv_idx][comp_idx] -= cq_s_effective.derivative(pv_idx + Indices::numEq); // input in transformed matrix
+        eqns.C()[seg][local_perf_index][pv_idx][comp_idx] -= cq_s_effective.derivative(pv_idx + Indices::numEq); // input in transformed matrix
 
         // the index name for the D should be eq_idx / pv_idx
         eqns.D()[seg][seg][comp_idx][pv_idx] += cq_s_effective.derivative(pv_idx + Indices::numEq);
@@ -372,7 +416,7 @@ assemblePerforationEq(const int seg,
 
     for (int pv_idx = 0; pv_idx < Indices::numEq; ++pv_idx) {
         // also need to consider the efficiency factor when manipulating the jacobians.
-        eqns.B()[seg][cell_idx][comp_idx][pv_idx] += cq_s_effective.derivative(pv_idx);
+        eqns.B()[seg][local_perf_index][comp_idx][pv_idx] += cq_s_effective.derivative(pv_idx);
     }
 }
 
