@@ -29,15 +29,15 @@
 
 #include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 
-#if COMPILE_BDA_BRIDGE
-#include <opm/simulators/linalg/bda/WellContributions.hpp>
+#if COMPILE_GPU_BRIDGE
+#include <opm/simulators/linalg/gpubridge/WellContributions.hpp>
 #endif
 
 #include <opm/simulators/linalg/istlsparsematrixadapter.hh>
 #include <opm/simulators/linalg/matrixblock.hh>
 #include <opm/simulators/linalg/SmallDenseMatrixUtils.hpp>
 
-#include <opm/simulators/wells/MSWellHelpers.hpp>
+#include <opm/simulators/wells/WellHelpers.hpp>
 #include <opm/simulators/wells/MultisegmentWellGeneric.hpp>
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
 
@@ -48,8 +48,10 @@ namespace Opm {
 
 template<class Scalar, int numWellEq, int numEq>
 MultisegmentWellEquations<Scalar,numWellEq,numEq>::
-MultisegmentWellEquations(const MultisegmentWellGeneric<Scalar>& well)
+MultisegmentWellEquations(const MultisegmentWellGeneric<Scalar>& well, const ParallelWellInfo<Scalar>& pw_info)
     : well_(well)
+    , pw_info_(pw_info)
+    , parallelB_(duneB_, pw_info)
 {
 }
 
@@ -107,7 +109,10 @@ init(const int numPerfs,
               end = duneC_.createend(); row != end; ++row) {
         // the number of the row corresponds to the segment number now.
         for (const int& perf : perforations[row.index()]) {
-            row.insert(perf);
+            const int local_perf_index = pw_info_.activeToLocal(perf);
+            if (local_perf_index < 0) // then the perforation is not on this process
+                continue;
+            row.insert(local_perf_index);
         }
     }
 
@@ -116,7 +121,10 @@ init(const int numPerfs,
               end = duneB_.createend(); row != end; ++row) {
         // the number of the row corresponds to the segment number now.
         for (const int& perf : perforations[row.index()]) {
-            row.insert(perf);
+            const int local_perf_index = pw_info_.activeToLocal(perf);
+            if (local_perf_index < 0) // then the perforation is not on this process
+                continue;
+            row.insert(local_perf_index);
         }
     }
 
@@ -142,29 +150,42 @@ apply(const BVector& x, BVector& Ax) const
 {
     BVectorWell Bx(duneB_.N());
 
-    duneB_.mv(x, Bx);
+    parallelB_.mv(x, Bx);
 
+    // It is ok to do this on each process instead of only on one,
+    // because the other processes would remain idle while waiting for
+    // the single process to complete the computation.
     // invDBx = duneD^-1 * Bx_
     const BVectorWell invDBx = mswellhelpers::applyUMFPack(*duneDSolver_, Bx);
-
-    // Ax = Ax - duneC_^T * invDBx
-    duneC_.mmtv(invDBx,Ax);
+    // Ax.size() == 0 indicates that there are no active perforations on this process.
+    // Then, Ax does not need to be updated by the following calculation.
+    if (Ax.size() > 0) {
+        // Ax = Ax - duneC_^T * invDBx
+        duneC_.mmtv(invDBx,Ax);
+    }
 }
 
 template<class Scalar, int numWellEq, int numEq>
 void MultisegmentWellEquations<Scalar,numWellEq,numEq>::
 apply(BVector& r) const
 {
-    // invDrw_ = duneD^-1 * resWell_
-    const BVectorWell invDrw = mswellhelpers::applyUMFPack(*duneDSolver_, resWell_);
-    // r = r - duneC_^T * invDrw
-    duneC_.mmtv(invDrw, r);
+    // r.size() == 0 indicates that there are no active perforations on this process.
+    // Then, r does not need to be updated by the following calculation.
+    if (r.size() > 0) {
+        // It is ok to do this on each process instead of only on one,
+        // because the other processes would remain idle while waiting for
+        // the single process to complete the computation.
+        // invDrw_ = duneD^-1 * resWell_
+        const BVectorWell invDrw = mswellhelpers::applyUMFPack(*duneDSolver_, resWell_);
+        // r = r - duneC_^T * invDrw
+        duneC_.mmtv(invDrw, r);
+    }
 }
 
 template<class Scalar, int numWellEq, int numEq>
 void MultisegmentWellEquations<Scalar,numWellEq,numEq>::createSolver()
 {
-#if HAVE_UMFPACK
+#if HAVE_SUITESPARSE_UMFPACK
     if (duneDSolver_) {
         return;
     }
@@ -185,6 +206,9 @@ template<class Scalar, int numWellEq, int numEq>
 typename MultisegmentWellEquations<Scalar,numWellEq,numEq>::BVectorWell
 MultisegmentWellEquations<Scalar,numWellEq,numEq>::solve() const
 {
+    // It is ok to do this on each process instead of only on one,
+    // because the other processes would remain idle while waiting for
+    // the single process to complete the computation.
     return mswellhelpers::applyUMFPack(*duneDSolver_, resWell_);
 }
 
@@ -192,6 +216,9 @@ template<class Scalar, int numWellEq, int numEq>
 typename MultisegmentWellEquations<Scalar,numWellEq,numEq>::BVectorWell
 MultisegmentWellEquations<Scalar,numWellEq,numEq>::solve(const BVectorWell& rhs) const
 {
+    // It is ok to do this on each process instead of only on one,
+    // because the other processes would remain idle while waiting for
+    // the single process to complete the computation.
     return mswellhelpers::applyUMFPack(*duneDSolver_, rhs);
 }
 
@@ -201,12 +228,16 @@ recoverSolutionWell(const BVector& x, BVectorWell& xw) const
 {
     BVectorWell resWell = resWell_;
     // resWell = resWell - B * x
-    duneB_.mmv(x, resWell);
+    parallelB_.mmv(x, resWell);
+
     // xw = D^-1 * resWell
+    // It is ok to do this on each process instead of only on one,
+    // because the other processes would remain idle while waiting for
+    // the single process to complete the computation.
     xw = mswellhelpers::applyUMFPack(*duneDSolver_, resWell);
 }
 
-#if COMPILE_BDA_BRIDGE
+#if COMPILE_GPU_BRIDGE
 template<class Scalar, int numWellEq, int numEq>
 void MultisegmentWellEquations<Scalar,numWellEq,numEq>::
 extract(WellContributions<Scalar>& wellContribs) const
@@ -222,7 +253,7 @@ extract(WellContributions<Scalar>& wellContribs) const
     Cvals.reserve(BnumBlocks * numEq * numWellEq);
     for (auto rowC = duneC_.begin(); rowC != duneC_.end(); ++rowC) {
         for (auto colC = rowC->begin(), endC = rowC->end(); colC != endC; ++colC) {
-            Ccols.emplace_back(colC.index());
+            Ccols.emplace_back(cells_[colC.index()]);
             for (int i = 0; i < numWellEq; ++i) {
                 for (int j = 0; j < numEq; ++j) {
                     Cvals.emplace_back((*colC)[i][j]);
@@ -252,7 +283,7 @@ extract(WellContributions<Scalar>& wellContribs) const
         for (auto rowB = duneB_.begin(); rowB != duneB_.end(); ++rowB) {
             int sizeRow = 0;
             for (auto colB = rowB->begin(), endB = rowB->end(); colB != endB; ++colB) {
-                Bcols.emplace_back(colB.index());
+                Bcols.emplace_back(cells_[colB.index()]);
                 for (int i = 0; i < numWellEq; ++i) {
                     for (int j = 0; j < numEq; ++j) {
                         Bvals.emplace_back((*colB)[i][j]);
@@ -402,6 +433,16 @@ extractCPRPressureMatrix(PressureMatrix& jacobian,
     }
 }
 
+template<class Scalar, int numWellEq, int numEq>
+void MultisegmentWellEquations<Scalar,numWellEq,numEq>::
+sumDistributed(Parallel::Communication comm)
+{
+    // accumulate resWell_ and duneD_ in parallel to get effects of all perforations (might be distributed)
+    // we need to do this for all segments in the residual and on the diagonal of D 
+    for (int seg = 0; seg < well_.numberOfSegments(); ++seg)
+        Opm::wellhelpers::sumDistributedWellEntries(duneD_[seg][seg], resWell_[seg], comm);
+}
+
 #define INSTANTIATE(T, numWellEq, numEq)                                                       \
     template class MultisegmentWellEquations<T,numWellEq,numEq>;                               \
     template void MultisegmentWellEquations<T,numWellEq,numEq>::                               \
