@@ -53,12 +53,13 @@ namespace Opm
 
 template<typename FluidSystem, typename Indices>
 MultisegmentWellEval<FluidSystem,Indices>::
-MultisegmentWellEval(WellInterfaceIndices<FluidSystem,Indices>& baseif)
+MultisegmentWellEval(WellInterfaceIndices<FluidSystem,Indices>& baseif, const ParallelWellInfo<Scalar>& pw_info)
     : MultisegmentWellGeneric<Scalar>(baseif)
+    , pw_info_(pw_info)
     , baseif_(baseif)
-    , linSys_(*this)
+    , linSys_(*this, pw_info)
     , primary_variables_(baseif)
-    , segments_(this->numberOfSegments(), baseif)
+    , segments_(this->numberOfSegments(), pw_info, baseif)
     , cell_perforation_depth_diffs_(baseif_.numPerfs(), 0.0)
     , cell_perforation_pressure_diffs_(baseif_.numPerfs(), 0.0)
 {
@@ -367,9 +368,7 @@ assembleICDPressureEq(const int seg,
     MultisegmentWellAssemble(baseif_).
         assemblePressureEq(seg, seg_upwind, outlet_segment_index,
                            pressure_equation, outlet_pressure,
-                           linSys_,
-                           FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx),
-                           FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx));
+                           linSys_);
 
     assembleAccelerationAndHydroPressureLosses(seg, well_state, use_average_density);
 }
