@@ -57,6 +57,19 @@ scaleSegmentRatesWithWellRates(const std::vector<std::vector<int>>& segment_inle
     auto& ws = well_state.well(baseif_.indexOfWell());
     auto& segments = ws.segments;
     auto& segment_rates = segments.rates;
+    Scalar sumTw = 0;
+    bool calculateSumTw = std::any_of(segment_rates.begin(), segment_rates.end(), [](const auto& unscaled_top_seg_rate) {
+        return std::abs(unscaled_top_seg_rate) <= 1e-12;
+    });
+    if (calculateSumTw) {
+        // Due to various reasons, the well/top segment rate can be zero for this phase.
+        // We can not scale this rate directly. The following approach is used to initialize the segment rates.
+        for (int perf = 0; perf < baseif_.numPerfs(); ++perf) {
+            sumTw += baseif_.wellIndex()[perf];
+        }
+        // We need to communicate here to scale the perf_phaserate_scaled with the contribution of all segments
+        sumTw = ws.parallel_info.get().communication().sum(sumTw);
+    }
     for (int phase = 0; phase < baseif_.numPhases(); ++phase) {
         const Scalar unscaled_top_seg_rate = segment_rates[phase];
         const Scalar well_phase_rate = ws.surface_rates[phase];
@@ -64,14 +77,7 @@ scaleSegmentRatesWithWellRates(const std::vector<std::vector<int>>& segment_inle
             for (int seg = 0; seg < numberOfSegments(); ++seg) {
                 segment_rates[baseif_.numPhases() * seg + phase] *= well_phase_rate / unscaled_top_seg_rate;
             }
-        } else {
-            // Due to various reasons, the well/top segment rate can be zero for this phase.
-            // We can not scale this rate directly. The following approach is used to initialize the segment rates.
-            Scalar sumTw = 0;
-            for (int perf = 0; perf < baseif_.numPerfs(); ++perf) {
-                sumTw += baseif_.wellIndex()[perf];
-            }
-
+        } else { // In this case, we calculated sumTw
             // only handling this specific phase
             constexpr Scalar num_single_phase = 1;
             std::vector<Scalar> perforation_rates(num_single_phase * baseif_.numPerfs(), 0.0);
@@ -81,7 +87,8 @@ scaleSegmentRatesWithWellRates(const std::vector<std::vector<int>>& segment_inle
             }
 
             std::vector<Scalar> rates;
-            WellState<Scalar>::calculateSegmentRates(segment_inlets,
+            WellState<Scalar>::calculateSegmentRates(ws.parallel_info,
+                                                     segment_inlets,
                                                      segment_perforations,
                                                      perforation_rates,
                                                      num_single_phase, 0, rates);
@@ -135,18 +142,14 @@ segmentNumberToIndex(const int segment_number) const
 }
 
 template<typename Scalar>
-void
+bool
 MultisegmentWellGeneric<Scalar>::
-detectOscillations(const std::vector<Scalar>& measure_history, bool& oscillate, bool& stagnate) const
+update_relaxation_factor(const std::vector<Scalar>& measure_history, Scalar& relaxation_factor, bool& regularize, DeferredLogger& deferred_logger) const
 {
     const auto it = measure_history.size() - 1;
     if ( it < 2 ) {
-        oscillate = false;
-        stagnate = false;
-        return;
+        return false;
     }
-
-    stagnate = true;
     const Scalar F0 = measure_history[it];
     const Scalar F1 = measure_history[it - 1];
     const Scalar F2 = measure_history[it - 2];
@@ -154,12 +157,60 @@ detectOscillations(const std::vector<Scalar>& measure_history, bool& oscillate,
     const Scalar d2 = std::abs((F0 - F1) / F0);
 
     const Scalar oscillaton_rel_tol = 0.2;
-    oscillate = (d1 < oscillaton_rel_tol) && (oscillaton_rel_tol < d2);
+    bool oscillate = (d1 < oscillaton_rel_tol) && (oscillaton_rel_tol < d2);
+    if (!oscillate)
+        return false;
+   
+    const Scalar min_relaxation_factor = 0.6;
+    std::ostringstream sstr;
+    if (relaxation_factor == min_relaxation_factor) {
+        sstr << " well " << baseif_.name() << " observes severe oscillation. Terminates after " << it << "iterations.\n";
 
-    const Scalar stagnation_rel_tol = 1.e-2;
-    stagnate = std::abs((F1 - F2) / F2) <= stagnation_rel_tol;
+    }
+    const Scalar reduction_mutliplier = 0.9;
+    relaxation_factor = std::max(relaxation_factor * reduction_mutliplier, min_relaxation_factor);
+
+    if (relaxation_factor == min_relaxation_factor) {
+        return true;
+    }
+    sstr << " well " << baseif_.name() << " observes oscillation in inner iteration " << it << "\n";
+    sstr << " relaxation_factor is " << relaxation_factor << "\n";
+    regularize = true;
+    deferred_logger.debug(sstr.str());
+    return false;
 }
 
+template<typename Scalar>
+bool
+MultisegmentWellGeneric<Scalar>::
+repeatedStagnation(const std::vector<Scalar>& measure_history, bool& regularize, DeferredLogger& deferred_logger) const
+{
+    const auto it = measure_history.size() - 1;
+    if ( it < 2 ) {
+        return false;
+    }
+    const Scalar F0 = measure_history[it];
+    const Scalar F1 = measure_history[it - 1];
+    const Scalar F2 = measure_history[it - 2];
+    const Scalar stagnation_rel_tol = 1.e-2;
+    bool stagnate = std::abs((F0 - F1) / F1) <= stagnation_rel_tol && std::abs((F1 - F2) / F2) <= stagnation_rel_tol;
+    if (!stagnate)
+        return false;
+
+    std::ostringstream sstr;
+    sstr << " well " << baseif_.name() << " observes stagnation in inner iteration " << it << "\n";
+    // we frist try to regularize and only stop iterating if it still stagnates
+    if (regularize ) {
+        sstr << "Inner iterations are terminated.\n";
+        return true;
+    }
+    regularize = true;
+    sstr << "Try to regularize the equation.\n";
+    deferred_logger.debug(sstr.str());
+    return false;
+}
+
+
 template<typename Scalar>
 bool
 MultisegmentWellGeneric<Scalar>::
