@@ -22,10 +22,7 @@
 #ifndef OPM_MULTISEGMENTWELL_GENERIC_HEADER_INCLUDED
 #define OPM_MULTISEGMENTWELL_GENERIC_HEADER_INCLUDED
 
-#include <functional>
-#include <optional>
 #include <vector>
-#include <array>
 
 namespace Opm
 {
@@ -52,7 +49,7 @@ public:
     int numberOfSegments() const;
 
 protected:
-    MultisegmentWellGeneric(WellInterfaceGeneric<Scalar>& baseif);
+    explicit MultisegmentWellGeneric(WellInterfaceGeneric<Scalar>& baseif);
 
     // scale the segment rates and pressure based on well rates and bhp
     void scaleSegmentRatesWithWellRates(const std::vector<std::vector<int>>& segment_inlets,
@@ -64,9 +61,8 @@ protected:
     WellSegmentCompPressureDrop compPressureDrop() const;
 
     /// Detect oscillation or stagnation based on the residual measure history
-    void detectOscillations(const std::vector<Scalar>& measure_history,
-                            bool& oscillate,
-                            bool& stagnate) const;
+    bool update_relaxation_factor(const std::vector<Scalar>& measure_history, Scalar& relaxation_factor, bool& regularize, DeferredLogger& deferred_logger) const;    
+    bool repeatedStagnation(const std::vector<Scalar>& measure_history, bool& regularize, DeferredLogger& deferred_logger) const;
 
     bool accelerationalPressureLossConsidered() const;
     bool frictionalPressureLossConsidered() const;
