@@ -54,7 +54,7 @@ resize(const int numSegments)
 
 template<class FluidSystem, class Indices>
 void MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
-init()
+setEvaluationsFromValues()
 {
     for (std::size_t seg = 0; seg < value_.size(); ++seg) {
         for (int eq_idx = 0; eq_idx < numWellEq; ++eq_idx) {
@@ -152,6 +152,7 @@ update(const WellState<Scalar>& well_state,
             }
         }
     }
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -208,6 +209,7 @@ updateNewton(const BVectorWell& dwells,
     if (stop_or_zero_rate_target) {
         value_[0][WQTotal] = 0.;
     }
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -225,7 +227,6 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
     const auto pvtReg = std::max(well_.wellEcl().pvt_table_number() - 1, 0);
 
     const PhaseUsage& pu = well_.phaseUsage();
-    assert( FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) );
     const int oil_pos = pu.phase_pos[Oil];
 
     auto& ws = well_state.well(well_.indexOfWell());
@@ -236,18 +237,34 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
     auto& segment_pressure = segments.pressure;
     for (std::size_t seg = 0; seg < value_.size(); ++seg) {
         std::vector<Scalar> fractions(well_.numPhases(), 0.0);
-        fractions[oil_pos] = 1.0;
 
-        if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            const int water_pos = pu.phase_pos[Water];
-            fractions[water_pos] = value_[seg][WFrac];
-            fractions[oil_pos] -= fractions[water_pos];
+        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+            fractions[oil_pos] = 1.0;
+            if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+                const int water_pos = pu.phase_pos[Water];
+                fractions[water_pos] = value_[seg][WFrac];
+                fractions[oil_pos] -= fractions[water_pos];
+            }
+
+            if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+                const int gas_pos = pu.phase_pos[Gas];
+                fractions[gas_pos] = value_[seg][GFrac];
+                fractions[oil_pos] -= fractions[gas_pos];
+            }
         }
+        else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+            const int water_pos = pu.phase_pos[Water];
+            fractions[water_pos] = 1.0;
 
-        if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+            if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+                const int gas_pos = pu.phase_pos[Gas];
+                fractions[gas_pos] = value_[seg][GFrac];
+                fractions[water_pos] -= fractions[gas_pos];
+            }
+        }
+        else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
             const int gas_pos = pu.phase_pos[Gas];
-            fractions[gas_pos] = value_[seg][GFrac];
-            fractions[oil_pos] -= fractions[gas_pos];
+            fractions[gas_pos] = 1.0;
         }
 
         // convert the fractions to be Q_p / G_total to calculate the phase rates
@@ -287,7 +304,7 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
 
         Scalar rsMax = 0.0;
         Scalar rvMax = 0.0;
-        if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+        if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) && FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
             // Both oil and gas active.
             rsMax = FluidSystem::oilPvt()
                 .saturatedGasDissolutionFactor(pvtReg, temperature, segment_pressure[seg]);
@@ -300,7 +317,8 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
         const auto& [Rs, Rv] = well_.rateConverter().inferDissolvedVaporisedRatio
             (rsMax, rvMax, segment_rates.begin() + seg * well_.numPhases());
 
-        if (! FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+        if ( (!FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) ||
+                  (!FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx))  ) {
             vapoil[seg] = disgas[seg] = 0.0;
         }
         else {
@@ -355,8 +373,10 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
         }
 
         // 5) Local condition phase viscosities.
-        segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Oil]] =
-            FluidSystem::oilPvt().viscosity(pvtReg, temperature, segment_pressure[seg], Rs);
+        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+            segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Oil]] =
+                    FluidSystem::oilPvt().viscosity(pvtReg, temperature, segment_pressure[seg], Rs);
+        }
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
             segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Water]] =
@@ -424,59 +444,71 @@ processFractions(const int seg)
 
     std::vector<Scalar> fractions(well_.numPhases(), 0.0);
 
-    assert(FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx));
-    const int oil_pos = pu.phase_pos[Oil];
-    fractions[oil_pos] = 1.0;
+    if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+        fractions[pu.phase_pos[Oil]] = 1.0;
 
-    if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
-        const int water_pos = pu.phase_pos[Water];
-        fractions[water_pos] = value_[seg][WFrac];
-        fractions[oil_pos] -= fractions[water_pos];
+        if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+            fractions[pu.phase_pos[Water]] = value_[seg][WFrac];
+            fractions[pu.phase_pos[Oil]] -= fractions[pu.phase_pos[Water]];
+        }
+
+        if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+            fractions[pu.phase_pos[Gas]] = value_[seg][GFrac];
+            fractions[pu.phase_pos[Oil]] -= fractions[pu.phase_pos[Gas]];
+        }
     }
+    else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
+        fractions[pu.phase_pos[Water]] = 1.0;
 
-    if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
-        const int gas_pos = pu.phase_pos[Gas];
-        fractions[gas_pos] = value_[seg][GFrac];
-        fractions[oil_pos] -= fractions[gas_pos];
+        if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+            fractions[pu.phase_pos[Gas]] = value_[seg][GFrac];
+            fractions[pu.phase_pos[Water]] -= fractions[pu.phase_pos[Gas]];
+        }
+    } else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
+        fractions[pu.phase_pos[Gas]] = 1.0;
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        const int water_pos = pu.phase_pos[Water];
-        if (fractions[water_pos] < 0.0) {
+        if (fractions[pu.phase_pos[Water]] < 0.0) {
             if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
-                fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[water_pos]);
+                fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[pu.phase_pos[Water]]);
             }
-            fractions[oil_pos] /= (1.0 - fractions[water_pos]);
-            fractions[water_pos] = 0.0;
+            if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+                fractions[pu.phase_pos[Oil]] /= (1.0 - fractions[pu.phase_pos[Water]]);
+            }
+            fractions[pu.phase_pos[Water]] = 0.0;
         }
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const int gas_pos = pu.phase_pos[Gas];
-        if (fractions[gas_pos] < 0.0) {
+        if (fractions[pu.phase_pos[Gas]] < 0.0) {
             if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
-                fractions[pu.phase_pos[Water]] /= (1.0 - fractions[gas_pos]);
+                fractions[pu.phase_pos[Water]] /= (1.0 - fractions[pu.phase_pos[Gas]]);
             }
-            fractions[oil_pos] /= (1.0 - fractions[gas_pos]);
-            fractions[gas_pos] = 0.0;
+            if ( FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) ) {
+                fractions[pu.phase_pos[Oil]] /= (1.0 - fractions[pu.phase_pos[Gas]]);
+            }
+            fractions[pu.phase_pos[Gas]] = 0.0;
         }
     }
 
-    if (fractions[oil_pos] < 0.0) {
-        if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
-            fractions[pu.phase_pos[Water]] /= (1.0 - fractions[oil_pos]);
+    if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+        if (fractions[pu.phase_pos[Oil]] < 0.0) {
+            if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
+                fractions[pu.phase_pos[Water]] /= (1.0 - fractions[pu.phase_pos[Oil]]);
+            }
+            if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
+                fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[pu.phase_pos[Oil]]);
+            }
+            fractions[pu.phase_pos[Oil]] = 0.0;
         }
-        if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
-            fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[oil_pos]);
-        }
-        fractions[oil_pos] = 0.0;
     }
 
-    if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
+    if constexpr (has_wfrac_variable) {
         value_[seg][WFrac] = fractions[pu.phase_pos[Water]];
     }
 
-    if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
+    if constexpr (has_gfrac_variable) {
         value_[seg][GFrac] = fractions[pu.phase_pos[Gas]];
     }
 }
@@ -485,7 +517,7 @@ template<typename FluidSystem, typename Indices>
 typename MultisegmentWellPrimaryVariables<FluidSystem,Indices>::EvalWell
 MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
 volumeFraction(const int seg,
-               const unsigned compIdx) const
+               const int compIdx) const
 {
     if (has_wfrac_variable && compIdx == Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx)) {
         return evaluation_[seg][WFrac];
@@ -554,7 +586,7 @@ typename MultisegmentWellPrimaryVariables<FluidSystem,Indices>::EvalWell
 MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
 getSegmentRateUpwinding(const int seg,
                         const int seg_upwind,
-                        const std::size_t comp_idx) const
+                        const int comp_idx) const
 {
     // the result will contain the derivative with respect to WQTotal in segment seg,
     // and the derivatives with respect to WFrac GFrac in segment seg_upwind.
