@@ -60,9 +60,10 @@ namespace Opm
 template<class FluidSystem, class Indices>
 MultisegmentWellSegments<FluidSystem,Indices>::
 MultisegmentWellSegments(const int numSegments,
+                         const ParallelWellInfo<Scalar>& parallel_well_info,
                          WellInterfaceGeneric<Scalar>& well)
     : perforations_(numSegments)
-    , perforation_depth_diffs_(well.numPerfs(), 0.0)
+    , local_perforation_depth_diffs_(well.numPerfs(), 0.0)
     , inlets_(well.wellEcl().getSegments().size())
     , depth_diffs_(numSegments, 0.0)
     , densities_(numSegments, 0.0)
@@ -85,7 +86,9 @@ MultisegmentWellSegments(const int numSegments,
     // the current implementation is a temporary solution for now, it should be corrected from the parser
     // side
     int i_perf_wells = 0;
-    well.perfDepth().resize(well_.numPerfs(), 0.);
+    // The perfDepth vector will contain the depths of all perforations across all processes of this well!
+    int num_perfs_whole_mswell = parallel_well_info.communication().sum(well.numPerfs());
+    well.perfDepth().resize(num_perfs_whole_mswell, 0.0);
     const auto& segment_set = well_.wellEcl().getSegments();
     for (std::size_t perf = 0; perf < completion_set.size(); ++perf) {
         const Connection& connection = completion_set.get(perf);
@@ -101,7 +104,9 @@ MultisegmentWellSegments(const int numSegments,
             perforations_[segment_index].push_back(i_perf_wells);
             well.perfDepth()[i_perf_wells] = connection.depth();
             const Scalar segment_depth = segment_set[segment_index].depth();
-            perforation_depth_diffs_[i_perf_wells] = well_.perfDepth()[i_perf_wells] - segment_depth;
+            int local_perf_index = parallel_well_info.activeToLocal(i_perf_wells);
+            if (local_perf_index > -1) // If local_perf_index == -1, then the perforation is not on this process
+                local_perforation_depth_diffs_[local_perf_index] = well_.perfDepth()[i_perf_wells] - segment_depth;
             i_perf_wells++;
         }
     }
@@ -342,10 +347,10 @@ getHydroPressureLoss(const int seg,
 template<class FluidSystem, class Indices>
 typename MultisegmentWellSegments<FluidSystem,Indices>::Scalar
 MultisegmentWellSegments<FluidSystem,Indices>::
-getPressureDiffSegPerf(const int seg,
-                       const int perf) const
+getPressureDiffSegLocalPerf(const int seg,
+                            const int local_perf_index) const
 {
-    return well_.gravity() * densities_[seg].value() * perforation_depth_diffs_[perf];
+    return well_.gravity() * densities_[seg].value() * local_perforation_depth_diffs_[local_perf_index];
 }
 
 template<class FluidSystem, class Indices>
@@ -512,7 +517,7 @@ getFrictionPressureLoss(const int seg,
     // at segment node while fraction derivatives are given at upwind node.
 
     if (seg != seg_upwind) {
-        if (!extra_reverse_flow_derivatives){
+        if (!extra_reverse_flow_derivatives) {
             constexpr int WQTotal = Indices::numEq + PrimaryVariables::WQTotal;
             constexpr int SPres = Indices::numEq + PrimaryVariables::SPres;
             density.setDerivative(WQTotal, 0.0);
@@ -520,12 +525,12 @@ getFrictionPressureLoss(const int seg,
             visc.setDerivative(WQTotal, 0.0);
             visc.setDerivative(SPres, 0.0);
         } else {
-            if (PrimaryVariables::has_water){
+            if constexpr (PrimaryVariables::has_wfrac_variable) {
                 constexpr int WFrac = Indices::numEq + PrimaryVariables::WFrac;
                 density.setDerivative(WFrac, 0.0);
                 visc.setDerivative(WFrac, 0.0);
             }
-            if (PrimaryVariables::has_gas){
+            if constexpr (PrimaryVariables::has_gfrac_variable) {
                 constexpr int GFrac = Indices::numEq + PrimaryVariables::GFrac;
                 density.setDerivative(GFrac, 0.0);
                 visc.setDerivative(GFrac, 0.0);
@@ -598,10 +603,10 @@ pressureDropSpiralICD(const int seg,
             zero_mask[PrimaryVariables::WQTotal] = true;
             zero_mask[PrimaryVariables::SPres] = true;
         } else {
-            if constexpr (PrimaryVariables::has_water){
+            if constexpr (PrimaryVariables::has_wfrac_variable) {
                 zero_mask[PrimaryVariables::WFrac] = true;
             }
-            if constexpr (PrimaryVariables::has_gas){
+            if constexpr (PrimaryVariables::has_gfrac_variable) {
                 zero_mask[PrimaryVariables::GFrac] = true;
             }
             // mass_rate has no extra derivatives (they are organized as in equations)
@@ -710,10 +715,10 @@ pressureDropAutoICD(const int seg,
             zero_mask[PrimaryVariables::WQTotal] = true;
             zero_mask[PrimaryVariables::SPres] = true;
         } else {
-            if (PrimaryVariables::has_water){
+            if constexpr (PrimaryVariables::has_wfrac_variable) {
                 zero_mask[PrimaryVariables::WFrac] = true;
             }
-            if (PrimaryVariables::has_gas){
+            if constexpr (PrimaryVariables::has_gfrac_variable) {
                 zero_mask[PrimaryVariables::GFrac] = true;
             }
             // mass_rate has no extra derivatives (they are organized as in equations)
@@ -785,7 +790,7 @@ pressureDropValve(const int seg,
     // For reference: the pressure equation assumes pressure/flow derivatives are given 
     // at segment node while fraction derivatives are given at upwind node. 
     if (seg != seg_upwind) {
-        if (!extra_reverse_flow_derivatives){
+        if (!extra_reverse_flow_derivatives) {
             constexpr int WQTotal = Indices::numEq + PrimaryVariables::WQTotal;
             constexpr int SPres = Indices::numEq + PrimaryVariables::SPres;
             density.setDerivative(WQTotal, 0.0);
@@ -793,12 +798,12 @@ pressureDropValve(const int seg,
             visc.setDerivative(WQTotal, 0.0);
             visc.setDerivative(SPres, 0.0);
         } else {
-            if (PrimaryVariables::has_water){
+            if constexpr (PrimaryVariables::has_wfrac_variable) {
                 constexpr int WFrac = Indices::numEq + PrimaryVariables::WFrac;
                 density.setDerivative(WFrac, 0.0);
                 visc.setDerivative(WFrac, 0.0);
             }
-            if (PrimaryVariables::has_gas){
+            if constexpr (PrimaryVariables::has_gfrac_variable) {
                 constexpr int GFrac = Indices::numEq + PrimaryVariables::GFrac;
                 density.setDerivative(GFrac, 0.0);
                 visc.setDerivative(GFrac, 0.0);
@@ -839,17 +844,17 @@ accelerationPressureLossContribution(const int seg,
     const int seg_upwind = upwinding_segments_[seg];
     EvalWell density = densities_[seg_upwind];    
     if (seg != seg_upwind) {
-        if (!extra_reverse_flow_derivatives){
+        if (!extra_reverse_flow_derivatives) {
             constexpr int WQTotal = Indices::numEq + PrimaryVariables::WQTotal;
             constexpr int SPres = Indices::numEq + PrimaryVariables::SPres;
             density.setDerivative(WQTotal, 0.0);
             density.setDerivative(SPres, 0.0);
         } else {
-            if (PrimaryVariables::has_water){
+            if constexpr (PrimaryVariables::has_wfrac_variable) {
                 constexpr int WFrac = Indices::numEq + PrimaryVariables::WFrac;
                 density.setDerivative(WFrac, 0.0);
             }
-            if (PrimaryVariables::has_gas){
+            if constexpr (PrimaryVariables::has_gfrac_variable) {
                 constexpr int GFrac = Indices::numEq + PrimaryVariables::GFrac;
                 density.setDerivative(GFrac, 0.0);
             }
@@ -857,7 +862,7 @@ accelerationPressureLossContribution(const int seg,
         }
     }
     const Scalar sign = mass_rate > 0 ? -1.0 : 1.0;
-    return sign*mswellhelpers::velocityHead(area, mass_rate, density);
+    return sign * mswellhelpers::velocityHead(area, mass_rate, density);
 }                                     
 
 template <class FluidSystem, class Indices>
@@ -931,10 +936,10 @@ mixtureDensity(const int seg) const
             continue;
         }
 
-        const auto compIdx = Indices::
-            canonicalToActiveComponentIndex(phIdx);
+        const auto active_comp_index = Indices::
+            canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(phIdx));
 
-        mixDens += q[compIdx].value() * rho[compIdx].value();
+        mixDens += q[active_comp_index].value() * rho[active_comp_index].value();
     }
 
     return mixDens;
@@ -975,7 +980,7 @@ mixtureDensityWithExponents(const AutoICD& aicd, const int seg) const
     for (const auto& [fsPhaseIdx, densityExponent] : densityExponents) {
         if (FluidSystem::phaseIsActive(fsPhaseIdx)) {
             const auto compIdx = Indices::
-                canonicalToActiveComponentIndex(fsPhaseIdx);
+                canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(fsPhaseIdx));
 
             // exp = (aicd.*densityExponent)() in native syntax.
             const auto exp = std::invoke(densityExponent, aicd);
