@@ -22,11 +22,12 @@
 #ifndef OPM_RATECONVERTER_HPP_HEADER_INCLUDED
 #define OPM_RATECONVERTER_HPP_HEADER_INCLUDED
 
-#include <opm/core/props/BlackoilPhases.hpp>
 #include <opm/grid/utility/RegionMapping.hpp>
-#include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
+
 #include <opm/simulators/wells/RegionAttributeHelpers.hpp>
 
+#include <opm/simulators/utils/BlackoilPhases.hpp>
+#include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/utils/ParallelCommunication.hpp>
 
 #include <dune/grid/common/gridenums.hh>
@@ -69,6 +70,8 @@ namespace Opm {
         template <class FluidSystem, class Region>
         class SurfaceToReservoirVoidage {
         public:
+            using Scalar = typename FluidSystem::Scalar;
+
             /**
              * Constructor.
              *
@@ -90,8 +93,8 @@ namespace Opm {
              * reservoir voidage rate.
              *
              */
-            template <typename ElementContext, class EbosSimulator>
-            void defineState(const EbosSimulator& simulator)
+            template <typename ElementContext, class Simulator>
+            void defineState(const Simulator& simulator)
             {
                 // create map from cell to region and set all attributes to
                 // zero
@@ -130,12 +133,12 @@ namespace Opm {
                     const auto& intQuants = elemCtx.intensiveQuantities(/*spaceIdx=*/0, /*timeIdx=*/0);
                     const auto& fs = intQuants.fluidState();
                     // use pore volume weighted averages.
-                    const double pv_cell =
-                            simulator.model().dofTotalVolume(cellIdx)
+                    const Scalar pv_cell =
+                            simulator.model().dofTotalVolume(simulator.vanguard().gridEquilIdxToGridIdx(cellIdx))
                             * intQuants.porosity().value();
 
                     // only count oil and gas filled parts of the domain
-                    double hydrocarbon = 1.0;
+                    Scalar hydrocarbon = 1.0;
                     const auto& pu = phaseUsage_;
                     if (RegionAttributeHelpers::PhaseUsed::water(pu)) {
                         hydrocarbon -= fs.saturation(FluidSystem::waterPhaseIdx).value();
@@ -145,7 +148,7 @@ namespace Opm {
                     assert(reg >= 0);
 
                     // sum p, rs, rv, and T.
-                    const double hydrocarbonPV = pv_cell*hydrocarbon;
+                    const Scalar hydrocarbonPV = pv_cell*hydrocarbon;
                     if (hydrocarbonPV > 0.) {
                         auto& attr = attributes_hpv[reg];
                         attr.pv += hydrocarbonPV;
@@ -190,7 +193,7 @@ namespace Opm {
                         }
                         attr.saltConcentration += fs.saltConcentration().value() * pv_cell;
                         if (FluidSystem::enableDissolvedGasInWater()) {
-                            attr.rsw += fs.Rsw().value() * pv_cell; 
+                            attr.rsw += fs.Rsw().value() * pv_cell;
                         }
                         if (FluidSystem::enableVaporizedWater()) {
                             attr.rvw += fs.Rvw().value() * pv_cell;
@@ -249,15 +252,15 @@ namespace Opm {
 
             template <class Coeff>
             void
-            calcCoeff(  const int pvtRegionIdx,
-                        const double        p,
-                        const double        rs,
-                        const double        rv,
-                        const double        rsw,
-                        const double        rvw,
-                        const double        T,
-                        const double saltConcentration,
-                        Coeff& coeff) const;
+            calcCoeff(  const int     pvtRegionIdx,
+                        const Scalar  p,
+                        const Scalar  rs,
+                        const Scalar  rv,
+                        const Scalar  rsw,
+                        const Scalar  rvw,
+                        const Scalar  T,
+                        const Scalar  saltConcentration,
+                        Coeff&        coeff) const;
 
             template <class Coeff>
             void
@@ -308,7 +311,7 @@ namespace Opm {
              * \param[in] rs Dissolved gas/oil ratio.
              *
              * \param[in] rv Vaporised oil/gas ratio.
-             *        
+             *
              * \param[in] rsw Dissolved gas/water ratio.
              *
              * \param[in] rwv Vaporised water/gas ratio.
@@ -327,20 +330,20 @@ namespace Opm {
              */
             template <typename SurfaceRates, typename VoidageRates>
             void calcReservoirVoidageRates(const int           pvtRegionIdx,
-                                           const double        p,
-                                           const double        rs,
-                                           const double        rv,
-                                           const double        rsw,
-                                           const double        rvw,
-                                           const double        T,
-                                           const double        saltConcentration,
+                                           const Scalar        p,
+                                           const Scalar        rs,
+                                           const Scalar        rv,
+                                           const Scalar        rsw,
+                                           const Scalar        rvw,
+                                           const Scalar        T,
+                                           const Scalar        saltConcentration,
                                            const SurfaceRates& surface_rates,
                                            VoidageRates&       voidage_rates) const;
 
             template <class Rates>
-            std::pair<double, double>
-            inferDissolvedVaporisedRatio(const double rsMax,
-                                         const double rvMax,
+            std::pair<Scalar, Scalar>
+            inferDissolvedVaporisedRatio(const Scalar rsMax,
+                                         const Scalar rvMax,
                                          const Rates& surface_rates) const;
 
             /**
@@ -357,13 +360,13 @@ namespace Opm {
              */
             template <class SolventModule>
             void
-            calcCoeffSolvent(const RegionId r, const int pvtRegionIdx, double& coeff) const
+            calcCoeffSolvent(const RegionId r, const int pvtRegionIdx, Scalar& coeff) const
             {
                 const auto& ra = attr_.attributes(r);
-                const double p = ra.pressure;
-                const double T = ra.temperature;
+                const Scalar p = ra.pressure;
+                const Scalar T = ra.temperature;
                 const auto& solventPvt = SolventModule::solventPvt();
-                const double bs = solventPvt.inverseFormationVolumeFactor(pvtRegionIdx, T, p);
+                const Scalar bs = solventPvt.inverseFormationVolumeFactor(pvtRegionIdx, T, p);
                 coeff = 1.0 / bs;
             }
 
@@ -406,15 +409,15 @@ namespace Opm {
                     return *this;
                 }
 
-                std::array<double,8> data;
-                double& pressure;
-                double& temperature;
-                double& rs;
-                double& rv;
-                double& rsw;
-                double& rvw;
-                double& pv;
-                double& saltConcentration;
+                std::array<Scalar,8> data;
+                Scalar& pressure;
+                Scalar& temperature;
+                Scalar& rs;
+                Scalar& rv;
+                Scalar& rsw;
+                Scalar& rvw;
+                Scalar& pv;
+                Scalar& saltConcentration;
             };
 
             void sumRates(std::unordered_map<RegionId,Attributes>& attributes_hpv,
