@@ -25,6 +25,7 @@
 
 #include <opm/simulators/timestepping/ConvergenceReport.hpp>
 #include <opm/simulators/wells/RateConverter.hpp>
+#include <opm/simulators/wells/RatioCalculator.hpp>
 #include <opm/simulators/wells/VFPInjProperties.hpp>
 #include <opm/simulators/wells/VFPProdProperties.hpp>
 #include <opm/simulators/wells/WellInterface.hpp>
@@ -136,8 +137,6 @@ namespace Opm
                           const std::vector<Scalar>& B_avg,
                           const bool changed_to_open_this_step) override;
 
-        void initPrimaryVariablesEvaluation() override;
-
         /// check whether the well equations get converged for this well
         virtual ConvergenceReport getWellConvergence(const Simulator& simulator,
                                                      const WellState<Scalar>& well_state,
@@ -234,7 +233,8 @@ namespace Opm
         computeBhpAtThpLimitProdWithAlq(const Simulator& ebos_simulator,
                                         const SummaryState& summary_state,
                                         const Scalar alq_value,
-                                        DeferredLogger& deferred_logger) const override;
+                                        DeferredLogger& deferred_logger,
+                                        bool iterate_if_no_solution) const override;
 
         void updateIPRImplicit(const Simulator& simulator,
                                WellState<Scalar>& well_state,
@@ -323,6 +323,7 @@ namespace Opm
                                     const WellState<Scalar>& well_state) const;
 
         bool computeWellPotentialsImplicit(const Simulator& ebos_simulator,
+                                           const WellState<Scalar>& well_state,
                                            std::vector<Scalar>& well_potentials,
                                            DeferredLogger& deferred_logger) const;               
 
@@ -465,60 +466,10 @@ namespace Opm
                                   const std::vector<EvalWell>& cq_s,
                                   const IntensiveQuantities& intQuants,
                                   DeferredLogger& deferred_logger) const;
-
-        template<class Value>
-        void gasOilPerfRateInj(const std::vector<Value>& cq_s,
-                               PerforationRates<Scalar>& perf_rates,
-                               const Value& rv,
-                               const Value& rs,
-                               const Value& pressure,
-                               const Value& rvw,
-                               DeferredLogger& deferred_logger) const;
-
-        template<class Value>
-        void gasOilPerfRateProd(std::vector<Value>& cq_s,
-                                PerforationRates<Scalar>& perf_rates,
-                                const Value& rv,
-                                const Value& rs,
-                                const Value& rvw) const;
-
-        template<class Value>
-        void gasWaterPerfRateProd(std::vector<Value>& cq_s,
-                                  PerforationRates<Scalar>& perf_rates,
-                                  const Value& rvw,
-                                  const Value& rsw) const;
-
-        template<class Value>
-        void gasWaterPerfRateInj(const std::vector<Value>& cq_s,
-                                 PerforationRates<Scalar>& perf_rates,
-                                 const Value& rvw,
-                                 const Value& rsw,
-                                 const Value& pressure,
-                                 DeferredLogger& deferred_logger) const;
-
-        template<class Value>
-        void disOilVapWatVolumeRatio(Value& volumeRatio,
-                                     const Value& rvw,
-                                     const Value& rsw,
-                                     const Value& pressure,
-                                     const std::vector<Value>& cmix_s,
-                                     const std::vector<Value>& b_perfcells_dense,
-                                     DeferredLogger& deferred_logger) const;
-
-        template<class Value>
-        void gasOilVolumeRatio(Value& volumeRatio,
-                               const Value& rv,
-                               const Value& rs,
-                               const Value& pressure,
-                               const std::vector<Value>& cmix_s,
-                               const std::vector<Value>& b_perfcells_dense,
-                               DeferredLogger& deferred_logger) const;
     };
 
 }
 
-#ifndef OPM_STANDARDWELL_IMPL_HEADER_INCLUDED
 #include "StandardWell_impl.hpp"
-#endif
 
 #endif // OPM_STANDARDWELL_HEADER_INCLUDED
