@@ -731,7 +731,7 @@ connectionRateFoam(const std::vector<EvalWell>& cq_s,
             }
             case Phase::SOLVENT: {
                 if constexpr (Indices::enableSolvent)
-                    return static_cast<unsigned>(Indices::contiSolventEqIdx);
+                    return Indices::contiSolventEqIdx;
                 else
                     OPM_DEFLOG_THROW(std::runtime_error, "Foam transport phase is SOLVENT but SOLVENT is not activated.", deferred_logger);
             }
@@ -756,7 +756,10 @@ std::tuple<typename StandardWellConnections<FluidSystem,Indices>::Eval,
            typename StandardWellConnections<FluidSystem,Indices>::Eval,
            typename StandardWellConnections<FluidSystem,Indices>::Eval>
 StandardWellConnections<FluidSystem,Indices>::
-connectionRatesMICP(const std::vector<EvalWell>& cq_s,
+connectionRatesMICP(Scalar& rate_m,
+                    Scalar& rate_o,
+                    Scalar& rate_u,
+                    const std::vector<EvalWell>& cq_s,
                     const std::variant<Scalar,EvalWell>& microbialConcentration,
                     const std::variant<Scalar,EvalWell>& oxygenConcentration,
                     const std::variant<Scalar,EvalWell>& ureaConcentration) const
@@ -769,6 +772,8 @@ connectionRatesMICP(const std::vector<EvalWell>& cq_s,
         cq_s_microbe *= std::get<EvalWell>(microbialConcentration);
     }
 
+    rate_m = cq_s_microbe.value();
+
     EvalWell cq_s_oxygen = cq_s[waterCompIdx];
     if (well_.isInjector()) {
         cq_s_oxygen *= std::get<Scalar>(oxygenConcentration);
@@ -776,6 +781,8 @@ connectionRatesMICP(const std::vector<EvalWell>& cq_s,
         cq_s_oxygen *= std::get<EvalWell>(oxygenConcentration);
     }
 
+    rate_o = cq_s_oxygen.value();
+
     EvalWell cq_s_urea = cq_s[waterCompIdx];
     if (well_.isInjector()) {
         cq_s_urea *= std::get<Scalar>(ureaConcentration);
@@ -783,6 +790,8 @@ connectionRatesMICP(const std::vector<EvalWell>& cq_s,
         cq_s_urea *= std::get<EvalWell>(ureaConcentration);
     }
 
+    rate_u = cq_s_urea.value();
+
     return {well_.restrictEval(cq_s_microbe),
             well_.restrictEval(cq_s_oxygen),
             well_.restrictEval(cq_s_urea)};
