@@ -34,13 +34,13 @@
 namespace Opm
 {
 
-class ParallelWellInfo;
+template<class Scalar> class ParallelWellInfo;
 template<class Scalar, int numEq> class StandardWellEquationAccess;
-#if COMPILE_BDA_BRIDGE
-class WellContributions;
+#if COMPILE_GPU_BRIDGE
+template<class Scalar> class WellContributions;
 #endif
-class WellInterfaceGeneric;
-class WellState;
+template<class Scalar> class WellInterfaceGeneric;
+template<class Scalar> class WellState;
 
 template<class Scalar, int numEq>
 class StandardWellEquations
@@ -65,15 +65,13 @@ public:
     // block vector type
     using BVector = Dune::BlockVector<Dune::FieldVector<Scalar,numEq>>;
 
-    StandardWellEquations(const ParallelWellInfo& parallel_well_info);
+    explicit StandardWellEquations(const ParallelWellInfo<Scalar>& parallel_well_info);
 
     //! \brief Setup sparsity pattern for the matrices.
-    //! \param num_cells Total number of cells
     //! \param numWellEq Number of well equations
     //! \param numPerfs Number of perforations
     //! \param cells Cell indices for perforations
-    void init(const int num_cells,
-              const int numWellEq,
+    void init(const int numWellEq,
               const int numPerfs,
               const std::vector<int>& cells);
 
@@ -99,10 +97,10 @@ public:
     //! \details xw = inv(D)*(rw - C*x)
     void recoverSolutionWell(const BVector& x, BVectorWell& xw) const;
 
-#if COMPILE_BDA_BRIDGE
+#if COMPILE_GPU_BRIDGE
     //! \brief Add the matrices of this well to the WellContributions object.
     void extract(const int numStaticWellEq,
-                 WellContributions& wellContribs) const;
+                 WellContributions<Scalar>& wellContribs) const;
 #endif
 
     //! \brief Add the matrices of this well to the sparse matrix adapter.
@@ -115,9 +113,9 @@ public:
                                   const BVector& weights,
                                   const int pressureVarIndex,
                                   const bool use_well_weights,
-                                  const WellInterfaceGeneric& well,
+                                  const WellInterfaceGeneric<Scalar>& well,
                                   const int bhp_var_index,
-                                  const WellState& well_state) const;
+                                  const WellState<Scalar>& well_state) const;
 
     //! \brief Get the number of blocks of the C and B matrices.
     unsigned int getNumBlocks() const;
@@ -150,6 +148,9 @@ private:
     // several vector used in the matrix calculation
     mutable BVectorWell Bx_;
     mutable BVectorWell invDrw_;
+
+    // Store the global index of well perforated cells
+    std::vector<int> cells_;
 };
 
 }
