@@ -97,7 +97,7 @@ namespace Opm {
 
 template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
-init()
+setEvaluationsFromValues()
 {
     for (int eqIdx = 0; eqIdx < numWellEq_; ++eqIdx) {
         evaluation_[eqIdx] =
@@ -223,6 +223,7 @@ update(const WellState<Scalar>& well_state,
 
     // BHP
     value_[Bhp] = ws.bhp;
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -239,6 +240,7 @@ updatePolyMW(const WellState<Scalar>& well_state)
             value_[Bhp + 1 + well_.numPerfs() + perf] = skin_pressure[perf];
         }
     }
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -301,6 +303,7 @@ updateNewton(const BVectorWell& dwells,
     // so that bhp constaint can be an active control when needed.
     constexpr Scalar bhp_lower_limit = 1. * unit::barsa - 1. * unit::Pascal;
     value_[Bhp] = std::max(value_[Bhp] - dx1_limited, bhp_lower_limit);
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -320,6 +323,7 @@ updateNewtonPolyMW(const BVectorWell& dwells)
             value_[pskin_index] -= relaxation_factor * dx_pskin;
         }
     }
+    setEvaluationsFromValues();
 }
 
 template<class FluidSystem, class Indices>
@@ -382,7 +386,7 @@ copyToWellState(WellState<Scalar>& well_state,
         if (scal > 0) {
             F[p] /= scal ;
         } else {
-            // this should only happens to injection wells
+            // this should only happen to injection wells
             F[p] = 0.;
         }
     }
@@ -446,7 +450,7 @@ copyToWellStatePolyMW(WellState<Scalar>& well_state) const
 template<class FluidSystem, class Indices>
 typename StandardWellPrimaryVariables<FluidSystem,Indices>::EvalWell
 StandardWellPrimaryVariables<FluidSystem,Indices>::
-volumeFraction(const unsigned compIdx) const
+volumeFraction(const int compIdx) const
 {
     if (FluidSystem::numActivePhases() == 1) {
         return EvalWell(numWellEq_ + Indices::numEq, 1.0);
@@ -456,7 +460,7 @@ volumeFraction(const unsigned compIdx) const
         return evaluation_[GFrac];
     }
 
-    if (Indices::enableSolvent && compIdx == (unsigned)Indices::contiSolventEqIdx) {
+    if (Indices::enableSolvent && compIdx == Indices::contiSolventEqIdx) {
         return evaluation_[SFrac];
     }
 
