@@ -171,10 +171,9 @@ findThpFromBhpIteratively(const std::function<Scalar(const Scalar, const Scalar)
     auto pressure_loss = getVfpBhpAdjustment(bhp + dp, thp_limit);
     auto thp = thp_func(bhp, pressure_loss);
     const Scalar tolerance = 1e-5 * unit::barsa;
-    bool do_iterate = true;
     int it = 1;
     int max_iterations = 50;
-    while (do_iterate) {
+    while (true) {
         if (it > max_iterations) {
             break;
         }
@@ -270,9 +269,9 @@ computeBhpAtThpLimitProd(const std::function<std::vector<Scalar>(const Scalar)>&
 
     // could not solve for the bhp-point, we could not continue to find the bhp
     if (!bhp_max.has_value()) {
-        deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
-                                "Robust bhp(thp) solve failed due to not being able to "
-                                "find bhp-point where production becomes non-zero for well " + well_.name());
+        deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
+                              "Robust bhp(thp) solve failed due to not being able to "
+                              "find bhp-point where production becomes non-zero for well " + well_.name());
         return std::nullopt;
     }
     const std::array<Scalar, 2> range {static_cast<Scalar>(controls.bhp_limit), *bhp_max};
@@ -558,8 +557,8 @@ computeBhpAtThpLimitInjImpl(const std::function<std::vector<Scalar>(const Scalar
         catch (...) {
             // Use previous value (or max value if at start) if we failed.
             bhp_samples.push_back(bhp_samples.empty() ? low : bhp_samples.back());
-            deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_EXTRACT_SAMPLES",
-                                    "Robust bhp(thp) solve failed extracting bhp values at flo samples for well " + well_.name());
+            deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_EXTRACT_SAMPLES",
+                                  "Robust bhp(thp) solve failed extracting bhp values at flo samples for well " + well_.name());
         }
     }
 
@@ -616,7 +615,7 @@ computeBhpAtThpLimitInjImpl(const std::function<std::vector<Scalar>(const Scalar
         // We are in the high flow regime where the bhp_samples
         // are all equal to the bhp_limit.
         assert(low == controls.bhp_limit);
-        deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE",
+        deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE",
                                 "Robust bhp(thp) solve failed for well " + well_.name());
         return std::nullopt;
     }
@@ -630,7 +629,7 @@ computeBhpAtThpLimitInjImpl(const std::function<std::vector<Scalar>(const Scalar
         return solved_bhp;
     }
     catch (...) {
-        deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE",
+        deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE",
                                 "Robust bhp(thp) solve failed for well " + well_.name());
         return std::nullopt;
     }
@@ -672,8 +671,8 @@ bhpMax(const std::function<Scalar(const Scalar)>& fflo,
             // Even at the BHP limit, we are injecting.
             // There will be no solution here, return an
             // empty optional.
-            deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
-                                    "Robust bhp(thp) solve failed due to inoperability for well " + well_.name());
+            deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
+                                  "Robust bhp(thp) solve failed due to inoperability for well " + well_.name());
             return std::nullopt;
         } else {
             // Still producing, even at high bhp.
@@ -701,8 +700,8 @@ bhpMax(const std::function<Scalar(const Scalar)>& fflo,
         if (it < maxit) {
             bhp_max = low;
         } else {
-            deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
-                                    "Bisect did not find the bhp-point where we produce for well " + well_.name());
+            deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
+                                  "Bisect did not find the bhp-point where we produce for well " + well_.name());
             return std::nullopt;
         }
     }
@@ -765,7 +764,7 @@ computeBhpAtThpLimit(const std::function<std::vector<Scalar>(const Scalar)>& fra
     }
 
     if (!finding_bracket) {
-        deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
+        deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_INOPERABLE",
                                 "Robust bhp(thp) solve failed due to not being able to "
                                 "bracket the bhp solution with the brute force search for " + well_.name());
         return std::nullopt;
@@ -781,7 +780,7 @@ computeBhpAtThpLimit(const std::function<std::vector<Scalar>(const Scalar)>& fra
         return solved_bhp;
     }
     catch (...) {
-        deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE",
+        deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE",
                                 "Robust bhp(thp) solve failed for well " + well_.name());
         return std::nullopt;
     }
@@ -847,13 +846,13 @@ bisectBracket(const std::function<Scalar(const Scalar)>& eq,
             const Scalar limit = 0.1 * unit::barsa;
             if (std::min(abs_low, abs_high) < limit) {
                 // Return the least bad solution if less off than 0.1 bar.
-                deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_BRACKETING_FAILURE",
-                                        "Robust bhp(thp) not solved precisely for well " + well_.name());
+                deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_BRACKETING_FAILURE",
+                                      "Robust bhp(thp) not solved precisely for well " + well_.name());
                 approximate_solution = abs_low < abs_high ? low : high;
             } else {
                     // Return failure.
-                deferred_logger.warning("FAILED_ROBUST_BHP_THP_SOLVE_BRACKETING_FAILURE",
-                                         "Robust bhp(thp) solve failed due to bracketing failure for well " +
+                deferred_logger.debug("FAILED_ROBUST_BHP_THP_SOLVE_BRACKETING_FAILURE",
+                                      "Robust bhp(thp) solve failed due to bracketing failure for well " +
                                          well_.name());
             }
         }
@@ -1051,7 +1050,7 @@ bruteForceBracketCommonTHP(const std::function<Scalar(const Scalar)>& eq,
                            Scalar& min_thp, Scalar& max_thp)
 {
     bool bracket_found = false;
-    constexpr int sample_number = 1000; 
+    constexpr int sample_number = 1000;
     constexpr Scalar interval = 1E5; 
     Scalar eq_low = eq(min_thp);
     Scalar eq_high = 0.0;
