@@ -24,8 +24,10 @@
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
+#include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/Schedule/Group/GPMaint.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
+#include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
@@ -35,6 +37,7 @@
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/utils/ParallelCommunication.hpp>
 
+#include <opm/simulators/wells/BlackoilWellModelConstraints.hpp>
 #include <opm/simulators/wells/FractionCalculator.hpp>
 #include <opm/simulators/wells/GroupState.hpp>
 #include <opm/simulators/wells/RegionAverageCalculator.hpp>
@@ -42,7 +45,7 @@
 #include <opm/simulators/wells/VFPProdProperties.hpp>
 #include <opm/simulators/wells/WellState.hpp>
 #include <opm/simulators/wells/GroupState.hpp>
-
+#include <opm/common/TimingMacros.hpp>
 #include <algorithm>
 #include <cassert>
 #include <cstddef>
@@ -75,21 +78,35 @@ namespace {
         return {oilRate, gasRate, waterRate};
     }
 
+} // namespace Anonymous
+
+namespace Opm {
+
     template<class Scalar>
-    Scalar sumWellPhaseRates(bool res_rates,
-                             const Opm::Group& group,
-                             const Opm::Schedule& schedule,
-                             const Opm::WellState<Scalar>& wellState,
-                             const int reportStepIdx,
-                             const int phasePos,
-                             const bool injector)
+    Scalar WellGroupHelpers<Scalar>::
+    sumWellPhaseRates(bool res_rates,
+                      const Opm::Group& group,
+                      const Opm::Schedule& schedule,
+                      const Opm::WellState<Scalar>& wellState,
+                      const int reportStepIdx,
+                      const int phasePos,
+                      const bool injector,
+                      const bool network)
     {
 
         Scalar rate = 0.0;
         for (const std::string& groupName : group.groups()) {
             const auto& groupTmp = schedule.getGroup(groupName, reportStepIdx);
-            const auto& gefac = groupTmp.getGroupEfficiencyFactor();
-            rate += gefac * sumWellPhaseRates(res_rates, groupTmp, schedule, wellState, reportStepIdx, phasePos, injector);
+            const auto& gefac = groupTmp.getGroupEfficiencyFactor(network);
+            rate += gefac * sumWellPhaseRates(res_rates, groupTmp, schedule, wellState, reportStepIdx, phasePos, injector, network);
+        }
+
+        // only sum satelite production once
+        if (wellState.isRank0() && !injector) {
+            const auto rateComp = selectRateComponent(wellState.phaseUsage(), phasePos);
+            if (rateComp.has_value()) {
+                rate += satelliteProduction(schedule[reportStepIdx], group.groups(), *rateComp);
+            }
         }
 
         for (const std::string& wellName : group.wells()) {
@@ -110,7 +127,7 @@ namespace {
             if (wellEcl.getStatus() == Opm::Well::Status::SHUT)
                 continue;
 
-            Scalar factor = wellEcl.getEfficiencyFactor();
+            const Scalar factor = wellEcl.getEfficiencyFactor(network) * wellState[wellEcl.name()].efficiency_scaling_factor;
             const auto& ws = wellState.well(well_index.value());
             if (res_rates) {
                 const auto& well_rates = ws.reservoir_rates;
@@ -128,9 +145,42 @@ namespace {
         }
         return rate;
     }
-} // namespace Anonymous
 
-namespace Opm {
+template <typename Scalar>
+Scalar WellGroupHelpers<Scalar>::
+satelliteProduction(const ScheduleState& sched,
+                    const std::vector<std::string>& groups,
+                    const GSatProd::GSatProdGroup::Rate rateComp)
+{
+    auto gsatProdRate = Scalar{};
+    const auto& gsatProd = sched.gsatprod();
+    for (const auto& group : groups) {
+        if (! gsatProd.has(group)) {
+            continue;
+        }
+        gsatProdRate += gsatProd.get(group).rate[rateComp];
+    }
+    return gsatProdRate;
+}
+
+template <typename Scalar>
+std::optional<GSatProd::GSatProdGroup::Rate> WellGroupHelpers<Scalar>::
+selectRateComponent(const PhaseUsage& pu, const int phasePos)
+{
+    using Rate = GSatProd::GSatProdGroup::Rate;
+
+    for (const auto& [phase, rateComp] : std::array {
+        std::pair { BlackoilPhases::Aqua, Rate::Water },
+        std::pair { BlackoilPhases::Liquid, Rate::Oil },
+        std::pair { BlackoilPhases::Vapour, Rate::Gas } })
+    {
+        if (pu.phase_used[phase] && (pu.phase_pos[phase] == phasePos)) {
+            return rateComp;
+        }
+    }
+
+    return std::nullopt;
+}
 
 template<class Scalar>
 void WellGroupHelpers<Scalar>::
@@ -263,7 +313,8 @@ sumSolventRates(const Group& group,
             continue;
 
         const auto& ws = wellState.well(well_index.value());
-        const Scalar factor = wellEcl.getEfficiencyFactor();
+        const Scalar factor = wellEcl.getEfficiencyFactor() *
+                              wellState[wellEcl.name()].efficiency_scaling_factor;
         if (injector)
             rate += factor * ws.sum_solvent_rates();
         else
@@ -284,6 +335,7 @@ updateGuideRatesForInjectionGroups(const Group& group,
                                    GuideRate* guideRate,
                                    DeferredLogger& deferred_logger)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateGuideRatesForInjectionGroups(groupTmp, schedule, summaryState, pu, reportStepIdx, wellState, group_state, guideRate, deferred_logger);
@@ -294,14 +346,14 @@ updateGuideRatesForInjectionGroups(const Group& group,
         if(!group.hasInjectionControl(phase))
             continue;
 
-        Scalar guideRateValue = 0.0;
+        std::optional<Scalar> guideRateValue;
         const auto& controls = group.injectionControls(phase, summaryState);
         switch (controls.guide_rate_def){
         case Group::GuideRateInjTarget::RATE:
             break;
         case Group::GuideRateInjTarget::VOID:
         {
-            guideRateValue = group_state.injection_vrep_rate(group.name());
+            guideRateValue = std::max(Scalar(0.0), group_state.injection_vrep_rate(group.name()));
             break;
         }
         case Group::GuideRateInjTarget::NETV:
@@ -309,17 +361,17 @@ updateGuideRatesForInjectionGroups(const Group& group,
             guideRateValue = group_state.injection_vrep_rate(group.name());
             const std::vector<Scalar>& injRES = group_state.injection_reservoir_rates(group.name());
             if (phase != Phase::OIL && pu.phase_used[BlackoilPhases::Liquid])
-                guideRateValue -= injRES[pu.phase_pos[BlackoilPhases::Liquid]];
+                guideRateValue = *guideRateValue - injRES[pu.phase_pos[BlackoilPhases::Liquid]];
             if (phase != Phase::GAS && pu.phase_used[BlackoilPhases::Vapour])
-                guideRateValue -= injRES[pu.phase_pos[BlackoilPhases::Vapour]];
+                guideRateValue = *guideRateValue - injRES[pu.phase_pos[BlackoilPhases::Vapour]];
             if (phase != Phase::WATER && pu.phase_used[BlackoilPhases::Aqua])
-                guideRateValue -= injRES[pu.phase_pos[BlackoilPhases::Aqua]];
+                guideRateValue = *guideRateValue - injRES[pu.phase_pos[BlackoilPhases::Aqua]];
+
+            guideRateValue = std::max(Scalar(0.0), *guideRateValue);
             break;
         }
         case Group::GuideRateInjTarget::RESV:
             OPM_DEFLOG_THROW(std::runtime_error, "GUIDE PHASE RESV not implemented. Group " + group.name(), deferred_logger);
-        case Group::GuideRateInjTarget::POTN:
-            break;
         case Group::GuideRateInjTarget::NO_GUIDE_RATE:
             break;
         default:
@@ -329,7 +381,9 @@ updateGuideRatesForInjectionGroups(const Group& group,
         }
 
         const UnitSystem& unit_system = schedule.getUnits();
-        guideRateValue = unit_system.from_si(UnitSystem::measure::rate, guideRateValue);
+        if (guideRateValue) {
+            guideRateValue = unit_system.from_si(UnitSystem::measure::rate, *guideRateValue);
+        }
         guideRate->compute(group.name(), phase, reportStepIdx, guideRateValue);
     }
 }
@@ -343,9 +397,11 @@ updateGroupTargetReduction(const Group& group,
                            const PhaseUsage& pu,
                            const GuideRate& guide_rate,
                            const WellState<Scalar>& wellState,
+                           const SummaryState& summaryState,
                            GroupState<Scalar>& group_state,
                            std::vector<Scalar>& groupTargetReduction)
 {
+    OPM_TIMEFUNCTION();
     const int np = wellState.numPhases();
     for (const std::string& subGroupName : group.groups()) {
         std::vector<Scalar> subGroupTargetReduction(np, 0.0);
@@ -357,6 +413,7 @@ updateGroupTargetReduction(const Group& group,
                                    pu,
                                    guide_rate,
                                    wellState,
+                                   summaryState,
                                    group_state,
                                    subGroupTargetReduction);
 
@@ -398,7 +455,7 @@ updateGroupTargetReduction(const Group& group,
                 const bool individual_control = (currentGroupControl != Group::InjectionCMode::FLD
                         && currentGroupControl != Group::InjectionCMode::NONE);
                 const int num_group_controlled_wells
-                        = groupControlledWells(schedule, wellState, group_state, reportStepIdx, subGroupName, "", !isInjector, phase);
+                        = groupControlledWells(schedule, wellState, group_state, summaryState, &guide_rate, reportStepIdx, subGroupName, "", !isInjector, phase);
                 if (individual_control || num_group_controlled_wells == 0) {
                     groupTargetReduction[phase_pos]
                         += subGroupEfficiency * sumWellSurfaceRates(subGroup, schedule, wellState, reportStepIdx, phase_pos, isInjector);
@@ -414,7 +471,7 @@ updateGroupTargetReduction(const Group& group,
             const bool individual_control = (currentGroupControl != Group::ProductionCMode::FLD
                                              && currentGroupControl != Group::ProductionCMode::NONE);
             const int num_group_controlled_wells
-                = groupControlledWells(schedule, wellState, group_state, reportStepIdx, subGroupName, "", !isInjector, /*injectionPhaseNotUsed*/Phase::OIL);
+                = groupControlledWells(schedule, wellState, group_state, summaryState, &guide_rate, reportStepIdx, subGroupName, "", !isInjector, /*injectionPhaseNotUsed*/Phase::OIL);
             if (individual_control || num_group_controlled_wells == 0) {
                 for (int phase = 0; phase < np; phase++) {
                     groupTargetReduction[phase]
@@ -453,8 +510,10 @@ updateGroupTargetReduction(const Group& group,
             continue;
         }
 
-        const Scalar efficiency = wellTmp.getEfficiencyFactor();
-        // add contributino from wells not under group control
+        const Scalar efficiency = wellTmp.getEfficiencyFactor() *
+                                  wellState[wellTmp.name()].efficiency_scaling_factor;
+
+        // add contribution from wells not under group control
         const auto& ws = wellState.well(well_index.value());
         if (isInjector) {
             if (ws.injection_cmode != Well::InjectorCMode::GRUP)
@@ -462,10 +521,13 @@ updateGroupTargetReduction(const Group& group,
                     groupTargetReduction[phase] += ws.surface_rates[phase] * efficiency;
                 }
         } else {
-            if (ws.production_cmode != Well::ProducerCMode::GRUP)
-                for (int phase = 0; phase < np; phase++) {
-                    groupTargetReduction[phase] -= ws.surface_rates[phase] * efficiency;
+                if ((ws.production_cmode != Well::ProducerCMode::GRUP)){
+                    if (!group.as_choke()) {
+                        for (int phase = 0; phase < np; phase++) {
+                            groupTargetReduction[phase] -= ws.surface_rates[phase] * efficiency;
+                        }
                 }
+            }
         }
     }
     if (isInjector)
@@ -484,6 +546,7 @@ updateWellRatesFromGroupTargetScale(const Scalar scale,
                                     const GroupState<Scalar>& group_state,
                                     WellState<Scalar>& wellState)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         bool individual_control = false;
         if (isInjector) {
@@ -554,6 +617,7 @@ updateVREPForGroups(const Group& group,
                     const WellState<Scalar>& wellState,
                     GroupState<Scalar>& group_state)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateVREPForGroups(groupTmp, schedule, reportStepIdx, wellState, group_state);
@@ -580,6 +644,7 @@ updateReservoirRatesInjectionGroups(const Group& group,
                                     const WellState<Scalar>& wellState,
                                     GroupState<Scalar>& group_state)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateReservoirRatesInjectionGroups(groupTmp, schedule, reportStepIdx, wellState, group_state);
@@ -606,6 +671,7 @@ updateSurfaceRatesInjectionGroups(const Group& group,
                                   const WellState<Scalar>& wellState,
                                   GroupState<Scalar>& group_state)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateSurfaceRatesInjectionGroups(groupTmp, schedule, reportStepIdx, wellState, group_state);
@@ -632,6 +698,7 @@ updateWellRates(const Group& group,
                 const WellState<Scalar>& wellStateNupcol,
                 WellState<Scalar>& wellState)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateWellRates(groupTmp, schedule, reportStepIdx, wellStateNupcol, wellState);
@@ -664,6 +731,7 @@ updateGroupProductionRates(const Group& group,
                            const WellState<Scalar>& wellState,
                            GroupState<Scalar>& group_state)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateGroupProductionRates(groupTmp, schedule, reportStepIdx, wellState, group_state);
@@ -676,6 +744,31 @@ updateGroupProductionRates(const Group& group,
     group_state.update_production_rates(group.name(), rates);
 }
 
+template<class Scalar>
+void WellGroupHelpers<Scalar>::
+updateNetworkLeafNodeProductionRates(const Schedule& schedule,
+                                     const int reportStepIdx,
+                                     const WellState<Scalar>& wellState,
+                                     GroupState<Scalar>& group_state)
+{
+    const auto& network = schedule[reportStepIdx].network();
+    if (network.active()) {
+        const int np = wellState.numPhases();
+        for (const auto& group_name : network.leaf_nodes()) {
+            std::vector<Scalar> network_rates(np, 0.0);
+            if (schedule[reportStepIdx].groups.has(group_name)) { // Allow empty leaf nodes that are not groups
+                const auto& group = schedule[reportStepIdx].groups.get(group_name);
+                if (group.numWells() > 0) {
+                    for (int phase = 0; phase < np; ++phase) {
+                        network_rates[phase] = sumWellPhaseRates(false, group, schedule, wellState, reportStepIdx, phase, /*isInjector*/ false, /*network*/ true);
+                    }
+                }
+            }
+            group_state.update_network_leaf_node_production_rates(group_name, network_rates);
+        }
+    }
+}
+
 template<class Scalar>
 void WellGroupHelpers<Scalar>::
 updateREINForGroups(const Group& group,
@@ -687,6 +780,7 @@ updateREINForGroups(const Group& group,
                     GroupState<Scalar>& group_state,
                     bool sum_rank)
 {
+    OPM_TIMEFUNCTION();
     const int np = wellState.numPhases();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
@@ -700,12 +794,10 @@ updateREINForGroups(const Group& group,
 
     // add import rate and subtract consumption rate for group for gas
     if (sum_rank) {
-        if (schedule[reportStepIdx].gconsump().has(group.name())) {
-            const auto& gconsump = schedule[reportStepIdx].gconsump().get(group.name(), st);
-            if (pu.phase_used[BlackoilPhases::Vapour]) {
-                rein[pu.phase_pos[BlackoilPhases::Vapour]] += gconsump.import_rate;
-                rein[pu.phase_pos[BlackoilPhases::Vapour]] -= gconsump.consumption_rate;
-            }
+        if (pu.phase_used[BlackoilPhases::Vapour]) {
+            const auto& [consumption_rate, import_rate] = group_state.gconsump_rates(group.name());
+            rein[pu.phase_pos[BlackoilPhases::Vapour]] += import_rate;
+            rein[pu.phase_pos[BlackoilPhases::Vapour]] -= consumption_rate;
         }
     }
 
@@ -723,6 +815,7 @@ updateGpMaintTargetForGroups(const Group& group,
                              const WellState<Scalar>& well_state,
                              GroupState<Scalar>& group_state)
 {
+    OPM_TIMEFUNCTION();
     for (const std::string& groupName : group.groups()) {
         const Group& groupTmp = schedule.getGroup(groupName, reportStepIdx);
         updateGpMaintTargetForGroups(groupTmp, schedule, regional_values, reportStepIdx, dt, well_state, group_state);
@@ -817,7 +910,7 @@ computeNetworkPressures(const Network::ExtNetwork& network,
                         const int report_time_step)
 {
     // TODO: Only dealing with production networks for now.
-
+    OPM_TIMEFUNCTION();
     if (!network.active()) {
         return {};
     }
@@ -852,23 +945,23 @@ computeNetworkPressures(const Network::ExtNetwork& network,
         // Starting with the leaf nodes of the network, get the flow rates
         // from the corresponding groups.
         std::map<std::string, std::vector<Scalar>> node_inflows;
+        const std::vector<Scalar> zero_rates(3, 0.0);
         for (const auto& node : leaf_nodes) {
-            node_inflows[node] = group_state.production_rates(node);
+            // Guard against empty leaf nodes (may not be present in GRUPTREE)
+            if (!group_state.has_production_rates(node)) {
+                node_inflows[node] = zero_rates;
+                continue;
+            }
+
+            node_inflows[node] = group_state.network_leaf_node_production_rates(node);
             // Add the ALQ amounts to the gas rates if requested.
             if (network.node(node).add_gas_lift_gas()) {
                 const auto& group = schedule.getGroup(node, report_time_step);
                 for (const std::string& wellname : group.wells()) {
                     const Well& well = schedule.getWell(wellname, report_time_step);
+
                     if (well.isInjector() || !well_state.isOpen(wellname)) continue;
-                    // Here we use the efficiency unconditionally, but if WEFAC item 3
-                    // for the well is false (it defaults to true) then we should NOT use
-                    // the efficiency factor. Fixing this requires not only changing the
-                    // code here, but also:
-                    //    - Adding a member to the well for this flag, and setting it in Schedule::handleWEFAC().
-                    //    - Making the wells' maximum flows (i.e. not time-averaged by using a efficiency factor)
-                    //      available and using those (for wells with WEFAC(3) true only) when accumulating group
-                    //      rates, but ONLY for network calculations.
-                    const Scalar efficiency = well.getEfficiencyFactor();
+                    const Scalar efficiency = well.getEfficiencyFactor(/*network*/ true) * well_state.getGlobalEfficiencyScalingFactor(wellname);
                     node_inflows[node][BlackoilPhases::Vapour] += well_state.getALQ(wellname) * efficiency;
                 }
             }
@@ -886,13 +979,14 @@ computeNetworkPressures(const Network::ExtNetwork& network,
                 // Add downbranch rates to upbranch.
                 std::vector<Scalar>& up = node_inflows[(*upbranch).uptree_node()];
                 const std::vector<Scalar>& down = node_inflows[node];
+                // We now also support NEFAC
+                const Scalar efficiency = network.node(node).efficiency();
                 if (up.empty()) {
-                    up = down;
-                } else {
-                    assert (up.size() == down.size());
-                    for (std::size_t ii = 0; ii < up.size(); ++ii) {
-                        up[ii] += down[ii];
-                    }
+                    up = std::vector<Scalar>(down.size(), 0.0);
+                }
+                assert (up.size() == down.size());
+                for (std::size_t ii = 0; ii < up.size(); ++ii) {
+                    up[ii] += efficiency*down[ii];
                 }
             }
         }
@@ -910,6 +1004,7 @@ computeNetworkPressures(const Network::ExtNetwork& network,
                 const Scalar up_press = node_pressures[(*upbranch).uptree_node()];
                 const auto vfp_table = (*upbranch).vfp_table();
                 if (vfp_table) {
+                    OPM_TIMEBLOCK(NetworkVfpCalculations);
                     // The rates are here positive, but the VFP code expects the
                     // convention that production rates are negative, so we must
                     // take a copy and flip signs.
@@ -1086,12 +1181,15 @@ int WellGroupHelpers<Scalar>::
 groupControlledWells(const Schedule& schedule,
                      const WellState<Scalar>& well_state,
                      const GroupState<Scalar>& group_state,
+                     const SummaryState& summary_state,
+                     const GuideRate* guideRate,
                      const int report_step,
                      const std::string& group_name,
                      const std::string& always_included_child,
                      const bool is_production_group,
                      const Phase injection_phase)
 {
+    OPM_TIMEFUNCTION();
     const Group& group = schedule.getGroup(group_name, report_step);
     int num_wells = 0;
     for (const std::string& child_group : group.groups()) {
@@ -1107,16 +1205,81 @@ groupControlledWells(const Schedule& schedule,
 
         if (included) {
             num_wells
-                += groupControlledWells(schedule, well_state, group_state, report_step, child_group, always_included_child, is_production_group, injection_phase);
+                += groupControlledWells(schedule, well_state, group_state, summary_state, guideRate, report_step, child_group, always_included_child, is_production_group, injection_phase);
         }
     }
     for (const std::string& child_well : group.wells()) {
         bool included = (child_well == always_included_child);
         if (is_production_group) {
-            included = included || well_state.isProductionGrup(child_well);
+                included = included || well_state.isProductionGrup(child_well) || group.as_choke();
         } else {
             included = included || well_state.isInjectionGrup(child_well);
         }
+        const auto ctrl1 = group_state.production_control(group.name());
+        if (group.as_choke() && ((ctrl1 == Group::ProductionCMode::FLD) || (ctrl1 == Group::ProductionCMode::NONE))){
+            // The auto choke group has not own group control but inherits control from an ancestor group.
+            // Number of wells should be calculated as zero when wells of auto choke group do not deliver target.
+            // This behaviour is then similar to no-autochoke group with wells not on GRUP control.
+            // The rates of these wells are summed up. The parent group target is reduced with this rate.
+            // This reduced target becomes the target of the other child group of this parent.
+            const PhaseUsage& pu = well_state.phaseUsage();
+            std::vector<Scalar> rates(pu.num_phases, 0.0);
+            for (int phase_pos = 0; phase_pos < pu.num_phases; ++phase_pos) {
+                 rates[phase_pos] = WellGroupHelpers<Scalar>::sumWellSurfaceRates(group,
+                                                                                  schedule,
+                                                                                  well_state,
+                                                                                  report_step,
+                                                                                  phase_pos,
+                                                                                  false);
+            }
+
+            // Get the ancestor of the auto choke group that has group control (cmode != FLD, NONE)
+            const auto& control_group_name = control_group(group, group_state, report_step, schedule);
+            const auto& control_group = schedule.getGroup(control_group_name, report_step);
+            const auto& ctrl = control_group.productionControls(summary_state);
+            const auto& control_group_cmode = ctrl.cmode;
+
+            const auto& group_guide_rate = group.productionControls(summary_state).guide_rate;
+
+            if (group_guide_rate > 0) {
+                // Guide rate is not default for the auto choke group
+                Scalar gratTargetFromSales = 0.0;
+                if (group_state.has_grat_sales_target(control_group_name))
+                    gratTargetFromSales = group_state.grat_sales_target(control_group_name);
+
+                std::vector<Scalar> resv_coeff(pu.num_phases, 1.0);
+                WGHelpers::TargetCalculator tcalc(control_group_cmode,
+                                                pu,
+                                                resv_coeff,
+                                                gratTargetFromSales,
+                                                group.name(),
+                                                group_state,
+                                                group.has_gpmaint_control(control_group_cmode));
+                auto deferred_logger = Opm::DeferredLogger();
+                const auto& control_group_target = tcalc.groupTarget(ctrl, deferred_logger);
+
+                // Calculates the guide rate of the parent group with control. 
+                // It is allowed that the guide rate of this group is defaulted. The guide rate will be derived from the children groups 
+                const auto& control_group_guide_rate = getGuideRate(control_group_name,
+                                                    schedule,
+                                                    well_state,
+                                                    group_state,
+                                                    report_step,
+                                                    guideRate,
+                                                    tcalc.guideTargetMode(),
+                                                    pu);
+
+                if (control_group_guide_rate > 0) {
+                    // Target rate for the auto choke group
+                    const Scalar target_rate = control_group_target * group_guide_rate / control_group_guide_rate;
+                    const Scalar current_rate = tcalc.calcModeRateFromRates(rates);
+
+                    if (current_rate < target_rate)
+                        included = false;
+                }
+            }
+        }
+
         if (included) {
             ++num_wells;
         }
@@ -1155,6 +1318,30 @@ groupChainTopBot(const std::string& bottom,
     return chain;
 }
 
+template<class Scalar>
+std::string
+WellGroupHelpers<Scalar>::
+control_group(const Group& group,
+              const GroupState<Scalar>& group_state,
+              const int reportStepIdx,
+              const Schedule& schedule)
+{
+    const Group::ProductionCMode& currentGroupControl = group_state.production_control(group.name());
+
+    if (currentGroupControl == Group::ProductionCMode::FLD || currentGroupControl == Group::ProductionCMode::NONE) {
+        const auto& parent_name = group.control_group();
+        if (parent_name) {
+            const auto& parent = schedule.getGroup(parent_name.value(), reportStepIdx);
+            return control_group(parent,
+                                group_state,
+                                reportStepIdx,
+                                schedule);
+        }
+    }
+
+    return group.name();
+}
+
 template<class Scalar>
 std::pair<bool, Scalar>
 WellGroupHelpers<Scalar>::
@@ -1181,6 +1368,9 @@ checkGroupConstraintsProd(const std::string& name,
     // part of. Later it is the accumulated factor including the group efficiency factor
     // of the child of group.
 
+    // NOTE: if name is a group, the rates-argument is the portion of the 'full' group rates that
+    // is potentially available for control by an ancestor, i.e. full rates minus reduction rates
+    OPM_TIMEFUNCTION();
     const Group::ProductionCMode& currentGroupControl = group_state.production_control(group.name());
 
     if (currentGroupControl == Group::ProductionCMode::FLD || currentGroupControl == Group::ProductionCMode::NONE) {
@@ -1232,6 +1422,7 @@ checkGroupConstraintsProd(const std::string& name,
     WGHelpers::FractionCalculator fcalc(schedule,
                                         wellState,
                                         group_state,
+                                        summaryState,
                                         reportStepIdx,
                                         guideRate,
                                         tcalc.guideTargetMode(),
@@ -1263,7 +1454,7 @@ checkGroupConstraintsProd(const std::string& name,
     // Assume we have a chain of groups as follows: BOTTOM -> MIDDLE -> TOP.
     // Then ...
     // TODO finish explanation.
-    const Scalar current_rate
+    const Scalar current_rate_available
         = -tcalc.calcModeRateFromRates(rates); // Switch sign since 'rates' are negative for producers.
     const auto chain = groupChainTopBot(name, group.name(), schedule, reportStepIdx);
     // Because 'name' is the last of the elements, and not an ancestor, we subtract one below.
@@ -1274,6 +1465,8 @@ checkGroupConstraintsProd(const std::string& name,
         const int num_gr_ctrl = groupControlledWells(schedule,
                                                      wellState,
                                                      group_state,
+                                                     summaryState,
+                                                     guideRate,
                                                      reportStepIdx,
                                                      chain[ii],
                                                      "",
@@ -1298,6 +1491,7 @@ checkGroupConstraintsProd(const std::string& name,
             }
         }
     }
+    // Compute portion of target corresponding to current_rate_available
     Scalar target = orig_target;
     for (std::size_t ii = 0; ii < num_ancestors; ++ii) {
         if ((ii == 0) || guideRate->has(chain[ii])) {
@@ -1309,18 +1503,18 @@ checkGroupConstraintsProd(const std::string& name,
             }
             // Add my reduction back at the level where it is included in the local reduction
             if (local_reduction_level == ii ) {
-                target += current_rate * efficiencyFactor;
+                target += current_rate_available * efficiencyFactor;
             }
         }
         target *= localFraction(chain[ii + 1]);
     }
     // Avoid negative target rates comming from too large local reductions.
-    const Scalar target_rate = std::max(Scalar(1e-12), target / efficiencyFactor);
+    const Scalar target_rate_available = std::max(Scalar(1e-12), target / efficiencyFactor);
     Scalar scale = 1.0;
-    if (current_rate > 1e-12)
-        scale = target_rate / current_rate;
+    if (current_rate_available > 1e-12)
+        scale = target_rate_available / current_rate_available;
 
-    return std::make_pair(current_rate > target_rate, scale);
+    return std::make_pair(current_rate_available > target_rate_available, scale);
 }
 
 template<class Scalar>
@@ -1350,6 +1544,9 @@ checkGroupConstraintsInj(const std::string& name,
     // part of. Later it is the accumulated factor including the group efficiency factor
     // of the child of group.
 
+    // NOTE: if name is a group, the rates-argument is the portion of the 'full' group rates that
+    // is potentially available for control by an ancestor, i.e. full rates minus reduction rates
+
     auto currentGroupControl = group_state.injection_control(group.name(), injectionPhase);
 
     if (currentGroupControl == Group::InjectionCMode::FLD || currentGroupControl == Group::InjectionCMode::NONE) {
@@ -1403,6 +1600,7 @@ checkGroupConstraintsInj(const std::string& name,
     WGHelpers::FractionCalculator fcalc(schedule,
                                         wellState,
                                         group_state,
+                                        summaryState,
                                         reportStepIdx,
                                         guideRate,
                                         tcalc.guideTargetMode(),
@@ -1435,7 +1633,7 @@ checkGroupConstraintsInj(const std::string& name,
     // Assume we have a chain of groups as follows: BOTTOM -> MIDDLE -> TOP.
     // Then ...
     // TODO finish explanation.
-    const Scalar current_rate
+    const Scalar current_rate_available
         = tcalc.calcModeRateFromRates(rates); // Switch sign since 'rates' are negative for producers.
     const auto chain = groupChainTopBot(name, group.name(), schedule, reportStepIdx);
     // Because 'name' is the last of the elements, and not an ancestor, we subtract one below.
@@ -1445,12 +1643,14 @@ checkGroupConstraintsInj(const std::string& name,
     for (std::size_t ii = 1; ii < num_ancestors; ++ii) {
         const int num_gr_ctrl = groupControlledWells(schedule,
                                                      wellState,
-                                                             group_state,
-                                                             reportStepIdx,
-                                                             chain[ii],
-                                                             "",
-                                                             /*is_producer*/ false,
-                                                             injectionPhase);
+                                                     group_state,
+                                                     summaryState,
+                                                     guideRate,
+                                                     reportStepIdx,
+                                                     chain[ii],
+                                                     "",
+                                                     /*is_producer*/ false,
+                                                     injectionPhase);
         if (guideRate->has(chain[ii], injectionPhase) && num_gr_ctrl > 0) {
             local_reduction_level = ii;
         }
@@ -1473,6 +1673,7 @@ checkGroupConstraintsInj(const std::string& name,
         }
     }
 
+    // Compute portion of target corresponding to current_rate_available
     Scalar target = orig_target;
     for (std::size_t ii = 0; ii < num_ancestors; ++ii) {
         if ((ii == 0) || guideRate->has(chain[ii], injectionPhase)) {
@@ -1485,18 +1686,18 @@ checkGroupConstraintsInj(const std::string& name,
 
             // Add my reduction back at the level where it is included in the local reduction
             if (local_reduction_level == ii ) {
-                target += current_rate * efficiencyFactor;
+                target += current_rate_available * efficiencyFactor;
             }
         }
         target *= localFraction(chain[ii + 1]);
     }
     // Avoid negative target rates comming from too large local reductions.
-    const Scalar target_rate = std::max(Scalar(1e-12), target / efficiencyFactor);
+    const Scalar target_rate_available = std::max(Scalar(1e-12), target / efficiencyFactor);
     Scalar scale = 1.0;
-    if (current_rate > 1e-12)
-        scale = target_rate / current_rate;
+    if (current_rate_available > 1e-12)
+        scale = target_rate_available / current_rate_available;
 
-    return std::make_pair(current_rate > target_rate, scale);
+    return std::make_pair(current_rate_available > target_rate_available, scale);
 }
 
 template<class Scalar>
@@ -1643,6 +1844,7 @@ updateGuideRates(const Group& group,
                  std::vector<Scalar>& pot,
                  DeferredLogger& deferred_logger)
 {
+    OPM_TIMEFUNCTION();
     guide_rate->updateGuideRateExpiration(sim_time, report_step);
     updateGuideRateForProductionGroups(group, schedule, pu, report_step, sim_time, well_state, group_state, comm, guide_rate, pot);
     updateGuideRatesForInjectionGroups(group, schedule, summary_state, pu, report_step, well_state, group_state, guide_rate, deferred_logger);
@@ -1662,6 +1864,7 @@ updateGuideRateForProductionGroups(const Group& group,
                                    GuideRate* guideRate,
                                    std::vector<Scalar>& pot)
 {
+    OPM_TIMEFUNCTION();
     const int np = pu.num_phases;
     for (const std::string& groupName : group.groups()) {
         std::vector<Scalar> thisPot(np, 0.0);
@@ -1739,6 +1942,7 @@ updateGuideRatesForWells(const Schedule& schedule,
                          const Parallel::Communication& comm,
                          GuideRate* guideRate)
 {
+    OPM_TIMEFUNCTION();
     for (const auto& well : schedule.getWells(reportStepIdx)) {
         std::array<Scalar,3> potentials{};
         auto& [oilpot, gaspot, waterpot] = potentials;
