@@ -22,8 +22,10 @@
 #define OPM_WELLGROUPHELPERS_HEADER_INCLUDED
 
 #include <opm/input/eclipse/Schedule/Group/GuideRate.hpp>
+#include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/simulators/utils/ParallelCommunication.hpp>
+#include <opm/input/eclipse/Schedule/ScheduleState.hpp>
 
 #include <map>
 #include <string>
@@ -47,6 +49,23 @@ template<class Scalar>
 class WellGroupHelpers
 {
 public:
+
+    static Scalar sumWellPhaseRates(bool res_rates,
+                                    const Opm::Group& group,
+                                    const Opm::Schedule& schedule,
+                                    const Opm::WellState<Scalar>& wellState,
+                                    const int reportStepIdx,
+                                    const int phasePos,
+                                    const bool injector,
+                                    const bool network = false);
+
+    static Scalar satelliteProduction(const ScheduleState& sched,
+                                      const std::vector<std::string>& groups,
+                                      const GSatProd::GSatProdGroup::Rate rateComp);
+
+    static std::optional<GSatProd::GSatProdGroup::Rate>
+    selectRateComponent(const PhaseUsage& pu, const int phasePos);
+
     static void setCmodeGroup(const Group& group,
                               const Schedule& schedule,
                               const SummaryState& summaryState,
@@ -96,6 +115,7 @@ public:
                                            const PhaseUsage& pu,
                                            const GuideRate& guide_rate,
                                            const WellState<Scalar>& wellState,
+                                           const SummaryState& summaryState,
                                            GroupState<Scalar>& group_state,
                                            std::vector<Scalar>& groupTargetReduction);
 
@@ -180,6 +200,12 @@ public:
                                            const WellState<Scalar>& wellState,
                                            GroupState<Scalar>& group_state);
 
+    static void updateNetworkLeafNodeProductionRates(const Schedule& schedule,
+                                                     const int reportStepIdx,
+                                                     const WellState<Scalar>& wellState,
+                                                     GroupState<Scalar>& group_state);
+
+
     static void updateWellRatesFromGroupTargetScale(const Scalar scale,
                                                     const Group& group,
                                                     const Schedule& schedule,
@@ -238,6 +264,8 @@ public:
     static int groupControlledWells(const Schedule& schedule,
                                     const WellState<Scalar>& well_state,
                                     const GroupState<Scalar>& group_state,
+                                    const SummaryState& summary_state,
+                                    const GuideRate* guideRate,
                                     const int report_step,
                                     const std::string& group_name,
                                     const std::string& always_included_child,
@@ -267,6 +295,12 @@ public:
                      const Schedule& schedule,
                      const int report_step);
 
+    static std::string
+    control_group(const Group& group,
+                  const GroupState<Scalar>& group_state,
+                  const int reportStepIdx,
+                  const Schedule& schedule);
+
     static std::pair<bool, Scalar>
     checkGroupConstraintsProd(const std::string& name,
                               const std::string& parent,
