@@ -37,6 +37,12 @@ namespace Opm {
 
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 
+#include <opm/material/fluidstates/BlackOilFluidState.hpp>
+
+#include <opm/models/blackoil/blackoilproperties.hh>
+
+#include <opm/simulators/linalg/linalgproperties.hh>
+
 #include <opm/simulators/wells/BlackoilWellModel.hpp>
 #include <opm/simulators/wells/GasLiftGroupInfo.hpp>
 #include <opm/simulators/wells/GasLiftSingleWell.hpp>
@@ -82,11 +88,7 @@ public:
     using SparseMatrixAdapter = GetPropType<TypeTag, Properties::SparseMatrixAdapter>;
     using RateVector = GetPropType<TypeTag, Properties::RateVector>;
     using GasLiftSingleWell = ::Opm::GasLiftSingleWell<TypeTag>;
-    using GLiftOptWells = typename BlackoilWellModel<TypeTag>::GLiftOptWells;
-    using GLiftProdWells = typename BlackoilWellModel<TypeTag>::GLiftProdWells;
-    using GLiftWellStateMap =
-        typename BlackoilWellModel<TypeTag>::GLiftWellStateMap;
-    using GLiftSyncGroups = typename GasLiftSingleWellGeneric<Scalar>::GLiftSyncGroups;
+    using GLiftEclWells = typename GasLiftGroupInfo<Scalar>::GLiftEclWells;
 
     using VectorBlockType = Dune::FieldVector<Scalar, Indices::numEq>;
     using MatrixBlockType = Dune::FieldMatrix<Scalar, Indices::numEq, Indices::numEq>;
@@ -149,8 +151,6 @@ public:
                       const std::vector<Scalar>& B_avg,
                       const bool changed_to_open_this_step);
 
-    virtual void initPrimaryVariablesEvaluation() = 0;
-
     virtual ConvergenceReport getWellConvergence(const Simulator& simulator,
                                                  const WellState<Scalar>& well_state,
                                                  const std::vector<Scalar>& B_avg,
@@ -190,7 +190,8 @@ public:
     computeBhpAtThpLimitProdWithAlq(const Simulator& ebos_simulator,
                                     const SummaryState& summary_state,
                                     const Scalar alq_value,
-                                    DeferredLogger& deferred_logger) const = 0;
+                                    DeferredLogger& deferred_logger,
+                                    bool iterate_if_no_solution) const = 0;
 
     /// using the solution x to recover the solution xw for wells and applying
     /// xw to update Well State
@@ -268,9 +269,6 @@ public:
                                          WellState<Scalar>& well_state,
                                          DeferredLogger& deferred_logger) const = 0;
 
-    virtual Scalar connectionDensity(const int globalConnIdx,
-                                     const int openConnIdx) const = 0;
-
     /// \brief Wether the Jacobian will also have well contributions in it.
     virtual bool jacobianContainsWellContributions() const
     {
@@ -297,6 +295,9 @@ public:
                      /* const */ WellState<Scalar>& well_state,
                      const GroupState<Scalar>& group_state,
                      WellTestState& welltest_state,
+                     const PhaseUsage& phase_usage,
+                     GLiftEclWells& ecl_well_map,
+                     std::map<std::string, double>& open_times,
                      DeferredLogger& deferred_logger);
 
     void checkWellOperability(const Simulator& simulator,
@@ -311,6 +312,9 @@ public:
 
     void gliftBeginTimeStepWellTestUpdateALQ(const Simulator& simulator,
                                              WellState<Scalar>& well_state,
+                                             const GroupState<Scalar>& group_state,
+                                             const PhaseUsage& phase_usage,
+                                             GLiftEclWells& ecl_well_map,
                                              DeferredLogger& deferred_logger);
 
     // check whether the well is operable under the current reservoir condition
@@ -350,16 +354,6 @@ public:
         return connectionRates_;
     }
 
-    virtual std::vector<Scalar> getPrimaryVars() const
-    {
-        return {};
-    }
-
-    virtual int setPrimaryVars(typename std::vector<Scalar>::const_iterator)
-    {
-        return 0;
-    }
-
     std::vector<Scalar> wellIndex(const int perf,
                                   const IntensiveQuantities& intQuants,
                                   const Scalar trans_mult,
@@ -473,6 +467,15 @@ protected:
                              WellState<Scalar>& well_state,
                              const GroupState<Scalar>& group_state,
                              DeferredLogger& deferred_logger);
+    
+
+    template<class GasLiftSingleWell>
+    std::unique_ptr<GasLiftSingleWell> initializeGliftWellTest_(const Simulator& simulator,
+                                                                WellState<Scalar>& well_state,
+                                                                const GroupState<Scalar>& group_state,
+                                                                const PhaseUsage& phase_usage,
+                                                                GLiftEclWells& ecl_well_map,
+                                                                DeferredLogger& deferred_logger);
 
     Eval getPerfCellPressure(const FluidState& fs) const;
 
@@ -503,8 +506,6 @@ protected:
 
 } // namespace Opm
 
-#ifndef OPM_WELLINTERFACE_IMPL_HEADER_INCLUDED
 #include "WellInterface_impl.hpp"
-#endif
 
 #endif // OPM_WELLINTERFACE_HEADER_INCLUDED
