@@ -19,10 +19,12 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifndef OPM_WELLINTERFACE_IMPL_HEADER_INCLUDED
+#define OPM_WELLINTERFACE_IMPL_HEADER_INCLUDED
+
 // Improve IDE experience
 #ifndef OPM_WELLINTERFACE_HEADER_INCLUDED
 #include <config.h>
-#define OPM_WELLINTERFACE_IMPL_HEADER_INCLUDED
 #include <opm/simulators/wells/WellInterface.hpp>
 #endif
 
@@ -74,7 +76,7 @@ namespace Opm
                                                   index_of_well,
                                                   perf_data)
     {
-        connectionRates_.resize(this->number_of_perforations_);
+        connectionRates_.resize(this->number_of_local_perforations_);
 
         if constexpr (has_solvent || has_zFraction) {
             if (well.isInjector()) {
@@ -171,12 +173,6 @@ namespace Opm
       return 0.0;
     }
 
-    // The urea injection concentration is scaled down by a factor of 10, since its value
-    // can be much bigger than 1 (not doing this slows the simulations). The
-    // corresponding values are scaled accordingly in blackoilmicpmodules.hh when computing
-    // the reactions and also when writing the output files (vtk and eclipse format, i.e.,
-    // vtkblackoilmicpmodule.hh and ecloutputblackoilmodel.hh respectively).
-
     template<typename TypeTag>
     typename WellInterface<TypeTag>::Scalar
     WellInterface<TypeTag>::
@@ -198,6 +194,7 @@ namespace Opm
                       const GroupState<Scalar>& group_state,
                       DeferredLogger& deferred_logger) /* const */
     {
+        OPM_TIMEFUNCTION();
         if (stoppedOrZeroRateTarget(simulator, well_state, deferred_logger)) {
             return false;
         }
@@ -213,8 +210,10 @@ namespace Opm
             from = WellProducerCMode2String(ws.production_cmode);
         }
         bool oscillating = std::count(this->well_control_log_.begin(), this->well_control_log_.end(), from) >= this->param_.max_number_of_well_switches_;
-
-        if (oscillating) {
+        const int episodeIdx = simulator.episodeIndex();
+        const int iterationIdx = simulator.model().newtonMethod().numIterations();
+        const int nupcol = schedule[episodeIdx].nupcol();
+        if (oscillating && iterationIdx > nupcol) {
             // only output frist time
             bool output = std::count(this->well_control_log_.begin(), this->well_control_log_.end(), from) == this->param_.max_number_of_well_switches_;
             if (output) {
@@ -278,6 +277,7 @@ namespace Opm
                                              const bool fixed_control, 
                                              const bool fixed_status)
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
         const auto& schedule = simulator.vanguard().schedule();
         auto& ws = well_state.well(this->index_of_well_);
@@ -307,8 +307,10 @@ namespace Opm
                     
                     const bool hasGroupControl = this->isInjector() ? inj_controls.hasControl(Well::InjectorCMode::GRUP) :
                                                                       prod_controls.hasControl(Well::ProducerCMode::GRUP);
-
-                    changed = this->checkIndividualConstraints(ws, summary_state, deferred_logger, inj_controls, prod_controls);
+                    bool isGroupControl = ws.production_cmode == Well::ProducerCMode::GRUP || ws.injection_cmode == Well::InjectorCMode::GRUP; 
+                    if (! (isGroupControl && !this->param_.check_group_constraints_inner_well_iterations_)) {
+                        changed = this->checkIndividualConstraints(ws, summary_state, deferred_logger, inj_controls, prod_controls);
+                    }
                     if (hasGroupControl && this->param_.check_group_constraints_inner_well_iterations_) {
                         changed = changed || this->checkGroupConstraints(well_state, group_state, schedule, summary_state,deferred_logger);
                     }
@@ -378,24 +380,41 @@ namespace Opm
                 /* const */ WellState<Scalar>& well_state,
                 const GroupState<Scalar>& group_state,
                 WellTestState& well_test_state,
+                const PhaseUsage& phase_usage,
+                GLiftEclWells& ecl_well_map,
+                std::map<std::string, double>& open_times,
                 DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         deferred_logger.info(" well " + this->name() + " is being tested");
 
         WellState<Scalar> well_state_copy = well_state;
         auto& ws = well_state_copy.well(this->indexOfWell());
 
+        const auto& summary_state = simulator.vanguard().summaryState();
+        const bool has_thp_limit = this->wellHasTHPConstraints(summary_state);
+        if (has_thp_limit) {
+            ws.production_cmode = Well::ProducerCMode::THP;
+        }
+        else {
+            ws.production_cmode = Well::ProducerCMode::BHP;
+        }
+
         updateWellStateWithTarget(simulator, group_state, well_state_copy, deferred_logger);
         calculateExplicitQuantities(simulator, well_state_copy, deferred_logger);
         updatePrimaryVariables(simulator, well_state_copy, deferred_logger);
-        initPrimaryVariablesEvaluation();
 
         if (this->isProducer()) {
             const auto& schedule = simulator.vanguard().schedule();
             const auto report_step = simulator.episodeIndex();
             const auto& glo = schedule.glo(report_step);
             if (glo.active()) {
-                gliftBeginTimeStepWellTestUpdateALQ(simulator, well_state_copy, deferred_logger);
+                gliftBeginTimeStepWellTestUpdateALQ(simulator, 
+                                                    well_state_copy, 
+                                                    group_state,
+                                                    phase_usage, 
+                                                    ecl_well_map, 
+                                                    deferred_logger);
             }
         }
 
@@ -462,13 +481,14 @@ namespace Opm
             deferred_logger.info(msg);
 
             // also reopen completions
-            for (auto& completion : this->well_ecl_.getCompletions()) {
+            for (const auto& completion : this->well_ecl_.getCompletions()) {
                 if (!welltest_state_temp.completion_is_closed(this->name(), completion.first))
                     well_test_state.open_completion(this->name(), completion.first);
             }
             // set the status of the well_state to open
             ws.open();
             well_state = well_state_copy;
+            open_times.try_emplace(this->name(), well_test_state.lastTestTime(this->name()));
         }
     }
 
@@ -484,6 +504,7 @@ namespace Opm
                          const GroupState<Scalar>& group_state,
                          DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
         const auto inj_controls = this->well_ecl_.isInjector() ? this->well_ecl_.injectionControls(summary_state) : Well::InjectionControls(0);
         const auto prod_controls = this->well_ecl_.isProducer() ? this->well_ecl_.productionControls(summary_state) : Well::ProductionControls(0);
@@ -519,6 +540,7 @@ namespace Opm
                                const GroupState<Scalar>& group_state,
                                DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
         bool is_operable = true;
         bool converged = true;
@@ -614,6 +636,7 @@ namespace Opm
                         const SummaryState& summary_state,
                         DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         // Given an unconverged well or closed well, estimate an operable bhp (if any)
         // Get minimal bhp from vfp-curve
         Scalar bhp_min =  WellBhpThpCalculator(*this).calculateMinimumBhpFromThp(well_state, this->well_ecl_, summary_state, this->getRefDensity());
@@ -637,6 +660,7 @@ namespace Opm
                      WellState<Scalar>& well_state,
                      DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         // Solve a well using single bhp-constraint (but close if not operable under this)
         auto group_state = GroupState<Scalar>(); // empty group
         auto inj_controls = Well::InjectionControls(0);
@@ -672,6 +696,7 @@ namespace Opm
                           WellState<Scalar>& well_state,
                           DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         // Solve a well as stopped
         const auto well_status_orig = this->wellStatus_;
         this->stopWell();
@@ -692,21 +717,9 @@ namespace Opm
                         const GroupState<Scalar>& group_state,
                         DeferredLogger& deferred_logger)
     {
-        // keep a copy of the original well state
-        const WellState<Scalar> well_state0 = well_state;
+        OPM_TIMEFUNCTION();
         const double dt = simulator.timeStepSize();
-        const auto& summary_state = simulator.vanguard().summaryState();
-        const bool has_thp_limit = this->wellHasTHPConstraints(summary_state);
-        bool converged;
-        if (has_thp_limit) {
-            well_state.well(this->indexOfWell()).production_cmode = Well::ProducerCMode::THP;
-            converged = gliftBeginTimeStepWellTestIterateWellEquations(
-                simulator, dt, well_state, group_state, deferred_logger);
-        }
-        else {
-            well_state.well(this->indexOfWell()).production_cmode = Well::ProducerCMode::BHP;
-            converged = iterateWellEquations(simulator, dt, well_state, group_state, deferred_logger);
-        }
+        const bool converged = iterateWellEquations(simulator, dt, well_state, group_state, deferred_logger);
         if (converged) {
             deferred_logger.debug("WellTest: Well equation for well " + this->name() +  " converged");
             return true;
@@ -714,7 +727,6 @@ namespace Opm
         const int max_iter = this->param_.max_welleq_iter_;
         deferred_logger.debug("WellTest: Well equation for well " + this->name() + " failed converging in "
                               + std::to_string(max_iter) + " iterations");
-        well_state = well_state0;
         return false;
     }
 
@@ -727,6 +739,7 @@ namespace Opm
                       const GroupState<Scalar>& group_state,
                       DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         if (!this->isOperableAndSolvable() && !this->wellIsStopped())
             return;
 
@@ -785,6 +798,7 @@ namespace Opm
                    const GroupState<Scalar>& group_state,
                    DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         prepareWellBeforeAssembling(simulator, dt, well_state, group_state, deferred_logger);
         assembleWellEqWithoutIteration(simulator, dt, well_state, group_state, deferred_logger);
     }
@@ -800,6 +814,7 @@ namespace Opm
                                    const GroupState<Scalar>& group_state,
                                    DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
         const auto inj_controls = this->well_ecl_.isInjector() ? this->well_ecl_.injectionControls(summary_state) : Well::InjectionControls(0);
         const auto prod_controls = this->well_ecl_.isProducer() ? this->well_ecl_.productionControls(summary_state) : Well::ProductionControls(0);
@@ -819,6 +834,7 @@ namespace Opm
                                 const GroupState<Scalar>& group_state,
                                 DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const bool old_well_operable = this->operability_status_.isOperableAndSolvable();
 
         if (this->param_.check_well_operability_iter_)
@@ -890,11 +906,11 @@ namespace Opm
         const bool well_operable = this->operability_status_.isOperableAndSolvable();
 
         if (!well_operable && old_well_operable) {
-            deferred_logger.info(" well " + this->name() + " gets STOPPED during iteration ");
+            deferred_logger.debug(" well " + this->name() + " gets STOPPED during iteration ");
             this->stopWell();
             changed_to_stopped_this_step_ = true;
         } else if (well_operable && !old_well_operable) {
-            deferred_logger.info(" well " + this->name() + " gets REVIVED during iteration ");
+            deferred_logger.debug(" well " + this->name() + " gets REVIVED during iteration ");
             this->openWell();
             changed_to_stopped_this_step_ = false;
             this->changed_to_open_this_step_ = true;
@@ -908,7 +924,7 @@ namespace Opm
         if(!this->isOperableAndSolvable() && !this->wellIsStopped())
             return;
 
-        for (int perfIdx = 0; perfIdx < this->number_of_perforations_; ++perfIdx) {
+        for (int perfIdx = 0; perfIdx < this->number_of_local_perforations_; ++perfIdx) {
             if (this->cells()[perfIdx] == cellIdx) {
                 for (int i = 0; i < RateVector::dimension; ++i) {
                     rates[i] += connectionRates_[perfIdx][i];
@@ -921,7 +937,7 @@ namespace Opm
     typename WellInterface<TypeTag>::Scalar
     WellInterface<TypeTag>::volumetricSurfaceRateForConnection(int cellIdx, int phaseIdx) const
     {
-        for (int perfIdx = 0; perfIdx < this->number_of_perforations_; ++perfIdx) {
+        for (int perfIdx = 0; perfIdx < this->number_of_local_perforations_; ++perfIdx) {
             if (this->cells()[perfIdx] == cellIdx) {
                 const unsigned activeCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(phaseIdx));
                 return connectionRates_[perfIdx][activeCompIdx].value();
@@ -943,7 +959,7 @@ namespace Opm
                          const WellState<Scalar>& well_state,
                          DeferredLogger& deferred_logger)
     {
-
+        OPM_TIMEFUNCTION();
         if (!this->param_.check_well_operability_) {
             return;
         }
@@ -970,6 +986,7 @@ namespace Opm
                                                    const GroupState<Scalar>& group_state,
                                                    DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const auto& well_name = this->name();
         assert(this->wellHasTHPConstraints(simulator.vanguard().summaryState()));
         const auto& schedule = simulator.vanguard().schedule();
@@ -999,9 +1016,13 @@ namespace Opm
     void
     WellInterface<TypeTag>::
     gliftBeginTimeStepWellTestUpdateALQ(const Simulator& simulator,
-                          WellState<Scalar>& well_state,
-                          DeferredLogger& deferred_logger)
+                                        WellState<Scalar>& well_state,
+                                        const GroupState<Scalar>& group_state,
+                                        const PhaseUsage& phase_usage,
+                                        GLiftEclWells& ecl_well_map,
+                                        DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
         const auto& well_name = this->name();
         if (!this->wellHasTHPConstraints(summary_state)) {
@@ -1014,28 +1035,44 @@ namespace Opm
         const auto& glo = schedule.glo(report_step_idx);
         if (!glo.has_well(well_name)) {
             const std::string msg = fmt::format(
-                "GLIFT WTEST: Well {} : Gas Lift not activated: "
+                "GLIFT WTEST: Well {} : Gas lift not activated: "
                 "WLIFTOPT is probably missing. Skipping.", well_name);
             deferred_logger.info(msg);
             return;
         }
         const auto& gl_well = glo.well(well_name);
-        auto& max_alq_optional = gl_well.max_rate();
-        Scalar max_alq;
-        if (max_alq_optional) {
-            max_alq = *max_alq_optional;
+
+        // Use gas lift optimization to get ALQ for well test
+        std::unique_ptr<GasLiftSingleWell> glift =
+            initializeGliftWellTest_<GasLiftSingleWell>(simulator,
+                                                        well_state,
+                                                        group_state,
+                                                        phase_usage,
+                                                        ecl_well_map,
+                                                        deferred_logger);
+        auto [wtest_alq, success] = glift->wellTestALQ();
+        std::string msg;
+        const auto& unit_system = schedule.getUnits();
+        if (success) {
+            well_state.setALQ(well_name, wtest_alq);
+            msg = fmt::format(
+                "GLIFT WTEST: Well {} : Setting ALQ to optimized value = {}",
+                well_name, unit_system.from_si(UnitSystem::measure::gas_surface_rate, wtest_alq));
         }
         else {
-            const auto& well_ecl = this->wellEcl();
-            const auto& controls = well_ecl.productionControls(summary_state);
-            const auto& table = this->vfpProperties()->getProd()->getTable(controls.vfp_table_number);
-            const auto& alq_values = table.getALQAxis();
-            max_alq = alq_values.back();
+            if (!gl_well.use_glo()) {
+                msg = fmt::format(
+                    "GLIFT WTEST: Well {} : Gas lift optimization deactivated. Setting ALQ to WLIFTOPT item 3 = {}",
+                    well_name, 
+                    unit_system.from_si(UnitSystem::measure::gas_surface_rate, well_state.getALQ(well_name)));
+                
+            }
+            else {
+                msg = fmt::format(
+                    "GLIFT WTEST: Well {} : Gas lift optimization failed, no ALQ set.",
+                    well_name);
+            }
         }
-        well_state.setALQ(well_name, max_alq);
-        const std::string msg = fmt::format(
-            "GLIFT WTEST: Well {} : Setting ALQ to max value: {}",
-            well_name, max_alq);
         deferred_logger.info(msg);
     }
 
@@ -1046,6 +1083,7 @@ namespace Opm
                           const WellState<Scalar>& well_state,
                           DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         if (this->param_.local_well_solver_control_switching_) {
             const bool success = updateWellOperabilityFromWellEq(simulator, well_state, deferred_logger);
             if (success) {
@@ -1082,6 +1120,7 @@ namespace Opm
                                     const WellState<Scalar>& well_state,
                                     DeferredLogger& deferred_logger)
     {
+        OPM_TIMEFUNCTION();
         // only makes sense if we're using this parameter is true
         assert(this->param_.local_well_solver_control_switching_);
         this->operability_status_.resetOperability();
@@ -1101,7 +1140,7 @@ namespace Opm
                               WellState<Scalar>& well_state,
                               DeferredLogger& deferred_logger) const
     {
-
+        OPM_TIMEFUNCTION();
         // only bhp and wellRates are used to initilize the primaryvariables for standard wells
         const auto& well = this->well_ecl_;
         const int well_index = this->index_of_well_;
@@ -1212,7 +1251,8 @@ namespace Opm
             {
                 assert(well.isAvailableForGroupControl());
                 const auto& group = schedule.getGroup(well.groupName(), this->currentStep());
-                const Scalar efficiencyFactor = well.getEfficiencyFactor();
+                const Scalar efficiencyFactor = well.getEfficiencyFactor() *
+                                                well_state[well.name()].efficiency_scaling_factor;
                 std::optional<Scalar> target =
                         this->getGroupInjectionTargetRate(group,
                                                           well_state,
@@ -1436,7 +1476,8 @@ namespace Opm
             {
                 assert(well.isAvailableForGroupControl());
                 const auto& group = schedule.getGroup(well.groupName(), this->currentStep());
-                const Scalar efficiencyFactor = well.getEfficiencyFactor();
+                const Scalar efficiencyFactor = well.getEfficiencyFactor() *
+                                                well_state[well.name()].efficiency_scaling_factor;
                 Scalar scale = this->getGroupProductionTargetRate(group,
                                                                   well_state,
                                                                   group_state,
@@ -1450,9 +1491,11 @@ namespace Opm
                     for (int p = 0; p<np; ++p) {
                         ws.surface_rates[p] *= scale;
                     }
-                    ws.trivial_target = false;
+                    ws.trivial_group_target = false;
                 } else {
-                    ws.trivial_target = true;
+                    // If group target is trivial we dont want to flip to other controls. To avoid oscillation we store
+                    // this information in the well state and explicitly check for this condition when evaluating well controls.
+                    ws.trivial_group_target = true;
                 }
                 break;
             }
@@ -1477,6 +1520,7 @@ namespace Opm
                             const WellState<Scalar>& well_state,
                             DeferredLogger& deferred_logger) const
     {
+        OPM_TIMEFUNCTION();
         // Check if well is under zero rate control, either directly or from group
         const bool isGroupControlled = this->wellUnderGroupControl(well_state.well(this->index_of_well_));
         if (!isGroupControlled) {
@@ -1525,6 +1569,7 @@ namespace Opm
     initialWellRateFractions(const Simulator& simulator,
                              const WellState<Scalar>& well_state) const
     {
+        OPM_TIMEFUNCTION();
         const int np = this->number_of_phases_;
         std::vector<Scalar> scaling_factor(np);
         const auto& ws = well_state.well(this->index_of_well_);
@@ -1542,10 +1587,12 @@ namespace Opm
         // if we don't have any potentials we weight it using the mobilites
         // We only need approximation so we don't bother with the vapporized oil and dissolved gas
         Scalar total_tw = 0;
-        const int nperf = this->number_of_perforations_;
+        const int nperf = this->number_of_local_perforations_;
         for (int perf = 0; perf < nperf; ++perf) {
             total_tw += this->well_index_[perf];
         }
+        total_tw = this->parallelWellInfo().communication().sum(total_tw);
+
         for (int perf = 0; perf < nperf; ++perf) {
             const int cell_idx = this->well_cells_[perf];
             const auto& intQuants = simulator.model().intensiveQuantities(cell_idx, /*timeIdx=*/0);
@@ -1573,6 +1620,7 @@ namespace Opm
                          WellState<Scalar>& well_state,
                          DeferredLogger& deferred_logger) const
     {
+        OPM_TIMEFUNCTION();
         // Check if the rates of this well only are single-phase, do nothing
         // if more than one nonzero rate.
         auto& ws = well_state.well(this->index_of_well_);
@@ -1623,9 +1671,12 @@ namespace Opm
               const Scalar                   trans_mult,
               const SingleWellState<Scalar>& ws) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         // Add a Forchheimer term to the gas phase CTF if the run uses
         // either of the WDFAC or the WDFACCOR keywords.
-
+        if (static_cast<std::size_t>(perf) >= this->well_cells_.size()) {
+            OPM_THROW(std::invalid_argument,"The perforation index exceeds the size of the local containers - possibly wellIndex was called with a global instead of a local perforation index!");
+        }
         auto wi = std::vector<Scalar>
             (this->num_components_, this->well_index_[perf] * trans_mult);
 
@@ -1705,7 +1756,7 @@ namespace Opm
 
         auto& d_factor = ws.perf_data.connection_d_factor;
 
-        for (int perf = 0; perf < this->number_of_perforations_; ++perf) {
+        for (int perf = 0; perf < this->number_of_local_perforations_; ++perf) {
             const int cell_idx = this->well_cells_[perf];
             const auto& intQuants = simulator.model().intensiveQuantities(cell_idx, /*timeIdx=*/ 0);
 
@@ -1771,7 +1822,7 @@ namespace Opm
         auto& tmult = ws.perf_data.connection_compaction_tmult;
         auto& ctf   = ws.perf_data.connection_transmissibility_factor;
 
-        for (int perf = 0; perf < this->number_of_perforations_; ++perf) {
+        for (int perf = 0; perf < this->number_of_local_perforations_; ++perf) {
             const int cell_idx = this->well_cells_[perf];
 
             const auto& intQuants = simulator.model()
@@ -1816,6 +1867,9 @@ namespace Opm
                                     return std::array<Eval,3>{};
                                 }
                             };
+        if (static_cast<std::size_t>(perf) >= this->well_cells_.size()) {
+            OPM_THROW(std::invalid_argument,"The perforation index exceeds the size of the local containers - possibly getMobility was called with a global instead of a local perforation index!");
+        }
         const int cell_idx = this->well_cells_[perf];
         assert (int(mob.size()) == this->num_components_);
         const auto& intQuants = simulator.model().intensiveQuantities(cell_idx, /*timeIdx=*/0);
@@ -1881,10 +1935,11 @@ namespace Opm
                                      WellState<Scalar>& well_state,
                                      DeferredLogger& deferred_logger) const
     {
+        OPM_TIMEFUNCTION();
         const auto& summary_state = simulator.vanguard().summaryState();
 
         auto bhp_at_thp_limit = computeBhpAtThpLimitProdWithAlq(
-            simulator, summary_state, this->getALQ(well_state), deferred_logger);
+            simulator, summary_state, this->getALQ(well_state), deferred_logger, /*iterate_if_no_solution */ false);
         if (bhp_at_thp_limit) {
             std::vector<Scalar> rates(this->number_of_phases_, 0.0);
             if (thp_update_iterations) {
@@ -1974,4 +2029,50 @@ namespace Opm
         connII[phase_pos] = connIICalc(mt * fs.invB(this->flowPhaseToModelPhaseIdx(phase_pos)).value());
     }
 
+    template<typename TypeTag>
+    template<class GasLiftSingleWell>
+    std::unique_ptr<GasLiftSingleWell> 
+    WellInterface<TypeTag>::
+    initializeGliftWellTest_(const Simulator& simulator,
+                             WellState<Scalar>& well_state,
+                             const GroupState<Scalar>& group_state,
+                             const PhaseUsage& phase_usage,
+                             GLiftEclWells& ecl_well_map,
+                             DeferredLogger& deferred_logger)
+    {
+        // Instantiate group info object (without initialization) since it is needed in GasLiftSingleWell
+        auto& comm = simulator.vanguard().grid().comm();
+        ecl_well_map.try_emplace(this->name(),  &(this->wellEcl()), this->indexOfWell());
+        GasLiftGroupInfo<Scalar> group_info {
+                ecl_well_map,
+                simulator.vanguard().schedule(),
+                simulator.vanguard().summaryState(),
+                simulator.episodeIndex(),
+                simulator.model().newtonMethod().numIterations(),
+                phase_usage,
+                deferred_logger,
+                well_state,
+                group_state,
+                comm,
+                false
+        };
+
+        // Return GasLiftSingleWell object to use the wellTestALQ() function
+        std::set<int> sync_groups;
+        const auto& summary_state = simulator.vanguard().summaryState();
+        return std::make_unique<GasLiftSingleWell>(*this, 
+                                                    simulator, 
+                                                    summary_state,
+                                                    deferred_logger, 
+                                                    well_state, 
+                                                    group_state,
+                                                    group_info, 
+                                                    sync_groups, 
+                                                    comm, 
+                                                    false);
+        
+    }
+
 } // namespace Opm
+
+#endif
