@@ -23,22 +23,31 @@
 #include <opm/simulators/wells/WellState.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
+
 #include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 
-#include <opm/simulators/utils/ParallelCommunication.hpp>
+#include <opm/output/data/Wells.hpp>
+
+#include <opm/simulators/wells/ConnFracStatistics.hpp>
 #include <opm/simulators/wells/ParallelWellInfo.hpp>
 #include <opm/simulators/wells/PerforationData.hpp>
+#include <opm/simulators/wells/RunningStatistics.hpp>
+
+#include <opm/simulators/utils/ParallelCommunication.hpp>
+
 #include <opm/grid/common/p2pcommunicator.hh>
-#include <opm/output/data/Wells.hpp>
 
 #include <algorithm>
 #include <cassert>
+#include <initializer_list>
 #include <numeric>
 #include <set>
 #include <stdexcept>
+#include <type_traits>
+#include <utility>
 #include <vector>
 
 #include <fmt/format.h>
@@ -239,6 +248,7 @@ void WellState<Scalar>::initSingleWell(const std::vector<Scalar>& cellPressures,
     if (!well_perf_data.empty()) {
         pressure_first_connection = cellPressures[well_perf_data[0].cell_index];
     }
+    // The following call is necessary to ensure that processes that do not contain the first perforation get the correct value
     pressure_first_connection = well_info.broadcastFirstPerforationValue(pressure_first_connection);
 
     if (well.isInjector()) {
@@ -246,6 +256,7 @@ void WellState<Scalar>::initSingleWell(const std::vector<Scalar>& cellPressures,
         if (!well_perf_data.empty()) {
             temperature_first_connection = cellTemperatures[well_perf_data[0].cell_index];
         }
+        // The following call is necessary to ensure that processes that do not contain the first perforation get the correct value
         temperature_first_connection = well_info.broadcastFirstPerforationValue(temperature_first_connection);
         this->initSingleInjector(well, well_info, pressure_first_connection, temperature_first_connection,
                                  well_perf_data, summary_state);
@@ -264,26 +275,35 @@ void WellState<Scalar>::init(const std::vector<Scalar>& cellPressures,
                              const int report_step,
                              const WellState* prevState,
                              const std::vector<std::vector<PerforationData<Scalar>>>& well_perf_data,
-                             const SummaryState& summary_state)
+                             const SummaryState& summary_state,
+                             const bool enableDistributedWells)
 {
-    // call init on base class
-    this->base_init(cellPressures, cellTemperatures, wells_ecl, parallel_well_info,
+    // Call init on base class.
+    this->base_init(cellPressures, cellTemperatures,
+                    wells_ecl, parallel_well_info,
                     well_perf_data, summary_state);
-    this->global_well_info = std::make_optional<GlobalWellInfo>(schedule,
-                                                                report_step,
-                                                                wells_ecl);
-    for (const auto& wname : schedule.wellNames(report_step))
-    {
+
+    this->enableDistributedWells_ = enableDistributedWells;
+
+    this->global_well_info.emplace(schedule, report_step, wells_ecl);
+
+    well_rates.clear();
+
+    this->permanently_inactive_well_names_ = schedule.getInactiveWellNamesAtEnd();
+
+    for (const auto& wname : schedule.wellNames(report_step)) {
         well_rates.insert({wname, std::make_pair(false, std::vector<Scalar>(this->numPhases()))});
     }
-    for (const auto& winfo: parallel_well_info)
-    {
+
+    for (const auto& winfo : parallel_well_info) {
         well_rates[winfo.get().name()].first = winfo.get().isOwner();
     }
 
-    const int nw = wells_ecl.size();
+    if (wells_ecl.empty()) {
+        return;
+    }
 
-    if( nw == 0 ) return ;
+    const int nw = wells_ecl.size();
 
     // Initialize perfphaserates_, which must be done here.
     const auto& pu = this->phaseUsage();
@@ -355,71 +375,75 @@ void WellState<Scalar>::init(const std::vector<Scalar>& cellPressures,
 
     // intialize wells that have been there before
     // order may change so the mapping is based on the well name
-    if (prevState && prevState->size() > 0) {
+    if ((prevState != nullptr) && (prevState->size() > 0)) {
         for (int w = 0; w < nw; ++w) {
-            const Well& well = wells_ecl[w];
-            if (well.getStatus() == Well::Status::SHUT) {
+            if (wells_ecl[w].getStatus() == Well::Status::SHUT) {
                 continue;
             }
+
+            const auto old_index = prevState->index(wells_ecl[w].name());
+            if (! old_index.has_value()) {
+                continue;
+            }
+
+            const auto& prev_well = prevState->well(*old_index);
+
             auto& new_well = this->well(w);
-            const auto& old_index = prevState->index(well.name());
-            if (old_index.has_value()) {
-                const auto& prev_well = prevState->well(old_index.value());
-                new_well.init_timestep(prev_well);
+            new_well.init_timestep(prev_well);
 
+            if (prev_well.status == Well::Status::SHUT) {
+                // Well was shut in previous state, do not use its values.
+                continue;
+            }
 
-                if (prev_well.status == Well::Status::SHUT) {
-                    // Well was shut in previous state, do not use its values.
-                    continue;
-                }
+            if (new_well.producer != prev_well.producer) {
+                // Well changed to/from injector from/to producer, do not
+                // use its previous values.
+                continue;
+            }
 
-                if (new_well.producer != prev_well.producer)
-                    // Well changed to/from injector from/to producer, do not use its privious values.
-                    continue;
+            // If new target is set using WCONPROD, WCONINJE etc. we use the new control
+            if (!new_well.events.hasEvent(WellState::event_mask)) {
+                new_well.injection_cmode = prev_well.injection_cmode;
+                new_well.production_cmode = prev_well.production_cmode;
+            }
 
-                // If new target is set using WCONPROD, WCONINJE etc. we use the new control
-                if (!new_well.events.hasEvent(WellState::event_mask)) {
-                    new_well.injection_cmode = prev_well.injection_cmode;
-                    new_well.production_cmode = prev_well.production_cmode;
-                }
+            new_well.surface_rates = prev_well.surface_rates;
+            new_well.reservoir_rates = prev_well.reservoir_rates;
+            new_well.well_potentials = prev_well.well_potentials;
 
-                new_well.surface_rates = prev_well.surface_rates;
-                new_well.reservoir_rates = prev_well.reservoir_rates;
-                new_well.well_potentials = prev_well.well_potentials;
+            // perfPhaseRates
+            //
+            // Copy perforation rates when the number of perforations is
+            // equal, otherwise initialize perfphaserates to well rates
+            // divided by the number of perforations.
+            //
+            // TODO: we might still need the values from the prev_well if
+            // the connection structure changes.
+            if (const auto num_perf_this_well = new_well.perf_data.size();
+                num_perf_this_well == prev_well.perf_data.size())
+            {
+                new_well.perf_data.try_assign(prev_well.perf_data);
+            }
+            else {
+                const auto global_num_perf_this_well =
+                    static_cast<Scalar>(wells_ecl[w].getConnections().num_open());
 
-                // perfPhaseRates
-                const int num_perf_old_well = prev_well.perf_data.size();
-                const int num_perf_this_well = new_well.perf_data.size();
-                const bool global_num_perf_same = (num_perf_this_well == num_perf_old_well);
-
-                // copy perforation rates when the number of
-                // perforations is equal, otherwise initialize
-                // perfphaserates to well rates divided by the
-                // number of perforations.
-                // TODO: we might still need the values from the prev_well if the connection structure changes
-                if (global_num_perf_same)
-                {
-                    auto& perf_data = new_well.perf_data;
-                    const auto& prev_perf_data = prev_well.perf_data;
-                    perf_data.try_assign( prev_perf_data );
-                } else {
-                    const int global_num_perf_this_well = well.getConnections().num_open();
-                    auto& perf_data = new_well.perf_data;
-                    auto& target_rates = perf_data.phase_rates;
-                    for (int perf_index = 0; perf_index < num_perf_this_well; perf_index++) {
-                        for (int p = 0; p < np; ++p) {
-                            target_rates[perf_index*np + p] = new_well.surface_rates[p] / Scalar(global_num_perf_this_well);
-                        }
+                auto target_rate = new_well.perf_data.phase_rates.begin();
+                for (auto perf_index = 0*num_perf_this_well; perf_index < num_perf_this_well; ++perf_index) {
+                    for (int p = 0; p < np; ++p, ++target_rate) {
+                        *target_rate = new_well.surface_rates[p] / global_num_perf_this_well;
                     }
                 }
+            }
 
-                // Productivity index.
-                new_well.productivity_index = prev_well.productivity_index;
+            // Productivity index.
+            new_well.productivity_index = prev_well.productivity_index;
 
-                // if there is no valid VFP table associated, we set the THP value to be 0.
-                if (well.vfp_table_number() == 0) {
-                    new_well.thp = 0.;
-                }
+            // If there is no valid VFP table associated, we set the THP
+            // value to zero.
+            if (wells_ecl[w].vfp_table_number() == 0) {
+                new_well.thp = Scalar{};
             }
         }
     }
@@ -437,7 +461,7 @@ void WellState<Scalar>::resize(const std::vector<Well>& wells_ecl,
                                const SummaryState& summary_state)
 {
     const std::vector<Scalar> tmp(numCells, 0.0); // <- UGLY HACK to pass the size
-    init(tmp, tmp, schedule, wells_ecl, parallel_well_info, 0, nullptr, well_perf_data, summary_state);
+    init(tmp, tmp, schedule, wells_ecl, parallel_well_info, 0, nullptr, well_perf_data, summary_state, this->enableDistributedWells_);
 
     if (handle_ms_well) {
         initWellStateMSWell(wells_ecl, nullptr);
@@ -515,6 +539,7 @@ WellState<Scalar>::report(const int* globalCellIdxMap,
         well.bhp = ws.bhp;
         well.thp = ws.thp;
         well.temperature = ws.temperature;
+        well.efficiency_scaling_factor = ws.efficiency_scaling_factor;
         well.filtrate.rate = ws.sum_filtrate_rate();
         well.filtrate.total = ws.sum_filtrate_total();
         well.filtrate.concentration = ws.filtrate_conc;
@@ -552,6 +577,12 @@ WellState<Scalar>::report(const int* globalCellIdxMap,
             well.rates.set(rt::brine, ws.sum_brine_rates());
         }
 
+        if (pu.has_micp) {
+            well.rates.set(rt::microbial, ws.sum_microbial_rates());
+            well.rates.set(rt::oxygen, ws.sum_oxygen_rates());
+            well.rates.set(rt::urea, ws.sum_urea_rates());
+        }
+
         if (ws.producer) {
             well.rates.set(rt::alq, getALQ(wname));
         }
@@ -573,8 +604,9 @@ WellState<Scalar>::report(const int* globalCellIdxMap,
             curr.inj  = ws.injection_cmode;
         }
 
-        const auto& pwinfo = ws.parallel_info.get();
-        if (pwinfo.communication().size() == 1) {
+        if (const auto& pwinfo = ws.parallel_info.get();
+            pwinfo.communication().size() == 1)
+        {
             reportConnections(well.connections, pu, well_index, globalCellIdxMap);
         }
         else {
@@ -596,84 +628,31 @@ WellState<Scalar>::report(const int* globalCellIdxMap,
 
 template<class Scalar>
 void WellState<Scalar>::reportConnections(std::vector<data::Connection>& connections,
-                                          const PhaseUsage &pu,
-                                          std::size_t well_index,
+                                          const PhaseUsage& pu,
+                                          const std::size_t well_index,
                                           const int* globalCellIdxMap) const
 {
-    using rt = data::Rates::opt;
     const auto& ws = this->well(well_index);
+
     const auto& perf_data = ws.perf_data;
-    const int num_perf_well = perf_data.size();
+    const auto  num_perf_well = perf_data.size();
+
     connections.resize(num_perf_well);
-    const auto& perf_rates = perf_data.rates;
-    const auto& perf_pressure = perf_data.pressure;
-    const auto& perf_mixing_rates = perf_data.phase_mixing_rates;
-    for (int i = 0; i < num_perf_well; ++i) {
-      const auto active_index = perf_data.cell_index[i];
-        auto& connection = connections[ i ];
-        connection.index = globalCellIdxMap[active_index];
-        connection.pressure = perf_pressure[i];
-        connection.reservoir_rate = perf_rates[i];
-        connection.trans_factor = perf_data.connection_transmissibility_factor[i];
-        connection.d_factor = perf_data.connection_d_factor[i];
-        connection.compact_mult = perf_data.connection_compaction_tmult[i];
-        connection.rates.set(rt::dissolved_gas, perf_mixing_rates[i][ws.dissolved_gas]);
-        connection.rates.set(rt::vaporized_oil, perf_mixing_rates[i][ws.vaporized_oil]);
-        if (!ws.producer) {
-            const auto& filtrate_data = perf_data.filtrate_data;
-            auto& filtrate = connection.filtrate;
-            filtrate.rate = filtrate_data.rates[i];
-            filtrate.total = filtrate_data.total[i];
-            filtrate.skin_factor = filtrate_data.skin_factor[i];
-            filtrate.thickness = filtrate_data.thickness[i];
-            filtrate.poro = filtrate_data.poro[i];
-            filtrate.perm = filtrate_data.perm[i];
-            filtrate.radius = filtrate_data.radius[i];
-            filtrate.area_of_flow = filtrate_data.area_of_flow[i];
-        }
+
+    for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+        connections[i].index = globalCellIdxMap[perf_data.cell_index[i]];
     }
 
-    const int np = pu.num_phases;
-    std::vector< rt > phs( np );
-    std::vector<rt> pi(np);
-    if (pu.phase_used[Water]) {
-        phs.at( pu.phase_pos[Water] ) = rt::wat;
-        pi .at( pu.phase_pos[Water] ) = rt::productivity_index_water;
+    this->reportConnectionFactors(well_index, connections);
+    this->reportConnectionPressuresAndRates(well_index, pu, connections);
+
+    if (! ws.producer) {
+        this->reportConnectionFilterCake(well_index, connections);
     }
 
-    if (pu.phase_used[Oil]) {
-        phs.at( pu.phase_pos[Oil] ) = rt::oil;
-        pi .at( pu.phase_pos[Oil] ) = rt::productivity_index_oil;
-    }
-
-    if (pu.phase_used[Gas]) {
-        phs.at( pu.phase_pos[Gas] ) = rt::gas;
-        pi .at( pu.phase_pos[Gas] ) = rt::productivity_index_gas;
-    }
-
-    std::size_t local_conn_index = 0;
-    for (auto& comp : connections) {
-        const auto * rates = &perf_data.phase_rates[np * local_conn_index];
-        const auto * connPI = &perf_data.prod_index[np * local_conn_index];
-
-
-        for (int i = 0; i < np; ++i) {
-            comp.rates.set( phs[ i ], rates[i] );
-            comp.rates.set( pi [ i ], connPI[i] );
-        }
-        if (pu.has_polymer) {
-            const auto& perf_polymer_rate = perf_data.polymer_rates;
-            comp.rates.set( rt::polymer, perf_polymer_rate[local_conn_index]);
-        }
-        if (pu.has_brine) {
-            const auto& perf_brine_rate = perf_data.brine_rates;
-            comp.rates.set( rt::brine, perf_brine_rate[local_conn_index]);
-        }
-        if (pu.has_solvent) {
-            const auto& perf_solvent_rate = perf_data.solvent_rates;
-            comp.rates.set( rt::solvent, perf_solvent_rate[local_conn_index] );
-        }
-        ++local_conn_index;
+    if (! perf_data.connFracStatistics.empty()) {
+        this->reportFractureStatistics(perf_data.connFracStatistics,
+                                       connections);
     }
 }
 
@@ -693,6 +672,9 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
     // what we do here, is to set the segment rates and perforation rates
     for (int w = 0; w < nw; ++w) {
         const auto& well_ecl = wells_ecl[w];
+        if (this->is_permanently_inactive_well(well_ecl.name()))
+            continue;
+
         auto& ws = this->well(w);
 
         if (well_ecl.isMultiSegment()) {
@@ -703,10 +685,13 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
             ws.segments = SegmentState<Scalar>{np, segment_set};
             const int well_nseg = segment_set.size();
             int n_activeperf = 0;
+            int n_activeperf_local = 0;
 
             // we need to know for each segment, how many perforation it has and how many segments using it as outlet_segment
             // that is why I think we should use a well model to initialize the WellState here
             std::vector<std::vector<int>> segment_perforations(well_nseg);
+            std::unordered_map<int,int> active_perf_index_local_to_global = {};
+            std::unordered_map<int,int> active_to_local = {};
             for (std::size_t perf = 0; perf < completion_set.size(); ++perf) {
                 const Connection& connection = completion_set.get(perf);
                 if (connection.state() == Connection::State::OPEN) {
@@ -720,9 +705,19 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                     }
 
                     segment_perforations[segment_index].push_back(n_activeperf);
+                    if (ws.parallel_info.get().globalToLocal(perf) > -1) {
+                        active_perf_index_local_to_global.insert({n_activeperf_local, n_activeperf});
+                        active_to_local.insert({n_activeperf, ws.parallel_info.get().globalToLocal(perf)});
+                        n_activeperf_local++;
+                    }
                     n_activeperf++;
                 }
             }
+            ws.parallel_info.get().setActiveToLocalMap(active_to_local);
+
+            if (!this->enableDistributedWells_ && static_cast<int>(ws.perf_data.size()) != n_activeperf)
+                throw std::logic_error("Distributed multi-segment wells cannot be initialized properly yet.");
+
 
             std::vector<std::vector<int>> segment_inlets(well_nseg);
             for (int seg = 0; seg < well_nseg; ++seg) {
@@ -746,14 +741,46 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                 // TODO: to see if this strategy can benefit StandardWell too
                 // TODO: it might cause big problem for gas rate control or if there is a gas rate limit
                 // maybe the best way is to initialize the fractions first then get the rates
-                for (int perf = 0; perf < n_activeperf; perf++)
+                for (std::size_t perf = 0; perf < perf_data.size(); perf++)
                     perf_rates[perf*np + gaspos] *= 100;
             }
 
             const auto& perf_rates = perf_data.phase_rates;
-            std::vector<Scalar> perforation_rates(perf_rates.begin(), perf_rates.end());
+            const auto& perf_press = perf_data.pressure;
+            // The function calculateSegmentRates as well as the loop filling the segment_pressure work
+            // with *global* containers. Now we create global vectors containing the phase_rates and
+            // pressures of all processes.
+            size_t number_of_global_perfs = 0;
+
+            if (ws.parallel_info.get().communication().size() > 1) {
+                number_of_global_perfs = ws.parallel_info.get().communication().sum(perf_data.size());
+            } else {
+                number_of_global_perfs = perf_data.size();
+            }
+
+            std::vector<Scalar> perforation_rates(number_of_global_perfs * np, 0.0);
+            std::vector<Scalar> perforation_pressures(number_of_global_perfs, 0.0);
+
+            assert(perf_data.size() == perf_press.size());
+            assert(perf_data.size() * np == perf_rates.size());
+            for (size_t perf = 0; perf < perf_data.size(); ++perf) {
+                if (auto candidate = active_perf_index_local_to_global.find(perf); candidate != active_perf_index_local_to_global.end()) {
+                    const int global_active_perf_index = candidate->second;
+                    perforation_pressures[global_active_perf_index] = perf_press[perf];
+                    for (int i = 0; i < np; i++) {
+                        perforation_rates[global_active_perf_index * np + i] = perf_rates[perf * np + i];
+                    }
+                } else {
+                    OPM_THROW(std::logic_error,fmt::format("Error when initializing MS Well state, there is no active perforation index for the local index {}", perf));
+                }
+            }
+            if (ws.parallel_info.get().communication().size() > 1) {
+                ws.parallel_info.get().communication().sum(perforation_rates.data(), perforation_rates.size());
+                ws.parallel_info.get().communication().sum(perforation_pressures.data(), perforation_pressures.size());
+            }
+
+            calculateSegmentRates(ws.parallel_info, segment_inlets, segment_perforations, perforation_rates, np, 0 /* top segment */, ws.segments.rates);
 
-            calculateSegmentRates(segment_inlets, segment_perforations, perforation_rates, np, 0 /* top segment */, ws.segments.rates);
             // for the segment pressure, the segment pressure is the same with the first perforation belongs to the segment
             // if there is no perforation associated with this segment, it uses the pressure from the outlet segment
             // which requres the ordering is successful
@@ -763,11 +790,13 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
                 // top segment is always the first one, and its pressure is the well bhp
                 auto& segment_pressure = ws.segments.pressure;
                 segment_pressure[0] = ws.bhp;
-                const auto& perf_press = perf_data.pressure;
+                // The segment_indices contain the indices of the segments, that are only available on one process.
+                std::vector<int> segment_indices;
                 for (int seg = 1; seg < well_nseg; ++seg) {
                     if (!segment_perforations[seg].empty()) {
-                        const int first_perf = segment_perforations[seg][0];
-                        segment_pressure[seg] = perf_press[first_perf];
+                        const int first_perf_global_index = segment_perforations[seg][0];
+                        segment_pressure[seg] = perforation_pressures[first_perf_global_index];
+                        segment_indices.push_back(seg);
                     } else {
                         // seg_press_.push_back(bhp); // may not be a good decision
                         // using the outlet segment pressure // it needs the ordering is correct
@@ -809,11 +838,12 @@ void WellState<Scalar>::initWellStateMSWell(const std::vector<Well>& wells_ecl,
 
 template<class Scalar>
 void WellState<Scalar>::
-calculateSegmentRates(const std::vector<std::vector<int>>& segment_inlets,
-                      const std::vector<std::vector<int>>& segment_perforations,
-                      const std::vector<Scalar>& perforation_rates,
-                      const int np, const int segment,
-                      std::vector<Scalar>& segment_rates)
+calculateSegmentRatesBeforeSum(const ParallelWellInfo<Scalar>& pw_info,
+                               const std::vector<std::vector<int>>& segment_inlets,
+                               const std::vector<std::vector<int>>& segment_perforations,
+                               const std::vector<Scalar>& perforation_rates,
+                               const int np, const int segment,
+                               std::vector<Scalar>& segment_rates)
 {
     // the rate of the segment equals to the sum of the contribution from the perforations and inlet segment rates.
     // the first segment is always the top segment, its rates should be equal to the well rates.
@@ -824,17 +854,34 @@ calculateSegmentRates(const std::vector<std::vector<int>>& segment_inlets,
     }
     // contributions from the perforations belong to this segment
     for (const int& perf : segment_perforations[segment]) {
-        for (int p = 0; p < np; ++p) {
-            segment_rates[np * segment + p] += perforation_rates[np * perf + p];
+        auto local_perf = pw_info.activeToLocal(perf);
+        // If local_perf == -1, then the perforation is not on this process.
+        // The perforation of the other processes are added in calculateSegmentRates.
+        if (local_perf > -1) {
+            for (int p = 0; p < np; ++p) {
+                segment_rates[np * segment + p] += perforation_rates[np * local_perf + p];
+            }
         }
     }
     for (const int& inlet_seg : segment_inlets[segment]) {
-        calculateSegmentRates(segment_inlets, segment_perforations, perforation_rates, np, inlet_seg, segment_rates);
+        calculateSegmentRatesBeforeSum(pw_info, segment_inlets, segment_perforations, perforation_rates, np, inlet_seg, segment_rates);
         for (int p = 0; p < np; ++p) {
             segment_rates[np * segment + p] += segment_rates[np * inlet_seg + p];
         }
     }
 }
+template<class Scalar>
+void WellState<Scalar>::
+calculateSegmentRates(const ParallelWellInfo<Scalar>& pw_info,
+                      const std::vector<std::vector<int>>& segment_inlets,
+                      const std::vector<std::vector<int>>& segment_perforations,
+                      const std::vector<Scalar>& perforation_rates,
+                      const int np, const int segment,
+                      std::vector<Scalar>& segment_rates)
+{
+    calculateSegmentRatesBeforeSum(pw_info, segment_inlets, segment_perforations, perforation_rates, np, segment, segment_rates);
+    pw_info.communication().sum(segment_rates.data(), segment_rates.size());
+}
 
 template<class Scalar>
 void WellState<Scalar>::stopWell(int well_index)
@@ -868,48 +915,44 @@ template<class Scalar>
 void WellState<Scalar>::communicateGroupRates(const Parallel::Communication& comm)
 {
     // Compute the size of the data.
-    std::size_t sz = 0;
-    for (const auto& [_, owner_rates] : this->well_rates) {
-        (void)_;
-        const auto& [__, rates] = owner_rates;
-        (void)__;
-        sz += rates.size();
-    }
+    std::size_t sz = std::accumulate(this->well_rates.begin(), this->well_rates.end(), std::size_t{0},
+                     [](const std::size_t acc, const auto& rates)
+                     { return acc + rates.second.second.size(); });
     sz += this->alq_state.pack_size();
 
     // Make a vector and collect all data into it.
     std::vector<Scalar> data(sz);
-    std::size_t pos = 0;
-    for (const auto& [_, owner_rates] : this->well_rates) {
-        (void)_;
-        const auto& [owner, rates] = owner_rates;
-        for (const auto& value : rates) {
-            if (owner)
-                data[pos++] = value;
-            else
-                data[pos++] = 0;
-        }
+    auto pos = data.begin();
+    std::for_each(this->well_rates.begin(), this->well_rates.end(),
+                  [&pos](const auto& input)
+                  {
+                      const auto& [owner, rates] = input.second;
+                      if (owner) {
+                          std::copy(rates.begin(), rates.end(), pos);
+                      }
+                      pos += rates.size();
+                  });
+
+    if (pos != data.end()) {
+        pos += this->alq_state.pack_data(&(*pos));
     }
-    if (!data.empty() && pos != sz) {
-        pos += this->alq_state.pack_data(&data[pos]);
-    }
-    assert(pos == sz);
+    assert(pos == data.end());
 
     // Communicate it with a single sum() call.
     comm.sum(data.data(), data.size());
 
-    pos = 0;
-    for (auto& [_, owner_rates] : this->well_rates) {
-        (void)_;
-        auto& [__, rates] = owner_rates;
-        (void)__;
-        for (auto& value : rates)
-            value = data[pos++];
+    pos = data.begin();
+    std::for_each(this->well_rates.begin(), this->well_rates.end(),
+                  [&pos](auto& input)
+                  {
+                      auto& rates = input.second.second;
+                      std::copy(pos, pos + rates.size(), rates.begin());
+                      pos += rates.size();
+                  });
+    if (pos != data.end()) {
+        pos += this->alq_state.unpack_data(&(*pos));
     }
-    if (!data.empty() && pos != sz) {
-        pos += this->alq_state.unpack_data(&data[pos]);
-    }
-    assert(pos == sz);
+    assert(pos == data.end());
 }
 
 template<class Scalar>
@@ -918,6 +961,7 @@ void WellState<Scalar>::updateGlobalIsGrup(const Parallel::Communication& comm)
     this->global_well_info.value().clear();
     for (std::size_t well_index = 0; well_index < this->size(); well_index++) {
         const auto& ws = this->well(well_index);
+        this->global_well_info.value().update_efficiency_scaling_factor(well_index, ws.efficiency_scaling_factor);
         if (ws.producer)
             this->global_well_info.value().update_producer(well_index, ws.status, ws.production_cmode);
         else
@@ -926,6 +970,16 @@ void WellState<Scalar>::updateGlobalIsGrup(const Parallel::Communication& comm)
     this->global_well_info.value().communicate(comm);
 }
 
+template<class Scalar>
+void WellState<Scalar>::
+updateEfficiencyScalingFactor(const std::string& wellName,
+                              const Scalar value)
+{
+    const auto idx = this->index(wellName);
+    this->global_well_info.value().update_efficiency_scaling_factor(*idx, value);
+    this->well(*idx).efficiency_scaling_factor = value;
+}
+
 template<class Scalar>
 data::Segment
 WellState<Scalar>::reportSegmentResults(const int well_id,
@@ -1006,6 +1060,168 @@ WellState<Scalar>::reportSegmentResults(const int well_id,
     return seg_res;
 }
 
+template <class Scalar>
+void WellState<Scalar>::
+reportConnectionFactors(const std::size_t well_index,
+                        std::vector<data::Connection>& connections) const
+{
+    const auto& perf_data = this->well(well_index).perf_data;
+    const auto num_perf_well = perf_data.size();
+
+    for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+        auto& connection = connections[i];
+
+        connection.trans_factor = perf_data.connection_transmissibility_factor[i];
+        connection.d_factor = perf_data.connection_d_factor[i];
+        connection.compact_mult = perf_data.connection_compaction_tmult[i];
+    }
+}
+
+template <class Scalar>
+void WellState<Scalar>::
+reportConnectionPressuresAndRates(const std::size_t well_index,
+                                  const PhaseUsage& pu,
+                                  std::vector<data::Connection>& connections) const
+{
+    using rt = data::Rates::opt;
+
+    const int np = pu.num_phases;
+    std::vector<rt> phs(np);
+    std::vector<rt> pi(np);
+
+    if (pu.phase_used[Water]) {
+        phs.at(pu.phase_pos[Water]) = rt::wat;
+        pi .at(pu.phase_pos[Water]) = rt::productivity_index_water;
+    }
+
+    if (pu.phase_used[Oil]) {
+        phs.at(pu.phase_pos[Oil]) = rt::oil;
+        pi .at(pu.phase_pos[Oil]) = rt::productivity_index_oil;
+    }
+
+    if (pu.phase_used[Gas]) {
+        phs.at(pu.phase_pos[Gas]) = rt::gas;
+        pi .at(pu.phase_pos[Gas]) = rt::productivity_index_gas;
+    }
+
+    const auto& ws = this->well(well_index);
+    const auto& perf_data = ws.perf_data;
+    const auto  num_perf_well = perf_data.size();
+
+    for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+        auto& connection = connections[i];
+
+        {
+            const auto* rates = &perf_data.phase_rates[np * i];
+            const auto* connPI = &perf_data.prod_index[np * i];
+
+            for (int p = 0; p < np; ++p) {
+                connection.rates.set(phs[p], rates [p]);
+                connection.rates.set(pi [p], connPI[p]);
+            }
+        }
+
+        connection.pressure = perf_data.pressure[i];
+        connection.reservoir_rate = perf_data.rates[i];
+
+        connection.rates.set(rt::dissolved_gas, perf_data.phase_mixing_rates[i][ws.dissolved_gas]);
+        connection.rates.set(rt::vaporized_oil, perf_data.phase_mixing_rates[i][ws.vaporized_oil]);
+    }
+
+    if (pu.has_polymer) {
+        for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+            connections[i].rates.set(rt::polymer, perf_data.polymer_rates[i]);
+        }
+    }
+
+    if (pu.has_brine) {
+        for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+            connections[i].rates.set(rt::brine, perf_data.brine_rates[i]);
+        }
+    }
+
+    if (pu.has_solvent) {
+        for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+            connections[i].rates.set(rt::solvent, perf_data.solvent_rates[i]);
+        }
+    }
+
+    if (pu.has_micp) {
+        for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+            connections[i].rates.set(rt::microbial, perf_data.microbial_rates[i]);
+            connections[i].rates.set(rt::oxygen, perf_data.oxygen_rates[i]);
+            connections[i].rates.set(rt::urea, perf_data.urea_rates[i]);
+        }
+    }
+    
+    if (pu.has_co2_or_h2store) {
+        for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+            connections[i].rates.set(rt::mass_gas, perf_data.gas_mass_rates[i]);
+        }
+    }
+}
+
+template <class Scalar>
+void WellState<Scalar>::
+reportConnectionFilterCake(const std::size_t well_index,
+                           std::vector<data::Connection>& connections) const
+{
+    const auto& perf_data = this->well(well_index).perf_data;
+    const auto num_perf_well = perf_data.size();
+
+    const auto& filtrate_data = perf_data.filtrate_data;
+
+    for (auto i = 0*num_perf_well; i < num_perf_well; ++i) {
+        auto& filtrate = connections[i].filtrate;
+
+        filtrate.rate = filtrate_data.rates[i];
+        filtrate.total = filtrate_data.total[i];
+        filtrate.skin_factor = filtrate_data.skin_factor[i];
+        filtrate.thickness = filtrate_data.thickness[i];
+        filtrate.poro = filtrate_data.poro[i];
+        filtrate.perm = filtrate_data.perm[i];
+        filtrate.radius = filtrate_data.radius[i];
+        filtrate.area_of_flow = filtrate_data.area_of_flow[i];
+    }
+}
+
+template <class Scalar>
+void WellState<Scalar>::
+reportFractureStatistics(const std::vector<ConnFracStatistics<Scalar>>& stats,
+                         std::vector<data::Connection>& connections) const
+{
+    using Quantity = typename ConnFracStatistics<Scalar>::Quantity;
+    using StatResult = data::ConnectionFracturing;
+
+    auto connIx = 0*connections.size();
+    for (auto& connection : connections) {
+        for (const auto& [q, result] : {
+                std::pair { Quantity::Pressure, &StatResult::press },
+                std::pair { Quantity::FlowRate, &StatResult::rate  },
+                std::pair { Quantity::Width   , &StatResult::width },
+            })
+        {
+            const auto& stat = stats[connIx].statistics(q);
+
+            if (stat.sampleSize() > 0) {
+                auto& x = connection.fract.*result;
+
+                x.avg = stat.mean();
+                x.min = stat.min();
+                x.max = stat.max();
+
+                if (const auto stdev = stat.stdev(); stdev.has_value()) {
+                    x.stdev = *stdev;
+                }
+
+                connection.fract.numCells = stat.sampleSize();
+            }
+        }
+
+        ++connIx;
+    }
+}
+
 template<class Scalar>
 bool WellState<Scalar>::wellIsOwned(std::size_t well_index,
                                     [[maybe_unused]] const std::string& wellName) const
@@ -1049,7 +1265,8 @@ bool WellState<Scalar>::operator==(const WellState& rhs) const
 {
     return this->alq_state == rhs.alq_state &&
            this->well_rates == rhs.well_rates &&
-           this->wells_ == rhs.wells_;
+           this->wells_ == rhs.wells_ &&
+           this->permanently_inactive_well_names_ == rhs.permanently_inactive_well_names_;
 }
 
 template<class Scalar>
