@@ -123,6 +123,7 @@ TEST_FOR_TYPE_NAMED_OBJ(ATS, AdaptiveTimeSteppingHardcoded, serializationTestObj
 TEST_FOR_TYPE_NAMED_OBJ(ATS, AdaptiveTimeSteppingPID, serializationTestObjectPID)
 TEST_FOR_TYPE_NAMED_OBJ(ATS, AdaptiveTimeSteppingPIDIt, serializationTestObjectPIDIt)
 TEST_FOR_TYPE_NAMED_OBJ(ATS, AdaptiveTimeSteppingSimple, serializationTestObjectSimple)
+TEST_FOR_TYPE_NAMED_OBJ(ATS, AdaptiveTimeStepping3rdOrder, serializationTestObject3rdOrder)
 
 namespace Opm { using BPV = BlackOilPrimaryVariables<Properties::TTag::TestTypeTag>; }
 TEST_FOR_TYPE_NAMED(BPV, BlackoilPrimaryVariables)
@@ -272,17 +273,10 @@ BOOST_AUTO_TEST_CASE(FlowGenericProblemFem)
     Opm::EclipseState eclState;
     Opm::Schedule schedule;
     Dune::CpGrid grid;
-#if DUNE_VERSION_GTE(DUNE_FEM, 2, 9)
     using GridPart = Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid, Dune::PartitionIteratorType(4), false>;
     using GridView = GridPart::GridViewType;
     auto gridPart = GridPart(grid);
     auto gridView = gridPart.gridView();
-#else
-    using GridPart = Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid, Dune::PartitionIteratorType(4), false>;
-    using GridView = Dune::GridView<Dune::Fem::GridPart2GridViewTraits<GridPart>>;
-    auto gridPart = GridPart(grid);
-    auto gridView = GridView(static_cast<GridView>(gridPart));
-#endif
     auto data_out
         = Opm::FlowGenericProblem<GridView, Opm::BlackOilFluidSystem<double, Opm::BlackOilDefaultIndexTraits>>::
             serializationTestObject(eclState, schedule, gridView);
@@ -304,12 +298,13 @@ class BlackoilWellModelGenericTest : public BlackoilWellModelGeneric<double>
 {
 public:
     BlackoilWellModelGenericTest(Schedule& schedule,
+                                 BlackoilWellModelGasLiftGeneric<double>& gaslift,
                                  const SummaryState& summaryState,
                                  const EclipseState& eclState,
                                  const PhaseUsage& phase_usage,
                                  const Parallel::Communication& comm,
                                  bool deserialize)
-        : BlackoilWellModelGeneric<double>(schedule, summaryState,
+        : BlackoilWellModelGeneric<double>(schedule, gaslift, summaryState,
                                            eclState, phase_usage, comm)
     {
         if (deserialize) {
@@ -331,9 +326,9 @@ public:
         active_wgstate_ = WGState<double>::serializationTestObject(dummy);
         last_valid_wgstate_ = WGState<double>::serializationTestObject(dummy);
         nupcol_wgstate_ = WGState<double>::serializationTestObject(dummy);
-        last_glift_opt_time_ = 5.0;
-        switched_prod_groups_ = {{"test4", "test5"}};
-        switched_inj_groups_ = {{{"test4", Phase::SOLVENT}, "test5"}};
+        switched_prod_groups_ = {{"test4", {Group::ProductionCMode::NONE, Group::ProductionCMode::ORAT}}};
+        const auto controls = {Group::InjectionCMode::NONE, Group::InjectionCMode::RATE, Group::InjectionCMode::RATE };
+        switched_inj_groups_ = {{"test4", {controls, {}, controls} }};
         closed_offending_wells_ = {{"test4", {"test5", "test6"}}};
     }
 
@@ -373,6 +368,16 @@ private:
     ParallelWellInfo<double> dummy;
 };
 
+class BlackoilWellModelGasLiftGenericTest : public BlackoilWellModelGasLiftGeneric<double>
+{
+public:
+    BlackoilWellModelGasLiftGenericTest()
+        : BlackoilWellModelGasLiftGeneric<double>(false)
+    {
+        this->last_glift_opt_time_ = 1234.5;
+    }
+};
+
 }
 
 BOOST_AUTO_TEST_CASE(BlackoilWellModelGeneric)
@@ -382,14 +387,15 @@ BOOST_AUTO_TEST_CASE(BlackoilWellModelGeneric)
     Opm::EclipseState eclState{};
     Opm::PhaseUsage phase_usage{};
     Opm::Parallel::Communication comm{};
-    Opm::BlackoilWellModelGenericTest data_out(schedule, summaryState,
+    Opm::BlackoilWellModelGasLiftGenericTest gaslift;
+    Opm::BlackoilWellModelGenericTest data_out(schedule, gaslift, summaryState,
                                                eclState, phase_usage, comm, false);
     data_out.setSerializationTestData();
     Opm::Serialization::MemPacker packer;
     Opm::Serializer ser(packer);
     ser.pack(data_out);
     const size_t pos1 = ser.position();
-    Opm::BlackoilWellModelGenericTest data_in(schedule, summaryState,
+    Opm::BlackoilWellModelGenericTest data_in(schedule, gaslift, summaryState,
                                               eclState, phase_usage, comm, true);
     ser.unpack(data_in);
     const size_t pos2 = ser.position();
@@ -419,7 +425,7 @@ public:
     {
         GenericTracerModelTest result(gridView, eclState, cartMapper, dofMapper, centroids);
         result.tracerConcentration_ = {{{1.0, 2.0}}, {{3.0, 4.0}}, {{5.0, 6.0}}};
-        result.wellTracerRate_.insert({{"foo", "bar"}, 4.0});
+        result.wellTracerRate_.emplace(1, std::vector{Opm::WellTracerRate<double>{"foo", 4.0}});
 
         return result;
     }
@@ -475,17 +481,10 @@ BOOST_AUTO_TEST_CASE(FlowGenericTracerModelFem)
     Opm::EclipseState eclState;
     Dune::CartesianIndexMapper<Dune::CpGrid> mapper(grid);
     auto centroids = [](int) { return std::array<double,Dune::CpGrid::dimensionworld>{}; };
-#if DUNE_VERSION_GTE(DUNE_FEM, 2, 9)
     using GridPart = Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid, Dune::PartitionIteratorType(4), false>;
     using GridView = GridPart::GridViewType;
     auto gridPart = GridPart(grid);
     auto gridView = gridPart.gridView();
-#else
-    using GridPart = Dune::Fem::AdaptiveLeafGridPart<Dune::CpGrid, Dune::PartitionIteratorType(4), false>;
-    using GridView = Dune::GridView<Dune::Fem::GridPart2GridViewTraits<GridPart>>;
-    auto gridPart = GridPart(grid);
-    auto gridView = GridView(static_cast<GridView>(gridPart));
-#endif
     Dune::MultipleCodimMultipleGeomTypeMapper<GridView> dofMapper(gridView, Dune::mcmgElementLayout());
     auto data_out = GenericTracerModelTest<Dune::CpGrid,
                                            GridView,
