@@ -20,10 +20,20 @@
   module for the precise wording of the license and the list of
   copyright holders.
 */
+
 #include "config.h"
 
 #define BOOST_TEST_MODULE Equil
 
+#include <boost/test/unit_test.hpp>
+
+#include <boost/version.hpp>
+#if (BOOST_VERSION / 100000 == 1) && ((BOOST_VERSION / 100) % 1000 < 71)
+#include <boost/test/floating_point_comparison.hpp>
+#else
+#include <boost/test/tools/floating_point_comparison.hpp>
+#endif
+
 #include <opm/grid/UnstructuredGrid.h>
 #include <opm/grid/GridManager.hpp>
 #include <opm/grid/cpgrid/GridHelpers.hpp>
@@ -58,42 +68,37 @@
 #include <string>
 #include <vector>
 
-#include <boost/test/unit_test.hpp>
-#include <boost/version.hpp>
-#if BOOST_VERSION / 100000 == 1 && BOOST_VERSION / 100 % 1000 < 71
-#include <boost/test/floating_point_comparison.hpp>
-#else
-#include <boost/test/tools/floating_point_comparison.hpp>
-#endif
-
-
 namespace Opm::Properties {
 
 namespace TTag {
 
-
 struct TestEquilTypeTag {
     using InheritsFrom = std::tuple<FlowBaseProblemBlackoil,
                                     BlackOilModel>;
 };
+
 struct TestEquilVapwatTypeTag {
     using InheritsFrom = std::tuple<FlowBaseProblemBlackoil,
                                     BlackOilModel>;
 };
-}
+
+} // namespace TTag
 
 template<class TypeTag>
 struct WellModel<TypeTag, TTag::TestEquilTypeTag> {
     using type = BlackoilWellModel<TypeTag>;
 };
+
 template<class TypeTag>
 struct EnableVapwat<TypeTag, TTag::TestEquilTypeTag> {
     static constexpr bool value = true;
 };
+
 template<class TypeTag>
 struct WellModel<TypeTag, TTag::TestEquilVapwatTypeTag> {
     using type = BlackoilWellModel<TypeTag>;
 };
+
 template<class TypeTag>
 struct EnableVapwat<TypeTag, TTag::TestEquilVapwatTypeTag> {
     static constexpr bool value = true;
@@ -101,18 +106,20 @@ struct EnableVapwat<TypeTag, TTag::TestEquilVapwatTypeTag> {
 
 } // namespace Opm::Properties
 
+namespace {
+
 template <class TypeTag>
 std::unique_ptr<Opm::GetPropType<TypeTag, Opm::Properties::Simulator>>
 initSimulator(const char *filename)
 {
     using Simulator = Opm::GetPropType<TypeTag, Opm::Properties::Simulator>;
 
-    std::string filenameArg = "--ecl-deck-file-name=";
-    filenameArg += filename;
+    const auto filenameArg = std::string {"--ecl-deck-file-name="} + filename;
 
     const char* argv[] = {
         "test_equil",
-        filenameArg.c_str()
+        filenameArg.c_str(),
+        "--check-satfunc-consistency=false",
     };
 
     Opm::setupParameters_<TypeTag>(/*argc=*/sizeof(argv)/sizeof(argv[0]), argv, /*registerParams=*/false);
@@ -123,7 +130,8 @@ initSimulator(const char *filename)
 }
 
 template <class GridView>
-static std::vector<std::pair<double,double>> cellVerticalExtent(const GridView& gridView)
+std::vector<std::pair<double,double>>
+cellVerticalExtent(const GridView& gridView)
 {
     using ElementMapper = Dune::MultipleCodimMultipleGeomTypeMapper<GridView>;
     ElementMapper elemMapper(gridView, Dune::mcmgElementLayout());
@@ -142,7 +150,7 @@ static std::vector<std::pair<double,double>> cellVerticalExtent(const GridView&
 }
 
 template <class TypeTag>
-static void initDefaultFluidSystem()
+void initDefaultFluidSystem()
 {
     using FluidSystem = Opm::GetPropType<TypeTag, Opm::Properties::FluidSystem>;
 
@@ -179,15 +187,16 @@ static void initDefaultFluidSystem()
     dryGasPvt.setNumRegions(/*numPvtRegion=*/1);
     dryGasPvt.setReferenceDensities(/*regionIdx=*/0, rhoRefO, rhoRefG, rhoRefW);
     dryGasPvt.setGasFormationVolumeFactor(/*regionIdx=*/0, Bg);
-    dryGasPvt.setGasViscosity(/*regionIdx=*/0, mug);
+    dryGasPvt.setGasViscosity(/*regionIdx=*/0, Opm::Tabulated1DFunction<double>(mug));
 
     auto oilPvt = std::make_shared<Opm::OilPvtMultiplexer<double>>();
     oilPvt->setApproach(Opm::OilPvtApproach::DeadOil);
     auto& deadOilPvt = oilPvt->getRealPvt<Opm::OilPvtApproach::DeadOil>();
     deadOilPvt.setNumRegions(/*numPvtRegion=*/1);
     deadOilPvt.setReferenceDensities(/*regionIdx=*/0, rhoRefO, rhoRefG, rhoRefW);
-    deadOilPvt.setInverseOilFormationVolumeFactor(/*regionIdx=*/0, Bo);
-    deadOilPvt.setOilViscosity(/*regionIdx=*/0, muo);
+    deadOilPvt.setInverseOilFormationVolumeFactor(/*regionIdx=*/0,
+                                                  Opm::Tabulated1DFunction<double>(Bo));
+    deadOilPvt.setOilViscosity(/*regionIdx=*/0, Opm::Tabulated1DFunction<double>(muo));
 
     auto waterPvt = std::make_shared<Opm::WaterPvtMultiplexer<double>>();
     waterPvt->setApproach(Opm::WaterPvtApproach::ConstantCompressibilityWater);
@@ -207,11 +216,12 @@ static void initDefaultFluidSystem()
     FluidSystem::initEnd();
 }
 
-static Opm::EquilRecord mkEquilRecord( double datd, double datp,
-                                       double zwoc, double pcow_woc,
-                                       double zgoc, double pcgo_goc )
+Opm::EquilRecord
+mkEquilRecord(const double datd, const double datp,
+              const double zwoc, const double pcow_woc,
+              const double zgoc, const double pcgo_goc)
 {
-    return Opm::EquilRecord( datd, datp, zwoc, pcow_woc, zgoc, pcgo_goc, true, true, 0, true);
+    return { datd, datp, zwoc, pcow_woc, zgoc, pcgo_goc, true, true, 0, true};
 }
 
 template <typename Simulator>
@@ -220,8 +230,6 @@ double centerDepth(const Simulator& sim, const std::size_t cell)
     return Opm::UgGridHelpers::cellCenterDepth(sim.vanguard().grid(), cell);
 }
 
-namespace {
-
 struct EquilFixture {
     EquilFixture() {
         int argc = boost::unit_test::framework::master_test_suite().argc;
@@ -253,7 +261,7 @@ struct EquilFixture {
                                                                         CartesianIndexMapper>;
 };
 
-}
+} // Anonymous namespace
 
 BOOST_GLOBAL_FIXTURE(EquilFixture);
 
@@ -845,7 +853,6 @@ BOOST_AUTO_TEST_CASE(DeckWithCO2STORE)
     }
 }
 
-
 BOOST_AUTO_TEST_CASE(DeckWithWetGas)
 {
     using TypeTag = Opm::Properties::TTag::TestEquilTypeTag;
