@@ -153,7 +153,6 @@ BOOST_AUTO_TEST_CASE(G1)
     well_model.calculateExplicitQuantities(deferred_logger);
     well_model.prepareTimeStep(deferred_logger);
     well_model.updateWellControls(false, deferred_logger);
-    well_model.initPrimaryVariablesEvaluation();
     Opm::WellInterface<TypeTag> *well_ptr = well_model.getWell("B-1H").get();
     StdWell *std_well = dynamic_cast<StdWell *>(well_ptr);
 
@@ -175,7 +174,8 @@ BOOST_AUTO_TEST_CASE(G1)
     WellState &well_state = well_model.wellState();
     const auto &group_state = well_model.groupState();
     GLiftEclWells ecl_well_map;
-    well_model.initGliftEclWellMap(ecl_well_map);
+    Opm::BlackoilWellModelGasLift<TypeTag>::
+        initGliftEclWellMap(well_model.localNonshutWells(), ecl_well_map);
     const int iteration_idx = simulator->model().newtonMethod().numIterations();
     const auto& comm = simulator->vanguard().grid().comm();
     GasLiftGroupInfo group_info {
